/***
*cp.h - Control parameters declarations
*
*$Id: cp.h 6.111 2008/08/22 17:05:54 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file declares the parameter structure, associated macro symbols,
*   and function prototypes associated with overall model control operations.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/04/22 | Added CPRDESCE and CPRDESCF, deleted CPRDESC            |
* | SFG | 07/11/07 | Added OUTBASMPRFLAG                                     |
* | SFG | 07/11/07 | Renamed OUTMPRFLAG to OUTVARMPRFLAG                     |
* | SFG | 07/11/05 | Add CREATEPARMFILES, OUTBASMPR                          |
* | CH  | 07/01/05 | Deleted the SAS output facility                         |
* | SB  | 05/12/21 | Remove BEEPFLAG                                         |
* | BBM | 05/01/11 | Added CPROFILE                                          |
* | BBM | 04/11/09 | Added CPRVER                                            |
* | LP  | 04/03/17 | labels, FAMEX -> SHS                                    |
* | BBM | 02/06/12 | Add after tax LICO option: LICOOPT parameter            |
* | EGA | 99/12/09 | implementation of INEQFLAG and INEQMEASURES             |
* | SFG | 98/04/27 | implement OUTMRSFRAC (preserve fractions always in MRS) |
* | SFG | 98/03/26 | implement marginal tax rate MARBASEFLAG capability      |
* | BBM | 95/06/19 | Parameter label changes                                 |
* | SFG | 93/11/04 | add UPDATEINT                                           |
* | SFG | 93/09/22 | add CLOSEFLAG                                           |
* | SFG | 92/08/26 | removed EX, CL, and related constants and code          |
* | SFG | 90/10/11 | added cpsmpreq for working SAMPLEREQ                    |
* | SFG | 90/10/09 | added cpuvdone                                          |
* | SFG | 90/09/20 | made cptotwgt & cptarwgt double                         |
* | SFG | 90/09/19 | added XTDBLFLAG                                         |
* | SFG | 90/09/07 | added CP.cpphase to make phase generally accessible     |
* | SFG | 90/08/07 | replaced REFVAR by INPREFVARS (variable string)         |
* | SFG | 90/04/24 | made roll-up context global in CP.cplevel               |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Dependencies:
*   #include <stdio.h>   Required for definition of FILE
*   #include "spsm.h"    Required for manifest constants and macros
*   #include "pm.h"      Required for cpc(), cpin(), cpd(), mpin() prototypes
*   #include "vsu.h"     Required by 'vs.h'
*   #include "vs.h"      Required for prototypes with 'hhld' argument
*   #include "cpu.h"     Required for declaration of 'cpu' substructure
*   #include "cp.h"
*
*****************************************************************************/



/* ==================== MANIFEST CONSTANTS ================================ */

/**
* UCNTSIZE determines the amount of space, in bytes, to be reserved
* for the user control structure (identified by 'UC' and declared in cpu.h).
* The difference between UCNTSIZE and the actual size of the user control
* structure (a substructure of the CP structure declared below) is filled
* by a compile-time filler array of char's.
**/

#define UCNTSIZE 601

/**
* Define array bounds.
**/

#define INCGPMAX  10        /* maximum of number of rows in INCGP vector   */
#define PVRATMAX  10        /* maximum of number of rows in PVRAT vector   */
#define DISTPMAX  21        /* maximum of number of rows in DISTP vector   */
#define INEQmax   10		/* maximum of number of rows in INEQMEASURE matrix */
#define INEQcol    2        /* number of colomns in INEQMEASURE            */

/**
Define codes used for BASMETH and VARMETH parameters.
**/

#define ME_NONE 0          /* all result variables set to zero            */
#define ME_READ 1          /* variables come from existing results file   */
#define ME_STD  2          /* variables come from standard algorithm      */
#define ME_ALT  3          /* variables come from alternate algorithm     */

/**
* Define codes used to tell initialisation routines which mode they
* are to perform.
**/

#define CHECK  0           /* routine is to perform a syntax check only */
#define INIT   1           /* routine is to perform initialisation      */

/**
* Define codes to indicate phase of execution.
**/

#define INIT_PHASE     1
#define DIALOGUE_PHASE 2
#define PREPARE_PHASE  3
#define MODEL_PHASE    4
#define REPORT_PHASE   5
#define COMPLETE_PHASE 6



/* ==================== STRUCTURE DECLARATION MACROS ====================== */

#define S_CP   struct CP_
#define P_CP   S_CP *



/* ==================== STRUCTURE DECLARATIONS ============================ */

/**
* The structure identified by the letters 'cp' holds parameter values
* used to control overall model operation.  Values for these parameters
* are contained in files with the '.cpr' extension.  Some of the structure
* members are not specified by the user, but are instead obtained from
* other sources.  Not all of these structure members are found in '.cpr'
* files.
**/

typedef struct CP_ {

	/*	2.1 Model Control Parameters */

	/*	2.1.1 General information */

	char  CPRDESCE[LABSIZE+1];    /* English description of SPSM run [string] */
	char  CPRDESCF[LABSIZE+1];    /* French description of SPSM run [string] */
	char  CPRVER[HDRSIZE+1];     /* SPSD/M Release Version [string]          */
	char  CPRSFILE[PATHSIZE+1];  /* Starting Control parameter file [string] */
	char  LICENSEE[HDRSIZE+1];   /* SPSD/M licensee [string]                 */
	char  AUTHOR[LABSIZE+1];     /* Name of person doing simulation [string] */
	char  OUTCPR[PATHSIZE+1];    /* Name of control parameter file (out) [string] */
	char  *ALGDESC;              /* Names of standard and alternate algorithms */
	int   BRKFLAG;               /* Activate interception of `Break' key     */
	int   CLOSEFLAG;             /* Close window on completion               */
	int   UPDATEINT;             /* Interval between screen updates (hhlds)  */
	int   ETAFLAG;               /* Activate fraction completed display      */
	int   ROUNDFLAG;             /* Round disposable and consumable income   */
	int   LOGFLAG;               /* Produce a .log file for this run         */
	char  OUTLOG[PATHSIZE+1];    /* Name of log file (out) [string]          */
	int   CREATEPARMFILES;       /* Force writing of parameter files         */

	/*	2.1.2 SPSD input files */

	int  DBVERSION;              /* Version of the database (x100)           */
	char INPSPD[PATHSIZE+1];     /* Name of SPSD file (in) [string]          */
	int  FXVFLAG;                /* Read SHS expenditure vector file         */
	char INPFXV[PATHSIZE+1];     /* Name of SHS vector file (in) [string]    */
	int  WGTFLAG;                /* Read weight file                         */
	char INPWGT[PATHSIZE+1];     /* Name of weight file (in) [string]        */

	/* 2.1.2.1 Reference value input file */

	int  REFFLAG;                /* Read reference results file              */
	char INPREF[PATHSIZE+1];     /* Name of reference results file (in) [string] */
	char *REFVARS;            /* Reference results file variables [string] */

	/*	2.1.3 Database adjustment */

	char AGENAME[IDSIZE+1];     /* Name of database adjustment algorithm [string] */
	char INPAPR[PATHSIZE+1];     /* Name of database adjustment parameter file (in) [string] */
	char OUTAPR[PATHSIZE+1];     /* Name of database adjustment parameter file (out) [string] */

	/*	2.1.4 Variant Information */

	char  VARALG[IDSIZE+1];      /* Name of variant algorithm [string]       */
	int   VARMETH;               /* Method of creating variant variables     */
	char  VARDESC[LABSIZE+1];    /* Description of variant parameters [string] */
	char INPVARMPR[PATHSIZE+1];  /* Name of variant tax/transfer parameter file (in) [string] */
	char OUTVARMPR[PATHSIZE+1];  /* Name of variant tax/transfer parameter file (out) [string] */
	int  OUTMRSFLAG;             /* Variant results file creation flag       */
	char OUTVARMRS[PATHSIZE+1];  /* Name of variant results file (out) [string] */
	char *OUTMRSVARS;            /* Variant results file variables [string]  */
	int  OUTMRSFRAC;             /* Variant results file preserves fractional part */

	/*	2.1.5 Base Information */

	char  BASALG[IDSIZE+1];      /* Name of base algorithm [string]          */
	int   BASMETH;               /* Method of creating base variables        */
	char  BASDESC[LABSIZE+1];    /* Description of base parameters [string]  */
	char INPBASMPR[PATHSIZE+1];  /* Name of base tax/transfer parameter file (in) [string] */
	char OUTBASMPR[PATHSIZE+1];  /* Name of base tax/transfer parameter file (out) [string] */
	char INPBASMRS[PATHSIZE+1];  /* Name of base results file (in) [string] */
	char *INPMRSVARS;            /* Base results file variables [string]     */

	/*	2.1.6 Subsampling, random number seed */

	NUMBER SAMPLEREQ;            /* Size of sample requested                 */
	NUMBER SAMPLE;               /* Size of sample obtained                  */
	NUMBER WGTTOT;               /* Sum of weights on tax file               */
	int SEEDrows;                /* Number of rows in SEED vector            */
	int SEED[NUMRAND];           /* Random number generator seeds            */

	/*	2.1.7 Record selection facility */

	int SELFLAG;                 /* Selection facility activation flag       */
	int SELUNIT;                 /* Selection facility family level          */
	char *SELSPEC;               /* Selection specification [string]         */
	int SELMAX;                  /* Selection facility maximum # of households */

	/*	2.1.8 Marginal Tax rate facility */

	int MARFLAG;                 /* Marginal tax rate facility activation flag */
	int MARBASEFLAG;             /* Save marginal results as base run flag */
	NUMBER MARAMT;               /* Amount to be added to variable for marginal calculation */
	char MARVAR[IDSIZE+1];       /* Variable to add MARAMT to [string]       */
	char *MARSPEC;               /* Expression identifying recipients [string] */

	/* 2.1.9 Turning Point Facility */

	int TPFLAG;                 /* Turning point facility activation flag */
	char *TPSPEC;               /* Expression identifying individuals to change [string] */
	char *TPVARS;               /* Variables to modify [string]       */
	int TPMETH;                 /* Method for modifying variables [1=additive, 2=multiplicative] */
	NUMBER TPLL;                /* Lower limit for modified variables */
	NUMBER TPUL;                /* Upper limit for modified variables */

	/*	2.1.10 User-defined Variables */

	int UVARFLAG;                  /* Activate UVAR parameter for expressions */
	char *UVAR;                    /* User statements [string]               */

	/*	2.1.11 Text output facility */

	int ASCFLAG;                 /* Text output facility activation flag     */
	char OUTASC[PATHSIZE+1];     /* Name of text output file (out) [string]  */
	int ASCUNIT;                 /* Text output family level                 */
	int ASCSTYLE;                /* Style of text output                     */
	char ASCDELIM[IDSIZE+1];     /* Field delimiter                          */
	int ASCEXTPRC;               /* Number of digits of extra precision      */
	char *ASCVARS;               /* Variables selected for text output [string] */

	/*	2.1.12 Reports */

	char OUTTBL[PATHSIZE+1];     /* Name of report file (out) [string]       */

	/* 2.1.13 Parameter reporting */

	int PRDFFLAG;                /* Parameter difference report activation flag */

	/* 2.1.14 Tabular reporting   */
	/*	2.1.14.1 Built-in tables */

	int T0FLAG;                  /* Canada totals table flag (Dollars)       */
	int T0AFLAG;                 /* Canada totals table flag (Units)         */
	int T1FLAG;                  /* Provincial totals table flag (Dollars)   */
	int T1AFLAG;                 /* Provincial totals table flag (Units)     */
	int T2FLAG;                  /* Income group totals table flag (Dollars) */
	int T2AFLAG;                 /* Income group totals table flag (Units)   */
	int T3FLAG;                  /* Family type totals table flag (Dollars)  */
	int T3AFLAG;                 /* Family type totals table flag (Units)    */
	int T4FLAG;                  /* LICO ratio group totals table flag (Dollars) */
	int T4AFLAG;                 /* LICO ratio group totals table flag (Units)   */
	int TABUNIT;                 /* Built-in tables family level             */
	NUMBER TABDELTA;             /* Built-in tables winner/loser threshold   */
	char INCVAR[IDSIZE+1];       /* Variable to use for table 2 [string]     */
	int INCGProws;               /* Number of rows in INCGP                  */
	NUMBER INCGP[INCGPMAX];	     /* Income cutpoints for table 2 [array]     */
	int    PVRATrows;            /* Number of rows in PVRAT                  */
	NUMBER PVRAT[PVRATMAX];      /* Family poverty ratio fractions for table 4 [array] */
	int LICOOPT;                 /* T4 LICO definition 1=BeforeTax 2=AfterTax */

	/*	2.1.14.2 User-specified Tabulation facility */

	int XTFLAG;                  /* X-tab facility activation flag           */
	char *XTSPEC;                /* X-tab specification [string]             */
	int XTDBLFLAG;               /* X-tab double precision activation flag   */
	int XTCOLS;                  /* X-tab desired print width                */
	int XTLINES;                 /* X-tab desired lines per page             */

	/*	2.1.14.3 Distributional Analysis Facility */

	int  DISTFLAG;               /* Distribution facility activation flag    */
	int  DISTUNIT;               /* Distribution facility family level       */
	char DISTVAR[IDSIZE+1];      /* Distribution facility variable [string]  */
	int  DISTSAMP;               /* Distribution facility sample size        */
	int  DISTZERO;               /* Distribution facility zero inclusion flag */
	int  DISTProws;              /* Number of rows in DISTP                  */
	int  DISTP[DISTPMAX];        /* Breakpoints for histogram plot [array]   */
	int  DISTPWID;               /* Width of histogram plot                  */
	int  DISTPHGT;               /* Height of histogram plot                 */
	int  INEQFLAG;               /* Inequality measures facility activation flag */
	int  INEQrows; 				  /* Number of rows in INEQMEASURE              */
	NUMBER INEQMEASURE[INEQmax][INEQcol];  /* Type of inequality measure [social preference, center of weight distribution] */



	/**
	* Following structure members are neither read nor output in
	* the .cpr file.
	**/

	char INPCPR[PATHSIZE+1];     /* Name of control parameter file (in) [string]  */
	int  OUTAPRFLAG;             /* Ageing parameter file creation flag           */
	int  OUTVARMPRFLAG;          /* Variant parameter file creation flag          */
	int  OUTBASMPRFLAG;          /* Base parameter file creation flag          */
	int CANCELFLAG;              /* Cancellation request flag                */


	/**
	* the following are global control variables which are not set
	* directly by the user - they are set by model control code.
	**/

	NUMBER cpseqhh;              /* Household currently being processed      */
	double cptotwgt;             /* Running sum of weights of selected households */
	double cptarwgt;             /* Target total weight for subsample        */
	NUMBER cpsmpreq;             /* Working copy of SAMPLEREQ                */
	NUMBER cpnhhsel;             /* Total number of selected households      */
	NUMBER cpninsel;             /* Total number of selected individuals     */
   double cpsumbas;             /* Cumulated base income                    */
   double cpsumvar;             /* Cumulated variant income                 */
	int    cpphase;              /* Phase of model operation (1-5)           */
	int    cpcalflg;             /* Algorithm variables are computed flag    */
	int    cplevel;              /* The current roll-up level                */
	int    cpreqlev[NUMLEV];     /* Family level cumulation request [array]  */
	long   cpnderr;              /* Count of domain errors                   */
	long   cpnxlev;              /* Count of bad cross-level references      */
	unsigned short cprand[NUMRAND]; /* current seed values for random number streams */
	int    cpuvdone;             /* User variables already evaluated         */

	/**
	* The following items include the user-defined control parameter structure
	* and a filler array which expands the size up to UCNTSIZE.
	**/

	S_UC UC;          /* user control parameters (see cpu.h) [struct] */
	char CPfill[UCNTSIZE-sizeof(UC_)]; /* filler to keep CP fixed in size [array] */


} CP_;



/* ==================== GLOBAL VARIABLE DECLARATIONS ====================== */

/**
* The global structure 'CP' holds parameter values used for model control.
* 'CP' is defined and initialised in 'global.c' .
**/

extern S_CP CP;


/**
* STDNAME is the global string used to describe the 'standard' set
* of tax/transfer algorithms used to produce the base or variant
* simulation values.  Similarily, ALTNAME is the global string used to
* identify the user-specified (glass-box mode) alternate system of
* tax/transfer algorithms.
* STDNAME  is defined and initialised in drv.c
* ALTNAME is defined and initialised in Adrv.c
**/

extern char STDNAME[IDSIZE+1];
extern char ALTNAME[IDSIZE+1];


/**
* MSGTEXT contains messages created by fatal(), etc.
**/

extern char MSGTEXT[];


/* ==================== FUNCTION PROTOTYPES =============================== */

#ifdef P_hh    /* declare prototypes taking hhld or sub-structures */
	extern  void maradd(P_hh hhld);
	extern  void marres(P_hh hhld);
	extern  void marrep(P_hh hhld);
	extern  void tpstrt(P_hh hhld);
	extern  int  tpnext(P_hh hhld);
	extern  void ascout(FILE *fp,P_hh hhld);
	extern  void class0(P_hh hhld);
	extern  void iminit(P_hh hhld);
	extern  void modsim(P_hh hhld);
	extern  int  select(P_hh hhld);
	extern  void tbl(P_hh hhld);
	extern  void uxeval(P_hh hhld);
	extern  void xtincr(P_hh hhld);
	extern  void dstupd(P_hh hhld);
#endif

#ifdef P_pc
	extern  int  cpc(P_pc pcp);
	extern  void cpin(P_pc pcp, FILE *fp, FILE *fpcom);
	extern  void cpd(P_pc pcp);
	extern  void mpin(P_pc pcbasp, P_pc pcvarp, FILE *fp, FILE *fpcom);
#endif

extern  int  mainline(int argc, char * * argv);
extern  void marini(void);
extern  void tpini(void);
extern  void tpfin(void);
extern  void ascini(void);
extern  void ascfin(void);
extern  void global(void);
extern  void selini(void);
extern  int  selprs(char *str,struct sel_spec *sp);
extern  void selfin(void);
extern  void tbi(void);
extern  void tbp(FILE *tblfp);
extern  int  uxini(void);
extern  void uxfin(void);
extern  int  xtinit(int mode);
extern  void xtprnt(FILE *xtfp);
extern  int  xttitl(FILE *xtfp);
extern  void dstini(void);
extern  void dstprt(FILE *dstfp);

