/***
*pm.h - Parameter manipulation package declarations
*
*$Id: pm.h 6.102 2008/08/22 17:05:54 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file declares structures, defines symbols, and gives prototypes
*   for the parameter manipulation facilities of SPSM.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | SFG | 04/10/19 | add optional lassification arguments to pmaddent        |
* | SFG | 98/05/12 | increase maximum rows in parms from 63 to 2047          |
* | SFG | 98/04/28 | Fix inappropriate Uninitialized messages on READ, etc.  |
* | SFG | 93/11/12 | Implement parameter name caching                        |
* | SFG | 90/03/19 | Made pecomtel long for big comment files.               |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Dependencies:
*   #include <stdio.h>   Required for definition of FILE
*   #include "spsm.h"    Required for manifest constants and macros
*   #include "pm.h"
*
*****************************************************************************/



/* ==================== MANIFEST CONSTANTS ================================ */

/* values for 'peaggtyp' member of 'pe' */

#define P_SCL   0
#define P_VCT   1
#define P_LKPXY 2    /* lookup table with X-Y pair input format */
#define P_LKPSL 3    /* lookup table with slope input format */
#define P_TBL   4
#define P_DUMMY 5    /* dummy entry to hold name of header string */


/* values for 'peedit' member of 'pe' */

#define E_NONE 0     /* no edit checks                        */
#define E_FIXL 1     /* no changes to array dimension allowed */
#define E_FLAG 2     /* must be 0 or 1                        */
#define E_FRCT 4     /* must be between 0.0 and 1.0 inclusive */
#define E_NOCH 8     /* no changes to value allowed           */
#define E_OPT  16    /* must be 1 to pemax inclusive          */


/* the following macros give the bit field width for elements of pe */

#define W_cols     12
#define W_aggtyp    3
#define W_inter     1  /* unsigned #1 */
#define W_max      11
#define W_edit      5
#define W_ctype     3  /* unsigned #2 */
#define W_inicnt    3
#define W_assign    1
#define W_format    4


/* the following constants tell pmeledit if interactive or not */

#define PMBATCH 0
#define PMINTER 1


/* the followin constants turn the name caching feature on or off */

#define PMCACHEOFF 0
#define PMCACHEON  1



/* ==================== STRUCTURE DECLARATION MACROS ====================== */
 
#define S_pe   struct pe_
#define P_pe   S_pe *

#define S_pc   struct pc_
#define P_pc   S_pc *



/* ==================== STRUCTURE DECLARATIONS ============================ */

/**
* The structure identified by the letters 'pe' holds data related to 
* the type, size and location in memory of a particular parameter
* value.  It also contains information on how the parameter is to be       
* formatted on output.  A number of 'pe' structures can be linked      
* together to form a simple list.
**/

typedef struct pe_ {
	P_pe  penext;                  /* next element in parameter list        */
	char  *pename;                 /* pointer to parameter name             */
	char  *pepval;                 /* pointer to parameter value            */
	char  *perowclass;             /* pointer to row classification name    */
	char  *pecolclass;             /* pointer to column classification name */
	int   *peprows;                /* pointer to number of rows             */
	long  pecomtel;                /* location of comments in comment file  */
	char  pecomcnt;                /* number of lines of comments           */
	unsigned pecols   :W_cols;     /* number of columns                     */
	unsigned peaggtyp :W_aggtyp;   /* type of parameter                     */
	unsigned peinter  :W_inter;    /* interactive mode flag                 */
	unsigned pemax    :W_max;      /* maximum value (for edits)             */
	unsigned peedit   :W_edit;     /* edit actions                          */
	unsigned pectype  :W_ctype;    /* C data type                           */
	unsigned peinicnt :W_inicnt;   /* number of times parm value changed    */
	unsigned peassign :W_assign;   /* a value has been assinged             */
	unsigned pewid0   :W_format;   /* width     #0                          */
	unsigned peprec0  :W_format;   /* precision #0                          */
	unsigned pewid1   :W_format;   /* width     #1                          */
	unsigned peprec1  :W_format;   /* precision #1                          */
	unsigned pewid2   :W_format;   /* width     #2                          */
	unsigned peprec2  :W_format;   /* precision #2                          */
} pe_;


/**
* The structure identified by the letters 'pc' maintains information 
* pertaining to an entire linked list of 'pe' structures.
**/

typedef struct pc_ {
	P_pe  pchead;         /* head of list [pointer]               */
	P_pe  pctail;         /* tail of list [pointer]               */
	P_pe  pclast;         /* entry last found by pmsrch [pointer] */
   int   pccount;        /* number of entries in parameter chain */
   int   pcstatus;       /* either PMCACHEON or PMCACHEOFF       */
   P_pe *pcsorted;       /* sorted array of pe's if pcstat==PMCACHEON */
} pc_;



/* ==================== FUNCTION PROTOTYPES =============================== */

extern  void pmaddent(P_pc pc, char NEAR * name,char *addr,\
	char NEAR * dumpformat,int aggtyp,int ctype,int edit,int max,\
	int *count,int tabsiz,\
	char NEAR *rowclass = NULL, char NEAR *colclass = NULL);
extern  void   pmcache(P_pc pc, int status);
extern  int CDECL pmpecmp(P_pe *arg1, P_pe *arg2);
extern  void   pmdump(P_pc pc, FILE *fp, FILE *fpcom, int chgmsg);
extern  void   pmdmp1(P_pe pe, FILE *fp, FILE *fpcom, int chgmsg, \
	int linwid, int indent);
extern  void   pmcomp(P_pc pc, void *base, void *p1, void *p2,\
	FILE *fp, FILE *fpcom);
extern  char * pmeldump(char *buf, unsigned bufmin, P_pe pe, int j, int k);
extern  int    pmeledit(char *s, P_pe pe, int j, int k, int intflag);
extern  char * pmelput(char *s, P_pe pe, int j, int k);
extern  int    pmget(P_pc pc, FILE *fp, FILE *fpcom, char *incext);
extern  void   pmhelp(P_pc pc);
extern  int    pmload(P_pc pc, FILE *fp, int modflag, FILE *fpcom);
extern  P_pe   pmsrch(P_pc pc, char *name);
extern  void   pmlkfill(P_pe pe);
extern  void   pmfree(P_pc pc);
