/***
*sub.h - Subroutine library declarations
*
*$Id: sub.h 6.101 2008/06/12 21:23:23 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This header file is included by every source code module in SPSM
*   immediately after the "spsm.h" include file.  Its primary function
*   is to define symbols and prototypes associated with the subroutine
*   library support package (source is in the directory SPSM/sub).
*   It also contains a number of stand-alone macros used to do
*   'bit-twiddling' operations on floating point numbers.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | SFG | 07/11/08 | implement cover macros for isupper, etc.                |
* | SFG | 98/04/06 | re-write to enable OEM / ANSI capabilities              |
* | SFG | 98/04/06 | add lputs() for unformatted logged output               |
* | SFG | 93/03/08 | added prototype for randrnd()                           |
* | SFG | 93/01/19 | added type argument to strclr(), new strnote(), etc.    |
* | SFG | 92/09/01 | added spcalloc                                          |
* | SFG | 92/07/28 | add prototypes for some funcs to avoid warnings in UNIX |
* | SFG | 92/07/15 | add LINEDELIM                                           |
* | SFG | 91/10/24 | always use trmalloc, etc. ; define flags for db_mem     |
* | SFG | 90/03/26 | Programmed spmalloc() and spfree() cover macros         |
* | SFG | 90/03/26 | Changed DEBUG_ON() to declare name as 'static char who' |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Dependencies:
*   #include <stdio.h>
*   #include "sub.h"
*
*Definitions:
*   spmalloc [macro ]  Allocate memory (cover macro)
*   spcalloc [macro ]  Allocate memory items (cover macro)
*   sprealloc [macro ]  Reallocate memory (cover macro)
*   spfree   [macro ]  Free memory (cover macro)
*   iszero   [macro ]  Is argument zero ?
*   isnzero  [macro ]  Is argument non-zero ?
*   isneg    [macro ]  Is argument negative ?
*   isnneg   [macro ]  Is argument non-negative ?
*   taxbak1  [macro ]  One level tax-back function
*   taxbak2  [macro ]  Two level tax-back function
*   taxbak3  [macro ]  Three level tax-back function
*   taxbak4  [macro ]  Four level tax-back function
*   nneg     [macro ]  Change negative numbers to zero
*   minn     [macro ]  Find the minimum of two numbers
*   maxn     [macro ]  Find the maximum of two numbers
*   round    [macro ]  Round to nearest integer
*   mini     [macro ]  Find the minimum of two quantities
*   maxi     [macro ]  Find the maximum of two quantities
*   sp_isalnum [macro ]  Is alphanumeric
*   sp_isalpha [macro ]  Is alphabetic
*   sp_isdigit [macro ]  Is digit
*   sp_islower [macro ]  
*   sp_isspace [macro ]  Is white space
*   sp_isupper [macro ]  Is uppercase
*   memmove  [global]  
*   unix_memmove [global]  
*   strcmpi  [global]  
*   unix_strcmpi [global]  
*   strrev   [global]  
*   unix_strrev [global]  
*   strupr   [global]  
*   unix_strupr [global]  
*   strlwr   [global]  
*   unix_strlwr [global]  
*   strstr   [global]  
*   unix_strstr [global]  
*
*****************************************************************************/

/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */
#include <stdarg.h>

/* ==================== MANIFEST CONSTANTS AND MACRO DEFINITIONS ========== */

/* define maximum chars in string MSGTEXT */

#define MSGSIZE    1023      /* maximum error message length              */


/* define maximum chars in LINEDELIM */

#define LINEDELIMSIZE 3


/* define maximum chars in fnlog */

#define FNLOGSIZE    20


/* define codes used for the global LANGUAGE variable */

#define ENGLISH 0
#define FRENCH  1


/* define the number of columns used by a lookup structure */

#define LKP_COLS 3

/* define bits used in the db_mem global variable */
#define DB_MEM_TRACE      1   // prints message on every malloc & free call
#define DB_MEM_INTEGRITY  2   // checks heap intergity on every malloc & free

/* define manifests used as arguments to strclr() */
#define SC_ALL   0  /* clear all stradd() entries */
#define SC_NEW   1  /* clear all stradd() entries made subsequent to last call to strnote() */


/**
* Define the debug and function tracing macros.  Note that the first
* argument of the DEBUGn macros (a format control string) is stored
* as a static variable in a local block.  This allows it to be placed
* in FAR memory (NEAR memory is at a premium).  The DEBUG_ON and DEBUG_OFF
* macros serve a double purpose.  They expand into appropriate fprintf
* statements if DEBUG statements are activated, and also keep a record
* of the function calling sequence if TRACE is activated.
**/

#ifdef DEBUG

#define DEBUG1(e1) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg);\
}
#define DEBUG2(e1,e2) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2);\
}
#define DEBUG3(e1,e2,e3) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3);\
}
#define DEBUG4(e1,e2,e3,e4) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3,e4);\
}
#define DEBUG5(e1,e2,e3,e4,e5) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3,e4,e5);\
}
#define DEBUG6(e1,e2,e3,e4,e5,e6) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3,e4,e5,e6);\
}
#define DEBUG7(e1,e2,e3,e4,e5,e6,e7) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3,e4,e5,e6,e7);\
}
#define DEBUG8(e1,e2,e3,e4,e5,e6,e7,e8) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3,e4,e5,e6,e7,e8);\
}
#define DEBUG9(e1,e2,e3,e4,e5,e6,e7,e8,e9) {\
	static char FAR db_e1[] = e1;\
	lprintf(db_e1,db_sg,e2,e3,e4,e5,e6,e7,e8,e9);\
}

#ifndef NTRACE
#define DEBUG_ON(e1) static char who[]=e1;\
	tr_fn[tr_lv++]=who;if(!dbinit){db_lv=db_sg+2;++dbinit;};\
	lprintf(db_emsg,db_sg,who);\
	*(db_lv++)='.';*(db_lv++)='.';
#define DEBUG_OFF(e1) tr_lv--;*(--db_lv)='\0';*(--db_lv)='\0';\
	lprintf(db_lmsg,db_sg,who);
#else
#define DEBUG_ON(e1) static char who[]=e1;\
	if(!dbinit){db_lv=db_sg+2;++dbinit;};\
	lprintf(db_emsg,db_sg,who);\
	*(db_lv++)='.';*(db_lv++)='.';
#define DEBUG_OFF(e1) *(--db_lv)='\0';*(--db_lv)='\0';\
	lprintf(db_lmsg,db_sg,who);
#endif

#else

#define DEBUG1(e1) ;
#define DEBUG2(e1,e2) ;
#define DEBUG3(e1,e2,e3) ;
#define DEBUG4(e1,e2,e3,e4) ;
#define DEBUG5(e1,e2,e3,e4,e5) ;
#define DEBUG6(e1,e2,e3,e4,e5,e6) ;
#define DEBUG7(e1,e2,e3,e4,e5,e6,e7) ;
#define DEBUG8(e1,e2,e3,e4,e5,e6,e7,e8) ;
#define DEBUG9(e1,e2,e3,e4,e5,e6,e7,e8,e9) ;

#ifndef NTRACE
#define DEBUG_ON(e1) static char who[]=e1;\
	{tr_fn[tr_lv]=who;tr_lv++;if(tr_lv>=50)tr_lv=0;}
#define DEBUG_OFF(e1) {tr_lv--;if(tr_lv<0)tr_lv=0;}
#else
#define DEBUG_ON(e1) static char who[]=e1;
#define DEBUG_OFF(e1) ;
#endif

#endif


/**
* Define the spmalloc(), spcalloc(), and spfree() cover macros.  These macros
* map directly to the corresponding trmalloc(), etc. functions at the moment.
**/

/*****************************************************************************
*spmalloc - Allocate memory (cover macro)
*
*Calls:
*   trmalloc Tracing version of malloc()
*
*Called by:
*   ConvertANSItoOEM converts ANSI -> OEM using local buffer
*   ascini   Specify SPSD/M variables in ASCII form
*   dstini   Parse distribution report request
*   dstprt   Print distribution report
*   exbrkfmt Create level labels from break values
*   exlvar   Perform special side effect actions for assignments
*   fmfxopen Open FAMEX (.fxv) file
*   fmriopen Open old SPSM result file (.mrs)
*   fmroopen Open new SPSM result file (.mrs)
*   fmrvopen Open reference value SPSM result file (.mrs)
*   fmspopen Open .spd binary file and prepare for reading
*   fmwiopen Open input weight file (.wgt)
*   gsub     Replace all occurences of a substring by another
*   hsort    Perform heap sort
*   indaie   Retrieve j'th element from linked list, extending list if necessary
*   nalloc   Allocate a new node for use in an expression tree
*   pmaddent Define and add a parameter
*   pmcache  Activate/de-activate caching for a parameter chain
*   pmdmp1   Write current value of a parameter
*   pmget    Query user for parameter changes
*   pmload   Read a parameter text file into a parameter list
*   strini   Initialise a message library
*   strspsm  Initialise the SPSM message library
*   tbi      Process user's hard-wired table requests
*   tpcreate Insert, remove, and adjust turning points
*   tpini    Process turning point facility request
*   tpinit   Create starting linked list for turning points
*   vmadd    Define an SPSM variable
*   xtdo1    Format and print one cross-tabulation
*   xtinit   Process user X-tab requests
*
*****************************************************************************/
#define spmalloc(e1) trmalloc(e1, who)

/*****************************************************************************
*spcalloc - Allocate memory items (cover macro)
*
*Calls:
*   trcalloc Tracing version of calloc()
*
*Called by:
*   ascini   Specify SPSD/M variables in ASCII form
*
*****************************************************************************/
#define spcalloc(e1,e2) trcalloc(e1, e2, who)

/*****************************************************************************
*sprealloc - Reallocate memory (cover macro)
*
*Calls:
*   trrealloc Tracing version of realloc()
*
*Called by:
*   ConvertANSItoOEM converts ANSI -> OEM using local buffer
*   cpalg    Create ALGDESC parameter from global module strings
*   exbrkfmt Create level labels from break values
*   exlvar   Perform special side effect actions for assignments
*   expbuf   Expand buffer size if possible
*   fmriopen Open old SPSM result file (.mrs)
*   fmrvopen Open reference value SPSM result file (.mrs)
*   gsub     Replace all occurences of a substring by another
*   pmdmp1   Write current value of a parameter
*   pmeldump Format a parameter as a string
*   pmelput  Convert a string into a parameter
*   pmget    Query user for parameter changes
*   pmload   Read a parameter text file into a parameter list
*
*****************************************************************************/
#define sprealloc(e1,e2) trrealloc(e1, e2, who)

/*****************************************************************************
*spfree - Free memory (cover macro)
*
*Calls:
*   trfree   Tracing version of free()
*
*Called by:
*   ascfin   Clean up after ASCII facility
*   dstprt   Print distribution report
*   exclr    Frees the space occupied by an expression tree
*   exlvar   Perform special side effect actions for assignments
*   fmfxclos Close FAMEX (.fxv) file
*   fmriclos Close old SPSM result file (.mrs)
*   fmroclos Close new SPSM result file (.mrs)
*   fmroopen Open new SPSM result file (.mrs)
*   fmrvclos Close reference value SPSM result file (.mrs)
*   fmrvopen Open reference value SPSM result file (.mrs)
*   fmspclos Close SPSD file (.spd)
*   fmwiclos Close input weight file (.wgt)
*   gsub     Replace all occurences of a substring by another
*   hsort    Perform heap sort
*   pmcache  Activate/de-activate caching for a parameter chain
*   pmdmp1   Write current value of a parameter
*   pmfree   Free up a complete parameter chain
*   pmget    Query user for parameter changes
*   pmload   Read a parameter text file into a parameter list
*   strini   Initialise a message library
*   tbp      Print hard-wired tables
*   tpcreate Insert, remove, and adjust turning points
*   tpfin    Clean up after ASCII facility
*   tpinit   Create starting linked list for turning points
*   vmcuvar  Clear user-defined variables from variable list
*   vmfree   Free memory used by variable entry ring
*   xtdo1    Format and print one cross-tabulation
*   xtinit   Process user X-tab requests
*   xtprnt   Format and print user X-tabs
*
*****************************************************************************/
#define spfree(e1)       trfree(e1, who)




/**
* Define the bit-twiddling macros for floats and doubles.
* What happens depends on the machine type and the macro NUMFLOAT.
* If NUMFLOAT (and not NUMDOUBLE) is defined, the macros operate on
* the C-type float instead of double.
**/

#if defined(_M_IX86)
#ifdef NUMFLOAT
#define SIGN_BIT 128  /* sign bit is 8th bit in EXP_BYTE    */
#define EXP_BYTE   3  /* offset to byte containing exponent */
#define MANT_HBYTE 2  /* offset to most significant byte of mantissa */
#else /* NUMDOUBLE */
#define SIGN_BIT 128  /* sign bit is 8th bit in EXP_BYTE    */
#define EXP_BYTE   7  /* offset to byte containing exponent */
#define MANT_HBYTE 6  /* offset to most significant byte of mantissa */
#endif
#endif

#ifdef I370
#define SIGN_BIT 128
#define EXP_BYTE   0
#define MANT_HBYTE 1
#endif


/*****************************************************************************
*iszero - Is argument zero ?
*
*Synopsis:
*   int iszero(arg);
*   NUMBER arg;
*
*Description:
*   This macro determines if the argument is zero, in an efficient way
*   using facts about the internal representation of floating point numbers.
*
*Return Value:
*   0 if false, non-zero if true.
*
*Called by:
*   mpc      Calculate derived model parameters and do edits
*   samod    Compute social assistance or guarantees
*   txqcalc  Calculate federal income tax
*   txqccea  Compute child care expense allowance (Quebec)
*
*****************************************************************************/
#if defined(_M_IX86) || defined(I370)
#define iszero(X) (!(*(((char *) &(X)) + EXP_BYTE)))
#else
#define iszero(X) ((X) == ZERO)
#endif

/**
* is sufficient on 8087 to test for exponent byte = 0 for number to = 0
* until we figure out 370 floating point, macro to equivalent C expression
**/


/*****************************************************************************
*isnzero - Is argument non-zero ?
*
*Synopsis:
*   int isnzero(arg);
*   NUMBER arg;
*
*Description:
*   This macro determines if the argument is non-zero, in an efficient way
*   using facts about the internal representation of floating point numbers.
*
*Return Value:
*   0 if false, non-zero if true.
*
*Called by:
*   adj      Perform SPSD database adjustment
*   ctmod    Compute commodity taxes
*   gis      Compute GIS/SPA for elderly
*   gist     Compute Provincial GIS top-ups for elderly
*   phase4   Perform simulation
*   pmlkfill Compute derived elements of a lookup table
*   samod    Compute social assistance or guarantees
*   taxbd1   One level tax-back function
*   taxbd2   Two level tax-back function
*   taxbd3   Three level tax-back function
*   taxbd4   Four level tax-back function
*   taxbf1   One level tax-back function
*   taxbf2   Two level tax-back function
*   taxbf3   Three level tax-back function
*   taxbf4   Four level tax-back function
*   tbl      Increment hard-wired tables
*   txalta   Compute provincial taxes for Alberta
*   txbc     Compute provincial taxes for British Columbia
*   txcalc   Calculate federal income tax
*   txccea   Compute child care expense allowance
*   txctc    Compute child tax credit
*   txhhexp  Compute and pro-rate household taxes, rent, etc.
*   txinet   Compute net income
*   txitax   Compute taxable income
*   txman    Compute provincial taxes for Manitoba
*   txnb     Compute provincial taxes for New Brunswick
*   txnfld   Compute provincial taxes for Newfoundland
*   txns     Compute provincial taxes for Nova Scotia
*   txont    Compute provincial taxes for Ontario
*   txpei    Compute provincial taxes for P.E.I.
*   txqcalc  Calculate federal income tax
*   txqccea  Compute child care expense allowance (Quebec)
*   txqhstr  Apply tax transfers between head and spouse (Quebec)
*   txqinet  Compute net income (Quebec)
*   txqitax  Compute taxable income (Quebec)
*   txsask   Compute provincial taxes for Saskatchewan
*   vmroll   Perform family cumulation to specified level
*
*****************************************************************************/
#if defined(_M_IX86) || defined(I370)
#define isnzero(X) ((*(((char *) &(X)) + EXP_BYTE)))
#else
#define isnzero(X) ((X) != ZERO)
#endif



/*****************************************************************************
*isneg - Is argument negative ?
*
*Synopsis:
*   int isneg(arg);
*   NUMBER arg;
*
*Description:
*   This macro determines if the argument is negative, in an efficient way
*   using facts about the internal representation of floating point numbers.
*
*Return Value:
*   0 if false, non-zero if true.
*
*Called by:
*   fxgrow   grow FAMEX data
*   memo1    Compute memo items for reporting
*
*****************************************************************************/
#if defined(_M_IX86) || defined(I370)
#define isneg(X) (*(((char *) &(X)) + EXP_BYTE) & SIGN_BIT)
#else
#define isneg(X) ((X) < ZERO)
#endif



/*****************************************************************************
*isnneg - Is argument non-negative ?
*
*Synopsis:
*   int isnneg(arg);
*   NUMBER arg;
*
*Description:
*   This macro determines if the argument is non-negative, in an efficient way
*   using facts about the internal representation of floating point numbers.
*
*Return Value:
*   0 if false, non-zero if true.
*
*Called by:
*   ctmod    Compute commodity taxes
*   txqinet  Compute net income (Quebec)
*
*****************************************************************************/
#if defined(_M_IX86) || defined(I370)
#define isnneg(X) (!(*(((char *) &(X)) + EXP_BYTE) & SIGN_BIT))
#else
#define isnneg(X) ((X) >= ZERO)
#endif




/* define different versions of the taxbak functions depending on NUMFLOAT */

#ifdef NUMFLOAT
/*****************************************************************************
*taxbak1 - One level tax-back function
*
*Synopsis:
*   NUMBER taxbak1(inc, exmpt, lvl1, rr1)
*   NUMBER inc;        income subject to tax back
*   NUMBER exmpt;      exemption on income for tax back
*   NUMBER *lvl1;      level of benefit (to be reduced)
*   NUMBER rr1;        reduction rate for tax back
*
*Description:
*
*Return Value:
*   The NUMBER value returned is the income in excess of the break-even value,
*   i.e. income not yet used for taxing back the benefit.
*
*Calls:
*   taxbf1   One level tax-back function
*   taxbd1   One level tax-back function
*
*Called by:
*   gis      Compute GIS/SPA for elderly
*   gist     Compute Provincial GIS top-ups for elderly
*   txalta   Compute provincial taxes for Alberta
*   txbc     Compute provincial taxes for British Columbia
*   txctc    Compute child tax credit
*   txfstc   Compute federal sales tax credit
*   txnb     Compute provincial taxes for New Brunswick
*   txnfld   Compute provincial taxes for Newfoundland
*   txont    Compute provincial taxes for Ontario
*   txpei    Compute provincial taxes for P.E.I.
*
*****************************************************************************/
#define taxbak1(a,b,c,d) (float) taxbf1(a,b,c,d)



/*****************************************************************************
*taxbak2 - Two level tax-back function
*
*Synopsis:
*   NUMBER taxbak2(inc, exmpt, lvl1, rr1, lvl2, rr2)
*   NUMBER inc;       income subject to tax back
*   NUMBER exmpt;     exemption on income for tax back
*   NUMBER *lvl1;     level (#1) of benefit (to be reduced)
*   NUMBER rr1;       reduction rate (#1) for tax back
*   NUMBER *lvl2;     level (#2) of benefit (to be reduced)
*   NUMBER rr2;       reduction rate (#2) for tax back
*
*Description:
*
*Return Value:
*   The NUMBER value returned is the income in excess of the break-even value,
*   i.e. income not yet used for taxing back the benefit.
*
*Calls:
*   taxbf2   Two level tax-back function
*   taxbd2   Two level tax-back function
*
*Called by:
*   gis      Compute GIS/SPA for elderly
*
*****************************************************************************/
#define taxbak2(a,b,c,d,e,f) (float) taxbf2(a,b,c,d,e,f)



/*****************************************************************************
*taxbak3 - Three level tax-back function
*
*Synopsis:
*   NUMBER taxbak3(inc, exmpt, lvl1, rr1, lvl2, rr2, lvl3, rr3)
*   NUMBER inc;       income subject to tax back
*   NUMBER exmpt;     exemption on income for tax back
*   NUMBER *lvl1;     level (#1) of benefit (to be reduced)
*   NUMBER rr1;       reduction rate (#1) for tax back
*   NUMBER *lvl2;     level (#2) of benefit (to be reduced)
*   NUMBER rr2;       reduction rate (#2) for tax back
*   NUMBER *lvl3;     level (#3) of benefit (to be reduced)
*   NUMBER rr3;       reduction rate (#3) for tax back
*
*Description:
*
*Return Value:
*   The NUMBER value returned is the income in excess of the break-even value,
*   i.e. income not yet used for taxing back the benefit.
*
*Calls:
*   taxbf3   Three level tax-back function
*   taxbd3   Three level tax-back function
*
*****************************************************************************/
#define taxbak3(a,b,c,d,e,f,g,h) (float) taxbf3(a,b,c,d,e,f,g,h)



/*****************************************************************************
*taxbak4 - Four level tax-back function
*
*Synopsis:
*   NUMBER taxbak3(inc, exmpt, lvl1, rr1, lvl2, rr2, lvl3, rr3, lvl4, rr4)
*   NUMBER inc;       income subject to tax back
*   NUMBER exmpt;     exemption on income for tax back
*   NUMBER *lvl1;     level (#1) of benefit (to be reduced)
*   NUMBER rr1;       reduction rate (#1) for tax back
*   NUMBER *lvl2;     level (#2) of benefit (to be reduced)
*   NUMBER rr2;       reduction rate (#2) for tax back
*   NUMBER *lvl3;     level (#3) of benefit (to be reduced)
*   NUMBER rr3;       reduction rate (#3) for tax back
*   NUMBER *lvl4;     level (#4) of benefit (to be reduced)
*   NUMBER rr4;       reduction rate (#4) for tax back
*
*Description:
*
*Return Value:
*   The NUMBER value returned is the income in excess of the break-even value,
*   i.e. income not yet used for taxing back the benefit.
*
*Calls:
*   taxbf4   Four level tax-back function
*   taxbd4   Four level tax-back function
*
*****************************************************************************/
#define taxbak4(a,b,c,d,e,f,g,h,i,j) (float) taxbf4(a,b,c,d,e,f,g,h,i,j)



#else

/**
* Define the double version of the taxbak macros.
* Descriptions are identical to those given above
**/

#define taxbak1(a,b,c,d) taxbd1(a,b,c,d)
#define taxbak2(a,b,c,d,e,f) taxbd2(a,b,c,d,e,f)
#define taxbak3(a,b,c,d,e,f,g,h) taxbd3(a,b,c,d,e,f,g,h)
#define taxbak4(a,b,c,d,e,f,g,h,i,j) taxbd4(a,b,c,d,e,f,g,h,i,j)
#endif



/* Define NUMBER-independent macros for certain functions */

/*****************************************************************************
*nneg - Change negative numbers to zero
*
*Synopsis:
*   NUMBER nneg(arg)
*   NUMBER arg;
*
*Return Value:
*   The NUMBER value returned equal to 'arg' if 'arg' is positive or zero.
*   If 'arg' is negative, the return value is zero.
*
*Calls:
*   nnegd    Change negative numbers to zero
*
*Called by:
*   ctmod    Compute commodity taxes
*   exevl    Evaluates a previously parsed expression
*   gissub   Compute income used to tax back GIS/SPA
*   gist     Compute Provincial GIS top-ups for elderly
*   samod    Compute social assistance or guarantees
*   taxbd2   Two level tax-back function
*   taxbd3   Three level tax-back function
*   taxbd4   Four level tax-back function
*   txalta   Compute provincial taxes for Alberta
*   txbc     Compute provincial taxes for British Columbia
*   txcalc   Calculate federal income tax
*   txccea   Compute child care expense allowance
*   txctc    Compute child tax credit
*   txfstc   Compute federal sales tax credit
*   txhstr   Apply tax transfers between head and spouse
*   txinet   Compute net income
*   txitax   Compute taxable income
*   txman    Compute provincial taxes for Manitoba
*   txnb     Compute provincial taxes for New Brunswick
*   txnfld   Compute provincial taxes for Newfoundland
*   txns     Compute provincial taxes for Nova Scotia
*   txont    Compute provincial taxes for Ontario
*   txpei    Compute provincial taxes for P.E.I.
*   txqcalc  Calculate federal income tax
*   txqccea  Compute child care expense allowance (Quebec)
*   txqhstr  Apply tax transfers between head and spouse (Quebec)
*   txqinet  Compute net income (Quebec)
*   txqitax  Compute taxable income (Quebec)
*   txsask   Compute provincial taxes for Saskatchewan
*   ui       Compute UI benefits
*   uiclm    Simulate a UI claim
*
*****************************************************************************/
#define nneg(X)   (NUMBER) nnegd(X)



/*****************************************************************************
*minn - Find the minimum of two numbers
*
*Synopsis:
*   NUMBER minn(arg1, arg2)
*   NUMBER arg1;
*   NUMBER arg2;
*
*Return Value:
*   The NUMBER value returned equal to the minimum of 'arg1' and 'arg2'.
*
*Calls:
*   mind     Minimum of two NUMBERs
*
*Called by:
*   gissub   Compute income used to tax back GIS/SPA
*   mpc      Calculate derived model parameters and do edits
*   samod    Compute social assistance or guarantees
*   txalta   Compute provincial taxes for Alberta
*   txbc     Compute provincial taxes for British Columbia
*   txcalc   Calculate federal income tax
*   txctc    Compute child tax credit
*   txfstc   Compute federal sales tax credit
*   txhstr   Apply tax transfers between head and spouse
*   txinet   Compute net income
*   txitax   Compute taxable income
*   txman    Compute provincial taxes for Manitoba
*   txnb     Compute provincial taxes for New Brunswick
*   txnfld   Compute provincial taxes for Newfoundland
*   txns     Compute provincial taxes for Nova Scotia
*   txont    Compute provincial taxes for Ontario
*   txpei    Compute provincial taxes for P.E.I.
*   txqcalc  Calculate federal income tax
*   txqhstr  Apply tax transfers between head and spouse (Quebec)
*   txqinet  Compute net income (Quebec)
*   txqitax  Compute taxable income (Quebec)
*   txsask   Compute provincial taxes for Saskatchewan
*   ui       Compute UI benefits
*   uiclm    Simulate a UI claim
*
*****************************************************************************/
#define minn(X,Y) (NUMBER) mind(X,Y)



/*****************************************************************************
*maxn - Find the maximum of two numbers
*
*Synopsis:
*   NUMBER maxn(arg1, arg2)
*   NUMBER arg1;
*   NUMBER arg2;
*
*Return Value:
*   The NUMBER value returned equal to the maximum of 'arg1' and 'arg2'.
*
*Calls:
*   maxd     Maximum of two NUMBERs
*
*Called by:
*   txman    Compute provincial taxes for Manitoba
*   txnfld   Compute provincial taxes for Newfoundland
*   txont    Compute provincial taxes for Ontario
*   txqcalc  Calculate federal income tax
*   ui       Compute UI benefits
*   uiclm    Simulate a UI claim
*
*****************************************************************************/
#define maxn(X,Y) (NUMBER) maxd(X,Y)



/*****************************************************************************
*round - Round to nearest integer
*
*Synopsis:
*   NUMBER round(arg)
*   NUMBER arg;
*
*Description:
*   This version relies on truncation of conversions from float or double
*   to longs.  The constant HALF is defined in spsm.h.
*
*Return Value:
*   The NUMBER value returned is equal to the nearest integer, but still
*   expressed as a floating-point quantity.
*
*Called by:
*   cceopt   Multiple calls of tax/transfer calculator
*   drvcall  Call driver and do requested marginal calculations
*   txinet   Compute net income
*   txqinet  Compute net income (Quebec)
*   uiclm    Simulate a UI claim
*
*****************************************************************************/
#define round(X) (NUMBER)(long)((X)+HALF)



/*****************************************************************************
*mini - Find the minimum of two quantities
*
*Synopsis:
*   mini(arg1, arg2)
*
*Description:
*   If using this macro, ensure that neither of the two arguments has
*   side effects.  Because one of the arguments will be evaluated twice,
*   unexpected things can happen unless this rule is followed.
*
*Return Value:
*   The value returned is equal to the minimum of 'arg1' and 'arg2'.
*
*Called by:
*   uiclm    Simulate a UI claim
*
*****************************************************************************/
#define mini(X,Y) (((X)<(Y)) ? (X) : (Y))



/*****************************************************************************
*maxi - Find the maximum of two quantities
*
*Synopsis:
*   maxi(arg1, arg2)
*
*Description:
*   If using this macro, ensure that neither of the two arguments has
*   side effects.  Because one of the arguments will be evaluated twice,
*   unexpected things can happen unless this rule is followed.
*
*Return Value:
*   The value returned is equal to the maximum of 'arg1' and 'arg2'.
*
*Called by:
*   oas      Compute OAS for elderly
*   pmeldump Format a parameter as a string
*   uiclm    Simulate a UI claim
*
*****************************************************************************/
#define maxi(X,Y) (((X)>(Y)) ? (X) : (Y))


/**
* Define cover macros for character classification routines.
* These versions are robust to French accented char values,
* which would otherwise be treated as invalid negative quantities
* by the underlying functions.
**/


/*****************************************************************************
*sp_isalnum - Is alphanumeric
*
*Synopsis:
*   sp_isalnum(arg1)
*
*Description:
*   Version robust to ANSI char values, e.g. French accented characters.
*
*Return Value:
*   As for underlying function.
*
*Called by:
*   fnexp    Construct expanded file name
*   pmload   Read a parameter text file into a parameter list
*   strmfe   Changes filename extension
*   xtinit   Process user X-tab requests
*
*C-Runtime Calls:
*   isalnum  Tests for alphanumeric character
*
*****************************************************************************/
#define sp_isalnum(X) isalnum((unsigned char)(X))



/*****************************************************************************
*sp_isalpha - Is alphabetic
*
*Synopsis:
*   sp_isalpha(arg1)
*
*Description:
*   Version robust to ANSI char values, e.g. French accented characters.
*
*Return Value:
*   As for underlying function.
*
*C-Runtime Calls:
*   isalpha  Tests for alphabetic character
*
*****************************************************************************/
#define sp_isalpha(X) isalpha((unsigned char)(X))



/*****************************************************************************
*sp_isdigit - Is digit
*
*Synopsis:
*   sp_isdigit(arg1)
*
*Description:
*   Version robust to ANSI char values, e.g. French accented characters.
*
*Return Value:
*   As for underlying function.
*
*Called by:
*   help     Displays help screens to user
*   pmload   Read a parameter text file into a parameter list
*   pmload1  Process information for a single parameter
*   pubmeta_hier Helper function for parameter hierarchy
*   strisnam Is string a name ?
*   strisnum Is string a number ?
*   vmlev    Produce information on levels of a specified integer variable
*
*C-Runtime Calls:
*   isdigit  Tests for decimal digit
*
*****************************************************************************/
#define sp_isdigit(X) isdigit((unsigned char)(X))



/*****************************************************************************
*sp_lower - Is lowercase
*
*Synopsis:
*   sp_islower(arg1)
*
*Description:
*   Version robust to ANSI char values, e.g. French accented characters.
*
*Return Value:
*   As for underlying function.
*
*****************************************************************************/
#define sp_islower(X) islower((unsigned char)(X))



/*****************************************************************************
*sp_isspace - Is white space
*
*Synopsis:
*   sp_isspace(arg1)
*
*Description:
*   Version robust to ANSI char values, e.g. French accented characters.
*
*Return Value:
*   As for underlying function.
*
*****************************************************************************/
#define sp_isspace(X) isspace((unsigned char)(X))



/*****************************************************************************
*sp_isupper - Is uppercase
*
*Synopsis:
*   sp_isuppercase(arg1)
*
*Description:
*   Version robust to ANSI char values, e.g. French accented characters.
*
*Return Value:
*   As for underlying function.
*
*****************************************************************************/
#define sp_isupper(X) isupper((unsigned char)(X))



/* Define macros to map routines in unix */
#if defined(UNIX)
	#define memmove(dest,src,count) unix_memmove(dest,src,count)
	#define strcmpi(s1,s2)          unix_strcmpi(s1,s2)
	#define strrev(src)             unix_strrev(src)
	#define strupr(src)             unix_strupr(src)
	#define strlwr(src)             unix_strlwr(src)
	#define strstr(s1,s2)           unix_strstr(s1,s2)
#endif


/* ==================== GLOBAL VARIABLE DECLARATIONS ====================== */

/* the following globals are defined in the file debug.c */

#ifdef DEBUG
extern char FAR db_emsg[];
extern char FAR db_lmsg[];
extern char *db_lv;
extern char db_sg[50];
extern short dbinit;
#endif

#ifndef NTRACE
extern int NEAR tr_lv;
extern char NEAR * NEAR tr_fn[50];
#endif


/**
* db_mem is a mask that controls memory debugging operations.  Manifest
* constants that define the bits used in the mask include DB_MEM_TRACE
* and DB_MEM_INTEGRITY and are defined above.  db_mem is defined and
* initialised in trmalloc.c
**/

extern int db_mem;


extern int breakflg;      /* An outstanding break request is pending  */
extern int pushctlc;      /* Push a Ctrl-C to the console buffer on break */


/**
* LANGUAGE is the global which identifies the language used for prompts,
* error messages, and text (see codes above). Defined in strini.c.
**/

extern int LANGUAGE;


/**
* LICENSEE is an identifier for the licensee of SPSD/M. Defined in strini.c.
**/

extern char LICENSEE[HDRSIZE+1];


/**
* LICTYPE is an identifier for the license type of SPSD/M.
* recognized values are "Site", "Demo", and "Individual".
* Defined in strini.c.
**/

extern char LICTYPE[IDSIZE+1];


/**
* LINEDELIM is the line delimiter string used by SPSM for output
**/

extern char LINEDELIM[LINEDELIMSIZE+1];


/**
* MSGTEXT contains text passed to the interface routines for display
**/
extern char MSGTEXT[MSGSIZE+1];


/**
* fnlog is the name of the temporary .log file created
* by the output routines (if activated).  The pattern is changed by mktemp().
* fplog is the associated file pointer.
**/
extern char fnlog[FNLOGSIZE+1];
extern FILE *fplog;


/* ==================== FUNCTION PROTOTYPES =============================== */


extern  void   strmfe(char *newf,char *old,char *ext);
extern  char * strtokk(char *s,char *brk);
extern  short  random(unsigned short *strm);
extern  void   srandom(unsigned short *strm, unsigned short seed);
extern  double randrnd(double fltnum, double randnum);
extern  int    strisnum(char *s);
extern  int    strisnam(char *s);
extern  double nnegd(double y);
extern  double mind(double x,double y);
extern  double maxd(double x,double y);
extern  char * hexdump(char *addr,int n);
extern  double lkup1(NUMBER (*tbl)[LKP_COLS],int tblnum,double input);
extern  double lkup2(NUMBER (*tbl)[LKP_COLS],int tblnum,double input);
extern  double taxbf1(double incd,double exmpt,float *lvl1,double rr1);
extern  double taxbf2(double incd,double exmpt,float *lvl1,double rr1,\
	float *lvl2,double rr2);
extern  double taxbf3(double incd,double exmpt,float *lvl1,double rr1,\
	float *lvl2,double rr2,float *lvl3,double rr3);
extern  double taxbf4(double incd,double exmpt,float *lvl1,double rr1,\
	float *lvl2,double rr2,float *lvl3,double rr3,float *lvl4,double rr4);
extern  double taxbd1(double inc,double exmpt,double *lvl1,double rr1);
extern  double taxbd2(double inc,double exmpt,double *lvl1,double rr1,\
	double *lvl2,double rr2);
extern  double taxbd3(double inc,double exmpt,double *lvl1,double rr1,\
	double *lvl2,double rr2,double *lvl3,double rr3);
extern  double taxbd4(double inc,double exmpt,double *lvl1,double rr1,\
	double *lvl2,double rr2,double *lvl3,double rr3,double *lvl4,double rr4);
extern  void   trace(void);
extern  FILE * strini(FILE *fp);
extern  void   strspsm(void);
extern  void   stradd(char *name, char *string);
extern  void   strclr(int type);
extern  void   strnote(void);
extern  char * STR(char *name);
extern  char * STRd(char *name);
extern  char * STRp(char *name);
extern  char * STRN(int num);
extern  long   hashit(char *ptr, unsigned len, long strt);
extern  int    exists(char * filespec);
extern  int    specok(char * filespec);
extern  void   help(char * topic);
extern  int    kbedit(FILE * fp,char ** pbuf,unsigned bufsiz,char * prompt,char * topic, int bNoLog);
extern  int    yesno(FILE * fp,char * prompt,char * topic);
void    CDECL  hsort(void *, size_t, size_t, int (CDECL *)(const void *, const void *));
extern  int    patch(char * fname, char * licensee, char * label);
extern  char * btime(char * etime);
extern  void   memrep(void);
extern  void * trmalloc(size_t size, char * whosit);
extern  void * trcalloc(size_t num, size_t size, char * whosit);
extern  void * trrealloc(void * p, size_t size, char * whosit);
extern  void   trfree(void * p, char * whosit);
extern  void   chkheap(void);
extern  char * fnexp(char * fn);
extern  char * gsub(char * srch, char * repl, char * target);
extern  int    lzcmpff(char *inpspec, char *outspec);
extern  int    lzexpff(char *inpspec, char *outspec);
extern  void   lzexpbfi(void);
extern  long   lzexpbf(FILE *output_file, char *inpbuf, unsigned int count);
extern  void * unix_memmove(void *dest, void *src, size_t count);
extern  int    unix_strcmpi(char *s1, char *s2);
extern  char * unix_strrev(char *src);
extern  char * unix_strupr(char *src);
extern  char * unix_strlwr(char *src);
extern  char * unix_strstr(char *str1, char *str2);
#if defined(UNIX)
int CDECL close(int);
long CDECL lseek(int, long, int);
int CDECL open(const char *, int, ...);
int CDECL read(int, void *, unsigned int);
int CDECL write(int, const void *, unsigned int);
#endif
extern  void CDECL lprintf(char *fmt, ...);
extern  void CDECL oprintf(char *fmt, ...);
extern  void CDECL vlprintf(char *fmt, va_list ap);
extern  void CDECL lputstr(char *s);
extern  void CDECL lputstrn(char *s, int n);
extern  void CDECL oputstr(char *s);
extern  void CDECL oputstrn(char *s, int n);
extern  void CDECL xputstr(char *s);
extern  void CDECL lputchar(unsigned int ch);
extern  void CDECL oputchar(unsigned int ch);
extern  void CDECL warning(char *fmt, ...);
extern  void CDECL error(char *fmt, ...);
extern  void CDECL fatal(char *fmt, ...);
extern  void CDECL fataltr(char *fmt, ...);

