/***
*vs.h - Household variable structure and sub-structures
*
*$Id: vs.h 6.284 2008/08/22 17:05:54 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1989, Minister of Regional Industrial Expansion.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file declares structures, defines symbols, and gives prototypes
*   associated with the variable structures and sub-structures used in
*   SPSD/M.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 08/06/11 | Added imqhptcp,imcgtcp,impcgtcp,imqnctcp for cg model   |
* | LP  | 08/06/10 | Added imotxtgr, Ontario Textbook and Technology Grant   |
* | LP  | 08/06/09 | Added imondc, immndc - kids 18 or younger in Man, Ont.  |
* | CH  | 08/06/05 | Added CTPCAR                                            |
* | LP  | 08/06/04 | Added immftb - Manitoba Family Tax Benefit              |
* | LP  | 08/06/04 | Added imqawrkpr - Que. Adapted Work Premium for disabled|
* | LP  | 08/06/03 | Added imoshptg, Ont. Sen. Homeowners' Property Tax Grant|
* | LP  | 08/05/12 | Added imnpfgb, imnpsb - NL newborn programs             |
* | LP  | 08/05/05 | Added extra pension income split var to match forms     |
* | LP  | 08/05/01 | Removed idcgtxc, this variable is now saved as idcgtc   |
* | LP  | 08/04/28 | Added imcatc - BC Refundable Climate Action Tax Credit  |
* | LP  | 08/04/21 | Added imclimdv - BC Climate Action Dividend             |
* | LP  | 08/04/15 | Added idcgtxc, idseqino, idalhrs; removed idcgless      |
* | CH  | 07/07/30 | Added imqbtax and imqtuitt                              |
* | CH  | 07/07/23 | Added ctpnew and ctfnew                                 |
* | CH  | 07/07/17 | Added imqctpc imqtctc imqtpcn                           |
* | LP  | 07/07/08 | Added Manitoba Child Benefit - immcb                    |
* | CH  | 07/06/25 | Added ctfgsths                                          |
* | CH  | 07/06/08 | Added fxgstidx                                          |
* | CH  | 07/06/06 | Added iddisab                                           |
* | CH  | 07/06/06 | Added fxpurprc fxselprc                                 |
* | LP  | 07/06/04 | Added Ontario Child Benefit - imocb and imocbyng        |
* | LP  | 07/04/24 | Added non-refundable child tax credit - imchdtxc        |
* | LP  | 07/04/04 | Added Working Income Tax Benefit - imwitb, imwitbs      |
* | CH  | 07/02/22 | Make pension income split fractional variables          |
* | CH  | 07/02/19 | Added commodity tax array variables                     |
* | CH  | 07/02/09 | Deleted extra space                                     |
* | CH  | 07/02/06 | Added pension income splitting                          |
* | LP  | 07/01/23 | Removed idninco, icninco and imninc - no income flags   |
* | LP  | 07/01/22 | Added 'eligible' to all large corporation dividend vars |
* | LP  | 07/01/16 | Fixed line number for imiqloss (234 not 243)            |
* | LP  | 07/01/15 | Removed impucctc, changed to impyctxc-yng child tx cred |
* | LP  | 06/06/14 | Added imbcss - B.C. Senior Supplement (part of SA)      |
* | LP  | 06/05/25 | Added N.S. Universal Child Care Benefit credit, impucctc|
* | LP  | 06/05/25 | Added Universal Child Care Benefit - imuccb             |
* | LP  | 06/05/15 | Added taxable dividend amounts from large/small corp    |
* |     |          | federal - imisdivt, imildivt; Quebec-imqldivt, imqsdivt |
* | LP  | 06/05/05 | Added Canada Employment Credit - imemptxc               |
* | LP  | 06/05/05 | Added federal text book non-ref tax credit - imtxttxc   |
* | LP  | 06/01/25 | Changed label for ucgotpa - received parental benefits  |
* | CH  | 06/01/04 | Added imqpsetc ad imqpsedd                              |
* | LP  | 05/12/20 | Added Quebec Parental Insurance Plan variables          |
* | CH  | 05/12/11 | Changed labels                                          |
* | LP  | 05/12/01 | Removed ctfoen & ctfmfg - pre 91 variables deleted      |
* | LP  | 05/11/09 | Added imictot, imicnet - tax form total and net income  |
* | LP  | 05/11/04 | Added imresreb - Alberta Resource Rebate amount         |
* | LP  | 05/10/21 | Added imfecb - Federal Energy Cost Benefit amount       |
* | LP  | 05/09/27 | Added ficloshs - redined ficlohhv                       |
* | LP  | 05/09/23 | Removed fxusecom                                        |
* | CH  | 05/09/12 | Deleted variables idfdsft and iddeppch                  |
* | LP  | 05/09/12 | Added fxusecom                                          |
* | LP  | 05/08/10 | Added imsaclaw - CTB/SA calculated clawback amount      |
* | LP  | 05/07/12 | Readded imashelt - Alberta Shelter Increase for CTB/SA  |
* | BBM | 05/07/03 | Added for 11.0:  idsamths idclergy                      |
* | LP  | 05/05/30 | Added imqncgtc, Que. Refund. Natural Caregivers tax cred|
* | LP  | 05/05/11 | Added imqempdd - earned income deduction for workers    |
* | CH  | 04/08/10 | Deleted imqodtc - no longer used                        |
* | CH  | 04/08/09 | Added income of designated dependant child              |
* | LP  | 04/06/18 | Added imqca and imqodtc - Que. Child Assistance cred    |
* | LP  | 04/06/09 | Added imqwrkpr - Que. Work Premium refundable tax cred  |
* | LP  | 04/06/04 | Added Provincial Health Premium amount - imphp          |
* | LP  | 04/06/02 | Added imopstc - Ont. property & sales tax credit amount |
* | LP  | 04/03/16 | fixed typos                                             |
* | LP  | 04/02/20 | Removed imashelt - dropped Alberta Shelter Increase     |
* | LP  | 04/02/20 | Removed imses - dropped Sask. Employment Supplement     |
* | CH  | 04/02/09 | Changed labels                                          |
* | SB  | 04/02/09 | Changed idcharit label (removed line number)            |
* | CH  | 04/01/22 | Changed iditoth label                                   |
* | LP  | 04/01/07 | Updated the WIDOWED code to match new idmarst in Ver 10 |
* | LP  | 04/01/06 | Removed imcgtcp and impcgtcp                            |
* | CH  | 03/12/29 | added fxrepr                                            |
* | LP  | 03/12/22 | Added impmeda - prov. medical expense allowed (computed)|
* | LP  | 03/12/08 | Added impothpe - prov. infirm depend. 18+ tax credit    |
* | CH  | 03/11/17 | Removed fshflg rgvlot, added alimo homexp anncon        |
* | LP  | 03/11/17 | Added imqintsl - Quebec interest on student loans credit|
* | LP  | 03/11/17 | Added imintsl - federal interest on student loans credit|
* | LP  | 03/11/14 | Added impintsl - provincial interest on student loans   |
* | CH  | 03/11/04 | Deleted iddisab                                         |
* | CH  | 03/10/17 | Added for 10.0: intstu cgtc deppch iinvrd fshflg        |
* |     |          | Dropped for 10.0: scfoas scfuib scfctc scfptc scfftc    |
* |     |          |                   noth iint scfint ioinv scfcqp itogv   |
* |     |          |                   inogv ivetp iscfsa foravg cintim      |
* |     |          |                   cluflg lystr nonlf efpub cfpub scfflg |
* | CH  | 03/05/16 | NS Taxpayer refund program implemented                  |
* | CH  | 03/05/15 | Changed label of idothpe                                |
* | LP  | 02/06/21 | Added imoccext - Ont. one-time Child Care extra Supp.   |
* | LP  | 02/06/17 | Added imnmbns - Nfld. Mother-Baby Nutrition Supplement  |
* | BBM | 02/06/12 | Add before or after tax LICO option: imatpinc,efpvthat  |
* | LP  | 02/06/05 | Added imashelt - Alta shelter increase (NCBS clawback)  |
* | LP  | 01/07/18 | Renamed imuiydeps to imuiydep and impmincar to impminco |
* | EGA | 01/07/10 | add net consumption SNA adjusted                        |
* | BBM | 01/06/21 | Drop brate and wkhr                                     |
* | BBM | 01/06/20 | Drop prov home ownership saving plan variable imphotc   |
* | CH  | 01/06/12 | Added ubp6 ubp6c                                        |
* | CH  | 01/06/07 | Changed UIPAR, added UIOTH                              |
* | LP  | 01/05/09 | Added imminamt,impmincar, imamtprv new min. tax variable|
* | BBM | 00/04/18 | Add brate, divwk, hrwk nere othbr othwk cgless for V9.0 |
* | BBM | 00/04/18 | Dropped idrooms idbdrms for version 9.0                 |
* | BBM | 00/04/18 | Dropped fxintpl,fxmorti,fxhmort,fxhmkt for version 9.0  |
* | LP  | 01/04/02 | Added impheatr - provincial home heating rebates        |
* | LP  | 01/03/05 | Removed pre-1988 program variables                      |
* | LP  | 01/03/05 | Removed Federal Senior's Benefit variables              |
* | CH  | 00/12/19 | Added imuiydeps                                         |                                          
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/10/24 | Added relief for heating expenses                       |
* | CH  | 00/06/14 | Added imothpe                                           |
* | CH  | 00/05/17 | Added imqcchc                                           |
* | LP  | 00/05/16 | Fixed label for imcgtcp                                 |
* | CH  | 00/05/15 | Added homeowners assistance                             |
* | CH  | 00/05/12 | Added taxpayer dividend                                 |
* | LP  | 00/05/10 | Added imiotax Ontario taxable income (CapGains concept) |
* | BBM | 00/04/18 | Added fxrgvlot for version 8.0                          |
* | CH  | 00/04/14 | Added imiloss imstkded imqiloss imqstkdd imqcapgx       |
* | LP  | 00/04/11 | Added Quebec CPP/QPP contrib. on self-employed income   |
* | CH  | 00/04/10 | Added Sask refundable sales tax credit                  |
* | CH  | 00/04/07 | Added Sask child and senior tax credits                 |
* | CH  | 00/03/30 | Renamed imqchartc to imqchatc to be under 8 characters  |
* | LP  | 00/03/21 | Added imqmedrc - Que. Ref. tax credit for med. expenses |
* | LP  | 00/03/17 | Added imncb - Nfld. child benefit                       |
* | CH  | 00/03/10 | Deleted imamtc                                          |
* | CH  | 00/03/07 | Added imcgtcp                                           |
* | CH  | 00/03/06 | Modified the IO_ categories                             |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/06/08 | Label changes                                           |
* | CH  | 99/05/26 | Add Alta married tax credit                             |
* | LP  | 99/05/25 | Add BC Earned Income Benefit imibceib                   |
* | CH  | 99/05/21 | Modified simplified tax system and added imqcalr        |
* | CH  | 99/04/20 | Add NS child benefit imnscb                             |
* | CH  | 99/04/15 | Added imbtcs                                            |
* | CH  | 99/03/22 | Added imlabtxc and implvctc                             |
* | CH  | 99/03/22 | Changed idlabtxc to idlabtxg                            |
* | EGA | 99/03/18 | Add iint, sa, cqp values from SCF                       |
* | BBM | 99/03/09 | Added Manifest constant for UI parental group UIPAR     |
* | BBM | 99/02/10 | Update variables for version7.0 (drop 5, add 11)        |
* | CH  | 98/05/01 | add imscb imses (Sask child benefit and emp. supp.)     |
* | SFG | 98/04/24 | increase glass box limits in 32-bit environments        |
* | EGA | 98/04/21 | add imqchartc                                           |
* | EGA | 98/04/07 | add imqapf (Quebec Anti-Poverty Fund)                   |
* | CH  | 98/04/07 | add imfsurtr                                            |
* | EGA | 98/04/02 | add imqlitc                                             |
* | CH  | 98/04/01 | deleted imoccec                                         |
* | EGA | 98/03/24 | add imcgtc                                              |
* | EGA | 98/03/09 | add imqtuitc,imqamt,imqamtflg                           |
* | EGA | 98/02/18 | add imqfatc                                             |
* | EGA | 98/02/13 | add imqduetc                                            |
* | CH  | 98/02/03 | added NS Direct Assistance iminsda                      |
* | CH  | 98/01/22 | immedref, Fed. Refundable medical expense supplement    |
* | EGA | 97/12/17 | added imqhptc                                           |
* | EGA | 97/11/05 | added ucuero                                            |
* | SFG | 97/11/03 | changed description of ididiv, idicapg                  |
* | EGE | 97/10/21 | changed description of imiuib                           |
* | CCH | 97/10/16 | changed description of ididiv, idicapg,                 |
* | BBM | 97/10/15 | added Ont. Refundable child care tax credit VARS imoccea|
* | BBM | 97/10/07 | Implement B.C. Family Bonus variable imibcfb            |
* | CCH | 97/10/03 | Add ubcalfs ubclmfs: family supplement received         |
* | BBM | 97/09/29 | Add vars for NB child tax ben imnbcben,imnbwis,imnbcben |
* | BBM | 97/09/12 | Alberta Family Employment Tax Credit variable (imiafetc)|
* | EGA | 97/09/11 | added imqstr                                            |
* | CCH | 97/09/10 | added imedrcv,imedtrf,immanltc                          |
* | CCH | 97/08/21 | added imigispa                                          |
* | CCH | 97/07/30 | dropped idwkhour, added ucwkhr                          |
* | CCH | 97/06/19 | dropped imeiwbp5                                        |
* | BBM | 97/05/27 | dropped idroyreb,idroytc,idstkccf,idsaskti,idrecssp     |
* | BBM | 97/05/27 | droped idstksav, fxhreno, uc[12]prvwk				     |
* | SFG | 97/05/26 | added variable idccqp, removed uc[12]gotrg              |
* | SFG | 97/05/06 | added new database UI variables                         |
* | SFG | 97/04/30 | MS-DOS: remove near/far & adjust overlay (memory probs.)|
* | CCH | 97/04/03 | Add EI refund variable                                  |
* | BBM | 96/05/12 | Add Seniors Benefit variables                           |
* | BBM | 95/05/19 | Add Nova Scotia Pharmacare program variables            |
* | BBM | 94/10/05 | Add Quebec refundable child care expense credit vars    |
* | BBM | 94/09/29 | add imqittr income tested tax reduction variable        |
* | BBM | 94/09/29 | add Quebec net income/tax credit vars imquictc imqcpptc |
* | BBM | 94/09/29 | add imqhsfc - health services fund contributions        |
* | GJC | 94/04/06 | add imuidpfg for assigning ui dependents to a claimant  |
* | RSE | 94/04/05 | add imiasb for Alberta seniors benefit                  |
* | GJC | 94/01/17 | remove ctpupst and ctpugas                              |
* | GJC | 93/02/17 | add other refundable tax credit and mod cap gains vars  |
* | GJC | 93/02/16 | add imfortxc foreign tax credit variable                |
* | GJC | 93/02/08 | add ui training pointers ubp5(c)                        |
* | BBM | 92/09/02 | Delete imccetxc and impctc                              |
* | BBM | 92/07/27 | replace imqmetc with imqmeda, add imqchara              |
* | BBM | 92/07/22 | Add prov non-refundable tax credit variable imnptc      |
* | BBM | 92/07/09 | Add prov net income tax variable impnit                 |
* | BBM | 92/07/08 | Add prov home ownership saving plan variable imphotc    |
* | BBM | 92/07/07 | imfcben, imfcbenb, imfcbene: child benefit vars added   |
* | BBM | 92/07/06 | impptxtc added: Provincial Property tax credit          |
* | BBM | 92/06/23 | impptc added: Provincial Political Contrib. Tax Credit  |
* | BBM | 92/06/23 | imfptc added: Federal Political Contribution Tax Credit |
* | SFG | 92/05/01 | NEARHHLD back to NEAR                                   |
* | SFG | 92/02/07 | ctupst -> ctpupst; ctugas -> ctpugas; also CTUPST,CTUGAS|
* | SFG | 92/02/06 | add ctnes, ctsave, ctothmon to complete expend. identity|
* | SFG | 92/02/06 | remove cttotexp; add ctnexp instead                     |
* | SFG | 92/02/06 | ctpct -> cttxpc_; ctfct -> cttxfc_; ctexp -> ctnexp     |
* | SFG | 92/02/06 | Delete ctpict; add ctupst, ctugas                       |
* | SFG | 91/08/07 | added imoasr imfar impctc                               |
* | SFG | 90/10/30 | removed unused icrefper                                 |
* | SFG | 90/10/04 | added ctexp[40]                                         |
* | SFG | 90/09/27 | added imqtfa                                            |
* | SFG | 90/09/26 | added imqrepay                                          |
* | SFG | 90/09/23 | removed idrefval                                        |
* | SFG | 90/09/20 | added NUMUV & uvbl[] array for black-box user vars.     |
* | SFG | 90/07/24 | removed unused variable imqchdn                         |
* | SFG | 90/03/20 | Added extra database id NUMBER variables idext_        |
* | SFG | 89/12/05 | Add imoftca variable                                    |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Dependencies:
*   #include "spsm.h"
*   #include "vsu.h"
*   #include "vs.h"
*
*****************************************************************************/



/* ==================== MANIFEST CONSTANTS ================================ */

/**
* UVGLSIZE determines the amount of space, in bytes, to be reserved
* for the user modelled variable structure (identified by 'vsu' and
* declared in vsu.h). The difference between UVGLSIZE and the actual size
* of the user modelled variable structure (a substructure of the
* md sub-structure declared below) is filled by a compile-time filler array
* of char's.
**/

#if defined(_WIN32)
#define UVGLSIZE 1000
#else
#define UVGLSIZE 500
#endif



/**
* NUMUV determines the number of variables, to be reserved for
* user-defined black box variables created using UVAR or EX & CL control
* parameters.  Analysis variables (stored as NUMBERS) and class variables
* (stored as int's) are both stored in the space occupied by a NUMBER
**/

#if defined(_WIN32)
#define NUMUV 150
#else
#define NUMUV 75
#endif


/* these are the values assumed by the hdprov variable */

#define PRNFLD 0
#define PRPEI  1
#define PRNS   2
#define PRNB   3
#define PRQUE  4
#define PRONT  5
#define PRMAN  6
#define PRSASK 7
#define PRALTA 8
#define PRBC   9


/* these are the values assumed by the prsex variable */

#define MALE    0
#define FEMALE  1


/* these is a value assumed by the prmarst variable */

#define WIDOWED 4


/* codes for claim type */

#define UIREG 1    /* regular    */
#define UISIC 2    /* sickness   */
#define UIMAT 3    /* maternity  */
#define UIPAR 4    /* parental */
#define UIFSH 5    /* fishing    */
#define UITRN 6    /* training   */
#define UIOTH 7    /* other types   */


/* codes for family type */

#define FT_CH1 0    /* children present, 1 adult  */
#define FT_CH2 1    /* children present, 2+ adult */
#define FT_EL1 2    /* elderly present, 1 adult   */
#define FT_EL2 3    /* elderly present, 2+ adult  */
#define FT_OT1 4    /* other, 1 adult             */
#define FT_OT2 5    /* other, 2+ adult            */


/* codes for indices in the fxio vector */

#define IO_FDNABV  0  /* food and non-alcoholic beverages */
#define IO_ALCBEX  1  /* alcoholic beverages */
#define IO_TOAC    2  /* tobacco */
#define IO_MBCLOTH 3  /* men's & boy's clothing */
#define IO_MBCLRA  4  /* men's and boys' clothing repair & alteration */
#define IO_WGCLOTH 5  /* women's and children's clothing */
#define IO_WGCLRA  6  /* women's clothing repair & alteration */
#define IO_TFOOTW  7  /* footwear */
#define IO_SHOER   8  /* shoe repair */
#define IO_GIRENT  9  /* gross imputed rent */
#define IO_GPRENT 10  /* gross rent paid */
#define IO_OTHLOG 11  /* Other shelter expenses */
#define IO_ELECTY 12  /* electricity */
#define IO_TGAS   13  /* natural gas */
#define IO_OTFUEL 14  /* other fuels */
#define IO_FURNC  15  /* furniture and floor covering */
#define IO_UFURNR 16  /* uphosltery and furniture repair */
#define IO_DHAPPL 17  /* houshold appliances */
#define IO_HEREP  18  /* houshold equipment repairs */
#define IO_SEMDUR 19  /* Semi-durable houshold furnishings */
#define IO_NONDUR 20  /* non-durable houshold supplies */
#define IO_DOMSRV 21  /* domestic and child care services */
#define IO_OHHSV  22  /* other houshold services */
#define IO_MEDCRE 23  /* medical care */
#define IO_HSPCRE 24  /* hospital care and the like */
#define IO_HLTIP  25  /* accident and sickness insurance */
#define IO_DRGSND 26  /* drugs and pharmaceutical products */
#define IO_NWAUTO 27  /* New and Used (net) Motor vehicles */
#define IO_AUTORP 28  /* Motor vehicles parts and accessories */
#define IO_MVREP  29  /* Motor vehicle reparis */
#define IO_GALOLG 30  /* motor fuels and lubricants */
#define IO_OTAURS 31  /* other motor vehicle related services */
#define IO_TSPLIC 32  /* purchased transportation */
#define IO_TELEPO 33  /* communications */
#define IO_RECSPT 34  /* recreation, sporting and camping equip */
#define IO_RECREP 35  /* recreation equipment repair and rentals */
#define IO_BKMAG  36  /* reading and entertainment supplies */
#define IO_RECSRV 37  /* recreational services */
#define IO_EDUCLT 38  /* educational and cultureal services */
#define IO_JEWLWR 39  /* jewelry and watches */
#define IO_JWREP  40  /* jewelry and watch repair */
#define IO_LEATH  41  /* leather goods & other personal effects */
#define IO_TOILAT 42  /* toilet articles and cosmetics */
#define IO_PERCRE 43  /* personal care */
#define IO_RSTEXP 44  /* restaurants and accomodation services */
#define IO_INTPLB 45  /* financial, legal, and other services */
#define IO_DUEGFT 46  /* operationg expenditures of non-profict orgs */
#define IO_UNUSED 47  /* net expenditure abroad */





/* ==================== MACRO DEFINITIONS ================================= */

/* NEARHHLD determines if the global household structure is NEAR or not */

/*#define NEARHHLD NEAR*/
#define NEARHHLD



/* ==================== STRUCTURE DECLARATION MACROS ====================== */

/* define the macros used to declare items */

#define S_hd   struct hd_
#define P_hd   S_hd NEARHHLD *

#define S_fx   struct fx_
#define P_fx   S_fx NEARHHLD *

#define S_ct   struct ct_
#define P_ct   S_ct NEARHHLD *

#define S_ef   struct ef_
#define P_ef   S_ef NEARHHLD *

#define S_cf   struct cf_
#define P_cf   S_cf NEARHHLD *

#define S_nf   struct nf_
#define P_nf   S_nf NEARHHLD *

#define S_uc   struct uc_
#define P_uc   S_uc NEARHHLD  *

#define S_id   struct id_
#define P_id   S_id NEARHHLD *

#define S_ub   struct ub_
#define P_ub   S_ub NEARHHLD  *

#define S_im   struct im_
#define P_im   S_im NEARHHLD *

#define S_ic   struct ic_
#define P_ic   S_ic NEARHHLD *

#define S_uv   struct uv_
#define P_uv   S_uv NEARHHLD *

#define S_in   struct in_
#define P_in   S_in NEARHHLD *

#define S_hh   struct hh_
#define P_hh   S_hh NEARHHLD *



/* ==================== STRUCTURE DECLARATIONS ============================ */


/**
* The structure identified by the letters 'hd' holds SPSD variables  
* pertaining to a household.  These variables are either read directly
* from the SPSD database, or are directly derived from other SPSD
* variables.
**/

typedef struct hd_ {
	int      hdnpers;         /* Number of persons in household             */
	int      hdnadult;        /* Number of adults in household              */
	int      hdneld;          /* Number of elderly in household             */
	int      hdnkids;         /* Number of children in household            */
	int      hdnearn;         /* Number of earners in household             */
	int      hdageeld;        /* Age of eldest in household                 */
	int      hdsexeld;        /* Sex of eldest in household                 */
	int      hdspoflg;        /* Household contains married couple          */
	int      hdtype;          /* Household type                             */
	NUMBER   hdseqhh;         /* Household sequence number                  */
	NUMBER   hdwgthh;         /* Household weight                           */
	NUMBER   hdwgthhs;        /* Sub-sample adjusted household weight       */
	NUMBER   hdwgtfx;         /* Sum of household weight for SHS group      */
	int      hdfrstfx;        /* First household in SHS group flag          */
	int      hdlastfx;        /* Last household in SHS group flag           */
	int      hdprov;          /* Province                                   */
	int      hdurb;           /* Size of urban area                         */
	int      hdclohh;         /* Number of SLID clones                      */
	int      hdtenur;         /* Tenure                                     */
	int      hdevmv;          /* Expenditure Vector Matching Variable       */
	NUMBER   hdunits;         /* Unit count                                 */
	NUMBER   hdnspsd;         /* SPSD records                               */
	NUMBER   hdnscf;          /* SLID records                               */
	NUMBER   hdnfxv;          /* SHS records                                */
	NUMBER   hdtpval;         /* Current value of adjustment (TP facility)  */
} hd_;


/**
* The structure identified by the letters 'fx' holds SPSD variables  
* pertaining to a single SHS expenditure vector.  These variables
* are read directly from the SPSD database, except for fxpsave and
* fxnsave, which are computed directly from fxncal.
**/

typedef struct fx_ {
	int      fxseqhv;         /* SHS record sequence number               */
	int      fxclohhv;        /* SHS cloning factor                       */
	int      fxcloshs;        /* SHS cloning factor                       */
	NUMBER   fxio[NUMCOM];    /* I/O expenditure categories [array]         */
	NUMBER   fxrecom;         /* real estate commissions */
	NUMBER   fxipac;          /* life ins prems and annuity contributions */ 
	NUMBER   fxgvpen;         /* gvt pension plan contributions */ 
	NUMBER   fxpvpen;         /* private pension plan contributions */
	NUMBER   fxcqp;           /* cpp - qpp contributions */
	NUMBER   fxuic;           /* UI contributions */
	NUMBER   fxintax;         /* Income taxes */
	NUMBER   fxprtax;         /* Property tax */
	NUMBER   fxtptax;         /* Transfer of Property taxes */
	NUMBER   fxrfees;         /* Registration and license fees */
	NUMBER   fxnes;           /* Not elsewhere stated */
	NUMBER   fxncal;          /* Net change in assets and liabilities */
	NUMBER   fxrrspt;         /* Total RRSP contributions (SHS) */
	NUMBER   fxfabd;          /* Account balancing difference */
	NUMBER   fxfomr;          /* Other money receipts */
	NUMBER   fxalimo;         /* Alimony and child support */
	NUMBER   fxhomexp;        /* Expenses assoc with principal residence */
	NUMBER   fxanncon;        /* Annuity contracts and transfers to RRIFs */
	NUMBER   fxrepr;          /* Repairs paid */
	NUMBER   fxpurprc;        /* Purchase price of home */
	NUMBER   fxselprc;        /* Selling price of home */
	NUMBER   fxgstidx;        /* Index for distributing GST on housing */
	NUMBER   fxpsave;         /* Positive savings                           */
	NUMBER   fxnsave;         /* Negative savings                           */
	NUMBER   fxsaldur;        /* Sale of durables                           */
} fx_;


/**
* The structure identified by the letters 'ct' holds modelled variables
* pertaining to a commodity taxes at the household level.
**/

typedef struct ct_ {
	int      ctseqhv;         /* SHS record sequence number                 */
	int      ctprov;          /* Province for COMTAX calculations           */
	NUMBER   ctfcid;          /* Federal custom import duties               */
	NUMBER   ctfexd;          /* Federal excise duties                      */
	NUMBER   ctfext;          /* Federal excise taxes                       */
	NUMBER   ctfgst;          /* Federal GST                                */
	NUMBER   ctfgsths;        /* Federal GST on housing                     */
	NUMBER   ctfnew;          /* New federal commodity tax                  */
	NUMBER   cttxfc;          /* Federal commodity taxes (total)            */
	NUMBER   cttxfc_[NUMCOM]; /* Federal commodity taxes [array]            */
	NUMBER   ctfcid_[NUMCOM]; /* Federal custom import duties [array]       */
	NUMBER   ctfexd_[NUMCOM]; /* Federal excise duties [array]              */
	NUMBER   ctfext_[NUMCOM]; /* Federal excise taxes [array]               */
	NUMBER   ctfgst_[NUMCOM]; /* Federal GST [array]                        */
	NUMBER   ctfnew_[NUMCOM]; /* New federal commodity tax [array]          */
	NUMBER   ctplgl;          /* Provincial liquor gallonage taxes          */
	NUMBER   ctpplq;          /* Provincial profits on liquor commissions   */
	NUMBER   ctpgas;          /* Provincial gasoline tax                    */
	NUMBER   ctpamu;          /* Provincial amusement tax                   */
	NUMBER   ctptob;          /* Provincial tobacco tax                     */
	NUMBER   ctprst;          /* Provincial retail sales tax                */
	NUMBER   ctpcar;          /* Provincial carbon tax                    */
	NUMBER   ctpnew;          /* New provincial commodity tax               */
	NUMBER   cttxpc;          /* Provincial commodity taxes (total)         */
	NUMBER   cttxpc_[NUMCOM]; /* Provincial commodity taxes [array]         */
	NUMBER   ctpgas_[NUMCOM]; /* Provincial gasoline tax [array]            */
	NUMBER   ctprst_[NUMCOM]; /* Provincial retail sales tax [array]        */
	NUMBER   ctpcar_[NUMCOM]; /* Provincial carbon tax [array]            */
	NUMBER   ctpnew_[NUMCOM]; /* New provincial commodity tax [array]       */
	NUMBER   ctnexp;          /* Household expenditure net of taxes         */
	NUMBER   ctnexp_[NUMCOM]; /* Household expenditure net of taxes [array] */
	NUMBER   ctnsna;          /* Household expenditure net of taxes -SNA adjusted  */
	NUMBER   ctnsna_[NUMCOM]; /* Household expenditure net of taxes [array] -SNA adjusted*/
	NUMBER   ctnes;           /* Household expenditure not elsewhere specified */
	NUMBER   ctlprop;         /* Local property taxes on owned dwellings    */
	NUMBER   ctsave;          /* Household savings                          */
	NUMBER   ctothmon;        /* Household money from other sources         */
	NUMBER   ctishrh;         /* Shared income concept (SHS)                */
	NUMBER   ctdfexp;         /* Deflated expenditure (tax exclusive)       */
} ct_;


/**
* The structure identified by the letters 'ef' holds SPSD variables  
* pertaining to an economic family.  These variables
* are either read directly from the SPSD database, or else directly
* derived from such variables.
**/

typedef struct ef_ {
	int      efnpers;         /* Number of persons in economic family       */
	int      efnadult;        /* Number of adults in economic family        */
	int      efneld;          /* Number of elderly in economic family       */
	int      efnkids;         /* Number of children in economic family      */
	int      efnearn;         /* Number of earners in economic family       */
	int      efageeld;        /* Age of eldest in economic family           */
	int      efsexeld;        /* Sex of eldest in economic family           */
	int      efspoflg;        /* Economic family contains married couple    */
	int      eftype;          /* Economic family type                       */
	P_in     efin;            /* First person in economic family [pointer]  */
	NUMBER   efpovthr;        /* Economic family low income threshold (Before tax) */
	NUMBER   efpvthat;        /* Economic family low income threshold (After tax) */
} ef_;


/**
* The structure identified by the letters 'cf' holds SPSD variables  
* pertaining to a census family.  These variables
* are either read directly from the SPSD database, or else directly
* derived from such variables.
**/

typedef struct cf_ {
	int      cfnpers;         /* Number of persons in census family         */
	int      cfnadult;        /* Number of adults in census family          */
	int      cfneld;          /* Number of elderly in census family         */
	int      cfnkids;         /* Number of children in census family        */
	int      cfnearn;         /* Number of earners in census family         */
	int      cfageeld;        /* Age of eldest in census family             */
	int      cfsexeld;        /* Sex of eldest in census family             */
	int      cfspoflg;        /* Census family contains married couple      */
	int      cftype;          /* Census family type                         */
	int      cfnchild;        /* Number of children (including 18+)         */
	int      cfcat;           /* Census family category                     */
	P_in     cfin;            /* First person in census family [pointer]    */
	P_in     cfineld;         /* Eldest person in census family [pointer]   */
	P_in     cfinspo;         /* Spouse of eldest [pointer]                 */
	P_in     cfinch;          /* First child in census family [pointer]     */
} cf_;


/**
* The structure identified by the letters 'nf' holds SPSD variables  
* pertaining to a nuclear family.  These variables
* are either read directly from the SPSD database, or else directly
* derived from such variables.
**/

typedef struct nf_ {
	int      nfnpers;         /* Number of persons in nuclear family         */
	int      nfnadult;        /* Number of adults in nuclear family          */
	int      nfneld;          /* Number of elderly in nuclear family         */
	int      nfnkids;         /* Number of children in nuclear family        */
	int      nfnearn;         /* Number of earners in nuclear family         */
	int      nfageeld;        /* Age of eldest in nuclear family             */
	int      nfsexeld;        /* Sex of eldest in nuclear family             */
	int      nfspoflg;        /* Nuclear family contains married couple      */
	int      nftype;          /* Nuclear family type                         */
	P_in     nfin;            /* First person in nuclear family [pointer]    */
	P_in     nfineld;         /* Eldest person in nuclear family [pointer]   */
	P_in     nfinspo;         /* Spouse of eldest [pointer]                  */
	P_in     nfinch;          /* First child in nuclear family [pointer]     */
} nf_;


/**
* The structure identified by the letters 'uc' holds SPSD variables  
* pertaining to a UI claim.  These variables are read directly from the
* SPSD database.
**/

typedef struct uc_ {
	int ucstat;               /* Claim status flag                           */
	int ucstart;              /* Week claim established                      */
	int ucrpeat;              /* Repeat claim flag                           */
	int ucbtyp;               /* Claim type                                  */
	int ucmtyp;               /* Main Claim type (majority of weeks)         */
	int uctpcng;              /* Also received regular benefits              */
	int ucweeks;              /* Weeks of benefits                           */
	int ucy1;                 /* Weeks on UI in first year before claim      */
	int ucy2;                 /* Weeks on UI in second year prior to claim   */
	int ucy3;                 /* Weeks on UI in third year prior to claim    */
	int ucy4;                 /* Weeks on UI in fourth year prior to claim   */
	int ucy5;                 /* Weeks on UI in fifth year prior to claim    */
	int ucwwork;              /* Weeks of work prior to claim                */
	int ucuer;                /* Local unemployment rate (x10)               */
	int ucuero;               /* Local unemployment rate (x10)-original value */
	int ucexhas;              /* Exhaustee flag                              */
	int uctrnwk;              /* Weeks of training benefits                  */
	int ucgotpa;              /* Received parental benefits                  */
	int ucdivwk;              /* Minimum divisor weeks                       */
	int uchrwk;               /* Hours worked prior to claim                 */
	int ucnere;               /* New entrant re-entrant flag                 */
	int ucothbr;              /* Other benefits weekly rate                  */
	int ucothwk;              /* Weeks of other benefits                     */
	NUMBER uctrnbr;           /* Training benefit weekly rate                */
	NUMBER ucern;             /* Insurable weekly earnings                   */
	NUMBER uceff;             /* Effective weekly rate                       */
} uc_;


/**
* The structure identified by the letters 'id' holds SPSD variables  
* pertaining to an individual.  These variables are read directly from the
* SPSD database.
**/

typedef struct id_ {
	P_in     idinspo;         /* Person's spouse [pointer]                  */
	P_hh     idhh;            /* Person's household [pointer]               */
	P_ef     idef;            /* Person's economic family [pointer]         */
	P_cf     idcf;            /* Person's census family [pointer]           */
	P_nf     idnf;            /* Person's nuclear family [pointer]          */
	int      idspoflg;        /* Person has spouse                          */
	int      idefseq;         /* Economic family sub-sequence number        */
	int      idcfseq;         /* Census family sub-sequence number          */
	int      idinseq;         /* Individual sub-sequence number             */
	int      idhhrh;          /* Relationship to head of household          */
	int      idefrh;          /* Relationship to economic family head       */
	int      idcfrh;          /* Relationship to census family head         */
	int      idage;           /* Age                                        */
	int      idalhrs;         /* Total Paid Hours of Work Last Year (imputed) */
	int      idcsa;           /* Converted social assistance                */
	int      idcuib;          /* Converted UI benefit                       */
	int      idccqp;          /* Converted CPP/QPP benefit                  */
	int      iddisab;         /* Disability status (age 16+)                */
	int      idedlev;         /* Educational level                          */
	int      idestat;         /* Educational status                         */
	int      idieflag;        /* Cloned institutionalized elderly person    */
	int      idimmi;          /* Years since immigration                    */
	int      idind;           /* Industry                                   */
	int      idlfst;          /* Labour force status                        */
	int      idlyfp;          /* Last year full/Part time                   */
	int      idlyun;          /* Weeks unemployed                           */
	int      idlyww;          /* Weeks worked                               */
	int      idmarst;         /* Marital status                             */
	int      idocc;           /* Occupation                                 */
	int      idschtp;         /* School type                                */
	int      idseqino;        /* SLID Individual Identifier (Original)      */
	int      idsex;           /* Sex                                        */
	int      idsynthi;        /* Synthetic high income person               */
	NUMBER   idnage;          /* Age                                        */
	NUMBER   idaddded;        /* Additional deductions from net income (256)*/
	NUMBER   idalexp;         /* Other allowable employment expenses (229)  */
	NUMBER   idcapgex;        /* Capital gains exemptions (254)             */
	NUMBER   idcarry;         /* Carrying charges (221)                     */
	NUMBER   idccet;          /* Child care expenses associated with child  */
	NUMBER   idccett;         /* Child care expenses (Limit A, Form T778)   */
	NUMBER   idcgtc;          /* Caregivers tax credit (database) (315)     */
	NUMBER   idcharit;        /* Charitable donations                       */
	NUMBER   idclergy;        /* Clergy residential deduction               */
	NUMBER   idcloss;         /* Allowable other years capital loss (253)   */
	NUMBER   idcount;         /* Person count                               */
	NUMBER   iddalimo;        /* Alimony paid (220)                         */
	NUMBER   iddisoth;        /* Disability amount for dependants (318)     */
	NUMBER   iddisslf;        /* Disability amount for self (316)           */
	NUMBER   iddues;          /* Union and professional dues (212)          */
	NUMBER   ideducfm;        /* Eligible full-time months of education     */
	NUMBER   ideducpm;        /* Eligible part-time months of education     */
	NUMBER   idemplo;         /* Employee home relocation loan dedn (248)   */
	NUMBER   idexplor;        /* Exploration and development expenses (224) */
	NUMBER   idforinc;        /* Net foreign income (508)                   */
	NUMBER   idfortx;         /* Foreign tax paid (507)                     */
	NUMBER   idgifts;         /* Gifts to Canada/provinces/culture (342)    */
	NUMBER   idgstreb;        /* GST rebate (457)                           */
	NUMBER   idhomstu;        /* College res/resdnt homeowner assist (558)  */
	NUMBER   idialimo;        /* Alimony income received                    */
	NUMBER   idicapg;         /* Capital gains (actual)                     */
	NUMBER   idicqp;          /* CPP/QPP income (114)                       */
	NUMBER   ididiv;          /* Dividend income (actual)                   */
	NUMBER   idiemp;          /* Wages & salaries                           */
	NUMBER   idiinvnd;        /* Interest and other investment income       */
	NUMBER   idintstu;        /* Interest paid on student loans (319)       */
	NUMBER   idiloss;         /* Business investment losses (217)           */
	NUMBER   idipens;         /* Pension income (115)                       */
	NUMBER   idisa;           /* Social assistance income                   */
	NUMBER   idise;           /* Self-employed income                       */
	NUMBER   iditc;           /* Federal investment tax credits (412)       */
	NUMBER   iditoth;         /* Other taxable income (130)                 */
	NUMBER   iditrrsp;        /* Taxable RRSP withdrawals                   */
	NUMBER   idiworkc;        /* Workers compensation benefits              */
	NUMBER   idlabtxg;        /* Gross Labour funds bought (413)            */
	NUMBER   idmedgro;        /* Medical expenses, gross (330)              */
	NUMBER   idmincar;        /* Minimum tax carryover (504)                */
	NUMBER   idmovexp;        /* Imputed moving expenses (219)              */
	NUMBER   idmschpd;        /* Manitoba school taxes paid                 */
	NUMBER   idnclos;         /* Allowable other years non-capital loss (252) */
	NUMBER   idnorth;         /* Northern deductions (255)                  */
	NUMBER   idothded;        /* Other deductions from total income (232)   */
	NUMBER   idothpe;         /* Amounts for Infirm Dependants (306)        */
	NUMBER   idpartlo;        /* Limited partnership losses (251)           */
	NUMBER   idpolcon;        /* Federal political contributions (409)      */
	NUMBER   idproptx;        /* Net property taxes paid (556)              */
	NUMBER   idprvftc;        /* Provincial foreign tax credit (Form T2036) */
	NUMBER   idprvpol;        /* Provincial political contributions (565)   */
	NUMBER   idrentpd;        /* Total rental payments (555)                */
	NUMBER   idrklyun;        /* Raking foundation: weeks unemployed        */
	NUMBER   idrklyww;        /* Raking foundation: weeks worked            */
	NUMBER   idrpp;           /* Registered pension plan contributions (207) */
	NUMBER   idrrsp;          /* RRSP calculated amount (208)               */
	NUMBER   idsamths;        /* Imputed number of months on SA             */
	NUMBER   idsapred;        /* Predicted amount of SA received            */
	NUMBER   idsarank;        /* Predicted ranked likelihood of getting SA  */
	NUMBER   idstkded;        /* Stock option deduction (249)               */
	NUMBER   idtuitn;         /* Tuition fees (320)                         */
	NUMBER   iduipred;        /* Predicted amount of UI received            */
	NUMBER   iduirank;        /* Predicted ranked likelihood of getting UI  */
	NUMBER   idext[NUMEXT];   /* Extra numbers [array]                      */
	NUMBER   idrand[NUMRAND]; /* Random numbers [array]                     */

	struct uc_ uc1;          /* UI claim #1 data [struct]                   */
	struct uc_ uc2;          /* UI claim #2 data [struct]                   */
} id_;


/**
* The structure identified by the letters 'ub' holds modelled variables  
* pertaining to a UI claim.
**/

typedef struct ub_ {
	int      ubclmwk;         /* Weeks on claim                             */
	int      ubcalwk;         /* Weeks on claim in calendar year            */
    int      ubeiwbp;         /* Weeks of past EI benefits */
	int      ubp1;            /* Week # of first payment                    */
	int      ubp2;            /* Week # of start of second phase            */
	int      ubp3;            /* Week # of start of third phase             */
	int      ubp4;            /* Week # of last payment                     */
	int      ubp5;            /* Week # of last other type of ui/ei payment */
	int      ubp6;            /* Week # of last training payment            */
	int      ubp1c;           /* Week # of first payment (windowed)         */
	int      ubp2c;           /* Week # of start of second phase (windowed) */
	int      ubp3c;           /* Week # of start of third phase (windowed)  */
	int      ubp4c;           /* Week # of last payment (windowed)          */
	int      ubp5c;           /* Week # of last other type of ui/ei payment (windowed) */
	int      ubp6c;           /* Week # of last training payment (windowed) */
	NUMBER   ubclmpd;         /* Benefits paid on claim                     */
	NUMBER   ubcalpd;         /* Benefits paid in calendar year             */
	NUMBER   ubclmfs;         /* Family supplement paid on claim            */
	NUMBER   ubcalfs;         /* Family supplement paid in calendar year    */
	NUMBER   ubern;           /* Modelled insurable weekly earnings         */
} ub_;


/**
* The structure identified by the letters 'im' holds modelled variables  
* pertaining to an individual.
**/

typedef struct im_ {
	NUMBER   imalexp;         /* Allowable employment expenses               */
	NUMBER   imamtdf;         /* Difference due to minimum tax               */
	NUMBER   imamtinc;        /* Net adjusted income used for minimum tax    */
	NUMBER   imamtprv;        /* Federal minimum tax concept used for provincial minimum tax */
	NUMBER   imashelt;        /* Alberta shelter increase under NCBS clawback */
	NUMBER   imatpinc;        /* After tax Income for low income measurement */
	NUMBER   imatxc;          /* Age tax credit                              */
	NUMBER   imatxcrt;        /* Total tax credits applied                   */
	NUMBER   imbft;           /* Basic federal tax                           */
	NUMBER   imbpt;           /* Basic provincial tax                        */
	NUMBER   imbtc;           /* Basic personal tax credit                   */
	NUMBER   imbtcs;          /* Basic personal tax credit supplement        */
	NUMBER   imcatc;          /* B.C. Refundable Climate Action Tax Credit   */
	NUMBER   imccea;          /* Child care expenses allowed                 */
	NUMBER   imccec;          /* Child care expenses claimed on behalf of child */
	NUMBER   imcceni;         /* Consumable income with CCE not zeroed       */
	NUMBER   imccezi;         /* Consumable income with CCE zeroed           */
	NUMBER   imcchc;          /* Child's non-refundable credit               */
	NUMBER   imcemc;          /* Child's equivalent to married credit        */
	NUMBER   imchara;         /* Allowable charitable donations and gifts (calculated) */
	NUMBER   imchartc;        /* Charitable donations tax credit             */
	NUMBER   imchclm;         /* Number of dependent children claimed        */
	NUMBER   imclimdv;        /* B.C. Climate Action Dividend                */
	NUMBER   imcppctc;        /* CPP contributions tax credit                */
	NUMBER   imcppse;         /* CPP contributions deduction for self-emp    */
	NUMBER   imcqppc;         /* CPP/QPP contributions                       */
	NUMBER   imctcben;        /* Federal child tax credit and child benefits */
	NUMBER   imctc;           /* Child tax credit                            */
	NUMBER   imctxcrt;        /* Tax credits transferred from children       */
	NUMBER   imctxcs;         /* Dependent children tax credits              */
	NUMBER   imcapgex;        /* Modelled capital gains deduction (254)      */
	NUMBER   imdedfn;         /* All deductions from net income              */
	NUMBER   imdedft;         /* Deductions from total income                */
	NUMBER   imdepni;         /* Dependant's net income                      */
	NUMBER   imdisatc;        /* Disability tax credit                       */
	NUMBER   imedrcv;         /* Education and tuition transferred from others*/
	NUMBER   imedtrf;         /* Education and tuition transferred to others  */
	NUMBER   imedtxc;         /* Education allowance tax credit              */
	NUMBER   imemptxc;        /* Canada Employment Credit                    */
	NUMBER   imfar;           /* Family allowance recovery                   */
	NUMBER   imfcben;         /* Total Federal Child Benefits                */
	NUMBER   imfcbenb;        /* Total Federal Child Benefits Base           */
	NUMBER   imfcbene;        /* Total Federal Child Benefit Supplement      */
	NUMBER   imfdtxc;         /* Federal dividend tax credit                 */
	NUMBER   imfedbal;        /* Federal taxes less transfers                */
	NUMBER   imfedtax;        /* Federal tax before tax credits              */
	NUMBER   imffa;           /* Federal portion of family allowances        */
	NUMBER   imfothtr;        /* Federal other trans income and ref. credits */
	NUMBER   imfoth;          /* Federal other government income             */
	NUMBER   imfptc;          /* Federal Political Contribution Tax Credit   */
	NUMBER   imfsa;           /* Federal social assistance                   */
	NUMBER   imfstc;          /* Federal sales tax credit                    */
	NUMBER   imfsur;          /* Federal surtax                              */
	NUMBER   imfsurtr;        /* Federal surtax reduction                    */
	NUMBER   imftax;          /* Federal taxes                               */
	NUMBER   imfortc;         /* Federal other refundable tax credits        */
	NUMBER   imftran;         /* Federal transfer income                     */
	NUMBER   imgisinc;        /* Individual's income for GIS/SPA reduction   */
	NUMBER   imgismax;        /* Maximum amount of GIS                       */
	NUMBER   imheatrl;        /* Federal relief for heating expenses         */
	NUMBER   imfecb;          /* Federal Energy Cost Benefit                 */
	NUMBER   imiafetc;        /* Alberta Family Employment Tax Credit Benefits */
	NUMBER   imibcfb;         /* B.C. Family Bonus                           */
	NUMBER   imibceib;        /* B.C. Earned Income Benefit                  */
	NUMBER   imicapgt;        /* Capital gains (taxable)                     */
	NUMBER   imicqp;          /* CPP/QPP payable                             */
	NUMBER   imicnet;         /* Net Income (line 236)                       */
	NUMBER   imictot;         /* Total Income (line 150)                     */
	NUMBER   imidivt;         /* Dividend income (taxable)                   */
	NUMBER   imisdivt;        /* Dividend income from small corporations (taxable) */
	NUMBER   imildivt;        /* Dividend income from large (eligible) corporations (taxable) */
	NUMBER   imiemp;          /* Wages and salaries                          */
	NUMBER   imiself;         /* Total self-employment income                */
	NUMBER   imigis;          /* GIS benefits                                */
	NUMBER   imigist;         /* GIS provincial top-up                       */
	NUMBER   imigispa;        /* GIS and spouse's allowance                  */
	NUMBER   imiasb;          /* Alberta seniors benefit                     */
	NUMBER   imiloss;         /* Modelled Business investment losses (217)   */
	NUMBER   iminet;          /* Net income                                  */
	NUMBER   iminsda;         /* NS Direct Assistance Program                */
	NUMBER   imintsl;         /* Federal interest on student loans tax credit */
	NUMBER   imioas;          /* OAS benefits                                */
	NUMBER   imiosa;          /* Other SA or guarantees                      */
	NUMBER   imfnewpg;        /* Federal new programs                        */
	NUMBER   imiotg;          /* Other taxable demogrants                    */
	NUMBER   imipnst;         /* Taxable pension income (after splitting)    */
	NUMBER   imiotax;         /* Ontario taxable income                      */
	NUMBER   imishri;         /* Shared income concept (SHS & SPSD)          */
	NUMBER   imisa;           /* Social assistance (or replacement program)  */
	NUMBER   imispa;          /* Spouse's allowance                          */
	NUMBER   imitax;          /* Taxable income                              */
	NUMBER   imitot;          /* Total income                                */
	NUMBER   imiuib;          /* Unemployment Insurance/Employment Insurance benefits */
	NUMBER   imlabtxc;        /* Federal labour-sponsored funds tax credit   */
	NUMBER   immanltc;        /* Manitoba learning tax credit                */
	NUMBER   immaramt;        /* Adjustment to income source                 */
	NUMBER   immartax;        /* Change in consumable income after adjustment */
	NUMBER   immartxc;        /* Married tax credit claimed                  */
	NUMBER   immcb;           /* Manitoba Child Benefit                      */
	NUMBER   immdisp;         /* Disposable income                           */
	NUMBER   immeda;          /* Medical expenses allowed (computed)         */
	NUMBER   immedatc;        /* Medical expenses allowed tax credit         */
	NUMBER   immedref;        /* Refundable medical expense supplement (452) */
	NUMBER   immemp;          /* All employment income                       */
	NUMBER   immhmas;         /* Manitoba Homeowners Assistance              */
	NUMBER   immicons;        /* Consumable income                           */
	NUMBER   imminamt;        /* Minimum Amount due to Federal Minimum Tax   */
	NUMBER   imminv;          /* Investment income                           */
	NUMBER   immmkt;          /* Market income                               */
	NUMBER   immoth;          /* Other income                                */
	NUMBER   immtax;          /* All taxes                                   */
	NUMBER   immtot;          /* Total income                                */
	NUMBER   immtran;         /* All transfer income                         */
	NUMBER   imnbcbb;		  /* NB child tax Benefits base amount           */
	NUMBER   imnbwis;		  /* NB child tax Benefits WIS                   */
	NUMBER   imnfach;         /* Number of family allowance children claimed */
	NUMBER   imnbcben;		  /* Total NB child tax Benefits                 */
	NUMBER   imncb;     	  /* Newfoundland Child Benefit                  */
	NUMBER   imnettr;         /* Net transfers to person                     */
	NUMBER   imnmbns;         /* Newfoundland Mother-Baby Nutrition Supplement */
	NUMBER   imnpfgb;         /* Newfoundland Progressive Family Growth Benefit */
	NUMBER   imnpsb;          /* Newfoundland Parental Support Benefit       */
	NUMBER   imnptc;          /* Non-refundable provincial tax credits       */
	NUMBER   imnscb;          /* Nova Scotia Child Benefit                   */
	NUMBER   imnstxrf;        /* Nova Scotia Taxpayer Refund                 */
	NUMBER   imoasr;          /* OAS recovery                                */
	NUMBER   imoasres;        /* Partial OAS fraction                        */
	NUMBER   imoccea;         /* Ont. Child Care Exp. credit allowed (Family)*/
	NUMBER   imoccext;        /* Ont. Child Care Supplement one-time extra benefit */
	NUMBER   imopstc;         /* Ont. Property and Sales Tax Credit          */
	NUMBER   imonteht;        /* Ontario Employers Heath Tax (Self-employed) */
	NUMBER   imothpe;         /* Other dependant tax credit (306)            */
	NUMBER   imotxdv;         /* Ontario Taxpayer Dividend                   */
	NUMBER   imfortxc;        /* Federal foreign tax credit (509)            */
	NUMBER   imoftca;         /* Other federal tax credits applied (416)     */
	NUMBER   imothrep;        /* Other federal repayments                    */
	NUMBER   impalltc;        /* All refundable provincial tax credits       */
	NUMBER   impatxc;         /* Prov Age amount tax credit                   */	
	NUMBER   impatxcr;        /* Prov total tax credits applied */
	NUMBER   impbtc;          /* Prov Basic amount                            */
	NUMBER   impcgtc;         /* Prov Caregiver Tax Credit Level                  */
	NUMBER   impchrtc;        /* Prov Charitable Donations tax credit         */
	NUMBER   impcpptc;        /* Prov CPP/QPP contributions tax credit        */
	NUMBER   impctxct;        /* Prov tax credits transferred from children   */
	NUMBER   impdistc;        /* Prov Disability tax credit                   */
	NUMBER   impdtxc;         /* Provincial dividend tax credit              */
	NUMBER   impedtxc;        /* Prov Education tax credit                    */
	NUMBER   impintsl;        /* Provincial interest on student loans tax credit */
	NUMBER   impeht;          /* Provincial elderly health tax               */
	NUMBER   impehtc;         /* Provincial elderly health tax credit        */
	NUMBER   impentxc;        /* Pension income tax credit                   */
	NUMBER   impnsph;        /* Pension income split for head                   */
	NUMBER   impnsps;        /* Pension income split for spouse                 */
	NUMBER   impfa;           /* Provincial family allowance                 */
	NUMBER   impfp;           /* Provincial family programs                  */
	NUMBER   impheatr;        /* Provincial Home Heating Fuel/Energy Rebate  */
	NUMBER   imphp;           /* Provincial Health Premium  */
	NUMBER   implvctc;        /* Provincial labour-sponsored funds tax credit*/
	NUMBER   impmartc;        /* Prov Married and Equivalent to Married tax credit */
	NUMBER   impmeda;         /* Prov Medical expenses allowed (computed)         */
	NUMBER   impmedtc;        /* Prov Medical Expenses tax credit             */
	NUMBER   impentrd;        /* Pension income deduction from transfer to spouse      */
	NUMBER   impentr;         /* Pension income transferred from spouse      */
	NUMBER   impminco;        /* Prov Minimum tax carryover                  */
	NUMBER   impnit;          /* Provincial net income tax                   */
	NUMBER   importc;         /* Other refundable provincial tax credits     */
	NUMBER   impoth;          /* Provincial other government income          */
	NUMBER   impothpe;        /* Provincial infirm dependents 18 or older tax credit  */
	NUMBER   impovinc;        /* Before Tax Income for low income measurement */
	NUMBER   imppentc;        /* Prov Pension Income tax credit               */
	NUMBER   impptc;          /* Provincial Political Contrib Tax Credit     */
	NUMBER   impptxtc;        /* Provincial Property tax tax credit          */
	NUMBER   impptg;          /* Provincial property tax grant for seniors   */
	NUMBER   improptx;        /* Imputed property tax paid                  */
	NUMBER   imprvbal;        /* Provincial taxes less transfers             */
	NUMBER   imprvtax;        /* Prov gross provincial tax    */
	NUMBER   impsa;           /* Provincial social assistance                */
	NUMBER   impstg;          /* Provincial sales tax grant for seniors      */
	NUMBER   impstxct;        /* Prov tax credits transferred from spouse */
	NUMBER   impsur;          /* Provincial surtax                           */
	NUMBER   imptax;          /* Provincial taxes                            */
	NUMBER   imptaxcr;        /* Prov total tax credits */
	NUMBER   imptc;           /* Refundable provincial tax credits           */
	NUMBER   imptr;           /* Provincial tax reduction                    */
	NUMBER   imptran;         /* Provincial transfer income                  */
	NUMBER   impttxct;        /* Prov total tax credits transferred */
	NUMBER   imptutxc;        /* Prov Tuition tax credit                      */
	NUMBER   impuictc;        /* Prov UI/EI contributions tax credit          */
	NUMBER   impyctxc;        /* Prov Young Child tax credit          */
	NUMBER   imqaafa;	      /* Quebec Availability Allowance FA Supplement */
	NUMBER   imqnbfa;	      /* Quebec newborn Allowance                    */
	NUMBER   imqca;           /* Quebec Child Assistance refundable tax credit */
	NUMBER   imqalexp;        /* Quebec allowable employment expenses        */
	NUMBER   imqatc;          /* Quebec age tax credit                       */
	NUMBER   imqbtax;         /* Quebec tax before credits                   */
	NUMBER   imqbtc;          /* Quebec basic tax credit                     */
	NUMBER   imqcapgx;        /* Quebec modelled capital gains deduction (254)  */
	NUMBER   imqcapgt;        /* Quebec taxable capital gains                */
	NUMBER   imqcalr;         /* Quebec combined age, living alone, retirement credit */
	NUMBER   imqccea;         /* Quebec child care expenses allowed (dedn)   */
	NUMBER   imqcceni;        /* Quebec net income for refundable cce credit calculation */
	NUMBER   imqccetc;        /* Quebec refundable child care expenses Tax Credit */
	NUMBER   imqcchc;         /* Quebec child's non-refundable credit        */
	NUMBER   imqchara;        /* Quebec allowable charitable donations(calc) */
	NUMBER   imqchatc;        /* Quebec allowable charitable donations tax credit */
	NUMBER   imqcpptc;        /* Quebec CPP/QPP contributions tax credit     */
	NUMBER   imqctpc;         /* Quebec child's transfer of recognized parental contribution */
	NUMBER   imqdctc;         /* Quebec dependent child tax credits          */
	NUMBER   imqdedea;        /* Quebec employment allowance                 */
	NUMBER   imqdedfn;        /* Quebec all deductions from net income       */
	NUMBER   imqdedft;        /* Quebec deductions from total income         */
	NUMBER   imqdepni;        /* Quebec dependant's net income               */
	NUMBER   imqdistc;        /* Quebec disability tax credit                */
	NUMBER   imqdtxc;         /* Quebec dividend tax credit                  */
	NUMBER   imqei;           /* Quebec eligible income for tax reduction    */
	NUMBER   imqempdd;        /* Quebec earned income deduction for workers  */ 
	NUMBER   imqftr;          /* Quebec family tax reduction                 */
	NUMBER   imqhsfc;         /* Quebec Health Services Fund Contributions   */
	NUMBER   imqhsftc;        /* Quebec Health Services Fund Cont.tax credit */
	NUMBER   imqidc;          /* Quebec income of designated dependent child */
	NUMBER   imqidivt;        /* Quebec taxable dividends                    */
	NUMBER   imqldivt;        /* Quebec taxable dividends from large (eligible) corporations  */
	NUMBER   imqsdivt;        /* Quebec taxable dividends from small corporations  */
	NUMBER   imqiloss;        /* Quebec Modelled Business investment losses (234)*/
	NUMBER   imqipnst;        /* Quebec taxable pension income (after splitting)    */
	NUMBER   imqinet;         /* Quebec net income                           */
	NUMBER   imqitax;         /* Quebec taxable income                       */
	NUMBER   imqitot;         /* Quebec total income                         */
	NUMBER   imqittr;         /* Quebec income tested tax reduction          */
	NUMBER   imqmeda;         /* Quebec medical expenses allowed             */
	NUMBER   imqmedrc;        /* Quebec refundable tax credit for medical expenses */
	NUMBER   imqmtc;          /* Quebec married tax credit                   */
	NUMBER   imqpentrd;       /* Quebec Pension income deduction from transfer to spouse      */
	NUMBER   imqpentr;        /* Quebec Pension income transferred from spouse      */
	NUMBER   imqpnsph;        /* Quebec pension income split for head                   */
	NUMBER   imqpnsps;        /* Quebec pension income split for spouse                 */
	NUMBER   imqptr;          /* Quebec property tax refund                  */
	NUMBER   imqritc;         /* Quebec retirement income tax credit         */
	NUMBER   imqstkdd;        /* Quebec modelled Stock option deduction (249)*/
	NUMBER   imqstr;          /* Quebec sales tax refund                     */
	NUMBER   imqta;           /* Quebec tax abatement (total)                */
	NUMBER   imqtaa;          /* Quebec tax abatement (applied)              */
	NUMBER   imqtar;          /* Quebec tax abatement (refundable)           */
	NUMBER   imqtca;          /* Quebec tax credits applied                  */
	NUMBER   imqtct;          /* Quebec tax credits transferable             */
	NUMBER   imqtctc;         /* Quebec tax credits transferred from child  */
	NUMBER   imqtcts;         /* Quebec tax credits transferred from spouse  */
	NUMBER   imqtfa;          /* Quebec taxable family allowances            */
	NUMBER   imqtottc;        /* Quebec total tax credits                    */
	int      imqtpcn;         /* Quebec number of children who transferred parental contribution */
	NUMBER   imqtuitt;        /* Quebec amount of tuition transferred to parents */
	NUMBER   imquictc;        /* Quebec UI contributions tax credit          */
	NUMBER   imqlatc;         /* Quebec living alone tax credit              */
	NUMBER   imqcppse;        /* Quebec self-employ. QPP/CPP Cont. deduction */
	NUMBER   imqintsl;        /* Quebec student loan interest tax credit     */
	NUMBER   imqwrkpr;        /* Quebec Work Premium refundable tax credit   */
	NUMBER   imqawrkpr;       /* Quebec Adapted Work Premium for Disabled refundable tax credit   */
	NUMBER   imrentpd;        /* Imputed rent paid                           */
	NUMBER   imrepay;         /* Social Benefits Repayments                  */
	NUMBER   imresreb;        /* Alberta Resource Rebate Amount              */
	NUMBER   imsaclaw;        /* Social Assistance NCB Clawback Amount       */
	NUMBER   imscb;           /* Saskatchewan Child Benefit                  */
	NUMBER   imsctxc;         /* Saskatchewan Non-refundable child tax credit*/
	NUMBER   imspamax;        /* Maximum amount of SPA                       */
	NUMBER   imssstxc;        /* Saskatchewan senior supplement tax credit   */
	NUMBER   imsstc;          /* Saskatchewan refundable sales tax credit    */
	NUMBER   imstkded;        /* Modelled Stock option deduction (249)       */
	NUMBER   imstxcrt;        /* Tax credits transferred from spouse         */
	NUMBER   imtaxcr;         /* Total tax credits                           */
	NUMBER   imtfa;           /* Taxable family allowances                   */
	NUMBER   imtutxc;         /* Tuition tax credit                          */
	NUMBER   imttxcrt;        /* Total tax credits transferred               */
	NUMBER   imtxf;           /* Federal income tax payable                  */
	NUMBER   imtxfc;          /* Federal commodity taxes                     */
	NUMBER   imtxp;           /* Provincial income tax payable               */
	NUMBER   imtxpc;          /* Provincial commodity taxes                  */
	NUMBER   imtxttxc;        /* Federal textbook tax credit                 */
	NUMBER   imuibr;          /* UI benefit recovery                         */
	NUMBER   imuic;           /* UIC contributions                           */
	NUMBER   imuicrf;         /* UI EI contribution refund					 */
	NUMBER   imuictc;         /* UIC contributions tax credit                */
	NUMBER   imqhptc;         /* Quebec Home Parent Tax Credit level         */
	NUMBER   imqhptcp;        /* Quebec Home Parent Tax Credit potential level    */
	NUMBER   imqncgtc;        /* Quebec Refundable Natural Caregivers Tax Credit  */
	NUMBER   imqnctcp;        /* Quebec Refundable Natural Caregivers Tax Credit potential amount  */
	NUMBER   imqduetc;        /* Quebec Dues Tax Credit                      */
	NUMBER   imqfatc;         /* Quebec Family Allowance Tax Credit          */
    NUMBER   imqtuitc;        /* Quebec Tuition Tax Credit                   */
    NUMBER   imqamt;          /* Quebec Minimum Tax Amount                   */
	NUMBER   imcgtc;          /* Caregiver Tax Credit Level                  */
	NUMBER   imcgtcp;         /* Dependent Caregiver Tax Credit Amount       */
	NUMBER   impcgtcp;        /* Provincial Dependent Caregiver Tax Credit Amount       */
	NUMBER   imqapf;          /* Quebec Anti-Poverty Fund                    */
	int      imqamtfg;        /* Quebec Minimum Tax flag                     */
	int      imamtfg;         /* Minimum tax flag                            */
	int      imccez;          /* Is CCE zeroed?                              */
	int      imfiler;         /* Taxable filer status                        */
	int      imgistyp;        /* Type of GIS entitlement                     */
	int      imoaspar;        /* Partial OAS residency flag                  */
	int      imoldtyp;        /* Type of GIS/SPA nuclear family              */
	int      imqfs;           /* Quebec family situation (1-5)               */
	int      imqndc;          /* Quebec number of dependent children         */
	int      imqsfflg;        /* Quebec simplified tax form flag 0=general 1=simplified */
	int      imspatyp;        /* Type of SPA entitlement                     */
	int      imuidpfg;        /* UI claimants has dependents flag            */
	int      imuiydep;        /* Number of dependents under 7 for family supplement (EI only) */
	int      imqpipt;         /* Quebec parental insurance plan type (basic/special) */
	NUMBER   imqpipcbn;       /* Quebec parental insurance plan calendar year benefits  */
	NUMBER   imqpiptbn;       /* Quebec parental insurance plan total benefits  */
	NUMBER   imqpipcwk;       /* Quebec parental insurance plan calendar year weeks  */
	NUMBER   imqpiptwk;       /* Quebec parental insurance plan total weeks  */
	NUMBER   imqpcmtb;        /* Quebec parental insurance plan calendar year maternity benefits */
	NUMBER   imqpcptb;        /* Quebec parental insurance plan calendar year paternity benefits */
	NUMBER   imqpcprb;        /* Quebec parental insurance plan calendar year parental benefits  */
	NUMBER   imqptmtb;        /* Quebec parental insurance plan total maternity benefits */
	NUMBER   imqptptb;        /* Quebec parental insurance plan total paternity benefits */
	NUMBER   imqptprb;        /* Quebec parental insurance plan total parental benefits  */
	NUMBER   imqpcmtw;        /* Quebec parental insurance plan calendar year maternity weeks */
	NUMBER   imqpcptw;        /* Quebec parental insurance plan calendar year paternity weeks */
	NUMBER   imqpcprw;        /* Quebec parental insurance plan calendar year parental weeks  */
	NUMBER   imqptmtw;        /* Quebec parental insurance plan total maternity weeks */
	NUMBER   imqptptw;        /* Quebec parental insurance plan total paternity weeks */
	NUMBER   imqptprw;        /* Quebec parental insurance plan total parental weeks  */
	NUMBER   imqpipp;         /* Quebec parental insurance plan premiums for paid workers */
	NUMBER   imqpippse;       /* Quebec parental insurance plan premiums for self-employed */
	NUMBER   imqpiptc;        /* Quebec parental insurance plan premium tax credit for paid workers (federal)*/
	NUMBER   imqpsetc;        /* Quebec parental insurance plan premium tax credit for self-employed (federal) */
	NUMBER   imqpsedd;        /* Quebec parental insurance plan premium deduction for self-employed (federal) */
	NUMBER   imqqpiptc;       /* Quebec parental insurance plan premium tax credit for paid workers (Quebec) */
	NUMBER   imqqpsetc;       /* Quebec parental insurance plan premium tax credit for self-employed (Quebec) */
	NUMBER   imqqpsedd;       /* Quebec parental insurance plan premium deduction for self-employed (Quebec) */
	NUMBER   imuccb;          /* Universal Child Care Benefit amount */
	NUMBER   imbcss;          /* British Columbia Seniors Supplement */
	NUMBER   imwitb;          /* Working Income Tax Benefit */
	NUMBER   imwitbs;         /* Working Income Tax Benefit Supplement for Disabilities */
	NUMBER   imchdtxc;        /* Non-refundable Child Tax Credit */
	NUMBER   imocb;           /* Ontario Child Benefit */
	NUMBER   imocbyng;        /* Ontario Child Benefit amount for Child Care Supplement dependents */
	NUMBER   imoshptg;        /* Ontario Senior Homeowners' Property Tax Grant */
	NUMBER   immftb;          /* Manitoba Family Tax Benefit */
	int      immndc;          /* Manitoba number of dependent children 18 or younger        */
	int      imondc;          /* Ontario number of dependent children 18 or younger        */
	NUMBER   imotxtgr;        /* Ontario Textbook and Technology Grant */



	struct ub_ ub1;          /* UI claim #1 results [struct]                 */
	struct ub_ ub2;          /* UI claim #2 results [struct]                 */

	struct uv_  uv;           /* user variables [struct]                     */
	char imfill[UVGLSIZE-sizeof(uv_)]; /* filler array for modeled variables [array]        */
} im_;


/**
* The structure identified by the letters 'ic' holds variables  
* pertaining to an individual used for model control functions.
**/

typedef struct ic_ {
	int      icselect;        /* Individual selected flag                   */
	NUMBER   icmarinc;        /* Income after adjustment                    */
	NUMBER   icmarold;        /* Saved old SPSD income before adjustment    */
	NUMBER   icmaramt;        /* Amount of SPSD income adjustment           */
	int      icrolled;        /* Individual already rolled up flag          */
	int      ictpflg;         /* Person will be changed (TP facility)       */
} ic_;


/**
* The structure identified by the letters 'in' holds all variables,
* both SPSD databse and modelled variables pertaining to an individual.
**/

typedef struct in_ {
	struct id_ id;       /* Individual SPSD variables [struct]               */
	struct im_ im;       /* Individual variant result variables [struct]     */
	struct im_ imbase;   /* Individual base result variables [struct]        */
	struct ic_ ic;       /* Individual model control variables [struct]      */
	NUMBER uvbl[NUMUV];  /* Black-box user variables [array]                 */
} IN_;


/**
* The structure identified by the letters 'hh' holds all variables,
* both SPSD database and modelled, that pertain to a given household.
**/

typedef struct hh_ {
	int    hhnef;            /* Number of economic families in household     */
	int    hhncf;            /* Number of census families in household       */
	int    hhnnf;            /* Number of nuclear families in household      */
	int    hhnin;            /* Number of individuals in household           */
	struct hd_ hd;           /* Housing characteristics data [struct]        */
	struct fx_ fx;           /* SHS data [struct]                            */
	struct ct_ ct;           /* Commodity tax variables [struct]             */
	struct ct_ ctbase;       /* Base commodity tax variables [struct]        */
	struct ef_ ef[MAXFAM];   /* Economic family data [array]                 */
	struct cf_ cf[MAXFAM];   /* Census family data [array]                   */
	struct nf_ nf[MAXFAM];   /* Nuclear family data [array]                  */
	struct in_ in[MAXPERS];  /* Individual data [array]                      */
} hh_;



/* ==================== GLOBAL VARIABLE DECLARATIONS ====================== */

/* the following global structures are defined in global.c */


/**
* HHLD is the global structure which holds SPSD/M variables for
* the current household.
**/

extern S_hh NEARHHLD HHLD;


/**
* CT_VAR saves the simulated household-level commodity tax data 
* for the variant model from the previous household, to allow the model
* to recompute it only when necessary (i.e. when the SHS expenditure
* record itself changes).  This structure is only used if the
* variant/base capability is activated.
**/

extern S_ct   FAR CT_VAR;



/* ==================== FUNCTION PROTOTYPES =============================== */

extern  void vsdd(void);
extern  void vsdm(void);
extern  void vsdu(void);




