/***
*Atxcalc.cpp - Calculate federal income tax
*
*$Id: Atxcalc.cpp 6.146 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 08/06/12 | Correct family conc. for QWITB-sing par treat as single |
* | LP  | 08/06/11 | Added caregivers modeled credit for CGTCOPT set to 3    |
* | LP  | 08/06/03 | Removed caregivers (imcgtc) from transfer of credits    |
* | LP  | 08/01/04 | Added B.C. and Quebec WITB programs                     |
* | LP  | 07/10/16 | Revised WITB disability supp. if both spouses eligible  |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/04/19 | Added non-ref child tax credit to transferable credits  |
* | LP  | 07/04/04 | Added Working Income Tax Benefit (WITB) - ref. credit   |
* | LP  | 07/01/22 | Added AMTDIVLC - AMT Dividend Rate for eligible dividend|
* | LP  | 07/01/18 | Removed imuccb from credit transfers test - uses imdepni|
* | LP  | 06/12/20 | Added clergy deduction to ref. medical tax credit test  |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/20 | Added imbcss (BC sen supp as part of sa) to income tests|
* | LP  | 06/06/07 | Subtract imuccb from imdepni for credit transfers test  |
* | LP  | 06/05/30 | Subtract imuccb from iminet for medical tax credit test |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate FDTCRLC|
* | LP  | 06/05/11 | Added Canada Employment Credit imemptxc to non-ref cred |
* | LP  | 06/05/08 | Added MEDRTD refundable medical expense income turndown |
* | LP  | 06/05/05 | Added textbook tax credit (imtxttxc) to credits/transfer|
* | CH  | 06/01/04 | Added imqpiptc                                          |
* | CH  | 05/09/12 | Deleted idfdsft                                         |
* | LP  | 05/04/15 | Added BXMEXT, MXMEXT, MXMTEXT to eligibility tests      |
* | LP  | 04/02/18 | Updated net income concept for ref. medical exp tax crd |
* | LP  | 03/11/18 | Removed caregivers tax credit loop                      |
* | LP  | 03/11/17 | Added imintsl to non-transferable tax credits           |
* | CH  | 03/10/17 | Deleted iddisab                                         |
* | CH  | 03/10/17 | Deleted foravg                                          |
* | LP  | 01/05/09 | Added new Federal Alternative Minimum Tax parameters    |
* |     |          |   and variables                                         |
* | LP  | 01/03/11 | Removed pre-1988 parameters and variables               |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/10/22 | Replaced MEDRTD with a calculation                      |
* | CH  | 00/06/14 | Added imothpe and changed foreign tax credit            |
* | BBM | 00/04/18 | Drop idfdfatc for version 8.0 database                  |
* | CH  | 00/04/14 | Apply CAPGIR to capgex stkded and iloss                 |
* | CH  | 00/03/07 | Added imamtinc and changed cgtc calculation for idrhcf=2|
* | CH  | 00/03/07 | Change calculation for Care Giver Tax Credit            |
* | EGA | 99/10/20 | Change calculation for Care Giver Tax Credit            |
* | LP  | 99/06/01 | Changed Care Giver Tax Credit to reflect Tax Form       |
* | CH  | 99/04/23 | Removed Manitoba Learning Tax Credit Maximum            |
* | CH  | 99/04/15 | Added personal amount supplement                        |
* | CH  | 99/03/26 | Added GST rebate to code                                |
* | CH  | 99/03/22 | Changed idlabtxc to idlabtxg                            |
* | BBM | 98/05/28 | Removed reference to idfsitc, idxii2                    |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/04/07 | Added Federal Surtax Reduction                          |
* | CH  | 98/04/06 | Let RRSP/RPP cont not trigger Alternative Minimm tax    |
* | EGA | 98/04/03 | Add Caregiver Tax Credit                                |
* | CH  | 98/04/03 | Add Maximum for education credits transfered (for Man   |
* |     |          |     Learning tax credit)                                |
* | CH  | 98/04/02 | Add FDFATCFLAG                                          |
* | BBM | 98/03/03 | Fix foreign tax credit bug (nneg the imtxf calculation) |
* | CH  | 98/01/23 | added federal refundable medical expense credit         |
* | CH  | 97/09/15 | add imedtrf,imedrcv and changed tranfered credit code   |
* | BBM | 94/11/25 | Fix foreign tax credit bug (move code within in loop)   |
* | SFG | 94/04/07 | deactivate overlay in MODEL_TEXT (getting too big)      |
* | GJC | 93/02/16 | add missing greenbook variables and imfortxc            |
* | BBM | 92/09/02 | Delete reference to imccetxc                            |
* | BBM | 92/06/15 | Fed political tax credit model based on total donations |
* | SFG | 89/12/07 | Improve appearance of minimum tax calculation code      |
* | SFG | 89/12/05 | Ensure that other fed tax credits can't reduce surtax   |
* | SFG | 89/12/05 | Compute imoftca (other fed tax credits applied          |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxcalc  [global]  Calculate federal income tax
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxcalc[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxcalc - Calculate federal income tax
*
*Synopsis:
*    void Atxcalc(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   minn     Find the minimum of two numbers
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   lkup1    Table look-up with interpolation/extrapolation
*
*Structure Members:
*   MP.AMTDIV       (i)  Alternative minimum tax: percentage of taxable dividends
*   MP.AMTDIVLC     (i)  Alternative minimum tax: percentage of eligible taxable dividends
*   MP.AMTEX        (i)  Alternate minimum tax: exemption level
*   MP.AMTFORTX     (i)  Alternative minimum tax: Foreign Income Tax Rate
*   MP.AMTINCRT     (i)  Alternative minimum tax: Inclusion Rate
*   MP.AMTNEGTI     (i)  Alternative minimum tax: Allow Negative Taxable Income Flag
*   MP.AMTPRVDF     (i)  Alternative minimum tax: prov. tax difference definition[prov]
*   MP.AMTRPFLG     (i)  RRSP/RPP included in Alternate minimum tax (1=included)
*   MP.AMTSTK       (i)  Alternative minimum tax: Stock Option Proportion
*   MP.AMTTX        (i)  Alternate minimum tax rate
*   MP.BXM          (i)  Basic personal exemption/amount
*   MP.BXMEXT       (i)  Basic personal amount extra top-up
*   MP.CAPGIR       (i)  Capital gains inclusion rate
*   MP.CGTC         (i)  Caregiver Tax Credit
*   MP.CGTCFLG      (i)  Caregiver Tax Credit activation flag
*   MP.CGTCOPT      (i)  Caregiver Tax Credit option (1=max, 2=greenbook, 3=model)
*   MP.CGTCTD       (i)  Caregiver Tax Credit income turndown
*   MP.CWITBFAM     (i)  B.C. Working Income Tax Benefit amount for couples/single parents
*   MP.CWITBFPO     (i)  B.C. WITB Family Net Income Phase-out Threshold
*   MP.CWITBMINE    (i)  B.C. Working Income Tax Benefit Minimum Earnings Threshold
*   MP.CWITBPIR     (i)  B.C. Working Income Tax Benefit Phase in Rate
*   MP.CWITBRR      (i)  B.C. Working Income Tax Benefit Reduction Rate
*   MP.CWITBSAMT    (i)  B.C. WITB Supplement for Disabilities Amount
*   MP.CWITBSCRR    (i)  B.C. WITB Supplement for Disabilities Reduction Rate for eligible couples
*   MP.CWITBSFPO    (i)  B.C. WITB Supplement for Disabilities Family Net Income Phase-out Threshold
*   MP.CWITBSMINE   (i)  B.C. WITB Supplement for Disabilities Minimum Earnings Threshold
*   MP.CWITBSNG     (i)  B.C. Working Income Tax Benefit amount for singles
*   MP.CWITBSPIR    (i)  B.C. WITB Supplement for Disabilities Phase in Rate
*   MP.CWITBSPO     (i)  B.C. WITB Single Net Income Phase-out Threshold
*   MP.CWITBSRR     (i)  B.C. WITB Supplement for Disabilities Reduction Rate
*   MP.CWITBSSPO    (i)  B.C. WITB Supplement for Disabilities Single Net Income Phase-out Threshold
*   MP.FDTCR        (i)  Federal dividend tax credit rate
*   MP.FDTCRLC      (i)  Federal dividend tax credit rate from large (eligible) corporations
*   MP.FLVCMAX      (i)  Maximum labour-sponsored funds tax credit allowed
*   MP.FLVCRT       (i)  Percent of labour-sponsored funds cost allowed as credit [prov]
*   MP.FNTCR        (i)  Federal non-refundable tax credit rate
*   MP.FPTC         (i)  Federal political contribution table [total donations,donation allowed]
*   MP.FPTCBEN      (i)  Maximum federal political tax credit allowed
*   MP.FPTCrows     (i)  number of rows in FPTC
*   MP.FSURL1       (i)  Federal surtax level 1
*   MP.FSURL2       (i)  Federal surtax level 2
*   MP.FSURR1       (i)  Federal surtax rate 1
*   MP.FSURR2       (i)  Federal surtax rate 2
*   MP.FSURTRMX     (i)  Maximum federal surtax reduction
*   MP.FSURTRRL     (i)  Federal surtax reduction reduction level
*   MP.FSURTRRR     (i)  Federal surtax reduction reduction rate
*   MP.FTX          (i)  Federal tax table [taxable income,basic federal tax]
*   MP.FTXrows      (i)  number of rows in FTX
*   MP.GSTREBFLAG   (i)  Database variable(gstreb) activation flag
*   MP.MAXDX        (i)  Maximum disability deduction/amount
*   MP.MAXET        (i)  Maximum on transfer of education and tuition amount
*   MP.MEDREMIN     (i)  Minimum earnings for refundable medical expense supplement
*   MP.MEDRMAX      (i)  Maximum expenses allowed for medical expense supplement
*   MP.MEDRRATE     (i)  Proportion of expenses allowed for medical expense supplement
*   MP.MEDRRR       (i)  Reduction rate for the medical expense supplement
*   MP.MEDRTD       (i)  Refundable medical expense supplement income turndown
*   MP.QTAP         (i)  Quebec tax abatement proportion of basic federal tax
*   MP.QWITBFAM     (i)  Quebec Working Income Tax Benefit amount for couples/single parents
*   MP.QWITBFPO     (i)  Quebec WITB Family Net Income Phase-out Threshold
*   MP.QWITBMINEF   (i)  Quebec Working Income Tax Benefit Minimum Earnings Threshold for families
*   MP.QWITBMINES   (i)  Quebec Working Income Tax Benefit Minimum Earnings Threshold for singles
*   MP.QWITBPIRF    (i)  Quebec Working Income Tax Benefit Phase in Rate for families
*   MP.QWITBPIRS    (i)  Quebec Working Income Tax Benefit Phase in Rate for singles
*   MP.QWITBRR      (i)  Quebec Working Income Tax Benefit Reduction Rate
*   MP.QWITBSAMT    (i)  Quebec WITB Supplement for Disabilities Amount
*   MP.QWITBSCRR    (i)  Quebec WITB Supplement for Disabilities Reduction Rate for eligible couples
*   MP.QWITBSFPO    (i)  Quebec WITB Supplement for Disabilities Family Net Income Phase-out Threshold
*   MP.QWITBSMINE   (i)  Quebec WITB Supplement for Disabilities Minimum Earnings Threshold
*   MP.QWITBSNG     (i)  Quebec Working Income Tax Benefit amount for singles
*   MP.QWITBSPIR    (i)  Quebec WITB Supplement for Disabilities Phase in Rate
*   MP.QWITBSPO     (i)  Quebec WITB Single Net Income Phase-out Threshold
*   MP.QWITBSRR     (i)  Quebec WITB Supplement for Disabilities Reduction Rate
*   MP.QWITBSSPO    (i)  Quebec WITB Supplement for Disabilities Single Net Income Phase-out Threshold
*   MP.WITBFAM      (i)  Working Income Tax Benefit amount for couples/single parents
*   MP.WITBFLAG     (i)  Working Income Tax Benefit activation flag
*   MP.WITBFPO      (i)  WITB Family Net Income Phase-out Threshold
*   MP.WITBMINE     (i)  Working Income Tax Benefit Minimum Earnings Threshold
*   MP.WITBPIR      (i)  Working Income Tax Benefit Phase in Rate
*   MP.WITBRR       (i)  Working Income Tax Benefit Reduction Rate
*   MP.WITBSAMT     (i)  WITB Supplement for Disabilities Amount
*   MP.WITBSCRR     (i)  WITB Supplement for Disabilities Reduction Rate for eligible couples
*   MP.WITBSFPO     (i)  WITB Supplement for Disabilities Family Net Income Phase-out Threshold
*   MP.WITBSMINE    (i)  WITB Supplement for Disabilities Minimum Earnings Threshold
*   MP.WITBSNG      (i)  Working Income Tax Benefit amount for singles
*   MP.WITBSPIR     (i)  WITB Supplement for Disabilities Phase in Rate
*   MP.WITBSPO      (i)  WITB Single Net Income Phase-out Threshold
*   MP.WITBSRR      (i)  WITB Supplement for Disabilities Reduction Rate
*   MP.WITBSSPO     (i)  WITB Supplement for Disabilities Single Net Income Phase-out Threshold
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   ef.efin         (i)  First person in economic family [pointer]
*   ef.efnpers      (i)  Number of persons in economic family
*   hd.hdprov       (i)  Province
*   hh.cf           (i)  Census family data [array]
*   hh.ef           (i)  Economic family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnef        (i)  Number of economic families in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idalexp      (i)  Other allowable employment expenses (229)
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.idclergy     (i)  Clergy residential deduction
*   id.idcloss      (i)  Allowable other years capital loss (253)
*   id.iddisab      (i)  Disability status (age 16+)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.iddues       (i)  Union and professional dues (212)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.idemplo      (i)  Employee home relocation loan dedn (248)
*   id.idexplor     (i)  Exploration and development expenses (224)
*   id.idforinc     (i)  Net foreign income (508)
*   id.idfortx      (i)  Foreign tax paid (507)
*   id.idgstreb     (i)  GST rebate (457)
*   id.idicapg      (i)  Capital gains (actual)
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idise        (i)  Self-employed income
*   id.iditc        (i)  Federal investment tax credits (412)
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idnclos      (i)  Allowable other years non-capital loss (252)
*   id.idothded     (i)  Other deductions from total income (232)
*   id.idpartlo     (i)  Limited partnership losses (251)
*   id.idpolcon     (i)  Federal political contributions (409)
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idspoflg     (i)  Person has spouse
*   im.imatxc       (i)  Age tax credit
*   im.imbcss       (i)  British Columbia Seniors Supplement
*   im.imbtc        (i)  Basic personal tax credit
*   im.imbtcs       (i)  Basic personal tax credit supplement
*   im.imcapgex     (i)  Modelled capital gains deduction (254)
*   im.imchartc     (i)  Charitable donations tax credit
*   im.imchdtxc     (i)  Non-refundable Child Tax Credit
*   im.imcppctc     (i)  CPP contributions tax credit
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imctxcs      (i)  Dependent children tax credits
*   im.imdedfn      (i)  All deductions from net income
*   im.imdepni      (i)  Dependant's net income
*   im.imdisatc     (i)  Disability tax credit
*   im.imedtxc      (i)  Education allowance tax credit
*   im.imemptxc     (i)  Canada Employment Credit
*   im.imicnet      (i)  Net Income (line 236)
*   im.imigis       (i)  GIS benefits
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.imiloss      (i)  Modelled Business investment losses (217)
*   im.iminet       (i)  Net income
*   im.imintsl      (i)  Federal interest on student loans tax credit
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imispa       (i)  Spouse's allowance
*   im.imitax       (i)  Taxable income
*   im.immartxc     (i)  Married tax credit claimed
*   im.immeda       (i)  Medical expenses allowed (computed)
*   im.immedatc     (i)  Medical expenses allowed tax credit
*   im.imothpe      (i)  Other dependant tax credit (306)
*   im.impentxc     (i)  Pension income tax credit
*   im.imqpiptc     (i)  Quebec parental insurance plan premium tax credit for paid workers (federal)
*   im.imqpsetc     (i)  Quebec parental insurance plan premium tax credit for self-employed (federal)
*   im.imstkded     (i)  Modelled Stock option deduction (249)
*   im.imtutxc      (i)  Tuition tax credit
*   im.imtxttxc     (i)  Federal textbook tax credit
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   im.imuictc      (i)  UIC contributions tax credit
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imamtdf      (io) Difference due to minimum tax
*   im.imbft        (io) Basic federal tax
*   im.imcgtc       (io) Caregiver Tax Credit Level
*   im.imcgtcp      (io) Dependent Caregiver Tax Credit Amount
*   im.imedtrf      (io) Education and tuition transferred to others
*   im.imfdtxc      (io) Federal dividend tax credit
*   im.imfedtax     (io) Federal tax before tax credits
*   im.imfortxc     (io) Federal foreign tax credit (509)
*   im.imfptc       (io) Federal Political Contribution Tax Credit
*   im.imfsur       (io) Federal surtax
*   im.imfsurtr     (io) Federal surtax reduction
*   im.imlabtxc     (io) Federal labour-sponsored funds tax credit
*   im.immedref     (io) Refundable medical expense supplement (452)
*   im.imoftca      (io) Other federal tax credits applied (416)
*   im.imqtaa       (io) Quebec tax abatement (applied)
*   im.imtaxcr      (io) Total tax credits
*   im.imttxcrt     (io) Total tax credits transferred
*   im.imtxf        (io) Federal income tax payable
*   im.imwitbs      (io) Working Income Tax Benefit Supplement for Disabilities
*   im.imamtfg      (o)  Minimum tax flag
*   im.imamtinc     (o)  Net adjusted income used for minimum tax
*   im.imamtprv     (o)  Federal minimum tax concept used for provincial minimum tax
*   im.imatxcrt     (o)  Total tax credits applied
*   im.imctxcrt     (o)  Tax credits transferred from children
*   im.imedrcv      (o)  Education and tuition transferred from others
*   im.imfortc      (o)  Federal other refundable tax credits
*   im.imminamt     (o)  Minimum Amount due to Federal Minimum Tax
*   im.imqtar       (o)  Quebec tax abatement (refundable)
*   im.imstxcrt     (o)  Tax credits transferred from spouse
*   im.imwitb       (o)  Working Income Tax Benefit
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void Atxcalc(
	P_hh hh
)
{
	register P_nf nf;
	register P_cf cf;
	register int nfi;


	register P_in in;
	register P_in ineld;
	register P_in inspo;
	P_ef ef;
			  
	register int efi;
	register int cfi;
	register int ini;
	NUMBER hftrt = ZERO;             /* elder's   federal tax reduction (if any)   */
	NUMBER sftrt = ZERO;             /* spouse's federal tax reduction (if any)   */
	NUMBER ctxcrt = ZERO;            /* children's total transferable tax credits */
	NUMBER txcrt = ZERO;             /* total transferable tax credits            */
	NUMBER ttxcrt = ZERO;            /* actual amount of transferable tax credits */
	NUMBER tfr1 = ZERO;              /* components of transferable tax credits    */
	NUMBER tfr2 = ZERO;              /* components of transferable tax credits    */
	NUMBER edtrf = ZERO;             /* education and tuition transfered          */
	NUMBER cedtrf = ZERO;            /* total children's education and tuition transfered */
	NUMBER bft = ZERO;               /* basic federal tax                         */
	NUMBER qta = ZERO;               /* Quebec tax abatement                      */
	NUMBER taxcr = ZERO;             /* federal tax credits (itc, fptc, fotc)     */
	NUMBER forinet = ZERO;           /* net income for foreign tax credit calc    */
	                                 /* MINIMUM TAX VARIABLES */
	NUMBER nndcl = ZERO;			 /* Minimum tax: net non-deducted capital losses */
	NUMBER adjtaxin = ZERO;			 /* Minimum tax: adjusted taxable income      */
	NUMBER ntpocg = ZERO;			 /* Minimum tax: non-taxable portion of capital gains */
	NUMBER taxinc = ZERO;			 /* Minimum tax: taxable income for minimum tax */
	NUMBER netadjin = ZERO;          /* Minimum tax: net adjusted taxable income  */
	NUMBER grminamt = ZERO;			 /* Minimum tax: gross minimum amount         */
	NUMBER bmtxcrd = ZERO;			 /* Minimum tax: basic minimum tax credits    */
	NUMBER basfedtx = ZERO;          /* Minimum tax: basic federal tax            */
	NUMBER minamt = ZERO;			 /* Minimum tax: minimum amount due to minimum tax */
	NUMBER taxcred = ZERO;           /* Minimum tax: Line 350 tax credits         */
	NUMBER maxcarry = ZERO;          /* Minimum tax: maximum carryover for taxation year */
	NUMBER mincarry = ZERO;			 /* Minimum tax: minimum carryover            */
	NUMBER fedtax = ZERO;   		 /* Minimum tax: federal tax                  */
	NUMBER sftc = ZERO;				 /* Minimum tax: special foreign tax credit   */
	NUMBER netminamt = ZERO;         /* Minimum tax: net minimum tax payable      */
	NUMBER regtaxpy = ZERO;          /* Minimum tax: regular tax payable          */
	NUMBER taxdiff = ZERO;           /* Minimum tax: difference between net min amt and reg. tax */
	NUMBER fsurval = ZERO;           /* Minimum tax: value used to calculate surtax */
	NUMBER addtaxmt = ZERO;          /* Minimum tax: additional tax due to minimum tax */
	NUMBER mbft = ZERO;              /* Minimum tax: basic federal tax            */
	NUMBER mfsur = ZERO;             /* Minimum tax: federal surtax               */
	NUMBER mtxf = ZERO;              /* Minimum tax: federal tax                  */
	NUMBER mfsurtr = ZERO;           /* Minimum tax: federal surtax reduction     */
	NUMBER divid = ZERO;             /* Minimum tax: dividend tax credits         */

	NUMBER medrtd = ZERO;            /* Medical supplement turndown               */

	NUMBER netinc = ZERO;
	NUMBER nfinet = ZERO;            /* net income for WITB */
	NUMBER nfiemp = ZERO;            /* employment income for WITB */ 
	NUMBER eldemp = ZERO;			 /* employment income for elder for WITB disability supplement */
	NUMBER spoemp = ZERO;			 /* employment income for spouse for WITB disability supplement */
	NUMBER witb = ZERO;              /* working income tax benefit */
	NUMBER witbs = ZERO;             /* working income tax benefit disability supplement */
	NUMBER witbmax = ZERO;           /* calculated WITB before reduction */
	NUMBER netincpo = ZERO;          /* net income phase out threshold for WITB */
	NUMBER netincspo = ZERO; 		 /* net income phase out threshold for WITB disability supplement */
	int spoelg;                      /* spouse eligible for WITB */
	int eldelg;                      /* elder eligible for WITB */
	int famstatus;                   /* family status for WITB thresholds: 0=single, 1=family */
	NUMBER efimcgtcp = ZERO;     

	DEBUG_ON("Atxcalc");

       /* Calculate the Caregiver tax credit */
       if (MP.CGTCFLG  && MP.CGTCOPT == 3) {
       
          int cgassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimcgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 65 and either a head or spouse or other family member who could be a parent (with takeup) */
                if (in->id.idage >=65 || (in->id.iddisab == 1 && in->id.idage>= 18 ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.imcgtcp = MP.FNTCR * minn(nneg(MP.CGTCTD -in->im.iminet),MP.CGTC);
                }
			   efimcgtcp += in->im.imcgtcp;
             }

			 cgassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (cgassign == 0 && in->id.idcgtc > 0 ) {
				   	if (in->id.idspoflg && inspo->im.imcgtcp > 0) {
				   		in->im.imcgtc = nneg(efimcgtcp - inspo->im.imcgtcp - in->im.imcgtcp );
				   		efimcgtcp = efimcgtcp - in->im.imcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.imcgtc = nneg(efimcgtcp - in->im.imcgtcp);
				   		efimcgtcp = efimcgtcp - in->im.imcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (cgassign == 0 && efimcgtcp > 0 && in->id.idcgtc > 0 && in->im.imcgtc == 0 ) {
		 			in->im.imcgtc = efimcgtcp;
		 			cgassign = 1;
		 	 	}
		    }
		 } 	 /* end of ef hh loop */
	}  /* end of cgtc loop */



	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

		/* do needed initialisations for each census family */
		hftrt = ZERO;
		sftrt = ZERO;
		ctxcrt = ZERO;
		cedtrf = ZERO;

		/* initialise elder and spouse pointers */
		ineld = cf->cfineld;
		inspo = cf->cfinspo;   /* NB: is NULL if spouse not present */


		/* do part 1 tax calculation for all persons in family      */

		/* process persons in census family */
		for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {

			DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
			         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);


			if (isnzero(in->id.ididiv)) {
				/* calculate federal dividend tax credit */
				in->im.imfdtxc = (in->im.imisdivt * MP.FDTCR)
								+ (in->im.imildivt * MP.FDTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.imfdtxc);
			}

			if (isnzero(in->im.imitax)) {
				/* calculate federal tax    */
				in->im.imfedtax = (NUMBER) lkup1(MP.FTX, MP.FTXrows, in->im.imitax);
				DEBUG2("%s fedtax =%.2f\n", in->im.imfedtax);
			}
			else {
				DEBUG1("%s fedtax = 0.0\n");
			}
			in->im.imbft = in->im.imfedtax;

			if (in->id.idcfrh >= 2) {
				/* total non-transferable tax credits applicable to dependent */
				in->im.imtaxcr = in->im.imbtc      /* basic personal tax credit       */
						   + in->im.imbtcs      /* basic personal tax credit supplement      */
				           + in->im.imatxc     /* age tax credit                  */
                           + in->im.imothpe    /* other dependent tax credits */
				           + in->im.imchartc   /* charitable donations tax credit */
				           + in->im.imcppctc   /* CPP contributions tax credit    */
				           + in->im.immedatc   /* medical expenses tax credit     */
				           + in->im.impentxc   /* pension income tax credit       */
				           + in->im.imuictc    /* UI contributions tax credit     */
				           + in->im.imfdtxc    /* federal dividend tax credit     */
				           + in->im.imcgtc     /* caregiver tax credit            */
				           + in->id.idmincar   /* minimum tax carryover (504)     */
						   + in->im.imintsl    /* interest on student loans tax credit */
                           + in->im.imqpiptc   /* Quebec parental insurance premiums */
                           + in->im.imqpsetc   /* Quebec parental insurance premiums - self-employed*/
						   + in->im.imemptxc   /* Canada Employment Credit */
				           ;
	    		DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
	    		        in->im.imtaxcr);
	    		in->im.imatxcrt = ZERO;
				if (isnzero(in->im.imtaxcr)) {
					/* if child's basic federal tax is more than transferable */
					/* tax credits, apply total amount to reduce tax          */
					if (in->im.imbft > in->im.imtaxcr) {
						in->im.imatxcrt += in->im.imtaxcr;
						in->im.imbft -= in->im.imtaxcr;
					}
					else { /* in->im.imbft <= in->im.imtaxcr */
						in->im.imatxcrt += in->im.imbft;
						in->im.imbft = ZERO;
					}
		    		DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
		    		        in->im.imbft, in->im.imatxcrt);
				}
	    		/* total transferable tax credits must be processed in 2 parts*/
				/* tfr2 contains the education allowance, tuition and textbook tax      */
				/*      credits which are subject to a maximum transferable   */
				/*      calculation dependent on the parameter MAXET          */
				/* tfr1 contains all other transferable tax credits           */
				tfr1 = in->im.imdisatc;
				tfr2 = in->im.imedtxc + in->im.imtutxc + in->im.imtxttxc;
				txcrt = tfr1 + tfr2;
	    		DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
				in->im.imtaxcr += txcrt;
				if (isnzero(txcrt)) {
					/* if child's basic federal tax is more than transferable */
					/* tax credits, apply total amount to reduce tax          */
					if (in->im.imbft > txcrt) {
						in->im.imatxcrt += txcrt;
						/* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
						in->im.imbft -= txcrt;
					}
					else { /* txcrt >= in->im.imbft */
						/* the child has some tax credit left for a parent    */
						/* reduce basic federal tax to zero and calculate     */
						/* actual amount of transferable tax credits          */
						in->im.imatxcrt += in->im.imbft;
						in->im.imbft = ZERO;
						tfr2 = minn(MP.FNTCR * MP.MAXET,tfr2);
						/* first calculate disability credit, then education amount */

						tfr1 = nneg(tfr1  /* disability credit */
						   - nneg( (MP.FNTCR * in->im.imitax)	  /* taxable income * credit tax rate 
						                                             to preserve idendity */
									- in->im.imbtc      /* basic personal tax credit       */
									- in->im.imbtcs      /* basic personal tax credit supplement    */
				                    - in->im.imatxc     /* age tax credit                  */
                                    - in->im.imothpe    /* other dependent tax credits */
									/* next two credits equal zero for CF children */
				           	      /*- in->im.immartxc      married tax credit */
				           		  /*- in->im.imctxcs       tax credit for dep children     */
				                    - in->im.imcppctc   /* CPP contributions tax credit    */
				                    - in->im.imuictc    /* UI contributions tax credit     */
				                    - in->im.impentxc   /* pension income tax credit       */
									- in->im.imemptxc));  /* Canada Employment Credit */
									
						tfr2 = nneg(tfr2  /* education, textbook and tuition */
						   - nneg( (MP.FNTCR * in->im.imitax)	  /* taxable income * credit tax rate 
						                                             to preserve idendity */
									- in->im.imbtc      /* basic personal tax credit       */
									- in->im.imbtcs      /* basic personal tax credit supplement      */
				                    - in->im.imatxc     /* age tax credit                  */
                                    - in->im.imothpe    /* other dependent tax credits */
									/* next two credits equal zero for CF children */
				           	      /*- in->im.immartxc      married tax credit */
				           		  /*- in->im.imctxcs       tax credit for dep children     */
				                    - in->im.imcppctc   /* CPP contributions tax credit    */
				                    - in->im.imuictc    /* UI contributions tax credit     */
				                    - in->im.impentxc   /* pension income tax credit       */
									- in->im.imemptxc));  /* Canada Employment Credit */

 					
			  			in->im.imttxcrt = tfr1+tfr2;

						ctxcrt += in->im.imttxcrt;


					    /* let education transfered be in total dollars, with the tax credit rate applied*/
						in->im.imedtrf = tfr2 / MP.FNTCR;
						cedtrf += in->im.imedtrf;

	

						DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
					}
				}
	    		DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
	    		       in->im.imbft, in->im.imatxcrt, in->im.imttxcrt);
	    	}

			if (in->id.idcfrh < 2) {
				/* total non-transferable tax credits applicable to parent */
				in->im.imtaxcr = in->im.imbtc      /* basic personal tax credit       */
				           + in->im.imbtcs     /* basic personal tax credit supplement */
                           + in->im.imothpe    /* other dependent tax credits */
				           + in->im.imchartc   /* charitable donations tax credit */
				           + in->im.imcppctc   /* CPP contributions tax credit    */
				           + in->im.imctxcs    /* tax credit for dep children     */
				           + in->im.immartxc   /* married tax credit              */
				           + in->im.immedatc   /* medical expenses tax credit     */
				           + in->im.imuictc    /* UI contributions tax credit     */
				           + in->im.imfdtxc    /* federal dividend tax credit     */
				           + in->id.idmincar   /* minimum tax carry over (504)    */
						   + in->im.imintsl    /* interest on student loans tax credit */
                           + in->im.imqpiptc   /* Quebec parental insurance premiums */
                           + in->im.imqpsetc   /* Quebec parental insurance premiums - self-employed*/
						   + in->im.imemptxc   /* Canada Employment Credit */
				           ;
	    		DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
	    		        in->im.imtaxcr);
	    		in->im.imatxcrt = ZERO;
				if (isnzero(in->im.imtaxcr)) {
					/* if parent's basic federal tax is more than transferable */
					/* tax credits, apply total amount to reduce tax          */
					if (in->im.imbft > in->im.imtaxcr) {
						in->im.imatxcrt += in->im.imtaxcr;
						in->im.imbft -= in->im.imtaxcr;
					}
					else { /* in->im.imbft <= in->im.imtaxcr */
						in->im.imatxcrt += in->im.imbft;
						in->im.imbft = ZERO;
					}
		    		DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
		    		        in->im.imbft, in->im.imatxcrt);
				}
				/* total transferable tax credits must be processed in 2 parts*/
				/* tfr2 contains the education allowance, tuition and textbook tax      */
				/*      credits which are subject to a maximum transferable   */
				/*      calculation dependent on the parameter MAXET          */
				/* tfr1 contains all other transferable tax credits           */
				tfr1 = in->im.imatxc     /* age tax credit                        */
				     + in->im.imdisatc   /* disability tax credit                 */
				     + in->im.impentxc   /* pension income tax credit */
				     + in->im.imchdtxc 	 /* child tax credit */
				     ;
				tfr2 = in->im.imedtxc + in->im.imtutxc + in->im.imtxttxc;
				txcrt = tfr1 + tfr2;
	    		DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
				in->im.imtaxcr += txcrt;
				if (isnzero(txcrt)) {
					/* if basic federal tax is more than transferable         */
					/* tax credits, apply total amount to reduce tax          */
					if (in->im.imbft > txcrt) {
						in->im.imatxcrt += txcrt;
						/* in->im.imttxcrt = 0.0 */
						in->im.imbft -= txcrt;
					}
					else { /* txcrt >= in->im.imbft */
						/* the parent has some tax credits left for a         */
						/* their spouse (if present) so reduce basic federal  */
						/* tax to zero and calculate actual amount of         */
						/* transferable tax credits                           */
						in->im.imatxcrt += in->im.imbft;
						in->im.imbft = ZERO; 

						tfr2 = minn(MP.FNTCR * MP.MAXET, tfr2);
						/* NB: use expanded net income (as on tax form) */
						/* follows Schedule 2 */
			  			in->im.imttxcrt = nneg(tfr1 + tfr2
			  			                  - MP.FNTCR * nneg(in->im.imdepni -in->im.imdedfn - (MP.BXM + MP.BXMEXT) -
			  			                                    (in->im.imcqppc - in->im.imcppse) - in->im.imuic));
						/* calculate amount of education, textbook and tuition which can be transfered */
						/* can't be more that total credits transfered */
						edtrf = minn(tfr2, in->im.imttxcrt);
						/* note that we want a value without the federal tax credit rate applied */
						in->im.imedtrf = edtrf / MP.FNTCR ;


					}
				}
	    		DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
	    		       in->im.imbft, in->im.imatxcrt, in->im.imttxcrt);
	    	}
		}

		if (cf->cfspoflg) {
			DEBUG1("%s spouse present\n");




             /* if federal taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbft > inspo->im.imbft) {
				txcrt = inspo->im.imttxcrt + ctxcrt;
				ineld->im.imtaxcr += txcrt;
				DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
				ineld->im.imstxcrt = inspo->im.imttxcrt;
				ineld->im.imttxcrt = ZERO;
				ineld->im.imctxcrt = ctxcrt;
				/* calculate education transfers received and set education transfered
				   to zero */
				/* the education tranferred had a maximum applied */
				ineld->im.imedrcv = inspo->im.imedtrf + cedtrf;
				ineld->im.imedtrf = ZERO;
				if (isnzero(txcrt)) {
					/* if basic federal tax is more than transfered           */
					/* tax credits, apply total amount to reduce tax          */
					if (ineld->im.imbft > txcrt) {
						ineld->im.imatxcrt += txcrt;
						ineld->im.imbft -= txcrt;
					}
					else { /* txcrt >= ineld->im.imbft */
						ineld->im.imatxcrt += ineld->im.imbft;
						ineld->im.imbft = ZERO;
					}
				}
					
				DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
			        ineld->im.imbft, ineld->im.imstxcrt, ineld->im.imctxcrt);

			}
			else {
				txcrt = ineld->im.imttxcrt + ctxcrt;
				inspo->im.imtaxcr += txcrt;
				DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
				inspo->im.imstxcrt = ineld->im.imttxcrt;
				inspo->im.imttxcrt = ZERO;
				inspo->im.imctxcrt = ctxcrt;
				/* calculate education transfers received and set education transfered
				   to zero */
				/* the education tranferred had a maximum applied */
				inspo->im.imedrcv = ineld->im.imedtrf + cedtrf;
				inspo->im.imedtrf = ZERO;
				if (isnzero(txcrt)) {
					/* if basic federal tax is more than transfered           */
					/* tax credits, apply total amount to reduce tax          */
					if (inspo->im.imbft > txcrt) {
						inspo->im.imatxcrt += txcrt;
						inspo->im.imbft -= txcrt;
					}
					else { /* txcrt >= inspo->im.imbft */
						inspo->im.imatxcrt += inspo->im.imbft;
						inspo->im.imbft = ZERO;
					}
				}
					  
				DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
			        inspo->im.imbft, inspo->im.imstxcrt, inspo->im.imctxcrt);
			}
		}
		else { /* no spouse present */
			txcrt = ctxcrt;
			ineld->im.imtaxcr += txcrt;
			DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
			ineld->im.imstxcrt = ZERO;
			ineld->im.imttxcrt = ZERO;
			ineld->im.imctxcrt = ctxcrt;
			/* calculate education transfers received and set education transfered
			   to zero */
			/* the education tranferred had a maximum applied */
			ineld->im.imedrcv = cedtrf;
			ineld->im.imedtrf = ZERO;
			if (isnzero(txcrt)) {
				/* if basic federal tax is more than transfered           */
				/* tax credits, apply total amount to reduce tax          */
				if (ineld->im.imbft > txcrt) {
					ineld->im.imatxcrt += txcrt;
					ineld->im.imbft -= txcrt;
				}
				else { /* txcrt >= ineld->im.imbft */
					ineld->im.imatxcrt += ineld->im.imbft;
					ineld->im.imbft = ZERO;
				}
			}


			DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
		        ineld->im.imbft, ineld->im.imstxcrt, ineld->im.imctxcrt);
		}



		/* do part 2 tax calculation for all persons in family      */

		/* process persons in census family */
		for (ini=0, in=cf->cfin; ini<cf->cfnpers; ini++, in++) {

			DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
			         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);


			/* calculate the federal foreign tax credit calc here */
			/* The income concept used for this credit is calculated as net income
			   less Can. forces personnel and police deduction (line 244)
					Employee home relocation deduction (line 248 - idemplo)
					Security options deductions (line 249 - imstkded)
					Net federal supplements, SA, WCB (line 250)
					Net capital losses of other years (line 253 - idcloss)
					Capital gains deduction (line 254 - imcapgex)  */

			forinet = nneg(in->im.imicnet 
							- in->id.idemplo
							- in->im.imstkded 
							- in->id.idcloss  
							- in->im.imcapgex
							- in->im.imisa
							- in->im.imigis
							- in->im.imispa
							- in->im.imbcss
							- in->id.idiworkc);

			if (forinet == ZERO) {
				in->im.imfortxc = ZERO;
			}
			else {
       			if (hh->hd.hdprov == PRQUE) {
    				qta = in->im.imbft * MP.QTAP;
                }
                else {
                    qta = ZERO;
                }

				in->im.imfortxc = minn(nneg(in->id.idfortx),
										( nneg(in->id.idforinc) / forinet)
										 * (in->im.imbft + in->im.imfdtxc - qta)
										);
			}

	   }

		/* do part 3 tax calculation for all persons in family */

		/* process persons in census family */
		for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {

			DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
			         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);
			/* save the value for the basic federa tax */
			bft = in->im.imbft;

			/* compute other tax credits */

			/* Federal political tax credit */
			if (in->id.idpolcon>ZERO) {
				in->im.imfptc = minn(MP.FPTCBEN,
               (NUMBER) lkup1(MP.FPTC, MP.FPTCrows, in->id.idpolcon));
				DEBUG3("%s idpolcon =%.2f imfptc =%.2f\n",
                  in->id.idpolcon, in->im.imfptc);
            }

			/* Federal labour sponsored tax credit */

			if (in->id.idlabtxg>ZERO) {
				in->im.imlabtxc = minn(MP.FLVCMAX, MP.FLVCRT[hh->hd.hdprov] * in->id.idlabtxg);
			}
         
            taxcr = in->im.imlabtxc
				   + in->im.imfptc + in->id.iditc;

			/* compute other credits applied to reduce tax */
			if (taxcr > bft) {
				in->im.imoftca = bft;
			}
			else {
				in->im.imoftca = taxcr;
			}

			/* calculate federal surtax        */
			in->im.imfsur = nneg(in->im.imbft - MP.FSURL1) * MP.FSURR1
			          + nneg(in->im.imbft - MP.FSURL2) * MP.FSURR2;

			/* calculate federal surtax reduction */
			in->im.imfsurtr = minn(in->im.imfsur,
			                      nneg(MP.FSURTRMX - nneg(MP.FSURTRRR * (in->im.imbft - MP.FSURTRRL))));

			in->im.imfsur -= in->im.imfsurtr;

			/* We used to reduce the surtax by amount of the additional foreign tax credit (511).
                This program was removed.
				(As of 1995(V7.0) the additional ITC (518) was too small to impute and so is no longer
				 removed from imfsur   */

			/* calculate final federal tax  */
			/* Do not let foreign tax credit reduce federal surtax */
			in->im.imtxf = nneg(bft - in->im.imfortxc - in->im.imoftca) + in->im.imfsur;
      		DEBUG2("%s txf    =%.2f\n", in->im.imtxf);

			if (MP.AMTTX != ZERO) {
				/* Calculate Minimum Tax  */

				/* note this section moved up for testing */
				ntpocg = ZERO;  /* non-taxable portion of capital gains */
				/* Prior to 2000, negative capital gains (capital losses) are claimed as 0 */
				/* for the purposes of alt. min. tax in the calculation of the non-taxable portion.  */
				/* Beginning in 2000, capital losses can be reported on the form however the non-taxable */
				/* portion is to shown without brackets and compared to the modeled capital gains */
				/* (which cannot be negative) and the lesser of the two represents the non-taxable portion */
				/* which will always be 0 in the event of negative idicapg.   */
				
				if (isnzero(in->id.idicapg)) {
					
					/* Only calculate a non-taxable portion of capital gains if */
					/*  an actual gain is realized.  If a capital loss, the result will */
					/*  always be zero in the model regardless of the changes in Form T691 in 2000. */

					ntpocg = in->id.idicapg * (MP.AMTINCRT - MP.CAPGIR);
				}


				/* Do some preliminary calculations - Part 9 - Net non-deducted capital losses
				/*  Do Part 9 first since the result is needed for Part 1.  */

				nndcl = ZERO;  /* net non-deducted capital losses */

 				if (isnzero(in->id.idcloss) || ntpocg > 0) {
					nndcl = nneg( in->id.idicapg - in->im.imcapgex - in->id.idcloss) * MP.AMTINCRT ;
				} 
				/* Calculated as the Net total capital gains (idicapg) less the
				   Capital Gains Deduction (imcapgex) and the net capital losses
				   from other years - Line 253 (idcloss). */
				/* verify if it can be negative */

				/* Part 1 - Adjusted Taxable Income and Minimum Amount. */

				adjtaxin = ZERO;  /* Adjusted Taxable Income */
				/* take the taxable income from line 260 and add back:   */
				/*  - RRSP contributions (idrrsp) (up to 1993 only),
				/*  - RPP contributions (idrpp) (up to 1993 only),
				/*  - Capital Cost allowance on film property,
				/*  - Carrying Charges for certain investment,
				/*  - Limited partnership and tax shelter losses (idpartlo),
				/*  - Resource property and flow-through shares (idexplor)
				/*  - non-taxable portion of capital gains reported in the year,
				/*  - employee home relocation loan deduction -line 248 (idemplo),
				/*  - Stock option and shares deduction -line 249 (imstkded),
				/*  - Limited partnership losses and non-capital losses attributable to
				/*    any capital cost allowance claimed on MURBs, certified feature films,
				/*    and certified productions, and to resource expenditures and 
				/*    depletion allowances (included in lines 251 and 252) (idpartlo, idnclos).

				/* First calculate the non-taxable portion of capital gains 
				/*   reported in the year - which is first set to equal the actual capital gains
				/*   (idicapg) times the applicable inclusion rate (AMTINCRT - CAPGIR). */


				/* Beginning in 1998, taxable income is allowed to be negative for */
				/* to be negative for the purposes of calculating the alternative minimum tax. */

				taxinc = ZERO;
				if (MP.AMTNEGTI) {
					/* Recalculate taxable income and allow it to be negative. */
					taxinc = in->im.imicnet - in->im.imdedfn;
				}
				else {   /* use the calculated value for taxable income. */
					taxinc = in->im.imitax;
				}

				/* Calculate the adjusted taxable income (adjtaxin). */

				adjtaxin = taxinc       /* taxable income from line 260 */
					+ (MP.AMTRPFLG?in->id.idrrsp:ZERO)   /* RRSP contributions */
					+ (MP.AMTRPFLG?in->id.idrpp:ZERO)    /* RPP contributions  */
					+ in->id.idexplor                    /* resource property and flow-through shares */
					+ in->id.idothded                    /* other deductions from total income */
					+ ntpocg                             /* non-taxable portion of capital gains */
					+ in->id.idemplo                     /* employee home relocation loan deduction */
					+ in->im.imstkded*MP.AMTSTK          /* stock option and shares deduction */
					+ in->id.idpartlo                    /* limited partnership losses */
					+ in->id.idnclos                     /* allowable other years non-capital losses */
					;
  
				/* The following items are subtracted from the calculation of adjusted
				/* taxable income: Line 120 - Taxable amount of dividends from taxable Canadian corporations
				/*  (imisdivt) times AMTDIV plus (imildivt) time AMTDIVLC  and 
				/*  Line 217 - Allowable deduction for business investment losses times applicable rate */

				divid = (in->im.imisdivt * MP.AMTDIV) + (in->im.imildivt * MP.AMTDIVLC);
				adjtaxin = nneg( adjtaxin - ( divid
				       + (in->im.imiloss * ((MP.AMTINCRT - MP.CAPGIR)/MP.CAPGIR) ) 
				       + nndcl ) );  /* net non-deductible capital losses calculated above */
				/* adjtaxin cannot be negative */
				
				/* Net adjusted taxable income is calculated as the non-negative result of the 
				/*  adjusted taxable income less the exemption */
				
				netadjin = nneg ( adjtaxin - MP.AMTEX ) ;
				
				/* The gross minimum amount of tax is equal to net adjusted taxable income times
				/*  the lowest federal tax rate.  */
				
				grminamt = netadjin * MP.AMTTX;
				
				/* From the gross minimum amount, subtract the Basic Minimum Tax Credit which is
				/*  equal to the total non-refundable tax credits (line 350) less the credit value
				/*  for the pension income amount - line 314 (impentxc), the disability amount
				/*  transferred from a dependant other than a spouse - line 318, tuition fees and 
				/*  education amount transferred from a child - line 324 and amounts transferred
				/*  from a spouse - line 326.  	   */

				bmtxcrd = (  in->im.imbtc    /* Line 300 basic personal amount */
							+ in->im.imatxc  /* Line 301 age amount */
							+ in->im.immartxc /* Line 303/305 Married or equivalent amount */
							+ in->im.imctxcs  /* Line 304 Amounts for dependent children */
							+ in->im.imothpe  /* Line 306 Amount for infirm dependents */
							+ in->im.imbtcs  /* Line 307 baisc personal amount supplement */
							+ in->im.imcppctc /* Line 308/310 CPP contributions */
							+ in->im.imuictc  /* Line 312 UI/EI premiums */
							+ in->im.imcgtc   /* Line 315 Caregiver amount */
							+ in->im.imedtxc  /* Line 323 Education amount */
							+ in->im.imtutxc  /* Line 323 Tuition amount */
							+ in->im.imtxttxc /* Textbook credit */ 
							+ in->im.immedatc /* Line 332 Medical expenses */
							+ in->im.imchartc /* Line 349 Donations and gifts */
							+ in->im.imemptxc /* Canada Employment Credit */
							) ;

				/* Add in the amount for a disability tax credit for self only if applicable */
				if (isnzero(in->id.iddisslf)) {
					bmtxcrd += MP.FNTCR * MP.MAXDX ;
				}

				/* The minimum amount is now calculated as the Gross Minimum Amount less
				/*  the basic minimum tax credit (bmtxcrd).   */

				minamt = nneg ( grminamt - bmtxcrd ) ;

				if (minamt > 0) {

					/* Part 2 - Basic Federal Tax Payable  */
	  				/* First calculate the basic federal tax from taxable income and tax rates */
					/*  and subtract from it the sum of Line 350 non-refundable tax credits, a  */
					/*  percentage of the dividend tax credit to get the tax payable before */
					/*  minimum tax carryover. */

					/* Line 350 equivalent to tax credits less federal dividend tax credit and */
					/*  the minimum tax carryover.  */

					taxcred = nneg(in->im.imtaxcr - in->im.imfdtxc - in->id.idmincar ) ; 
					
				   	basfedtx = nneg(in->im.imfedtax - ( taxcred + in->im.imfdtxc )); 
					
					/* The basic federal tax is further reduced by the minimum tax carryover applied in year */
					/*  as calculated in Part 8.   */
	  				/*  Part 8 - Applying a minimum tax carryover from previous years against current year tax payable */
					/*  Maximum carryover that can be applied to taxation year is defined as the 
					/*  basic federal tax before minimum carryover less the minimum amount (minamt). */
					
					maxcarry = nneg(basfedtx - minamt);
					mincarry = minn(maxcarry, in->id.idmincar);
					
					/* reduce the basic federal tax by the minimum tax carryover applied in taxation year (mincarry) */
					
					basfedtx -= mincarry;	
					
					/* Now subtract the foreign tax credit from basic federal tax to get the Federal Tax */
					
					fedtax = nneg(basfedtx - in->im.imfortxc);
					
					/* Also subtract the sum of the federal political contribution tax credit (line 410), */
					/*  the investment tax credit (line 412) and the labour-sponsored funds tax credit (line 414) */
					/*  to determine the regular tax payable.   */	

					regtaxpy = nneg ( fedtax - (in->im.imfptc        /* federal political contribution tax credit */
										    + in->id.iditc		   /* federal investment tax credits */
											+ in->im.imlabtxc) );  /* labour-sponsored funds tax credit */
	  				
					/*  Part 4 - Special Foreign Tax Credit  */
					/*  the special foreign tax credit is equal to the maximum of the federal foreign tax credit */
					/*   and the minimum of the following amounts for foreign income limits and foreign taxes paid.  */
					
					sftc = maxn ( minn( (in->id.idforinc * MP.FNTCR), in->id.idfortx * MP.AMTFORTX ), 
							in->im.imfortxc);

					/* Part 5 - Obligation to pay alternative minimum tax  */
					 
					netminamt = nneg(minamt - sftc);  /* minimum amount less special foreign tax credit */
					
					/* if the difference between the net minimum amount payable and the regular */
					/*  tax payable is negative, no alternative minimum tax need be paid */
					
					taxdiff = netminamt - regtaxpy ;
					if (taxdiff > 0) {
						/* Need to pay the alternative minimum tax */
						/* Part 6 - Federal Tax Payable Under Alternative Minimum Tax */
											
						/* to calculate the surtax, use the greater of the minimum amount and the basic federal tax */

						fsurval = maxn ( minamt , basfedtx ) ;
						mfsur = nneg(fsurval - MP.FSURL1) * MP.FSURR1
							   + nneg(fsurval - MP.FSURL2) * MP.FSURR2;

						/* Recalculate the federal surtax reduction */
						mfsurtr = minn( mfsur, 
								nneg(MP.FSURTRMX - nneg(MP.FSURTRRR * (fsurval - MP.FSURTRRL))));
						mfsur -= mfsurtr;

						/* indicate that minimum tax was applied */
						in->im.imamtfg = 1 ;

						/* Part7 - Additional taxes paid for minimum tax carryover */
						/* The form indicates that the difference between special foreign tax credit and the federal */
						/* foreign tax credit be multiplied by the following ratio:   */
						/*  foreign taxes paid for special foreign tax credit = in->id.idfortx*MP.AMTFORTX    */
						/*  ------------------------------------------------------------------------------    */
						/*  foreign taxes paid                                = in->id.idfortx                */
						/* Thus the formula is simplified by simply multiplying the difference by AMTFORTX.   */
						/*  This simplification is required since the appropriate detail of foreign business  */
						/*  and non-business income and taxes are currently not present on the database.      */

	  			 		in->im.imamtdf = nneg(minamt - (basfedtx + ((sftc - in->im.imfortxc) * MP.AMTFORTX) ) ); 
				 
						/*  Since provinces use various methods to calculate their minimum tax
						    save the proper amount in the variable when minimum tax is calculated   
							by a fraction of the difference in tax amounts.  Provinces have
							incorporated various small changes.   */

						if (MP.AMTPRVDF[hh->hd.hdprov] == 0 ) {
							/* when set to 0, use the difference in tax */
							in->im.imamtprv = in->im.imamtdf;
						}
						else if ( MP.AMTPRVDF[hh->hd.hdprov] == 1 ) {
							      /* when set to 1, use the additional taxes paid for minimum taxes
							         as per the form  */
							     in->im.imamtprv =  taxdiff;
						}
						else if ( MP.AMTPRVDF[hh->hd.hdprov] == 2 ) {
								  /* when set to 2, difference is set to minimum amount
								     less regular tax payable */
								 in->im.imamtprv = minamt - regtaxpy;
						}
						else if ( MP.AMTPRVDF[hh->hd.hdprov] == 3 ) {
								  /* when set to 3, difference is set to minimum amount
								     less basic federal tax */
								  in->im.imamtprv = minamt - basfedtx;
						}

						/* Save/replace results of required variables */
						in->im.imamtinc = netadjin;   /* net adjusted taxable income */
						/* The federal tax is set to the net minimum amount  */
					   	in->im.imfedtax = netminamt;	
						in->im.imminamt = minamt;    /* minimum amount */
					    in->im.imatxcrt = bmtxcrd; 
						in->im.imbft = basfedtx;     /* basic federal tax */
						in->im.imoftca = ZERO; 
						in->im.imfsur = mfsur;       /* Federal individual surtax */
						in->im.imtxf = nneg(basfedtx + mfsur) ;
						in->im.imfsurtr = mfsurtr;

	  				}
				}
			}

			/**
			* calculate QUE tax abatement:
			* non-refundable portion is considered as a credit reducing federal
			* tax payable; refundable portion will be accounted as a transfer
			* in memo1.c .
			**/

			if (hh->hd.hdprov == PRQUE) {
				qta = in->im.imbft * MP.QTAP;
				DEBUG2("%s qta=%.2f\n", qta);
				if (in->im.imtxf > qta) {
					DEBUG1("%s federal tax exceeds QUE tax abatement\n");
					in->im.imqtaa = qta;
					in->im.imqtar = ZERO;
				}
				else {
					DEBUG1("%s QUE tax abatement exceeds federal tax\n");
					in->im.imqtar = qta - in->im.imtxf;
					in->im.imqtaa = in->im.imtxf;
				}
				DEBUG4("%s qtaa=%.2f, qtar=%.2f, txf=%.2f\n",
				       in->im.imqtaa, in->im.imqtar, in->im.imtxf);

				in->im.imtxf    -= in->im.imqtaa;
				DEBUG2("%s txf=%.2f\n", in->im.imtxf);
			}

			/**
			* add up other federal refundable tax credits:
			* these are the obscure credits found in lines 444 through 478.
			* since they are not used to reduce net federal tax (line 420), we
			* have chosen to treat them as refundable credits. they are added to
			* the federal transfers in memo1.c
			*
			* Version 7.0 no longer removes the part XII.2 tax credit (Trusts) (456)
			* as it was too small to impute for that year  
			* Version 8.0 no longer removes the forward averaging tax credit
			* as it was too small to impute for that year  
			**/

			in->im.imfortc =  (MP.GSTREBFLAG?in->id.idgstreb:ZERO);
												/* GST rebate (457) */

			/* calculate refundable medical expense supplement (452) */
			/* it is only available to persons aged 18 and more who have earnings over MEDREMIN
			   and who had allowable medical expenses */
			if (in->im.immeda>0 && in->id.idage>=18 && 
 		    (nneg(in->id.idiemp - in->id.idrpp - in->id.iddues - in->id.idalexp
 		    		- in->id.idclergy) 
 		     + nneg(in->id.idise) ) >= MP.MEDREMIN) {
			 /* note that cleric's residence should also be subtracted from earned income */

                /* calculate medical supplement turndown.  It is equal to the sum of the 
                basic amount, the spousal amount, and the basic disability tax credit amount */
				medrtd = MP.MEDRTD;

				/* calculate the net income concept as per tax forms */
				/* the uccb payments are excluded from the income test in 452 worksheet */

				netinc = nneg(in->im.imicnet - in->im.imuccb);

				if (in->id.idspoflg) {
				    netinc += nneg(in->id.idinspo->im.imicnet - in->id.idinspo->im.imuccb);
					in->im.immedref = nneg(minn(in->im.immeda * MP.MEDRRATE, MP.MEDRMAX)
							- (nneg(netinc-medrtd)* MP.MEDRRR));
				}
				else {
					in->im.immedref = nneg(minn(in->im.immeda * MP.MEDRRATE, MP.MEDRMAX)
							- (nneg(netinc-medrtd)* MP.MEDRRR));
				}

				in->im.imfortc += in->im.immedref;
			}


		}
	}

	/* calculate the working income tax benefit (WITB) - introduced in 2007 */
	if (MP.WITBFLAG) {
        /* process each nuclear family in household */
	    for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
			     
           	/* Initialize local variables and head and spouse pointers */
		   	ineld = nf->nfineld;
		   	inspo = nf->nfinspo;
			witb = 0;
			witbmax = 0;
			netincpo = 0;
			nfinet = 0;
			nfiemp = 0;
			eldemp = 0;
			spoemp = 0;
			witbs = 0;
			netincspo = 0;
			famstatus = 0;   /* set famstatus to single (0) initially */
			eldelg = 1;      /* initialize elder to be eligible */
			spoelg = 1;      /* initialize spouse to be eligible */

			/* first determine if a spouse exists at all and set eligibility flag for spouse */
		 	if (nf->nfspoflg == NULL) {
				spoelg = 0;
			} 

		   	/* determine whether to treat as single or couple (including single parents) */
			/* students with at least 3 months of full time study are ineligible for WITB */
			if (nf->nfspoflg != NULL && nf->nfnkids == 0 && ineld->id.idage >= 19 && ineld->id.ideducfm > 2) {
			    famstatus = 0;
				eldelg = 0;
			}
			
			/* must be at least 19 to be eligible */
		   if (ineld->id.idage < 19) {
				eldelg = 0;
				famstatus = 0;
			}
			
			if (nf->nfspoflg != NULL && inspo->id.idage < 19) {
				spoelg = 0; 
				famstatus = 0;
			}

			if (nf->nfspoflg != NULL && nf->nfnkids == 0 && inspo->id.idage >= 19 && inspo->id.ideducfm > 2) {
				famstatus = 0;
				spoelg = 0;
			}

			/* if children exist, then treat as a family for all provinces except Quebec */
			switch(hh->hd.hdprov) {
				case PRQUE:
					if (eldelg == 1 && nf->nfspoflg != NULL && spoelg == 1) {
						famstatus = 1;
					}

					break;
				 default:
					if (nf->nfnkids > 0 || eldelg == 1 && nf->nfspoflg != NULL && spoelg == 1) {
						famstatus = 1;
					}
					break;
			}

		    if ((eldelg == 0 && nf->nfspoflg == NULL) || (eldelg == 0 && nf->nfspoflg != NULL && spoelg == 0)) {
				continue;
			} 

           	/* Compute Family net income and family employment income */
			/* include the elder's income in these tests if eligible */
			if (eldelg == 1) {
		   		nfinet = nneg( ineld->im.imicnet - ineld->im.imuccb);
           		nfiemp = ineld->id.idiemp + nneg(ineld->id.idise) ;
  				/* Compute individual earned income for WITB supplement for disabilities */
            	eldemp = ineld->id.idiemp + nneg(ineld->id.idise);
			} 

           	if (nf->nfspoflg && spoelg == 1) {
				nfinet += nneg( inspo->im.imicnet - inspo->im.imuccb);
	          	nfiemp += inspo->id.idiemp + nneg(inspo->id.idise) ;
           		spoemp =  inspo->id.idiemp + nneg(inspo->id.idise) ;
           	}


			/* Some provinces have chosen to adapt the WITB */

			switch(hh->hd.hdprov)  {


				case PRQUE:
		   			if (famstatus == 1) {  /* treat as couple / single parent */
						witbmax = MP.QWITBFAM;
						netincpo = MP.QWITBFPO;
						netincspo = MP.QWITBSFPO;
		   			}
		   			else {  /* treat as single, lower maximums */
						witbmax = MP.QWITBSNG;
						netincpo = MP.QWITBSPO;
						netincspo = MP.QWITBSSPO;
		   			}
				
					break;
				case PRBC:
		   			if (famstatus == 1) {  /* treat as couple / single parent */
						witbmax = MP.CWITBFAM;
						netincpo = MP.CWITBFPO;
						netincspo = MP.CWITBSFPO;
		   			}
		   			else {  /* treat as single, lower maximums */
						witbmax = MP.CWITBSNG;
						netincpo = MP.CWITBSPO;
						netincspo = MP.CWITBSSPO;
		   			}
				
					break;

				default:
					if (famstatus == 1) {  /* treat as couple / single parent */
						witbmax = MP.WITBFAM;
						netincpo = MP.WITBFPO;
						netincspo = MP.WITBSFPO;
		   			}
		   			else {  /* treat as single, lower maximums */
						witbmax = MP.WITBSNG;
						netincpo = MP.WITBSPO;
						netincspo = MP.WITBSSPO;
		   			}
					
					break;
			 }


			/* First assign the disability supplement */
			/* WITB Supplement for persons with disabilities */
			/* Eligibility rules: non-dependent who is eligible for the disability tax credit */
			/* and who meets other eligibility requirements for the WITB */

			/* Eligible individuals must meet the minimum earnings criteria */
			/* Note that the ramp up is based on the individual's earned income */
			/* and not the family earned income (which is used for WITB) */

			switch(hh->hd.hdprov) {
				case PRQUE:

					if (isnzero(ineld->id.iddisslf) && eldemp > MP.QWITBSMINE && eldelg == 1 ) {
						witbs = minn(MP.QWITBSAMT, nneg((eldemp - MP.QWITBSMINE) * MP.QWITBSPIR));
					
						/* now check if any of it has to be reduced by net income */
						if (witbs > 0 && nfinet > netincspo) {

							/* check if there is a spouse who is eligible for disability deduction */
							/* if so, the reduction rate is divided by 2 */
							if (nf->nfspoflg && isnzero(inspo->id.iddisslf)) {
								witbs = nneg(witbs - (MP.QWITBSCRR * (nfinet - netincspo)));
							}
							else {
								witbs = nneg(witbs - (MP.QWITBSRR * (nfinet - netincspo)));
				   			}
				   		}

						/* if there is still some WITB supplement remaining, assign to elder */
						ineld->im.imwitbs = witbs;
						ineld->im.imfortc += witbs;
					}
		   			
					if (nf->nfspoflg) {
		 		   		if (isnzero(inspo->id.iddisslf) && spoemp > MP.QWITBSMINE && spoelg == 1) {
							 witbs = minn(MP.QWITBSAMT, nneg( (spoemp - MP.QWITBSMINE) * MP.QWITBSPIR) );
					
							/* now check if any of it has to be reduced by net income */
							if (witbs > 0 && nfinet > netincspo) {
								/* check if the other spouse is eligible for disability deduction */
								/* if so, the reduction rate is divided by 2 */
								if (isnzero(ineld->id.iddisslf)) {
									witbs = nneg(witbs - (MP.QWITBSCRR * (nfinet - netincspo)));
								}
								else {
									witbs = nneg(witbs - (MP.QWITBSRR * (nfinet - netincspo)));
				   				}
							}

							/* if there is still some WITB supplement remaining, assign to elder */
					 		inspo->im.imwitbs = witbs;
							inspo->im.imfortc += witbs;
						}	
					}	 
					break;

				case PRBC:

					if (isnzero(ineld->id.iddisslf) && eldemp > MP.CWITBSMINE && eldelg == 1 ) {
						witbs = minn(MP.CWITBSAMT, nneg((eldemp - MP.CWITBSMINE) * MP.CWITBSPIR));
					
						/* now check if any of it has to be reduced by net income */
						if (witbs > 0 && nfinet > netincspo) {

							/* check if there is a spouse who is eligible for disability deduction */
							/* if so, the reduction rate is divided by 2 */
							if (nf->nfspoflg && isnzero(inspo->id.iddisslf)) {
								witbs = nneg(witbs - (MP.CWITBSCRR * (nfinet - netincspo)));
							}
							else {
								witbs = nneg(witbs - (MP.CWITBSRR * (nfinet - netincspo)));
				   			}
				   		}

						/* if there is still some WITB supplement remaining, assign to elder */
						ineld->im.imwitbs = witbs;
						ineld->im.imfortc += witbs;
					}
		   			
					if (nf->nfspoflg) {
		 		   		if (isnzero(inspo->id.iddisslf) && spoemp > MP.CWITBSMINE && spoelg == 1) {
							 witbs = minn(MP.CWITBSAMT, nneg( (spoemp - MP.CWITBSMINE) * MP.CWITBSPIR) );
					
							/* now check if any of it has to be reduced by net income */
							if (witbs > 0 && nfinet > netincspo) {
								/* check if the other spouse is eligible for disability deduction */
								/* if so, the reduction rate is divided by 2 */
								if (isnzero(ineld->id.iddisslf)) {
									witbs = nneg(witbs - (MP.CWITBSCRR * (nfinet - netincspo)));
								}
								else {
									witbs = nneg(witbs - (MP.CWITBSRR * (nfinet - netincspo)));
				   				}
							}

							/* if there is still some WITB supplement remaining, assign to elder */
					 		inspo->im.imwitbs = witbs;
							inspo->im.imfortc += witbs;
						}	
					}	 
					break;

				default:
					if (isnzero(ineld->id.iddisslf) && eldemp > MP.WITBSMINE && eldelg == 1 ) {
						witbs = minn(MP.WITBSAMT, nneg((eldemp - MP.WITBSMINE) * MP.WITBSPIR));
					
						/* now check if any of it has to be reduced by net income */
						if (witbs > 0 && nfinet > netincspo) {

							/* check if there is a spouse who is eligible for disability deduction */
							/* if so, the reduction rate is divided by 2 */
							if (nf->nfspoflg && isnzero(inspo->id.iddisslf)) {
								witbs = nneg(witbs - (MP.WITBSCRR * (nfinet - netincspo)));
							}
							else {
								witbs = nneg(witbs - (MP.WITBSRR * (nfinet - netincspo)));
				   			}
				   		}

						/* if there is still some WITB supplement remaining, assign to elder */
						ineld->im.imwitbs = witbs;
						ineld->im.imfortc += witbs;
					}
		   			
					if (nf->nfspoflg) {
		 		   		if (isnzero(inspo->id.iddisslf) && spoemp > MP.WITBSMINE && spoelg == 1) {
							 witbs = minn(MP.WITBSAMT, nneg( (spoemp - MP.WITBSMINE) * MP.WITBSPIR) );
					
							/* now check if any of it has to be reduced by net income */
							if (witbs > 0 && nfinet > netincspo) {
								/* check if the other spouse is eligible for disability deduction */
								/* if so, the reduction rate is divided by 2 */
								if (isnzero(ineld->id.iddisslf)) {
									witbs = nneg(witbs - (MP.WITBSCRR * (nfinet - netincspo)));
								}
								else {
									witbs = nneg(witbs - (MP.WITBSRR * (nfinet - netincspo)));
				   				}
							}

							/* if there is still some WITB supplement remaining, assign to elder */
					 		inspo->im.imwitbs = witbs;
							inspo->im.imfortc += witbs;
						}	
					}	 
					break;
			}

			
			/* now determine the basic working income tax benefit */
			/* couples and singles must have at least the minimum threshold of earnings */
			/* programs differ for BC and Quebec */

			switch(hh->hd.hdprov) {
				case PRQUE:

					/* Quebec has separate minimum earnings thresholds and phase in rates for single and family */

					if (famstatus == 1)	{
						if (nfiemp >= MP.QWITBMINEF ) {
							witb = minn(witbmax, nneg((nfiemp - MP.QWITBMINEF) * MP.QWITBPIRF));
							/* now check if any of it has to be reduced by net income */
							if (witb > 0 && nfinet > netincpo) {
								witb = nneg(witb - (MP.QWITBRR * (nfinet - netincpo)));
		   					}
						}
					}
					else {
						if (nfiemp >= MP.QWITBMINES ) {
							witb = minn(witbmax, nneg((nfiemp - MP.QWITBMINES) * MP.QWITBPIRS));
							/* now check if any of it has to be reduced by net income */
							if (witb > 0 && nfinet > netincpo) {
								witb = nneg(witb - (MP.QWITBRR * (nfinet - netincpo)));
		   					}
						}
					}
					break;
				case PRBC:

					if (nfiemp >= MP.CWITBMINE ) {
						witb = minn(witbmax, nneg((nfiemp - MP.CWITBMINE) * MP.CWITBPIR));
						/* now check if any of it has to be reduced by net income */
						if (witb > 0 && nfinet > netincpo) {
							witb = nneg(witb - (MP.CWITBRR * (nfinet - netincpo)));
		   				}
					}

					break;

				default:
					if (nfiemp >= MP.WITBMINE ) {
						witb = minn(witbmax, nneg((nfiemp - MP.WITBMINE) * MP.WITBPIR));
						/* now check if any of it has to be reduced by net income */
						if (witb > 0 && nfinet > netincpo) {
							witb = nneg(witb - (MP.WITBRR * (nfinet - netincpo)));
		   				}
					}

					break;
			 }

			/* Give the basic WITB to the spouse with the disability supplement */
			/* if the spouse is the only recipient of the disability supplement */
			if (nf->nfspoflg != NULL && inspo->im.imwitbs > 0 && ineld->im.imwitbs == 0) {
			   /* here there is a spouse who is WITB eligible and received supplement */
			   /* give the WITB to this spouse */
			   inspo->im.imwitb = witb;
			   inspo->im.imfortc += witb;
			}
			else {
				/* the eligible spouse with the higher working income should claim the WITB */
				if (eldelg == 1 && spoelg == 1)  {
					if (eldemp >= spoemp) {
						ineld->im.imwitb = witb;
						ineld->im.imfortc += witb;
					}
					else {  /* the spouse has the higher working income, spouse claims credit */
						inspo->im.imwitb = witb;
						inspo->im.imfortc += witb;
					}
				}
				else { 
			
					/* all other cases, give to elder if eligible */
					if (eldelg == 1) {
						ineld->im.imwitb = witb;
						ineld->im.imfortc += witb;
					}
					else {	/* give to the spouse if exists and eligible */
						if (nf->nfspoflg != NULL && spoelg == 1) {
			  				inspo->im.imwitb = witb;
			   				inspo->im.imfortc += witb;
						}
					}
				}
			}


		} /* end of nuclear family loop */								 
	}  /* end of WITB loop */


	DEBUG_OFF("Atxcalc");
}
