/***
*Atxhstr.cpp - Compute family-related deductions or credits
*
*$Id: Atxhstr.cpp 6.109 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/04/24 | Added non-refundable child tax credit - imchdtxc        |
* | LP  | 07/01/18 | Removed imuccb adjustment - actual net income in tests  |
* | LP  | 06/12/13 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/20 | Added imbcss (BC sen supp as part of sa) to income tests|
* | LP  | 06/06/07 | Subtract universal child care benefit from income tests |
* | LP  | 05/04/15 | Added MXMEXT, MXMTEXT, EMXMEXT, EMXMTEXT                |
* | LP  | 04/02/17 | Added idiworkc to dependent net income concept imdepni  |
* | LP  | 01/03/12 | Removed pre-1988 programs (PEROPT, etc.)                |
* | CH  | 00/12/04 | Changed imbtcs                                          |
* | CH  | 99/04/15 | Added imbtcs                                            |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 97/04/30 | MS-DOS: remove near/far & adjust overlay (memory probs.)|
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxhstr  [global]  Apply tax transfers between head and spouse
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== MANIFEST CONSTANTS ================================ */

#define HUGE_VAL 9999999999.0		/* huge value */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxhstr[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxhstr - Apply tax transfers between head and spouse
*
*Synopsis:
*    void Atxhstr(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   minn     Find the minimum of two numbers
*
*Structure Members:
*   MP.BXM          (i)  Basic personal exemption/amount
*   MP.BXMS         (i)  Basic personal exemption supplement
*   MP.BXMSPI       (i)  Basic personal exemption supplement phase in
*   MP.BXMSRR       (i)  Basic personal exemption supplement reduction rate
*   MP.CTCAMTPC     (i)  Child Tax Credit amount per child
*   MP.EMXM         (i)  Married equivalent exemption/amount
*   MP.EMXMEXT      (i)  Married equivalent amount extra top-up
*   MP.EMXMT        (i)  Married equivalent exemption/amount turndown level
*   MP.EMXMTEXT     (i)  Married equivalent amount turndown level extra top-up
*   MP.FNTCR        (i)  Federal non-refundable tax credit rate
*   MP.MXM          (i)  Married exemption/amount
*   MP.MXMEXT       (i)  Married amount extra top-up
*   MP.MXMT         (i)  Married exemption/amount turndown level
*   MP.MXMTEXT      (i)  Married amount turndown level extra top-up
*   MP.YCXM1        (i)  Dependent child amount (1st child)
*   MP.YCXM2        (i)  Dependent child amount (2nd child)
*   MP.YCXM3        (i)  Dependent child amount (3rd ,etc.)
*   MP.YCXMT        (i)  Dependent child exemption/amount turndown level
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfspoflg     (i)  Census family contains married couple
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   im.imicnet      (i)  Net Income (line 236)
*   im.imdepni      (io) Dependant's net income
*   im.imbtcs       (o)  Basic personal tax credit supplement
*   im.imcchc       (o)  Child's non-refundable credit
*   im.imcemc       (o)  Child's equivalent to married credit
*   im.imchclm      (o)  Number of dependent children claimed
*   im.imchdtxc     (o)  Non-refundable Child Tax Credit
*   im.imctxcs      (o)  Dependent children tax credits
*   im.immartxc     (o)  Married tax credit claimed
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void Atxhstr(
	P_hh hh
)
{
	register P_in in;
	register P_in ineld;
	register P_in inspo;
	P_cf cf;

	register int cfi;
	register int ini;

	NUMBER inet = ZERO;
	NUMBER cdedt = ZERO;
	NUMBER marex = ZERO;
	NUMBER cdeds = ZERO;
	NUMBER cmininet = ZERO;
	NUMBER xminet = ZERO;
	NUMBER cdedm = ZERO;
	NUMBER exmt = ZERO;
	NUMBER exmtpi = ZERO;
	NUMBER dedt = ZERO;
	NUMBER chclm = ZERO;
	NUMBER ctxct = ZERO;
	NUMBER ctxcs = ZERO;
	NUMBER ctxcm = ZERO;
	NUMBER yctc  = ZERO;   /* young child tax credit amount */

	/**
	* Pre-reform variables
	**/

	NUMBER cchx  = ZERO;   /* amount of (non-refundable) child tax credit */
	NUMBER cemx  = ZERO;   /* equivalent to married for child */
	NUMBER cchxt = ZERO;   /* cumulated cchx for all children */
	NUMBER cchxm = ZERO;   /* optimal cchx */
	NUMBER cemxm = ZERO;   /* optimal cemx */
	NUMBER cbest = ZERO;   /* indicator for optimal emx child */
	NUMBER ctcclm = ZERO;  /* number of under 18 children to claim for non-ref child tax credit */

	DEBUG_ON("Atxhstr");

	/**
	* First off, we compute the net income concept to use for reducing
	* dependant credits.  This variable will also be used to reduce
	* the spousal tax credit in txcalc().
	**/

	/* process persons in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
		in->im.imdepni = in->im.imicnet;	
	}



	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

		/**
	   	* The algorithm applied here is as follows:
	   	*  (1) find the child with the least net income,
	   	*      for potential use as the equivalent to married tax credit.
	   	*  (2) find the children with the two greatest net incomes.
	   	*      They will be used as child #1 and child #2, in order to
	   	*      maximize the benefits of the higher credits for third and
	   	*      subsequent children.
	   	*  (3) Pass through the kids again, and assign them to child #1,
	   	*      child #2, child #3, etc. or equivalent to married
	   	*      in such a way as to optimize the credits received.
	   	*  (4) Give the credits to one or the other of the spouses.
	   	**/


	   	/* Do needed declarations/initialisations for each census family */

	   	P_in   p_max1 = NULL;   /* kid with largest net income       */
	   	P_in   p_max2 = NULL;   /* kid with 2nd largest net income   */
	   	P_in   p_min  = NULL;   /* kid with smallest net income      */
	   	int    chclm  = 0;      /* number of children claimed        */
	   	NUMBER ctxcs  = ZERO;   /* total value of child tax credits  */
	   	NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
	   	NUMBER cchc   = ZERO;   /* Child's non-refundable credit             */
	   	NUMBER cmininet   = ZERO;   /* Income of child with lowest income */
		int    ctcclm = 0;      /* number of children 18 and under */


	   	DEBUG1("%s process census family children Pass #1\n");

	   	/* process children in census family */
	   	for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {

	   		DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
	   	         	ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

	   		/* Filter condition (ineligible for dependent or EM credit)  */

	   		if (in->id.idage > 18) {
	   			continue;
	   		}

			
			inet = in->im.imdepni;

	   		/* Update pointers for highest and second highest income kids */

	   		if (!cf->cfspoflg && (p_min == NULL || p_min->im.imdepni > inet)) {
	   			if (p_max1 == NULL || p_max1->im.imdepni <= p_min->im.imdepni) {
	   				p_max2 = p_max1;
	   				p_max1 = in;
	   			}
	   			else if (p_max2 == NULL || p_max2->im.imdepni <= p_min->im.imdepni) {
	   				p_max2 = in;
	   			}
	   			p_min = in;
	   		}
	   		else if (p_max1 == NULL || p_max1->im.imdepni <= inet) {
	   			p_max2 = p_max1;
	   			p_max1 = in;
	   		}
	   		else if (p_max2 == NULL || p_max2->im.imdepni <= inet) {
	   			p_max2 = in;
	   		}
	  	}
	   
	  	DEBUG1("%s process census family children Pass #2\n");

	  	/* Initialise tax credit variables */
	  	chclm = 0;      /* number of children claimed */
	  	ctxcs = ZERO;   /* total value of child tax credits */

	  	/* process children in census family */
	  	for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {


	  		/* 	Calculate Personal Amount Supplement */
	  		/* 	Note, on the tax form, it appears that you can only apply to this program 
	  			when single when you have an income  greater than the basic personal amount.
	  			This is not coded here, which means that there may be more total tax credits
	  			in the model than in the greenbook.  But since these people do not have any
	  			tax owing, there will be no difference in applied tax credits */
	  		/* The Supplement for Adults is done later in the module */

	  		/* calculate for children */
	  		in->im.imbtcs = MP.FNTCR * MP.BXMSPI * nneg(MP.BXMS - 
	  							(nneg(in->im.imdepni-(MP.BXM+MP.BXMS)) * MP.BXMSRR));


	  		/* Filter condition (ineligible for dependent or EM credit)  */

	  		if (in->id.idage > 18) {
	  			continue;
	  		}

	  		/* create working variables */
	  		inet = in->im.imdepni;

			/* save a count of the number of children under 18 
			   for the 2007 announced non-refundable child tax credit */
			if (in->id.idage < 18) {
				ctcclm++;
			}		  

	  		/* update number of children claimed */
	  		chclm++;

	  		/* assign credits to children as appropriate */
	  		if (cf->cfspoflg) {

	  			/**
	  			* Married couples claim kid with highest net income as kid #1,
	  			* kid with second highest income as kid #2, and remaining kids
	  			* as kid #3, kid #4, etc.
	  			**/

	  			if (in == p_max1) {
	  				cchc = MP.FNTCR * nneg(MP.YCXM1 - nneg(inet - MP.YCXMT));
	  			}
	  			else if (in == p_max2) {
	  				cchc = MP.FNTCR * nneg(MP.YCXM2 - nneg(inet - MP.YCXMT));
	  			}
	  			else {
	  				cchc = MP.FNTCR * nneg(MP.YCXM3 - nneg(inet - MP.YCXMT));
	  			}
	  			in->im.imcchc = cchc;
	  			ctxcs += cchc;
	  		}

	  		else /* Single parent family */ {

	  			/**
	  			* Single parent claims kid with lowest income for the
	  			* equivalent to married exemption, kid with highest net income
	  			* as kid #2, and all other kids (necessarily) as kid #3,
	  			* kid #4, etc.
	  			**/

	  			if (in == p_min) {
	  				cchc = ZERO;
	  				cemc = MP.FNTCR * nneg((MP.EMXM + MP.EMXMEXT) -
	  						 nneg(inet - (MP.EMXMT + MP.EMXMTEXT)));
	  				in->im.imcemc = cemc;
                    cmininet = inet;
	  			}
	  			else if (in == p_max1) {
	  				cchc = MP.FNTCR * nneg(MP.YCXM2 - nneg(inet - MP.YCXMT));
	  			}
	  			else {
	  				cchc = MP.FNTCR * nneg(MP.YCXM3 - nneg(inet - MP.YCXMT));
	  			}

	  			in->im.imcchc = cchc;
	  			ctxcs += cchc;
	  		}

	  	}


	  	DEBUG1("%s Apply credits to head (and spouse if present)\n");

	  	/* initialise pointers for head and spouse modelled variables */
	  	ineld = cf->cfineld;
	  	inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

	  	if (cf->cfspoflg) {

	  		DEBUG1("%s spouse present\n");

	  		/**
	  		* If the net income of the head is greater than the net
	  		* income of the spouse, the head gets the married
	  		* exemption and the exemptions of the wholly dependent
	  		* children.
	  		**/

	  		if (ineld->im.imicnet >= inspo->im.imicnet) {
	  			DEBUG1("%s head receives married tax credit\n");
	  			ineld->im.immartxc = MP.FNTCR * nneg((MP.MXM + MP.MXMEXT) - 
	  			           nneg(inspo->im.imdepni - (MP.MXMT + MP.MXMTEXT)));
	  			ineld->im.imctxcs = ctxcs;
	  			ineld->im.imchclm = (NUMBER) chclm;
				ineld->im.imchdtxc = (NUMBER) ctcclm * MP.CTCAMTPC * MP.FNTCR;
	  			DEBUG3("%s head's martxc=%.2f, ctxcs=%.2f\n",
	  			      ineld->im.immartxc, ineld->im.imctxcs);

	  		}
	  		else {
	  			DEBUG1("%s spouse receives married tax credit\n");
	  			inspo->im.immartxc = MP.FNTCR * nneg((MP.MXM + MP.MXMEXT) - 
	  			            nneg(ineld->im.imdepni - (MP.MXMT + MP.MXMTEXT)));
	  			inspo->im.imctxcs = ctxcs;
	  			inspo->im.imchclm = (NUMBER) chclm;
				inspo->im.imchdtxc = (NUMBER) ctcclm * MP.CTCAMTPC * MP.FNTCR;
	   			DEBUG3("%s spouse's martxc=%.2f, ctxcs=%.2f\n",
	   			      inspo->im.immartxc, inspo->im.imctxcs);
	   		}

	   		/* 	Calculate Personal Amount Supplement */
	   		/* 	Note, on the tax form, it appears that you can only apply to this program 
	   			when single when you have an income  greater than the basic personal amount.
	   			This is not coded here, which means that there may be more total tax credits
	   			in the model than in the greenbook.  But since these people do not have any
	   			tax owing, there will be no difference in applied tax credits */

	   		/* calculate for eldest */
	   		ineld->im.imbtcs = MP.FNTCR * MP.BXMSPI * nneg((MP.BXMS + minn(nneg((MP.BXM+MP.BXMS)- inspo->im.imdepni),MP.BXMS)) - 
	   						(nneg(ineld->im.imdepni-(MP.BXM+MP.BXMS+nneg((MP.BXM+MP.BXMS)-inspo->im.imdepni)))*MP.BXMSRR));

	   		/* calculate for youngest */
	   		inspo->im.imbtcs = MP.FNTCR * MP.BXMSPI * nneg((MP.BXMS + minn(nneg((MP.BXM+MP.BXMS)- ineld->im.imdepni),MP.BXMS)) - 
	   						(nneg(inspo->im.imdepni-(MP.BXM+MP.BXMS+nneg((MP.BXM+MP.BXMS)-ineld->im.imdepni)))*MP.BXMSRR));

	   	}

	   	else { /* no spouse present */

	   		ineld->im.immartxc = cemc;
	   		ineld->im.imctxcs = ctxcs;
	   		ineld->im.imchclm = (NUMBER) chclm;
			ineld->im.imchdtxc = (NUMBER) ctcclm * MP.CTCAMTPC * MP.FNTCR;

	   		DEBUG3("%s head's martxc=%.2f, ctxcs=%.2f\n",
	   		      ineld->im.immartxc, ineld->im.imctxcs);

	   		/* 	Calculate Personal Amount Supplement */
	   		/* 	Note, on the tax form, it appears that you can only apply to this program 
	   			when single when you have an income  greater than the basic personal amount.
	   			This is not coded here, which means that there may be more total tax credits
	   			in the model than in the greenbook.  But since these people do not have any
	   			tax owing, there will be no difference in applied tax credits */

	   		/* supplement if single parent */
	   		if (chclm>0) {
	   			ineld->im.imbtcs = MP.FNTCR * MP.BXMSPI * nneg((MP.BXMS + minn(nneg((MP.BXM+MP.BXMS)-cmininet),MP.BXMS)) - 
	   							(nneg(ineld->im.imdepni-(MP.BXM+MP.BXMS+nneg((MP.BXM+MP.BXMS)-cmininet)))*MP.BXMSRR));
	   		}
	   		/* supplement if single and no kids */
	   		else {
	   			ineld->im.imbtcs = MP.FNTCR * MP.BXMSPI * nneg(MP.BXMS - 
	   							(nneg(ineld->im.imdepni-(MP.BXM+MP.BXMS))*MP.BXMSRR));
	   		}
	   	}
	}

	DEBUG_OFF("Atxhstr");
}
