/***
*Atxqcalc.cpp - Calculate income tax (Quebec)
*
*$Id: Atxqcalc.cpp 6.181 2008/08/22 17:07:46 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 08/06/11 | Added Model code for housing parent/natural caregiver   |
* | LP  | 08/06/04 | Added Adapted Work Premium for disabled (QAWPFLAG+)     |
* | CH  | 07/07/3- | Added transfer of tuition tax credit                    |
* | LP  | 07/07/23 | Added a nneg condition to imqitot usage                 |
* | LP  | 07/07/20 | Added 2 flags to CCE - QCCEEIFLG and QCCEEITST          |
* | CH  | 07/07/17 | Added new option for complementary amount               |
* | CH  | 07/07/17 | Added tranfer of credit to parents                      |
* | LP  | 07/02/13 | Added QSTRMAGE - min age flag for sales tax credit      |
* | LP  | 07/02/11 | Que. pension credit now based on imqipnst, not idipens  |
* | LP  | 07/02/02 | Removed QAMTINCRT, added QAMTCAPR                       |
* | LP  | 07/01/25 | Deduct idclergy from earn inc test for ref. med. tx.cred|
* | LP  | 07/01/16 | Deleted imuccb from income tests, imuccb only in tax inc|
* | LP  | 07/01/12 | Removed parameter QDIVRAT - no longer needed            |
* | LP  | 07/01/12 | Expanded the ref. nat. caregiver tx cred to idothpe     |
* | CH  | 07/01/08 | Minor bug fix                                           |
* | LP  | 06/12/22 | Minor bug fix, immtot changed to imqitot                |
* | LP  | 06/07/11 | Modifications to work premium as per tax form verify    |
* | LP  | 06/06/13 | Add nneg to income tests that are reduced by imuccb     |
* | LP  | 06/06/01 | Removed imuccb from income tested credits               |
* | LP  | 06/05/15 | Added Que. large corporation tax credit rate (QDTCRLC)  |
* | CH  | 06/01/04 | Fixed QPIP pointers                                     |
* | CH  | 06/01/04 | Modified QPIP for self-employed                         |
* | LP  | 05/12/16 | Added QPIP premium tax credit/deduction for all workers |
* | LP  | 05/11/29 | Added QFATCOPT == 3 to Child Assist. Refundable credit  |
* | LP  | 05/11/17 | Removed QSTROPT - pre 1991 option                       |
* | LP  | 05/05/30 | Added QRNCGFLG, QRNCGTC refund natural caregivers tx crd|
* | LP  | 04/08/31 | Added qstrext to imptc, minor bug fix in imqfs assign.  |
* | CH  | 04/08/06 | Modified Child assistance and other credits             |
* | LP  | 04/07/14 | Added QAMTSTTC - spouse transfer tax credit flag for AMT|
* | LP  | 04/06/18 | Added Child Assistance - revised other dep. credits and |
* |     |          | Added new non-ref dependent credits for older dependents|
* | LP  | 04/06/09 | Added Work Premium - refundable tax credit (QWPFLAG +)  |
* | LP  | 04/05/13 | Added QSTSCA - Que Single Tax System Complementary Amt  |
* | LP  | 04/02/11 | Updated income concepts for family tax credits and ccea |
* | LP  | 03/12/08 | Revised Housing Parent tax credit - added QHPTCOPT      |
* | LP  | 03/11/17 | Added imqintsl (student loan int.) to total tax credits |
* | CH  | 03/10/17 | Deleted iddisab                                         |
* | CH  | 03/05/22 | Added Extra QSTR                                        |
* | LP  | 02/05/15 | Fixed family income concept (imqcppse for spouse)       |
* | LP  | 02/04/30 | Added QSFTXCR transfer credits between spouses expanded |
* | LP  | 02/04/30 | Added QSFMEDFLAG - include medical exp. credit in simp. |
* | LP  | 01/05/18 | Changes and new parameters for alternative minimum. tax |
* |     |          |   and removed QCAPRAT                                   |
* | LP  | 01/03/28 | Added QPCTR, QPCOPT to political contribution calc. and |
* |     |          |   modified the spouse's calculation of eligible income  |
* | LP  | 01/03/12 | Removed pre-1988 programs (QREFOPT)                     |
* | LP  | 00/11/16 | Apply QSPBFLAG, social prog benefits in total inc. flag |
* | LP  | 00/10/18 | Added QFAMNETI to calc. of family allowance tax credit  |
* | CH  | 00/06/19 | Modified Quebec tax reduction                           |
* | CH  | 00/06/02 | Modified imptc for people with Housing Parent credit    |
* | CH  | 00/05/17 | Modifed refundable tax credits                          |
* | LP  | 00/05/03 | Apply QEEXPDED to employment expenses                   |
* | CH  | 00/04/14 | Apply QCAPGIR to capgex stkded and iloss                |
* | LP  | 00/04/13 | Applied improved system for self-employed workers       |
* | CH  | 00/04/06 | Phase out of family allowance tax credit                |
* | CH  | 00/04/04 | Rewrote the family allowance tax credit                 |
* | CH  | 00/03/30 | Renamed imqchartc to imqchatc to be under 8 characters  |
* | CH  | 00/03/28 | Modifed child care expense credit                       |
* | LP  | 00/03/21 | Added Refundable tax credit for medical expenses        |
* | CH  | 99/05/28 | Stock Option Deduction option in Alternate Minimum Tax  |
* | CH  | 99/05/28 | Disability credit can be transfered to spouse           |
* | CH  | 99/05/21 | Modified Simplifed tax system code and added combined   |
* |     |          |    age, living alone, retirement tax credit             |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | EGA | 98/04/24 | imcapgex instead of idcapgex                            |
* | EGA | 98/04/22 | Added Que max to gifts to crowns                        |
* | EGA | 98/04/07 | Anti-Poverty Fund: QAPFFLG, QAPFRAT                     |
* | EGA | 98/04/06 | Supplement for young kids in large families in QFATC    |
* | EGA | 98/04/03 | 1998 tax simplification: QLICRFLG,Q:ICR, QFAMFLG,QFAMRAT|
* | EGA | 98/03/27 | mod. of Quebec political contribution: QPCOPT, QPCTR    |
* | EGA | 98/03/24 | Minor change to QHPTC                                   |
* | EGA | 98/03/10 | Minor change to Quebec Alternative Minimum Tax          |
* | EGA | 98/03/09 | Quebec Family Allowance Tax Credit revisited            |
* | EGA | 98/03/09 | Quebec Minimum Tax                                      |
* | EGA | 98/02/18 | Quebec Family Allowance Tax Credit                      |
* | EGA | 98/02/13 | add imqduetc (if QDUESOPT==2)                           |
* | EGA | 98/02/05 | Quebec Home Parent Tax Credit revisited                 |
* | EGA | 97/12/17 | Quebec Home Parent Tax Credit                           |
* | EGA | 97/10/15 | rename QSTFTYP3 and 4 to QSTRFTP3 and 4                 |
* | EGA | 97/10/12 | Add new QSRTXXX parameters                              |
* | EGA | 97/10/11 | Add Sales tax credit (QSRTXXX parameters) and imqstr    |
* | SFG | 97/04/30 | MS-DOS: remove near/far & adjust overlay (memory probs.)|
* | BBM | 94/09/30 | Add cpp/ui non-refundable tax credit option             |
* | BBM | 94/09/30 | Add income tested tax reduction                         |
* | BBM | 94/09/29 | Add health services fund tax                            |
* | BBM | 94/09/28 | Add two tier high income surtax                         |
* | SFG | 90/10/05 | reactivate use of fx data                               |
* | SFG | 90/09/26 | implement Quebec family allowance claw back             |
* | SFG | 90/09/26 | correct elderly amount QEIAA in eligible income calc.   |
* | SFG | 90/09/18 | disable use of fx data                                  |
* | SFG | 90/04/27 | compute imbpt, imtxp, imptr consistently for pre-reform |
* | SFG | 89/12/05 | Remember imqftr into imptr                              |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxqcalc [global]  Calculate federal income tax
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxqcalc[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxqcalc - Calculate federal income tax
*
*Synopsis:
*    void Atxqcalc(        
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   nneg     Change negative numbers to zero
*   maxn     Find the maximum of two numbers
*   lkup1    Table look-up with interpolation/extrapolation
*   minn     Find the minimum of two numbers
*   lkup2    Table look-up without interpolation
*   isnzero  Is argument non-zero ?
*   txhhexp  Compute and pro-rate household taxes, rent, etc.
*   iszero   Is argument zero ?
*
*Structure Members:
*   MP.CCEAOAGE     (i)  CCEA Maximum eligible age for oldest child
*   MP.CCEATFLG     (i)  CCEA eligible->claimable transformation activation flag
*   MP.CCEATFR      (i)  CCEA eligible->claimable fraction
*   MP.CCEATLL      (i)  CCEA eligible->claimable lower limit
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.QAMTCAPR     (i)  Quebec Alternative Minimum Tax Capital inclusion rate
*   MP.QAMTCBX      (i)  Quebec Alternative Minimum Tax Calculate Basic Exemption
*   MP.QAMTDIV      (i)  Quebec Alternative Minimum Tax Taxable Dividend Limit
*   MP.QAMTNEGTI    (i)  Quebec Alternative Minimum Tax Allow Negative Taxable Income
*   MP.QAMTRITC     (i)  Quebec Alternative Minimum Tax Retirement Income Credit Deduction
*   MP.QAMTRPFLG    (i)  RRSP/RPP included in Quebec Alternative Minimum Tax (1=include)
*   MP.QAMTSTTC     (i)  Quebec Alternative Minimum Tax Spouse Transfer of Tax Credits Flag
*   MP.QAMTTX       (i)  QAMTX tax rate
*   MP.QAMTX        (i)  QAMTX exemption level
*   MP.QAMTXFLG     (i)  Quebec alternative minimum tax, QAMTX, activation flag
*   MP.QAMTXREF     (i)  Quebec alternative minimum tax 1995 reform indicator (flag)
*   MP.QAMTXSTKFLG  (i)  QAMTX stock option deduction inclusion (1=stk can be deducted)
*   MP.QAPFFLG      (i)  Quebec anti-poverty fund activation flag
*   MP.QAPFRAT      (i)  Quebec anti-poverty fund rate
*   MP.QAWPCR       (i)  Quebec Adapted Work Premium for Disabled Credit Rate [family type]
*   MP.QAWPEIPI     (i)  Quebec Adapted Work Premium for Disabled Employment Income Phase In [family type]
*   MP.QAWPFLAG     (i)  Quebec Adapted Work Premium for Disabled refundable tax credit flag
*   MP.QAWPMAX      (i)  Quebec Adapted Work Premium for Disabled Maximum [family type]
*   MP.QAWPRR       (i)  Quebec Adapted Work Premium for Disabled Reduction Rate
*   MP.QAWPTD       (i)  Quebec Adapted Work Premium for Disabled Income Turndown [family type]
*   MP.QAXM         (i)  Quebec age exemption/amount
*   MP.QBXM         (i)  Quebec basic personal exemption/amount
*   MP.QCAFLAG      (i)  Quebec Child Assistance refundable tax credit flag
*   MP.QCALRFLG     (i)  Quebec combined age, living alone retirement credit
*   MP.QCAMAX       (i)  Quebec Child Assistance Maximum [number of children]
*   MP.QCAMIN       (i)  Quebec Child Assistance Minimum [number of children]
*   MP.QCAPAMTX     (i)  QAMTX taxable capital ratio
*   MP.QCARR        (i)  Quebec Child Assistance Reduction Rate
*   MP.QCASPMAX     (i)  Quebec Child Assistance Maximum Single Parent additional credit
*   MP.QCASPMIN     (i)  Quebec Child Assistance Minimum Single Parent additional credit
*   MP.QCATDC       (i)  Quebec Child Assistance Income Turndown for couples
*   MP.QCATDS       (i)  Quebec Child Assistance Income Turndown for single parents
*   MP.QCCEAOLD     (i)  Quebec CCEA old child limit
*   MP.QCCEAYNG     (i)  Quebec CCEA young child limit
*   MP.QCCEEIFLG    (i)  Quebec Refundable CCE EI Benefits in Earned Income flag
*   MP.QCCEEITST    (i)  Quebec Refundable CCE Earned Income Test flag
*   MP.QCCEOPT      (i)  Quebec CCE calculation option [1=deduction,2=refundable credit]
*   MP.QCCETCR      (i)  Refundable CCE credit rate [net income, % cce claimable]
*   MP.QCCETCRrows  (i)  number of rows in QCCETCR
*   MP.QCMPOPT      (i)  Quebec Complementary Credit option
*   MP.QCRDFLG      (i)  Quebec income credit flag [1=if social contribution are deducted]
*   MP.QDTCR        (i)  Quebec dividend tax credit rate
*   MP.QDTCRLC      (i)  Quebec dividend tax credit rate from large (eligible) corporations
*   MP.QDUESOPT     (i)  Quebec Dues Tax Credit option
*   MP.QEIA1        (i)  Quebec eligible income family type #1
*   MP.QEIA2        (i)  Quebec eligible income family type #2
*   MP.QEIA3        (i)  Quebec eligible income family type #3
*   MP.QEIA4        (i)  Quebec eligible income family type #4
*   MP.QEIA5        (i)  Quebec eligible income family type #5
*   MP.QEIAA        (i)  Quebec eligible income aged amount
*   MP.QESSNEEDS    (i)  Quebec essential needs for transfer of parental contributions
*   MP.QFAMDED      (i)  Quebec Family related deduction on income in refundable credits
*   MP.QFAMFLG      (i)  Quebec family income deduction activation flag
*   MP.QFAMNETI     (i)  Quebec family allowance tax credit net income concept flag
*   MP.QFAMRAT      (i)  Quebec family income rate
*   MP.QFATCHIRR    (i)  QFATC High income reduction rate
*   MP.QFATCHITD    (i)  QFATC High income turndown
*   MP.QFATCK6      (i)  QFATC Supplement for kids under 6 in large families [#kids][#kids<6]
*   MP.QFATCK6PYR   (i)  QFATC supplement for kids under 6 phase out year proportion
*   MP.QFATCK6YR    (i)  QFATC Supplement for kids under 6 phase out year
*   MP.QFATCMIN     (i)  QFATC minimum benefit paid
*   MP.QFATCMNK     (i)  QFATC Minimum benefits by kid rank
*   MP.QFATCMNKrows (i)  number of rows in QFATCMNK
*   MP.QFATCMRR     (i)  QFATC turn down income levels and reduction rate for married parents
*   MP.QFATCMRRrows (i)  number of rows in QFATCMRR
*   MP.QFATCMXK     (i)  QFATC Maximum benefits by kid rank
*   MP.QFATCMXKrows (i)  number of rows in QFATCMXK
*   MP.QFATCOPT     (i)  Quebec Family Allowance Tax Credit Option
*   MP.QFATCSRR     (i)  QFATC turn down income levels and reduction rate for single parent
*   MP.QFATCSRRrows (i)  number of rows in QFATCSRR
*   MP.QFATCSUP     (i)  Quebec Family Allowance Tax Credit Supplement for single parent
*   MP.QFATCTK      (i)  QFATC Take-up rates by total income
*   MP.QFATCTKrows  (i)  number of rows in QFATCTK
*   MP.QFATD4K      (i)  QFATC turn down income for 4 children and more
*   MP.QFATD4KI     (i)  QFATC turn down income increase by child for 4 children and more
*   MP.QFTRA1       (i)  Quebec family tax reduction family type #1
*   MP.QFTRA2       (i)  Quebec family tax reduction family type #2
*   MP.QFTRA3       (i)  Quebec family tax reduction family type #3
*   MP.QFTRA4       (i)  Quebec family tax reduction family type #4
*   MP.QFTRFLAG     (i)  Quebec family tax reduction activation flag
*   MP.QFTRRR       (i)  Quebec family tax reduction reduction rate
*   MP.QGIFTOPT     (i)  Quebec gifts as tax credit
*   MP.QHPTC        (i)  Quebec Housing Parent Tax Credit
*   MP.QHPTCFLG     (i)  Quebec Housing Parent Tax Credit activation flag
*   MP.QHPTCOPT     (i)  Quebec Housing Parent Tax Credit Calculation option
*   MP.QHSCI        (i)  Quebec surtax first cut-in level
*   MP.QHSCI2       (i)  Quebec surtax second cut-in level
*   MP.QHSF         (i)  Quebec surtax first level rate
*   MP.QHSF2        (i)  Quebec surtax second level rate
*   MP.QITSFSEW     (i)  Quebec improved tax system for self-employed worker activation flag
*   MP.QLAXM        (i)  Quebec living alone exemption/amount
*   MP.QLAXOLDDEP   (i)  Quebec living alone amount if all dependents 18 or over
*   MP.QLVCMAX      (i)  Maximum Quebec labour-sponsored funds tax credit allowed
*   MP.QLVCRT       (i)  Percent of Quebec labour-sponsored funds cost allowed as credit
*   MP.QMEDINC      (i)  Income definition for Que. refundable tax credit for medical expenses
*   MP.QMEDRATE     (i)  Proportion of expenses allowed for refundable tax credit for medical expenses
*   MP.QMEDRMAX     (i)  Maximum expenses allowed for Que. refundable tax credit for medical expenses
*   MP.QMEDRMIN     (i)  Minimum earnings for Quebec refundable tax credit for medical expenses
*   MP.QMEDRRR      (i)  Reduction rate for the Que. refundable tax credit for medical expenses
*   MP.QMEDRTD      (i)  Turndown level for Que. refundable tax credit for medical expenses
*   MP.QMRTCFLG     (i)  Quebec refundable tax credits for medical expenses activation flag
*   MP.QNORFAMI     (i)  Quebec Northern Deductions from Family Tax Credit Income flag
*   MP.QNTCR        (i)  Quebec nominal tax credit rate
*   MP.QPCOPT       (i)  Quebec political contribution option
*   MP.QPCTR        (i)  Quebec political contribution tax rates
*   MP.QPCTRrows    (i)  number of rows in QPCTR
*   MP.QPSXM        (i)  Quebec post-secondary exemption/amount
*   MP.QPTC         (i)  Quebec political contribution table [total donations,donation allowed]
*   MP.QPTCBEN      (i)  Maximum Quebec political tax credit allowed
*   MP.QPTRGISB     (i)  Quebec property tax rebate GIS bonus
*   MP.QPTRMTP      (i)  Quebec property tax minimum tax per person
*   MP.QPTRMTR      (i)  Quebec property tax maximum tax for reduction
*   MP.QPTRRR       (i)  Quebec property tax rebate reduction rate
*   MP.QPTRTF       (i)  Quebec property tax fraction
*   MP.QREPFAMI     (i)  Quebec repayment of UI/OAS benefits deduct from income flag
*   MP.QRNCGFLG     (i)  Quebec Natural Caregivers Tax Credit activation flag
*   MP.QRNCGMR      (i)  Quebec Natural Caregivers Tax Credit maximum reduction
*   MP.QRNCGOPT     (i)  Quebec Natural Caregivers Tax Credit option (1=max, 2=model)
*   MP.QRNCGTC      (i)  Quebec Natural Caregivers Tax Credit
*   MP.QRNCGTCR     (i)  Quebec Natural Caregivers Tax Credit rate
*   MP.QRNCGTD      (i)  Quebec Natural Caregivers Tax Credit income turndown
*   MP.QRTRFLAG     (i)  Quebec real estate rebate activation flag
*   MP.QSFDED       (i)  Quebec Simplified Form - additional deductions for net income calculation flag
*   MP.QSFDISFLAG   (i)  Quebec Simplified Form Disability tax credit inclusion flag
*   MP.QSFMEDFLAG   (i)  Quebec Simplified Form medical expense tax credit inclusion flag
*   MP.QSFOPT       (i)  Quebec Simplified Form Option (1=general 2=simplified 3=optimized)
*   MP.QSFTC        (i)  Quebec Simplified Form tax credit
*   MP.QSFTXCR      (i)  Quebec Simplified Form - transfer unused credits to spouse under either tax system
*   MP.QSPBFLAG     (i)  Quebec inclusion of social program benefits in income flag
*   MP.QSTRBCRD     (i)  Quebec sales tax rebate base credit
*   MP.QSTRCHILD    (i)  Quebec sales tax rebate children dependent credit
*   MP.QSTREXT      (i)  Quebec sales tax rebate extra amount
*   MP.QSTRFLAG     (i)  Quebec sales tax rebate activation flag
*   MP.QSTRFTP3     (i)  Quebec sales tax rebate family type 3 credit
*   MP.QSTRFTP4     (i)  Quebec sales tax rebate family type 4 credit
*   MP.QSTRMAGE     (i)  Quebec sales tax rebate minimum age flag
*   MP.QSTRRR       (i)  Quebec sales tax rebate reduction rate
*   MP.QSTSCA       (i)  Quebec Single Tax System Complementary Amount
*   MP.QTPCFLG      (i)  Quebec transfer of recognized parental contribution to education activation flag
*   MP.QTRBE        (i)  Quebec Tax Reduction Breakeven
*   MP.QTROPT       (i)  Quebec Tax Reduction calculation option 1=universal 2=income tested
*   MP.QTRP         (i)  Quebec tax Reduction proportion
*   MP.QTRSAFAM     (i)  Quebec Social Assistance Included in Family Tax Credit Income flag
*   MP.QTRWCFAM     (i)  Quebec Workers Compensation Included in Family Tax Credit Income flag
*   MP.QTUIOPT      (i)  Quebec tuition tax credit option
*   MP.QTUITRFFLG   (i)  Quebec flag to allow transfer of tuition tax credit to parents
*   MP.QTX          (i)  Quebec income tax table [taxable income,basic provincial tax]
*   MP.QTXrows      (i)  number of rows in QTX
*   MP.QWPCR        (i)  Quebec Work Premium Credit Rate [family type]
*   MP.QWPEIPI      (i)  Quebec Work Premium Employment Income Phase In [family type]
*   MP.QWPFLAG      (i)  Quebec Work Premium refundable tax credit flag
*   MP.QWPMAX       (i)  Quebec Work Premium Maximum [family type]
*   MP.QWPRR        (i)  Quebec Work Premium Reduction Rate
*   MP.QWPTD        (i)  Quebec Work Premium Income Turndown [family type]
*   MP.QYPDL        (i)  Quebec deduction limit for pension income
*   MP.TARGETYEAR   (i)  Year of analysis
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   ef.efin         (i)  First person in economic family [pointer]
*   ef.efnpers      (i)  Number of persons in economic family
*   hd.hdnadult     (i)  Number of adults in household
*   hd.hdnpers      (i)  Number of persons in household
*   hh.cf           (i)  Census family data [array]
*   hh.ef           (i)  Economic family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnef        (i)  Number of economic families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idcarry      (i)  Carrying charges (221)
*   id.idccet       (i)  Child care expenses associated with child
*   id.idccett      (i)  Child care expenses (Limit A, Form T778)
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.idclergy     (i)  Clergy residential deduction
*   id.idcloss      (i)  Allowable other years capital loss (253)
*   id.iddisab      (i)  Disability status (age 16+)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.idemplo      (i)  Employee home relocation loan dedn (248)
*   id.idexplor     (i)  Exploration and development expenses (224)
*   id.idicapg      (i)  Capital gains (actual)
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idiloss      (i)  Business investment losses (217)
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idise        (i)  Self-employed income
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idnorth      (i)  Northern deductions (255)
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idpartlo     (i)  Limited partnership losses (251)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idrand       (i)  Random numbers [array]
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idsex        (i)  Sex
*   id.idspoflg     (i)  Person has spouse
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imigis       (i)  GIS benefits
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imispa       (i)  Spouse's allowance
*   im.imiuib       (i)  Unemployment Insurance/Employment Insurance benefits
*   im.improptx     (i)  Imputed property tax paid
*   im.imqalexp     (i)  Quebec allowable employment expenses
*   im.imqatc       (i)  Quebec age tax credit
*   im.imqbtc       (i)  Quebec basic tax credit
*   im.imqcapgt     (i)  Quebec taxable capital gains
*   im.imqcapgx     (i)  Quebec modelled capital gains deduction (254)
*   im.imqcchc      (i)  Quebec child's non-refundable credit
*   im.imqchatc     (i)  Quebec allowable charitable donations tax credit
*   im.imqcppse     (i)  Quebec self-employ. QPP/CPP Cont. deduction
*   im.imqcpptc     (i)  Quebec CPP/QPP contributions tax credit
*   im.imqdctc      (i)  Quebec dependent child tax credits
*   im.imqdedfn     (i)  Quebec all deductions from net income
*   im.imqdedft     (i)  Quebec deductions from total income
*   im.imqdistc     (i)  Quebec disability tax credit
*   im.imqduetc     (i)  Quebec Dues Tax Credit
*   im.imqhsfc      (i)  Quebec Health Services Fund Contributions
*   im.imqhsftc     (i)  Quebec Health Services Fund Cont.tax credit
*   im.imqidivt     (i)  Quebec taxable dividends
*   im.imqinet      (i)  Quebec net income
*   im.imqintsl     (i)  Quebec student loan interest tax credit
*   im.imqipnst     (i)  Quebec taxable pension income (after splitting)
*   im.imqitax      (i)  Quebec taxable income
*   im.imqitot      (i)  Quebec total income
*   im.imqlatc      (i)  Quebec living alone tax credit
*   im.imqldivt     (i)  Quebec taxable dividends from large (eligible) corporations
*   im.imqmeda      (i)  Quebec medical expenses allowed
*   im.imqmtc       (i)  Quebec married tax credit
*   im.imqndc       (i)  Quebec number of dependent children
*   im.imqpipcbn    (i)  Quebec parental insurance plan calendar year benefits
*   im.imqqpiptc    (i)  Quebec parental insurance plan premium tax credit for paid workers (Quebec)
*   im.imqqpsedd    (i)  Quebec parental insurance plan premium deduction for self-employed (Quebec)
*   im.imqqpsetc    (i)  Quebec parental insurance plan premium tax credit for self-employed (Quebec)
*   im.imqritc      (i)  Quebec retirement income tax credit
*   im.imqsdivt     (i)  Quebec taxable dividends from small corporations
*   im.imqsfflg     (i)  Quebec simplified tax form flag 0=general 1=simplified
*   im.imqstkdd     (i)  Quebec modelled Stock option deduction (249)
*   im.imqtuitc     (i)  Quebec Tuition Tax Credit
*   im.imquictc     (i)  Quebec UI contributions tax credit
*   im.imrepay      (i)  Social Benefits Repayments
*   nf.nfinch       (i)  First child in nuclear family [pointer]
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imbpt        (io) Basic provincial tax
*   im.implvctc     (io) Provincial labour-sponsored funds tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.impsur       (io) Provincial surtax
*   im.imqapf       (io) Quebec Anti-Poverty Fund
*   im.imqbtax      (io) Quebec tax before credits
*   im.imqcceni     (io) Quebec net income for refundable cce credit calculation
*   im.imqctpc      (io) Quebec child's transfer of recognized parental contribution
*   im.imqdtxc      (io) Quebec dividend tax credit
*   im.imqei        (io) Quebec eligible income for tax reduction
*   im.imqfs        (io) Quebec family situation (1-5)
*   im.imqftr       (io) Quebec family tax reduction
*   im.imqhptc      (io) Quebec Home Parent Tax Credit level
*   im.imqhptcp     (io) Quebec Home Parent Tax Credit potential level
*   im.imqidc       (io) Quebec income of designated dependent child
*   im.imqittr      (io) Quebec income tested tax reduction
*   im.imqmedrc     (io) Quebec refundable tax credit for medical expenses
*   im.imqncgtc     (io) Quebec Refundable Natural Caregivers Tax Credit
*   im.imqnctcp     (io) Quebec Refundable Natural Caregivers Tax Credit potential amount
*   im.imqtca       (io) Quebec tax credits applied
*   im.imqtct       (io) Quebec tax credits transferable
*   im.imqtcts      (io) Quebec tax credits transferred from spouse
*   im.imqtottc     (io) Quebec total tax credits
*   im.imqtpcn      (io) Quebec number of children who transferred parental contribution
*   im.imqtuitt     (io) Quebec amount of tuition transferred to parents
*   im.imtxp        (io) Provincial income tax payable
*   im.imptc        (o)  Refundable provincial tax credits
*   im.imptr        (o)  Provincial tax reduction
*   im.imqamt       (o)  Quebec Minimum Tax Amount
*   im.imqamtfg     (o)  Quebec Minimum Tax flag
*   im.imqawrkpr    (o)  Quebec Adapted Work Premium for Disabled refundable tax credit
*   im.imqca        (o)  Quebec Child Assistance refundable tax credit
*   im.imqcalr      (o)  Quebec combined age, living alone, retirement credit
*   im.imqccetc     (o)  Quebec refundable child care expenses Tax Credit
*   im.imqfatc      (o)  Quebec Family Allowance Tax Credit
*   im.imqptr       (o)  Quebec property tax refund
*   im.imqstr       (o)  Quebec sales tax refund
*   im.imqtctc      (o)  Quebec tax credits transferred from child
*   im.imqwrkpr     (o)  Quebec Work Premium refundable tax credit
*
*Called by:
*   txque    Compute provincial taxes for Quebec
*
*****************************************************************************/
void Atxqcalc(
    P_hh hh
)
{
    register P_in in;
    register P_in ineld;
    register P_in inspo;
    register P_nf nf;
    P_cf cf;

    register int cfi;
    register int ini;
    register int nfi;
	P_ef ef;
	register int efi;

    NUMBER lvctc = ZERO;   /* Labour-sponsored venture capital tax cred */
	NUMBER efimcgtcp = ZERO;  /* combined cgtc potential amounts for ef */   
	NUMBER efimncgtcp = ZERO;  /* combined cgtc potential amounts for ef */   

    DEBUG_ON("Atxqcalc");


	/* Quebec Child Assistance */
	/* This new refundable tax credit was announced in the 2004 Quebec Budget
	   and will be implemented in January 2005.  It replaces the Quebec Family 
	   Allowance Tax Credit, the tax reduction for families and the non-refundable
	   tax credit for children.  */ 
	if (MP.QCAFLAG && MP.QFATCOPT == 3) {

		NUMBER faminc = ZERO;    /* family aggregate income */
		NUMBER incthrs = ZERO;   /* family income threshold for reduction */
		NUMBER caspf = ZERO;     /* maximum benefit for single parent status */
		NUMBER caspfmin = ZERO;  /* minimum benefit for single parent status */
		NUMBER cabase = ZERO;    /* maximum child assistance credit */
		NUMBER camin = ZERO;     /* minimum child assistance credit */
		NUMBER cacred = ZERO;    /* calculated value of child assistance credit */

      /* process each nuclear family in household */
      for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
     	
     		/* initialise elder and spouse pointers */
       		ineld = nf->nfineld;
        	inspo = nf->nfinspo;   /* NB: is NULL if spouse not present */

			/* proceed only for census families with kids under 18 years of age */
			if (nf->nfnkids > 0) {
				
				/* Compute Family income - head and spouse */
				faminc = MP.PYINC * nneg(ineld->im.imqitot);
				if (nf->nfspoflg) {
					faminc += MP.PYINC * nneg(inspo->im.imqitot);
				}

				/* Determine family income reduction threshold based on family structure */
				if (nf->nfspoflg) {
					incthrs = MP.QCATDC;
					caspf = 0;
					caspfmin = 0;
				}
				else {
					incthrs = MP.QCATDS;
					caspf = MP.QCASPMAX;
					caspfmin = MP.QCASPMIN;
				}

				/* Calculate the base amount and minimum amount based on # kids */
				if (nf->nfnkids == 1) {
					cabase = MP.QCAMAX[0] + caspf;
					camin = MP.QCAMIN[0] + caspfmin;
				}
				else if (nf->nfnkids == 2) {
					cabase = MP.QCAMAX[0] + MP.QCAMAX[1] + caspf;
					camin = MP.QCAMIN[0] + MP.QCAMIN[1] + caspfmin;
				}
				else if (nf->nfnkids == 3) {
					cabase = MP.QCAMAX[0] + MP.QCAMAX[1] + MP.QCAMAX[2] + caspf;
					camin = MP.QCAMIN[0] + MP.QCAMIN[1] + MP.QCAMIN[2] + caspfmin;
				}
				else {
					cabase = MP.QCAMAX[0] + MP.QCAMAX[1] + MP.QCAMAX[2] 
							+ ((nf->nfnkids - 3) * MP.QCAMAX[3]) + caspf;
					camin = MP.QCAMIN[0] + MP.QCAMIN[1] + MP.QCAMIN[2]
							+ ((nf->nfnkids - 3) * MP.QCAMIN[3]) + caspfmin;
				}

				/* Determine the calculated credit amount taking into account the income
					turndown, maximum and minimum credit amounts */

				cacred = maxn(cabase - (MP.QCARR * nneg(faminc - incthrs)), camin);

				/* Presumption in favour of the mother */
				/* The person who is mainly responsible for the care and eduction
				   of the child is entitled to claim the child assistance payment.  */
			
				if (nf->nfspoflg) {
					if (ineld->id.idsex == FEMALE ) {
						ineld->im.imqca = cacred;
						ineld->im.imptc += cacred;
					}
					else {
						inspo->im.imqca = cacred;
						inspo->im.imptc += cacred;
					}
				}
				else {
					ineld->im.imqca = cacred;
					ineld->im.imptc += cacred;
				}	   					

			}

		}

	}

    /* process each census family in household */
    for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {
    
        NUMBER maxpar = ZERO;   /* Maximum amount of transfer of parental contributions to education */
        NUMBER qctct = ZERO;   /* Amount from children 18+ which can be transfered  */
        int qtpcn = 0;  /* Number of kids who tranfer basic credit to their parents */
        NUMBER amttrf = ZERO;  /* Amount which can be tranferred to parent*/
        NUMBER txcrt = ZERO;  /* Amount which can be tranfered from spouse or child */
        NUMBER maxtax=ZERO; /* Max taxes with minimum credits applied for tuition credit */
        NUMBER maxtuit =ZERO; /* Max tuition fees that can be transfered to parents */
        NUMBER maxroom = ZERO; /* Max room to transfer fees */
        NUMBER maxtuitc =ZERO; /* Max tuition credit that can be transfered */

        /* initialise elder and spouse pointers */
        ineld = cf->cfineld;
        inspo = cf->cfinspo;   /* NB: is NULL if spouse not present */


        /* process persons in census family */
        for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {

            /* calculate Quebec provincial tax */
            in->im.imqbtax = (NUMBER) lkup1(MP.QTX, MP.QTXrows, in->im.imqitax);
            in->im.imbpt = in->im.imqbtax;

            /* compute Quebec dividend tax credit */
            in->im.imqdtxc = (in->im.imqsdivt * MP.QDTCR) 
            				 + (in->im.imqldivt * MP.QDTCRLC);

            /* Non-refundable Quebec Political Contribution Tax Credit */
            if (in->id.idprvpol>0) {
                if(MP.QPCOPT== 1) {
                    in->im.impptc = minn(MP.QPTCBEN, in->id.idprvpol * MP.QPTC); 
                    DEBUG3("%s idpolcon =%.2f impptc =%.2f\n",
                    in->id.idprvpol, in->im.impptc);

                }
                else {	  // QPCOPT = 2, credit varies with the level of contributions
                    in->im.impptc = minn(MP.QPTCBEN, in->id.idprvpol * ((NUMBER) lkup2(MP.QPCTR, MP.QPCTRrows, in->id.idprvpol))); 
                    DEBUG3("%s idpolcon =%.2f impptc =%.2f\n",
                    in->id.idprvpol, in->im.impptc);

                }
            }

            /* Que labour-sponsored venture capital credit */
            /* Also known as the Fonds de solidarite des travailleurs du Quebec */
            if ( isnzero(in->id.idlabtxg) ) {
               in->im.implvctc = minn(MP.QLVCMAX, MP.QLVCRT * in->id.idlabtxg);
            }

			/* Calculate the QPIP tax credit on premiums for paid workers */

            /* If the simplified tax system is in place, this calculates a person's benefit */
            /* person was estimated likely to benefit from simplified tax system */
            if (in->im.imqsfflg) {
                /* compute tax credits */
                in->im.imqtottc =  in->im.imqbtc  /* basic tax credit */
                            + MP.QNTCR*MP.QSFTC   /* flat amount */
                            + in->im.imqatc       /* age tax credit */
                            + in->im.imqlatc      /* person living alone tax credit */
                            + in->im.imqritc      /* retirement income tax credit */
                            + in->im.imqdctc      /* dependent children tax credits */
                                                  /* other dependant tax credits */
                            + in->im.imqchatc     /* charitable donations */
                            + in->im.impptc       /* political contribution */
                            + in->im.implvctc    /* labour sponsored venture captial tax credit FSTQ */
                            + (MP.QSFDISFLAG?in->im.imqdistc:ZERO) /* disability tax credit */
							+ (MP.QSFMEDFLAG?in->im.imqmeda*MP.QNTCR:ZERO)  /* medical expenses tax credit if QSFMED=1 */
                            ;
            }
            /* general tax system */
            else { 
                /* compute tax credits */
                in->im.imqtottc = in->im.imqbtc   /* basic tax credit */
                            + in->im.imqatc   /* age tax credit */
                            + in->im.imqlatc  /* person living alone tax credit */
                            + in->im.imqmtc   /* married tax credit */
                            + in->im.imqdctc  /* dependent children tax credits */
                                              /* other dependant tax credits including older children and infirmities */
                            + in->im.imqritc  /* retirement income tax credit */
                            + in->im.imqdistc /* disability tax credit */
                            + in->im.imqdtxc  /* dividend tax credit */
                            + (in->im.imqmeda * MP.QNTCR) /* medical (estimate) */
                            + in->im.impptc   /* political contribution */
                            + in->im.implvctc /* labour sponsored venture captial tax credit FSTQ */
							+ in->im.imqintsl /* student loan interest tax credit */
                            ;

                if (MP.QDUESOPT == 2) in->im.imqtottc += in->im.imqduetc; /* union and profession dues */
                if (MP.QTUIOPT == 2) in->im.imqtottc += in->im.imqtuitc; /* tuition tax credit */
                if (MP.QGIFTOPT == 2) in->im.imqtottc += in->im.imqchatc; /* charity and gifts as tax credits */


                /* Prior to 2005, you had tax credits  for CPP contributions, EI contributions and Health Services Fund contribution */
				/* In 2005, the single tax system was introduced with a complementary
				amount to be claimed by everyone which replaces the flat amount in the 
				simplified tax system.  This amount will replace the tax credits for
				CPP contributions, EI contributions and Health Services Fund contribution.
				In cases where the tax credits for QPP, EI and HSF are higher than the
				complementary amount, taxpayers may claim the full value of those amounts instead. Starting in 
				2006, QPIP contributions are also on this list.*/
                /* Starting in 2008, the basic amount was increased and the other credits were dropped */

                if (MP.QCMPOPT==1) {
                    /* general system before complementary amount introduced */
					in->im.imqtottc += (in->im.imqhsftc + in->im.imqcpptc + in->im.imquictc + in->im.imqqpiptc + in->im.imqqpsetc);
                }
                else if (MP.QCMPOPT==2) {
                    /* complementary amount */
					in->im.imqtottc += maxn(MP.QSTSCA*MP.QNTCR, (in->im.imqhsftc +
										in->im.imqcpptc + in->im.imquictc + in->im.imqqpiptc + in->im.imqqpsetc));
				}
				/* Otherwise, basic amount increased and the other credits unused */
                
                DEBUG2("%s tax credits=%.2f\n", in->im.imqtottc);
										
            }

            /* initialise applied credits to zero */
            in->im.imqtca = ZERO;

            /**
            * Apply tax credits.
            * If provincial tax is more than tax credits, apply total amount
            * to reduce tax, otherwise reduce tax to zero.  Keep track of
            * the amount of tax credits actually applied, as well as (for
            * spouses), the potentially transferable amount.
            **/
            if (isnzero(in->im.imqtottc)) {
                if (in->im.imbpt > in->im.imqtottc) {
                    in->im.imqtca += in->im.imqtottc;
                    in->im.imbpt -= in->im.imqtottc;
                }
                else { /* in->im.imbpt <= in->im.imqtottc */
                    in->im.imqtca += in->im.imbpt;
                    in->im.imbpt = ZERO;

                    if (in->id.idcfrh>=2) {
                       /* Adult students can transfer the amount relating fo the transfer of
                           parental contributions (replaces the dependant tax credit ) */
                        if (MP.QTPCFLG==1 && in->id.idage>=18 && in->id.ideducfm>=4 ) {
                            /* This is a student who is eligible to transfer */
                            /* max amount which can be transfered */
                            /* two credits */
                            if (in->id.ideducfm>=8) {
                                maxpar = MP.QNTCR * MP.QESSNEEDS;
                            }
                            else {
                                maxpar = MP.QNTCR * nneg(MP.QESSNEEDS - MP.QPSXM);
                            }
                            amttrf = minn(in->im.imqtottc - in->im.imqtca, maxpar);
                            in->im.imqtct = amttrf;
                            /* add to total amount of credits which are being tranfered from children */
                            qctct += in->im.imqtct;
                            if (amttrf>0) {
                                /* Counter of all kids who transfer credit */
                                qtpcn++;
                                /* Is a kid who tranfered credit */
                                in->im.imqctpc = amttrf;
                            }
                            
                        }
                        /* Adult students can also transfer tuition amounts to their parents */
                        if (MP.QTUITRFFLG) {
    						/* The maximum amount which can be transfered is QNTCR time the 
    						   amount of tuition fees which exceeds the amount of tax payable calculated
    						   taking into account only certain credits */
                            maxtax = nneg(in->im.imqbtax - (
                                        in->im.imqbtc   /* basic tax credit */
                                        + in->im.imqatc   /* age tax credit */
                                        + in->im.imqlatc  /* person living alone tax credit */
                                        + (in->im.imqmeda * MP.QNTCR) /* medical (estimate) */
                                        + in->im.imqduetc /* union and profession dues */
                                        + in->im.imqdistc /* disability tax credit */
                                        + in->im.imqchatc)); /* charity and gifts as tax credits */
                            /* The maximum amount which can be transfered is QNTCR * tuition, 
                               and the tax payable using the above credits */
    						maxtuit = nneg((MP.QNTCR * in->id.idtuitn) - maxtax);  
                            /* The SPSM will only transfer the amount after people's taxes go to zero  */
                            /* So it's the difference between the total credits, the credits applied, and the amount transfered
                               to parents in recognition of educational contribution (assume that one goes first
                               since tuition can be banked for later use, the other cannot) */
                            maxroom = nneg(in->im.imqtottc - in->im.imqtca - in->im.imqctpc);
                            maxtuitc=minn(maxtuit, maxroom);
                            in->im.imqtuitt = maxtuitc;
                            /* Add to amount which can be tranfered */
                            in->im.imqtct += maxtuitc;
                            /* add to total amount of credits which are being tranfered from children */
                            qctct += in->im.imqtuitt;
                        }
                    }                            

                    if (cf->cfspoflg && in->id.idcfrh < 2) {
                        /**
                        * The transferable amount is the lesser of any unused
                        * tax credits and the value of transferable credits.
                        **/

                        /* in the simplified tax system, if both spouses use the 
                            simplified tax system, then all extra can be transfered */
						/* non-refundable tax credits not used can be transfered to a spouse
						   if QSFTXCR = 1 regardless of the tax system used */
                        if (in->im.imqsfflg || MP.QSFTXCR==1 ) {
                            in->im.imqtct = in->im.imqtottc - in->im.imqtca;
                        }
                        else {
                        /* in the general tax system, only the age, retirement, and 
                           disability credits are transferable */
                             in->im.imqtct = minn(in->im.imqtottc - in->im.imqtca,
                                             in->im.imqatc + in->im.imqritc + in->im.imqdistc);
                        }
                    }
                }
                DEBUG3("%s bpt   =%.2f, qtca=%.2f\n",
                        in->im.imbpt, in->im.imqtca);
            }
        }
        
        /* apply spousal and children transfers of unused credits */
        if (cf->cfspoflg) {
                if ( (isnzero(qctct)) 
                    || (
                    (isnzero(inspo->im.imqtct) || isnzero(ineld->im.imqtct)) 
                      && 
                    (MP.QSFOPT==1 || MP.QSFTXCR == 1 ||(ineld->im.imqsfflg==inspo->im.imqsfflg))
                    )) {
                /* when the simplified tax system is in place, credits can only
                   be transferred between spouses if they use the same tax form */
			    /* beginning in 2003, QSFTXCR is set to 1, allowing the transfer of 
			       credits between spouses regardless of the tax system used */

                /* Note that the dedcuction relating to an amount tranferred by 
                   an eligible student is supposed to be applied after the basic tax credit, 
                   the tax credit for minor children engaged in vocation training or 
                   post-secondary studies, and the tax credit for other dependants.  In the 
                   SPSM, it is applied after all other non-tranfered credits */


                /* give credits to spouse with the highest taxes */
                if (ineld->im.imbpt > inspo->im.imbpt) {
                    DEBUG1("%s transfering credits from spouse to elder\n");

                    ineld->im.imqtcts = inspo->im.imqtct;
                    ineld->im.imqtctc = qctct;
                    ineld->im.imqtpcn = qtpcn;
                    txcrt = inspo->im.imqtct + qctct;

                    if (ineld->im.imbpt > txcrt ) {
                        ineld->im.imqtca += txcrt;
                        ineld->im.imbpt -= txcrt;
                    }
                    else {
                        ineld->im.imqtca += ineld->im.imbpt;
                        ineld->im.imbpt = ZERO;
                    }
                }

                else if (isnzero(inspo->im.imbpt)) {
                    DEBUG1("%s transfering credits from elder to spouse\n");
                    inspo->im.imqtcts = ineld->im.imqtct;
                    inspo->im.imqtctc = qctct;
                    inspo->im.imqtpcn = qtpcn;
                    txcrt = ineld->im.imqtct + qctct;

                    if (inspo->im.imbpt > txcrt) {
                        inspo->im.imqtca += txcrt;
                        inspo->im.imbpt -= txcrt;
                    }
                    else {
                        inspo->im.imqtca += inspo->im.imbpt;
                        inspo->im.imbpt = ZERO;
                    }
                }
                else { /* no one owes taxes.  Should save the amount of kids for later */
                    ineld->im.imqtpcn = qtpcn;
                }
            }
        }
        /* no spouse, but may have kids with tranferable credits*/
        else {
            if (qctct>0.0) {
                ineld->im.imqtctc = qctct;
                ineld->im.imqtpcn = qtpcn;
 
                if (ineld->im.imbpt > qctct) {
                    ineld->im.imqtca += qctct;
                    ineld->im.imbpt -= qctct;
                }
                else {
                    ineld->im.imqtca += ineld->im.imbpt;
                    ineld->im.imbpt = ZERO;
                }
            }
        }

        /* The Alternative minimum tax: calculation of the level */
                    /* Calculate Quebec Minimum Tax                                          */
                    /* - Take Taxable Income, add back:                               */
                    /*   - RRSP contributions,                                        */
                    /*   - RPP contributions,(should lump sum payments and payments to*/
                    /*     Deferred Income Plans, but they are all lumped together in */
                    /*     RPP),                                                      */
                    /*   - a portion of taxable Capital Gains						  */
                    /*   - Employee home relocation loan dedn (248)                   */
                    /*   - Stock option deduction (249)                               */
                    /*   - Pension and Interest Income Deductions                     */
                    /*   - Deductions Transferred from spouse                         */
                    /*                                                                */
                    /* - Allow the following items as deductions:                     */
                    /*   - The grossed-up portion of Dividends from Canadian          */
                    /*     Corporations                                               */
                    /*   - Allowable Business Investment Losses						  */
                    /*   - Net Capital losses from previous years                     */
                    /*                                                                */
                    /*   In 1995 a reform, QAMTXREF, includes more income             */
				    /*   In 1999, taxable income is allowed to be negative            */
		
        if (MP.QAMTXFLG != ZERO ) {	

            /* process persons in census family */
           for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {

			  /* determine if the individual may have to pay alternative minimum tax */
              if (in->im.imqsfflg==0 && ((in->im.imqidivt > MP.QAMTDIV) || in->id.idicapg > ZERO || 
           		(in->im.imqstkdd > ZERO && !MP.QAMTXSTKFLG) || in->id.idexplor > ZERO
           		|| (MP.QAMTXREF && (in->id.idpartlo > ZERO || in->id.idcarry > ZERO)) )) {
			 
				   NUMBER taxinc;
				   NUMBER adjtaxinc;
				   NUMBER basex;
				   NUMBER nrtc;
				   NUMBER altmintx;
				   taxinc = ZERO;
				   adjtaxinc = ZERO;
				   basex = ZERO;
				   nrtc = ZERO;
				   altmintx = ZERO;
			   	
				   /*  First recalculate taxable income allowing it to go negative if applicable */
				   if (MP.QAMTNEGTI) {
						taxinc = in->im.imqitot - in->im.imqdedft - in->im.imqdedfn ;
				   }
				   else {
				   		taxinc = in->im.imqitax;	 
				   }

				   /* Calculate the adjusted taxable income */	
				   /* In 2006, dividend income was split with varied gross up rates.  The amount of 
				      dividend income to deduct in the calculation of adjusted taxable income for
					  Que. alternative minimum tax has always been the equal to the difference
					  between the grossed up amount and the actual amount.  Removing QDIVRAT and
					  replacing the taxable amount less the actual amount in the adjustment as
					  per tax forms in 2006.  The formula works for all years. */

				   adjtaxinc = taxinc
								+ (MP.QAMTRPFLG?in->id.idrpp:ZERO)    /* contributions to RPP */
								+ (MP.QAMTRPFLG?in->id.idrrsp:ZERO)   /* contributions to RRSP */
								+ in->id.idexplor                     /* exploration and development expenses */
								+ in->id.idemplo                  /* home relocation loan deduction */
								+ (!MP.QAMTXSTKFLG?in->im.imqstkdd:ZERO)  /* Stock option deduction */
								+ nneg(in->im.imqcapgt * MP.QAMTCAPR)  /*  proportion of taxable capital gains */
								+ (MP.QAMTXREF?in->id.idpartlo:ZERO)  /* allowable partnership losses */
								+ (MP.QAMTXREF?in->id.idcarry:ZERO)   /* carrying charges  */
								- ( (in->im.imqidivt - in->id.ididiv)         /* grossed up amount of taxable dividends */
									+ (in->id.idiloss * MP.QAMTCAPR)   /* proportion of business losses  */
									+ (in->id.idcloss * MP.QAMTCAPR))  /* proportion of net capital losses of other years */
								;

				   /* Calculate the basic exemption if applicable */
				   if (MP.QAMTCBX) {
						basex = (nneg(in->im.imqinet - in->im.imqcapgx)/MP.QCAPAMTX)
								+ MP.QAMTX;
				   }
				   else {  /* use the basic exemption value */
				        basex = MP.QAMTX;
				   }

				   /* Calculate the value of the non-refundable tax credits */
				   /*  equal to total tax credits less refundable credits - 
				       dividend tax credit, political contribution and labour sponsored credits */

				   nrtc = nneg(in->im.imqtottc - in->im.impptc - in->im.imqdtxc - in->im.implvctc);
				   
				   /* Subtract the basic exemption from adjusted taxable income and 
				      multiply by the alternative minimum tax rate and reduce by non-refundable tax credits */

				   altmintx = nneg((nneg(adjtaxinc - basex) * MP.QAMTTX)-nrtc );

				   /* Add back tax credits from retirement income and transferred from spouse if applicable */
				   
				   if (!MP.QAMTRITC) { /* if this parameter is off, add back the retirement income credit */
				       altmintx += in->im.imqritc;  
				   }

				   /* Add back a portion of the credits transferred from a spouse 
				      when QAMTSTTC is flagged on.  There was a retroactive change
				      to AMT calculation in 2004, retroactive to 2003.  */
				   if (MP.QAMTSTTC) {
				   		altmintx += in->im.imqtcts ;	 
				   }		   
				   /* Compare to basic provincial tax payable */
				   
		           if (altmintx > in->im.imbpt ) {
		                in->im.imqamt=altmintx - in->im.imbpt;
		                in->im.imqamtfg = 1;
		                in->im.imbpt = altmintx;
		           }  
	          }       
		   }	   	
        }     
    

      /* process persons in census family */
      for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
          /* copy basic provincial tax to provincial tax payable */
          in->im.imtxp = in->im.imbpt;
        
           /* Quebec minimum tax: second part as a correction to imptx */
        
           /* add Quebec Health Services Fund contributions (tax) to imtxp */
           in->im.imtxp += in->im.imqhsfc;

      }
    }

    /* calculate tax reduction, real estate rebate, and sales tax rebate 
       and combined living alone, age, retirement credit */ 
    if (MP.QFTRFLAG || MP.QRTRFLAG || MP.QHSF>0 || MP.QSTRFLAG || MP.QCALRFLG ) {

        int dcflag;           /* dependent children in family flag */
        int qndc;             /* number of dependents */
        int qfs;              /* family situation code */
        NUMBER qei  = ZERO;   /* eligible income */
        P_in   indch = NULL;  /* designated child for family tax reduction */
        NUMBER amtpt = ZERO;  /* amount of property tax */
        NUMBER qptr = ZERO;   /* property tax rebate */
        NUMBER qstr = ZERO;   /* sales tax rebate */
        NUMBER qstrext = ZERO;   /* Extra sales tax rebate */
        NUMBER qtrtxp = ZERO; /* Tax payable on which provincial tax reduction is calculated */
        NUMBER calr = ZERO;    /* Combined age, living alone, and retirement credit */

        DEBUG1("%s Performing eligible income calculation\n");

        /* process each nuclear family in household */
        for (nfi=0, nf=&hh->nf[0]; nfi<hh->hhnnf; nfi++, nf++) {

            /* initialise elder and spouse pointers */
            ineld = nf->nfineld;
            inspo = nf->nfinspo;   /* NB: is NULL if spouse not present */

            /**
            * Determine family situation and eligible income
            * for family tax reduction and property tax rebate.
            **/

            /* Determine family situation */

            /**
            * Determine if there are dependent children by looking for
            * the corresponding deduction or tax credit.  This is preferable
            * to re-doing looking at full-time university attendance, etc.
            **/

			dcflag = 0;
			qndc = 0; 

            if (ineld->im.imqndc>0) {
                /* there are dependent children */
                dcflag = 1;
                qndc = ineld->im.imqndc;
            }
            else if (nf->nfspoflg) {
                if (inspo->im.imqndc>0 ) {
                    /* there are dependent children */
                    dcflag = 1;
                    qndc = inspo->im.imqndc ;
                }
            }

            /* Kids 18+ in school who transfer their basic amount also considered dependants */
            if (ineld->im.imqtpcn>0) {
                /* there are dependent children */
                dcflag = 1;
                qndc += ineld->im.imqtpcn;
            }
            else if (nf->nfspoflg ) {
                if (inspo->im.imqtpcn>0) {
                    /* there are dependent children */
                    dcflag = 1;
                    qndc += inspo->im.imqtpcn;
                }
            }

            if (nf->nfspoflg && dcflag) {
                qfs = 1;
                DEBUG2("%s couple with children, type=%d\n", qfs);
            }
            else if (nf->nfspoflg) {
                qfs = 2;
                DEBUG2("%s couple without children, type=%d\n", qfs);
            }
            else if (dcflag && hh->hd.hdnpers == (1 + qndc) ) {
                qfs = 3;
                DEBUG2("%s living alone with children, type=%d\n", qfs);
            }
            else if (dcflag) {
                qfs = 4;
                DEBUG2("%s single parent (not alone) with children, type=%d\n", qfs);
            }
            else if (hh->hd.hdnadult == 1) {
                qfs = 5;
                DEBUG2("%s no dependents, not married, lives alone, type=%d\n", qfs);
            }
            else {
                qfs = 6;
                DEBUG2("%s no dependents, not married, does not live alone, type=%d\n", qfs);
            }

            /* record the family type */
            ineld->im.imqfs = qfs;
            if (nf->nfspoflg) {
                inspo->im.imqfs = qfs;
            }

            /**
            * Determine eligible income for elder and spouse, as well
            * as family eligible income.
            **/
			if (MP.QSFDED == 1 ) {  // use net income, same for simplified and general forms
				ineld->im.imqei = nneg(ineld->im.imqinet - MP.QFAMDED );
			}
			else {  
	            if (MP.QFAMFLG == 0 ) {

	                ineld->im.imqei = nneg(
	                      ineld->im.imqinet
	                    + ineld->id.idrpp
	                    + ineld->id.idrrsp
	                    + (MP.QTRSAFAM?ineld->im.imisa:ZERO)  /* Add Social Assistance if not already included in net income */
	                    + (MP.QSPBFLAG?ZERO:ineld->im.imigis) /* Add GIS benefits if not already included in net income */
	                    + (MP.QSPBFLAG?ZERO:ineld->im.imispa)  /* Add Spouse Allowances if not already included in net income */
						+ (MP.QTRWCFAM?ineld->id.idiworkc:ZERO));  /* Add Workers Compensation Benefits included in net income */
	                /* subtract amount for family situation from elder */
	                switch (qfs) {
	                    /* couple with kids */
	                    case 1:
	                        ineld->im.imqei -= MP.QEIA1;
	                        break;
	                    /* couple with no kids */
	                    case 2:
	                        ineld->im.imqei -= MP.QEIA2;
	                        break;
	                    /* 1 adult with kids */
	                    case 3:
	                        /* eligible for amount for person living alone */
	                            ineld->im.imqei -= MP.QEIA3;
	                        break;
	                    /* single parent */
	                    case 4:
	                        ineld->im.imqei -= MP.QEIA4;
	                        break;
	                    /* no kids, not married, lives alone */
	                    case 5:
	                        ineld->im.imqei -= MP.QEIA5;
	                        break;
	                    /* no kids, not married, lives with others */
	                    case 6:
	                        ineld->im.imqei -= MP.QEIA5;
	                        break;
	                }
	                /* add amount for elderly */
	                if (ineld->id.idage >= 65) {
	                    ineld->im.imqei -= MP.QEIAA;
	                }
	                /**
	                * Subtract other amounts. The division by QNTCR converts credits
	                * back to amounts.
	                **/
					if (MP.QNORFAMI) {   /* northern deductions for years 1995-97 */
					   ineld->im.imqei -= ineld->id.idnorth; /* northern deduction*/
	                }

	                ineld->im.imqei -=
	                        (
	                        + ineld->im.imqcpptc * MP.QCRDFLG /* CPP tax credit */
	                        + ineld->im.imquictc * MP.QCRDFLG /* UI Tax Credit */
	                        + ineld->im.imqhsfc * MP.QCRDFLG  /* Heatlh Services Fund contributions */
	                        + ineld->im.imqlatc     /* living alone */
	                        + ineld->im.imqdctc     /* dependent children */
	                                                /* other dependants */

	                        ) / MP.QNTCR;

	                 if (MP.QDUESOPT == 2) {
	                    ineld->im.imqei -= ineld->im.imqduetc/MP.QNTCR;
	                 }
	            }
	            else {	 /* QFAMFLG == 1  */
	                ineld->im.imqei = nneg(ineld->im.imqitot
	                        - ineld->id.idrpp
	                        - ineld->id.idrrsp 
							- (MP.QREPFAMI?ineld->im.imrepay:ZERO)    /* repayment of UI/OAS */
	                        - (MP.QITSFSEW?ineld->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
	                        - ineld->im.imqqpsedd  
	                        - MP.QFAMDED)
	                        ;
	            }
			}

            /* initialise family eligible income with elder's value */
            qei = ineld->im.imqei ;

            if (nf->nfspoflg) {
				if (MP.QSFDED == 1) {
					inspo->im.imqei = inspo->im.imqinet ;
				}
				else {

	                if (MP.QFAMFLG == 0) {
	                    inspo->im.imqei =
	                      nneg(inspo->im.imqinet
	                    + inspo->id.idrpp
	                    + inspo->id.idrrsp
	                    + (MP.QTRSAFAM?inspo->im.imisa:ZERO)  /* Add Social Assistance if not already included in net income */
	                    + (MP.QSPBFLAG?ZERO:inspo->im.imigis) /* Add GIS benefits if not already included in net income */
	                    + (MP.QSPBFLAG?ZERO:inspo->im.imispa)  /* Add Spouse Allowances if not already included in net income */
						+ (MP.QTRWCFAM?inspo->id.idiworkc:ZERO));  /* Add Workers Compensation if not already included in net income */

	                    /* subtract amount for elderly */
	                    if (inspo->id.idage >= 65) {
	                        inspo->im.imqei -= MP.QEIAA;
	                    }
						if (MP.QNORFAMI) {   /* Northern deductions subtracted for years 1995 to 1997 */
	                    	inspo->im.imqei -= inspo->id.idnorth; /* northern deduction*/
						}
	                    inspo->im.imqei -=
	                        (
	                         + inspo->im.imqcpptc * MP.QCRDFLG /* CPP tax credit */
	                         + inspo->im.imquictc * MP.QCRDFLG /* UI Tax Credit */
	                         + inspo->im.imqhsfc * MP.QCRDFLG  /* Heatlh Services Fund contributions */
	                         + inspo->im.imqdctc     /* dependent children */
	                                                /* other dependants */
	                        ) / MP.QNTCR;
						

	                    if (MP.QDUESOPT == 2) {
	                        inspo->im.imqei -= inspo->im.imqduetc/MP.QNTCR;
	                    }

	                }
	                else {	/* QFAMFLG == 1 */
	                    inspo->im.imqei = nneg(inspo->im.imqitot
	                            - inspo->id.idrpp
	                            - inspo->id.idrrsp  
								- (MP.QREPFAMI?inspo->im.imrepay:ZERO)  /* Repayment of OAS/UI */
	        					- (MP.QITSFSEW?inspo->im.imqcppse:ZERO) /* improved tax system for self-employed workers */
                                - inspo->im.imqqpsedd  
                                )
	                            ;
	                }
				}

                /* add to family amount */
                qei += inspo->im.imqei;
            }

            /* make zero if negative */
            qei = nneg(qei);

            DEBUG2("%s family eligible income = %.2f\n", qei);

            /*************
            ***  Combined credit for age, person living alone, or for retirement income 
            **************/
            if (MP.QCALRFLG){

                calr=ZERO;

                if (hh->hd.hdnpers==1 || ineld->im.imqfs == 3) {
                    /* eligible for living alone amount */
                    calr = MP.QLAXM;
                    /* if no child assistance but older dependants, add supplement */
                    if (nf->nfnkids==0 && ineld->im.imqtpcn>0) {
                        calr += MP.QLAXOLDDEP;
                    }
                }
                /* age amount for self */
                if (ineld->id.idage >= 65) {
                    calr += MP.QAXM;
                }
                /* age amount for spouse */
                if (nf->nfspoflg) {
                    if (inspo->id.idage >= 65) {
                        calr += MP.QAXM;
                    }
                }
                /* retirement amount for self */
                if (isnzero(ineld->im.imqipnst)) {
                    calr += minn(ineld->im.imqipnst, MP.QYPDL);
                }
                /* retirement amount for spouse */
                if (nf->nfspoflg) {
                    if (isnzero(inspo->im.imqipnst)) {
                        calr += minn(inspo->im.imqipnst, MP.QYPDL);
                    }
                }
                calr -= MP.QFAMRAT * qei;
                calr = calr * MP.QNTCR;
                if (calr < 0) calr=0;

                /* need to split between spouses. Start by giving to spouse with highest taxes */
                if (nf->nfspoflg && calr>0) {
                    /* elder has largest taxes */
                    if (ineld->im.imtxp > inspo->im.imtxp) {
                        /* deduct credit from elder's taxes */
                        if (ineld->im.imtxp > calr) {
                            ineld->im.imqcalr = calr;
                            ineld->im.imtxp -= calr;
                            calr = ZERO;
                        }
                        else {
                            ineld->im.imqcalr = ineld->im.imtxp;
                            ineld->im.imtxp  = ZERO;
                            calr -= ineld->im.imtxp;
                        }
                        /* give remainder to spouse */
                        if (inspo->im.imtxp > calr) {
                            inspo->im.imqcalr = calr;
                            inspo->im.imtxp -= calr;
                        }
                        else {
                            inspo->im.imqcalr = inspo->im.imtxp;
                            inspo->im.imtxp  = ZERO;
                            calr -= inspo->im.imtxp;
                        }
                    }
                    /* spouse has highest taxes */
                    else {
                        /* deduct credit from spouse's taxes */
                        if (inspo->im.imtxp > calr) {
                            inspo->im.imqcalr = calr;
                            inspo->im.imtxp -= calr;
                            calr = ZERO;
                        }
                        else {
                            inspo->im.imqcalr = inspo->im.imtxp;
                            inspo->im.imtxp  = ZERO;
                            calr -= inspo->im.imtxp;
                        }
                        /* give remainder to eldest */
                        if (ineld->im.imtxp > calr) {
                            ineld->im.imqcalr = calr;
                            ineld->im.imtxp -= calr;
                        }
                        else {
                            ineld->im.imqcalr = ineld->im.imtxp;
                            ineld->im.imtxp  = ZERO;
                            calr -= ineld->im.imtxp;
                        }
                    }
                }
                /* no spouse present */
                else if (calr>0) {
                    /* deduct credit from elder's taxes */
                    if (ineld->im.imtxp > calr) {
                        ineld->im.imqcalr = calr;
                        ineld->im.imtxp -= calr;
                        calr = ZERO;
                    }
                    else {
                        ineld->im.imqcalr = ineld->im.imtxp;
                        ineld->im.imtxp  = ZERO;
                        calr -= ineld->im.imtxp;
                    }
                }
            }


            /*************
            /* Real Estate Rebate 
            *************/
            if (MP.QRTRFLAG) {

                /* call the function which performs the inputation of paid rent
                *  and property taxes to individuals in the hhld
                */
                txhhexp(hh);

                /* compute property tax rebate */
                qptr = ZERO;

                /**
                * Compute amount of property tax.
                **/

                amtpt = ineld->im.improptx;

                /* subtract minimum tax/person */
                amtpt -= MP.QPTRMTP;
                if (nf->nfspoflg) {
                    amtpt -= MP.QPTRMTP;
                }

                /* compute maximum tax refund */
                qptr = MP.QPTRTF * minn(MP.QPTRMTR, nneg(amtpt));

                if(MP.QFAMFLG == 0) {
                    /* reduce refund based on family eligible income */
                    if (ineld->im.imqfs == 5 && ineld->id.idage < 65)
                        { 
                        qptr = nneg(qptr - MP.QPTRRR * nneg(qei - MP.QFAMDED));
                        }
                    else 
                        { 
                        qptr = nneg(qptr - MP.QPTRRR * (qei ));
                        }


                    /* add on amount for gis & spa recipients */
                    if (isnzero(ineld->im.imigis) || isnzero(ineld->im.imispa)) {
                        qptr += MP.QPTRGISB;
                    }
                    if (nf->nfspoflg &&
                        (isnzero(inspo->im.imigis) || isnzero(inspo->im.imispa))) {
                        qptr += MP.QPTRGISB;
                    }                               
                }
                else {
                    qptr = nneg(qptr - MP.QPTRRR * qei);
                
                }

                /**
                * Since this amount is refundable, arbitrarily apply the amount to
                * the elder.  The family effect is in any case the same.
                **/
                ineld->im.imqptr = qptr;
                ineld->im.imptc += qptr;
            }

            /*************
            /* Compute the Quebec sales tax credit 
            **************/

            if (MP.QSTRFLAG) {
                qstr = ZERO;
                qstrext = ZERO;

                /* if the person was claimed as a dependent because they were in school full time
                   and living with their parents, then skip */
				/* When QSTRMAGE is flagged on, eligibility also includes a minimum age of
				   19 unless married or have kids (starts in 2001) */
		        if (ineld->im.imqcchc > 0 || ineld->im.imqctpc >0 || (MP.QSTRMAGE && ineld->id.idage < 19 && 
		        	 (nf->nfspoflg == NULL || nf->nfnkids == 0) ) ) {
			       DEBUG1("%s skipping nuclear families headed by 'dependent''\n");
			       continue;
		        }

                /* compute base sales tax rebate */
                qstr = MP.QSTRBCRD;

                if (nf->nfspoflg ) qstr +=MP.QSTRBCRD;

                /* dependant children credit */

                qstr += qndc* MP.QSTRCHILD;

                /* no spouse and dependent child */    
                if (ineld->im.imqfs == 3 || ineld->im.imqfs==4 ) qstr += MP.QSTRFTP3;

                /* 1 adult household */
                if (((ineld->im.imqfs == 3) || (ineld->im.imqfs == 5))) {
                      qstr +=MP.QSTRFTP4;
                }

                /* Reduce the sales tax credit */
                if (MP.QFAMFLG == 0) {
                    if (ineld->im.imqfs == 5 &&  ineld->id.idage < 65) {                         
                        qstr = nneg(qstr - (MP.QSTRRR * nneg(qei - MP.QFAMDED)));
                    }
                    else { 
                        qstr = nneg(qstr - (MP.QSTRRR * qei));
                    }
                }
                else {
                    qstr = nneg(qstr - (MP.QSTRRR * qei * MP.PYINC));
                }

                /**
                * Families who receive the credit (or who would have received the credit)
                *  had they not received SA get extra
                **/
                if (qstr>0) {
                    qstrext=MP.QSTREXT;
                    if (nf->nfspoflg) qstrext += MP.QSTREXT;
                }


                /**
                *  Families who receive social assistance receive the credit as
                *  part of their sa.  
                ***/

                if (ineld->im.imisa > 0) {
                    qstr = ZERO;
                }
                if (nf->nfspoflg) {
                    if (inspo->im.imisa > 0) {
                        qstr = ZERO;
                    }
                }

                /**
                * Since this amount is refundable, arbitrarily apply the amount to
                * the elder.  The family effect is in any case the same.
                **/

				ineld->im.imqstr = qstr + qstrext;
                ineld->im.imptc += qstr + qstrext;

            }
         }

         /* need a census family loop for anti-poverty, surtax, and family tax reductions */

         /* process each census family in household */
         for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

            /* initialise elder and spouse pointers */
            ineld = cf->cfineld;
            inspo = cf->cfinspo;   /* NB: is NULL if spouse not present */

            /*************
            *****  Family tax reduction
            **************/
            if (MP.QFTRFLAG) {
                DEBUG1("%s computing consumer tax credit / family tax reduction\n");

                NUMBER qftr = ZERO;   /* family tax reduction */
                qei = ZERO;             /* set family income back to zero */

                switch (ineld->im.imqfs) {
                    case 1:
                        qftr = MP.QFTRA1;
                        break;
                    case 2:
                        qftr = MP.QFTRA2;
                        break;
                    case 3:
                        qftr = MP.QFTRA3;
                        break;
                    case 4:
                        qftr = MP.QFTRA4;
                        break;
                    default:
                        qftr = ZERO;
                }

                /* calculate income */
                qei = ineld->im.imqei;
                if (cf->cfspoflg) {
                    qei += inspo->im.imqei;
                }
                qei = nneg(qei);

                /* reduce benefit by percentage of eligible income */
                if (isnzero(qftr)) {
                    if (MP.QFAMFLG == 0) {
                        qftr = nneg(qftr - MP.QFTRRR * qei);

                        if (isnzero(qftr) && MP.QFTRFLAG) {
                            /**
                            * Determine designated child. Just pick first
                            * (ie. youngest) child for now.
                            **/

                            indch = cf->cfinch;

                            if (indch != NULL) {
                                /* reduce benefit by social assistance received by child */
                                qftr = nneg(qftr - indch->im.imisa);

                                /* and by net income in excess of basic amount */
                                qftr = nneg(qftr - nneg(indch->im.imqinet - MP.QBXM ));

                            }
                            else {
                                qftr = ZERO;
                            }
                        }
                    }
                    else {
                        /* this insure the consistency of concepts without implementing 
                         * new parameters and previous year income instead of current*/
                        qftr = nneg(qftr - MP.QFTRRR * qei);
                    }
                }


                /**
                * Arbitrarily, apply family tax reduction to reduce spouse's tax
                * payable first, then apply any unused tax reduction to the elder.
                * The family effect is in any case unaffected by who
                * gets the tax reduction.
                **/

                if (isnzero(qftr)) {
                    /**
                    * Only apply as much to the spouse as is necessary to reduce
                    * the spouse's tax to zero.
                    **/
                    if (cf->cfspoflg && isnzero(inspo->im.imtxp)) {
                        if (inspo->im.imtxp > qftr) {
                            inspo->im.imqftr = qftr;
                            inspo->im.imtxp -= qftr;
                            qftr = ZERO;
                        }
                        else {
                            inspo->im.imqftr = inspo->im.imtxp;
                            qftr -= inspo->im.imtxp;
                            inspo->im.imtxp  = ZERO;
                        }
                        inspo->im.imptr += inspo->im.imqftr;
                    }

                    /**
                    * Apply any remaining amount to the elder.
                    **/
                    if (ineld->im.imtxp > qftr) {
                        ineld->im.imqftr = qftr;
                        ineld->im.imtxp -= qftr;
                    }
                    else {
                        ineld->im.imqftr = ineld->im.imtxp;
                        qftr -= ineld->im.imtxp;
                        ineld->im.imtxp  = ZERO;
                    }
                    ineld->im.imptr += ineld->im.imqftr;
                }
            }

            /***********
            *** Surtax and anti-poverty fund
            ************/
            /* process persons in census family */
            for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
            
                /**
                * Calculate two tier high income surtax
                **/     
                if (MP.QHSF>0 && in->im.imtxp > MP.QHSCI) {
                    in->im.impsur = (MP.QHSF  * nneg(in->im.imtxp - MP.QHSCI))
                                  + (MP.QHSF2 * nneg(in->im.imtxp - MP.QHSCI2));
                    /* apply surtax */
                    in->im.imtxp += in->im.impsur;
                }   

                /**
                 *  Calculate the anti-poverty fund 
                **/
                if (MP.QAPFFLG == 1) {
                    in->im.imqapf = MP.QAPFRAT * in->im.imtxp;
                    in->im.imtxp += in->im.imqapf;
                }

                /**
                * Calculate Provincial tax reduction
                **/
                if (MP.QTROPT==2) {
                    
                    qtrtxp = ZERO; 
                    /**
                    * Provincial income tested tax reduction (Budget 1994)
                    *
                    * Calculate Tax payable for purposes of determining the tax reduction 
                    * We start using basic provincial tax from table as the budget is not
                    * explicit on whether other taxes (ie surtax, health services) are included
                    * in tax payable, or whether other tax reductions (family/property tax..)
                    * Tax reduction may affect calculation of these other credits
                    **/
                    qtrtxp=in->im.imbpt;

                    in->im.imqittr += minn(in->im.imtxp, MP.QTRP * nneg(MP.QTRBE - qtrtxp));
                
                    /* apply tax reduction */
                    in->im.imptr += in->im.imqittr;
                    in->im.imtxp -= in->im.imqittr;
                }   
            }
        }
   }    

   if (MP.QHPTCFLG ) {
       
         /**
         * Housing parents refundable tax credit 
         **/

       /* Calculate the Modeled tax credit */
       if ( MP.QHPTCOPT == 3) {
       
          int hpassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimcgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 70 or disabled and over 60 */
                if (in->id.idage >=70 || (in->id.iddisab == 1 && in->id.idage>= 60 ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.imqhptcp = MP.QHPTC;
                }
			   efimcgtcp += in->im.imqhptcp;
             }

			 hpassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (hpassign == 0 && in->id.idcgtc > 0 ) {
				   	if (in->id.idspoflg && inspo->im.imqhptcp > 0) {
				   		in->im.imqhptc = nneg(efimcgtcp - inspo->im.imqhptcp - in->im.imqhptcp );
						in->im.imptc += in->im.imqhptc;
				   		efimcgtcp = efimcgtcp - in->im.imqhptc;
				   		if (efimcgtcp <= 0) {
				   			hpassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.imqhptc = nneg(efimcgtcp - in->im.imqhptcp);
						in->im.imptc += in->im.imqhptc;
				   		efimcgtcp = efimcgtcp - in->im.imqhptc;
				   		if (efimcgtcp <= 0) {
				   			hpassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (hpassign == 0 && efimcgtcp > 0 && in->id.idcgtc > 0 && in->im.imqhptc == 0 && in->im.imqhptcp == 0 ) {
		 			in->im.imqhptc = efimcgtcp;
					in->im.imptc += in->im.imqhptc;
		 			hpassign = 1;
		 	 	}
		    }
		  } 	 /* end of ef hh loop */
	   }  /* end of model loop */

 	   /* process each person in household */
       for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		    if (isnzero(in->id.idcgtc) && MP.QHPTCOPT < 3){
				/* if QHPTCOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				if (MP.QHPTCOPT == 1) {
					in->im.imqhptc = MP.QHPTC ;
				}
				else {   /* use the greenbook imputed amount */
					in->im.imqhptc = in->id.idcgtc;
				}
				DEBUG2("%s hptc =%.2f\n", in->im.imqhptc);

		        /*  add to refundable tax credits */
		        in->im.imptc += in->im.imqhptc;
			}
	   }
   }

	/* Announced in the 2005 Quebec budget, a new refundable tax credit for natural
	   caregivers.  This new credit will replace the refundable tax credit for housing
	   a parent and the transfer to a natural caregiver of the unused portion of the 
	   non-refundable tax credit respecting a severe and prolonged mental or 
	   physical impairment (imqdistc when granted due to iddisoth>0). */

   /*  Implementation - for the person in the household with the federal amount
       for caregivers, give the maximum Quebec amount.  Assuming only one cared
	   person in the household.  This will have to be revised when disability 
	   status is added to database, then we can add the income test and minimum
	   credit amount */

   /*  Tax form validation - this credit also replaces the Que. infirm dependent 
       amount (18 and over - QODISTC from Schedule A).  Expanding this credit to
	   those with a federal amount in line 316 for infirm dependents 18 and over. */

   if (MP.QRNCGFLG) {
		if (MP.QRNCGOPT == 2) {  /* model the credit */

          int cgassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimncgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 70 or disabled and over 18 */
                if (in->id.idage >=70 || (in->id.idage>= 18 && (isnzero(in->im.imqdistc) || in->id.iddisab == 1) ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.imqnctcp = MP.QRNCGTC - minn(((nneg(in->im.imqinet - MP.QRNCGTD)) * MP.QRNCGTCR), MP.QRNCGMR);
                }
			   efimncgtcp += in->im.imqnctcp;
             }

			 cgassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (cgassign == 0 && (in->id.idcgtc > 0 || in->id.idothpe > 0) ) {
				   	if (in->id.idspoflg && inspo->im.imqnctcp > 0) {
				   		in->im.imqncgtc = nneg(efimncgtcp - inspo->im.imqnctcp - in->im.imqnctcp );
						in->im.imptc += in->im.imqncgtc;
				   		efimncgtcp = efimncgtcp - in->im.imqncgtc;
				   		if (efimncgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.imqncgtc = nneg(efimncgtcp - in->im.imqnctcp);
						in->im.imptc += in->im.imqncgtc;
				   		efimncgtcp = efimncgtcp - in->im.imqncgtc;
				   		if (efimncgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (cgassign == 0 && efimncgtcp > 0 && in->id.idcgtc > 0 && in->im.imqncgtc == 0 && in->im.imqnctcp == 0 ) {
		 			in->im.imqncgtc = efimncgtcp;
					in->im.imptc += in->im.imqncgtc;
		 			cgassign = 1;
		 	 	}
		    }
		  } 	 /* end of ef hh loop */
		}
		else { /* QNCGTCOPT is set to 1 for giving the max */

			/* process for each person in household */
			for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
				if (isnzero(in->id.idcgtc)) {
					in->im.imqncgtc = MP.QRNCGTC;

					DEBUG2("%s ncgtc =%.2f\n", in->im.imqncgtc);

		        	/*  add to refundable tax credits */
		        	in->im.imptc += in->im.imqncgtc;

				}
				/* testing an additional amount for infirm dependents 18+ */
				if (isnzero(in->id.idothpe)) {
					in->im.imqncgtc += MP.QRNCGTC;
					DEBUG2("%s ncgtc =%.2f\n", in->im.imqncgtc);

		        	/*  add to refundable tax credits */
		        	in->im.imptc += in->im.imqncgtc;

				}
			}
		}
			    		
	}  /* End of new natural caregivers code */

    /* Quebec refundable tax credit for medical expenses */
    /*  As of the 1997 taxation year, a refundable tax credit of up to QMEDRATE
    *   of the amount for medical expenses to a maximum credit of QMEDRMAX */

    if (MP.QMRTCFLG == 1) {

        /*  entitlement to the refundable tax credit for medical expenses
        *   is determined by the presense of an allowable medical expense (imqmeda)
        *   and employment related income of at least QMEDRMIN.  Employment
        *   related income corresponds to the total employment income minus
        *   contributions to a registered pension plan and employment expenses
        *   and deductions (including clergy deduction).  This income also includes
        *	 net income from self employment.  One must also be at least 18 at year end. */

        /*  This tax credit is equal to QMEDRATE times the amount of allowable
        *   medical expenses up to a maximum of QMEDRMAX.  It may be reduced
        *   on the basis of an income test.  The parameter QMEDINC determines 
        *   the definition of the income to be used.  When QMEDINC is 1, 
        *   the income test uses the aggregate of the net income of the individual
        *   and their spouse.   When QMEDINC is 2, the test is based on total income  */

     NUMBER famnet = ZERO;
     for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {
       for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {

        if (in->im.imqmeda > 0 && in->id.idage>=18 && 
            (nneg(in->id.idiemp - in->id.idrpp -  in->im.imqalexp - in->id.idclergy) 
             + nneg(in->id.idise)) >= MP.QMEDRMIN) {
            /* initialize spouse pointer */
            inspo = in->id.idinspo;
                if (in->id.idspoflg) {
                    if (MP.QMEDINC == 1 || MP.QSFDED == 1) {
                        famnet = in->im.imqinet + inspo->im.imqinet;
                    }
                    
                    else if (MP.QMEDINC == 2) {
                        famnet = nneg(in->im.imqitot + inspo->im.imqitot
                                 - in->id.idrpp - inspo->id.idrpp
                                 - in->id.idrrsp - inspo->id.idrrsp 
                                 - (MP.QREPFAMI?in->im.imrepay:ZERO)
                                 - (MP.QREPFAMI?inspo->im.imrepay:ZERO)  
								 - (MP.QITSFSEW?in->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
								 - (MP.QITSFSEW?inspo->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
        	                     - in->im.imqqpsedd  
        	                     - inspo->im.imqqpsedd  
                                 );
                    }
                }
                else {
                    if (MP.QMEDINC == 1 || MP.QSFDED == 1) {
                       famnet = in->im.imqinet ;
                    }
                    else if (MP.QMEDINC == 2) {
                       famnet = nneg(in->im.imqitot
                                    - in->id.idrpp
                                    - in->id.idrrsp
									- (MP.QREPFAMI?in->im.imrepay:ZERO)
 									- (MP.QITSFSEW?in->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
            	                    - in->im.imqqpsedd  
                                    );
                    }
                } 
                
                /*  Calculate the refundable tax credit with the appropriate income definition */

                in->im.imqmedrc = nneg(minn(in->im.imqmeda * MP.QMEDRATE, MP.QMEDRMAX)
                        - (nneg(famnet - MP.QMEDRTD)* MP.QMEDRRR));
                /* Add the calculated refundable tax credit for medical expense to 
                *   the variable for provincial tax credits.  */
            in->im.imptc += in->im.imqmedrc;
        }
       }
     }
    }
    

    if (MP.QCCEOPT==TXCR) {
        /**
        * Child care expenses are calculated as a Refundable Tax Credit
        * (Introduced for calendar 1994 in 1994 Quebec Budget) They are included here
        * (not txqccea) because they need information from txqhstr.  
        **/

        int spogets;            /* 0=elder gets, 1=spouse gets */
        int kidsclmd;           /* number of children eligible for CCE */
        NUMBER ccee = ZERO;     /* eligible child care expenses for child */
        NUMBER ccec = ZERO;     /* child care expenses claimed for child */
        NUMBER ccea = ZERO;     /* child care expenses claimed for entire family */
        NUMBER ccetr = ZERO;    /* CCE tax credit rate of allowable expenses */
        NUMBER limitA = ZERO;   /* limitation `A' in form (amount paid) */
        NUMBER limitB = ZERO;   /* Earned income */
        NUMBER limitC = ZERO;   /* limitation `C' in form (amounts per young/old child) */
        NUMBER qcceni = ZERO;   /* Family net income for determining refundable tax credit rate */
        NUMBER ccetc = ZERO;    /* Family refundable child care expense tax credit claimed */
        NUMBER limitBe = ZERO;   /* Earned income of elder  */
        NUMBER limitBs = ZERO;   /* Earned income of spouse */
        
        /* process each census family in household */
        for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {

            /* skip to next family if no kids here */
            if (!cf->cfnkids) {
                continue;
            }

            limitA = ZERO;   /* total child care expenses    */
            limitB = ZERO;   /* earned income limit */
            limitC = ZERO;   /* allowable amounts per child */
			limitBe = ZERO;  /* earned income of elder */
			limitBs = ZERO;  /* earned income of spouse */

            /* Use FAMEX-derived SPSD amount that varies with child, and fudge */

            /* first determine allowed amount for family */
            /* process kids in census family */
            kidsclmd = 0;
            for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++) {
                ini++;

             /* determine total amount of child care expenses per child */
            if (MP.CCEATFLG) {
                /* fudge eligible amount from FAMEX total child care expenses */
                if (in->id.idccet < MP.CCEATLL) {
                       ccee = ZERO;
                }
                else {
                    ccee = MP.CCEATFR * in->id.idccet;
                   }
                }
             else {  /* Use Greenbook imputed amount per child */
                ccee = in->id.idccett;
             }

                DEBUG4("%s processing child aged %d, ccet=%.2f ccee=%.2f\n",
                    in->id.idage, in->id.idccet, ccee);

             /* subject to limits as a function of age */
             if (in->id.idage <= 6) {
                 limitC += MP.QCCEAYNG;
                 ccec = ccee;
                 kidsclmd++;
             }
             else if (in->id.idage <= MP.CCEAOAGE) {
             		limitC += MP.QCCEAOLD;
                    ccec = ccee;
                    if (isnzero(ccec)) {
                        kidsclmd++;
                    }
             }
             else {
                  /* child too old to be eligible */
                 continue;
             }

                /* cumulate amount for family */
                limitA += ccec;
            }

            
            /**
            * let ccea be the minimum of limitations A and C 
            **/
            ccea = (limitA < limitC) ? limitA : limitC;


            /* continue with next family if ccea is zero */
            if (iszero(ccea)) {
                continue;
            }

            /* get elder's and spouses pointer */
            ineld = cf->cfineld;
                        
            if (cf->cfspoflg) {
                inspo  = cf->cfinspo;
            }

            /**
            * Determine earned income
            **/

            limitBe = ineld->id.idiemp + nneg(ineld->id.idise);
            /* as of 2000, employment insurance benefits are included in the earned income test */
			/* this will also include QPIP benefits when this is flagged on */
			if (MP.QCCEEIFLG) {
				limitBe += ineld->im.imiuib + ineld->im.imqpipcbn;
			}
            if (cf->cfspoflg) {
                limitBs = inspo->id.idiemp + nneg(inspo->id.idise) ;
				if (MP.QCCEEIFLG) {
					limitBs += inspo->im.imiuib + inspo->im.imqpipcbn;
				}
            }

			/* the amount of child care expenses allowed is limited by the */
			/* lower of the earned income of the 2 spouses if they exist */

			if (cf->cfspoflg) {
				limitB = minn(limitBe, limitBs);
			}
			else {
				/* only one spouse, limitB is equal to elder's earned income */
				limitB = limitBe;
			}

			/* As of 2007, the earned income test is eliminated */
			if (MP.QCCEEITST) {
            	ccea = minn(ccea, limitB);
			}
			
			/* continue with next family if ccea is zero */
            if (iszero(ccea)) {
                continue;
            }

            /**
            * Determine eligible income for elder and spouse, as well
            * as family eligible (income less deductions) income.
            **/

			if (MP.QSFDED) {
				ineld->im.imqcceni = nneg(ineld->im.imqinet - MP.QFAMDED);
			}
            else if (MP.QFAMFLG==0) {
                ineld->im.imqcceni = nneg(
                      ineld->im.imqinet
                    + ineld->id.idrpp
                    + ineld->id.idrrsp
            		) ;

                /**
                * Deduct amounts.  Tax credits are used, this is a post reform program 
                * 
                **/
            
                ineld->im.imqcceni -=
                    (
                      ineld->im.imqbtc      /* basic tax credit */
                    + ineld->im.imqcpptc    /* CPP tax credit */
                    + ineld->im.imquictc    /* UI Tax Credit */
                    + ineld->im.imqhsfc     /* Heatlh Services Fund contributions */
                    + ineld->im.imqlatc     /* living alone */
                    + ineld->im.imqdctc     /* dependent children  and other infirm dependant */
                    ) / MP.QNTCR;           /* Expand back to credits amounts */
                    if (MP.QDUESOPT == 2) {
                        ineld->im.imqcceni -= ineld->im.imqduetc/MP.QNTCR;
                    }
					if (MP.QNORFAMI == 1) { 
						ineld->im.imqcceni -= ineld->id.idnorth;   /* Reduce income by idnorth 1995 to 1997 */
					}
            }
            else   {  /* QFAMFLG==1 */
                ineld->im.imqcceni =  nneg(ineld->im.imqitot
                        - ineld->id.idrpp
                        - ineld->id.idrrsp
                       	- (MP.QITSFSEW?ineld->im.imqcppse:ZERO)  /* improved tax systme for self-employed workers */
                        - ineld->im.imqqpsedd  
                        - (MP.QREPFAMI?ineld->im.imrepay:ZERO)   /* Repayment of UI/OAS */
                        - MP.QFAMDED )
                        ;
            }

            /* initialise family eligible income with elder's value */
            qcceni = ineld->im.imqcceni;

            if (cf->cfspoflg) {
				if (MP.QSFDED) {
					inspo->im.imqcceni = inspo->im.imqinet;
				}
                else if (MP.QFAMFLG==0) {
                    inspo->im.imqcceni = nneg(
                           inspo->im.imqinet
                        +  inspo->id.idrpp
                        +  inspo->id.idrrsp );	
                
                    inspo->im.imqcceni -=
                        (
                          inspo->im.imqbtc    /* Basic tax credit */
                        + inspo->im.imqcpptc    /* CPP tax credit */
                        + inspo->im.imquictc    /* UI Tax Credit */
                        + inspo->im.imqhsfc     /* Heatlh Services Fund contributions */
                        + inspo->im.imqdctc     /* dependent children */
                                                /* other dependants */
                    
                        ) / MP.QNTCR;
                     if (MP.QDUESOPT == 2) {
                        inspo->im.imqcceni -= inspo->im.imqduetc/MP.QNTCR;
                     }
					 if (MP.QNORFAMI == 1) {
						inspo->im.imqcceni -= inspo->id.idnorth;
					 }
                  }
                  else  {  /* QFAMFLG == 1 */
                    inspo->im.imqcceni =  nneg(inspo->im.imqitot
                            - inspo->id.idrpp
                            - inspo->id.idrrsp    
                            - (MP.QITSFSEW?inspo->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
                            - inspo->im.imqqpsedd
            			    - (MP.QREPFAMI?inspo->im.imrepay:ZERO)   /* repayment of UI/OAS */
                         	);
                  }
				  
			    /* add to family amount */
                qcceni += inspo->im.imqcceni;
            }

            /* make zero if negative */
            qcceni = nneg(qcceni);

            DEBUG2("%s family eligible income = %.2f\n", qcceni);

			/**
            * Determine if spouse is present in census family,
            * and perform appropriate calculations.
            **/

            /* initialise who claims CCE to elder */
            spogets = 0;

            if (cf->cfspoflg) {

                /**
                * Optimize by assigning CCE to spouse with higher net income,
                * since in this way the family as a whole optimizes their
                * tax situation.
                **/

                DEBUG1("%s partner with higher net income receives ccea\n");
                if (inspo->im.imqinet >= ineld->im.imqinet) {
                    DEBUG2("%s spouse claims ccea=%.2f\n", ccea);
                    spogets = 1;
                }
                else {
                    DEBUG2("%s elder claims ccea=%.2f\n", ccea);
                }
            }
			            
            /**
            * Look up refundable CCE tax credit rate for family using qcceni 
            * and determine family tax credit and assign to elder or spouse
            **/
            
            ccetr = (NUMBER) lkup2(MP.QCCETCR, MP.QCCETCRrows, (double) qcceni) ;
            ccetc = ccea * ccetr;
            
            if (spogets) {
                inspo->im.imqccetc = ccetc;
                inspo->im.imptc += ccetc;
            }
            else {
                ineld->im.imqccetc = ccetc; 
                ineld->im.imptc += ccetc;
            }
        }
    }

    /* Quebec family allowance tax credit */

    if (MP.QFATCOPT == 2)    {
 
       NUMBER cben  ;               /* Calculated level of benefit */    
       NUMBER nfinet;               /* Family net income */
       int     spfflag ;            /* Flag to indicate single parent family */
       NUMBER  maxben;              /* Maximum benefits per family */
       NUMBER  minben;              /* Minimum benefits per family */
       int     kids ;               /* Indicators of number of kids */
       int     tdrrrows ;            /* Number of turndowns and reduction rates */
       NUMBER  td[10];               /* Temporary vector turndowns */
       NUMBER  rr[10];               /* Temporary vector reduction rates */
       int     kidlt6  ;            /* Counter of the number of kids lt 6 */
       int     i  ;            /* Counter of the number of kids lt 6 */
       int     j  ;            /* Counter of the number of kids lt 6 */

       /* process each nuclear family in household */
      for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
            DEBUG4("%s cben: nf %d spoflg=%d, nkids=%d\n",
            nfi, nf->nfspoflg, nf->nfnkids);
         kidlt6 = 0;
         for (ini=0, in= nf->nfinch;  ini<nf->nfnkids;  in++,ini++) {
            /* this program was being phased out from the beginning.  Hence, it's only good
               for children under 6 which were born before August 1, 1997. */
            if (in->id.idage< 6 ) {
                if (/* born in 1997 but before August */
                    (((MP.TARGETYEAR-MP.QFATCK6YR)==in->id.idage) && (in->id.idrand[1]<MP.QFATCK6PYR)) || 
                    /* born before 1997 */
                    ((MP.TARGETYEAR-MP.QFATCK6YR)<in->id.idage)) {
                           kidlt6 +=1;
                }
            }
         }
         /* Proceed only for nf with kids */
         if (nf->nfnkids > 0) {
                   DEBUG1("%s cben: skipping nuclear family with no `children'\n");
                   
            inspo = nf->nfinspo;
            ineld = nf->nfineld;
            kids=nf->nfnkids;
            /* Compute Family net income and family employment income and single parent status*/
 			/* Beginning in 1999, since this program is based on previous year's net income, 
 			   the Family net income concept will be the same regardless of whether the 
 			   individuals completed the general or the simplified tax form. */

			if ( MP.QFAMNETI == 1 ) {
				nfinet = MP.PYINC * nneg( ineld->im.imqitot 
                                    - ineld->id.idrpp
                                    - ineld->id.idrrsp
 									- (MP.QITSFSEW?ineld->im.imqcppse:ZERO)  /* improved tax system for self-employed workers */
                                    - ineld->im.imqqpsedd
		                            - (MP.QREPFAMI?ineld->im.imrepay:ZERO)   /* Repayment of UI/OAS */
                                    );
				if ( nf->nfspoflg == 1 ) {
					nfinet += MP.PYINC * nneg(inspo->im.imqitot 
										- inspo->id.idrpp
										- inspo->id.idrrsp
										- ( MP.QITSFSEW?inspo->im.imqcppse:ZERO ) /* improved tax system for self-employed workers */
                                        - inspo->im.imqqpsedd
                        				- (MP.QREPFAMI?inspo->im.imrepay:ZERO)   /* Repayment of UI/OAS */
										);
				}
			}
			else {
	            nfinet = MP.PYINC * ineld->im.imqinet ;

    	        if (nf->nfspoflg == 1) {
        	       nfinet += MP.PYINC * inspo->im.imqinet;
            	}
		   }

            /* Initialize Temporary Variables */
            spfflag = 0;
            tdrrrows = 0;  
            cben  = ZERO;
            maxben = ZERO;
            minben = ZERO;
            ineld = nf->nfineld;
            inspo = nf->nfinspo;
            if (nf->nfspoflg == 0) {
               spfflag = 1;  
            }

            /**
            * Get the maximum and minimum benefits for this family
            * Benefits are added in a loop
            **/

            for (i=1; i <(kids+1); i++) {
               maxben +=(NUMBER) lkup2(MP.QFATCMXK,MP.QFATCMXKrows,i); 
               minben +=(NUMBER) lkup2(MP.QFATCMNK,MP.QFATCMNKrows,i); 
            }
            maxben += (spfflag * MP.QFATCSUP);
            /* Supplement for large families with kids lt 6 */
            maxben += (NUMBER) MP.QFATCK6[kids][kidlt6];
            /* if income over the high income turndown, the benefit is equal
               to the minimum benefit and the high income turndown and reduction rate
               apply */
            if (nfinet>MP.QFATCHITD) {
               cben = nneg(minben - MP.QFATCHIRR*(nfinet-MP.QFATCHITD));
            }
            else {
               /* there are a few turndowns and reduction rates.  These will vary 
               according to the type of family (single parent or not) plus the
               turndowns will also vary if it's a family with more than 4 children.
               
               The method will be to first decide on the number of turndowns
               and reduction rates.  Secondly, we assign the turndown and reduction
               rates to vectors.  Then the benefit is calculated */

               /* if single parent */
               if (spfflag == 1 ) {
                  /* number of turndowns */
                  tdrrrows = MP.QFATCSRRrows;
                  /* set turndowns and reduction rates */
                  for (i=0;i<MP.QFATCSRRrows;i++) {
                     td[i]=MP.QFATCSRR[i][0];
                     /* turndown equal to the special turndown for families
                        with four or more children, then add to the turndown */
                     if (td[i] == MP.QFATD4KI && kids>3) {
                        td[i] += MP.QFATD4K * (kids-3);
                     }
                     rr[i]=MP.QFATCSRR[i][1];
                  }
               }
               /* if married couple */
               else {
                  /* number of turndowns */
                  tdrrrows = MP.QFATCMRRrows;
                  /* set turndowns and reduction rates */
                  for (i=0;i<MP.QFATCMRRrows;i++) {
                     td[i]=MP.QFATCMRR[i][0];
                     /* turndown equal to the special turndown for families
                       with four or more children, then add to the turndown */
                     if (td[i] == MP.QFATD4KI && kids>3) {
                        td[i] += MP.QFATD4K * (kids-3);
                     }
                     rr[i]=MP.QFATCMRR[i][1];
                  }
               }

               /* set benefit to maximum */                                
               cben = maxben;
               /* reduce benefit.  Loop through the turndown vector
                  starting at the greatest value: td-1 */
               for (i=tdrrrows; i>0; i--) {
                  if (nfinet>td[i-1]) {
                     /* if income is greater than the turndown, then reduce the
                        benefit by this amount.  We'll also want to reduce the benefit
                        according to any smaller turndowns.

                        For example, married couples in 1998 had 2 turndowns,
                        income over 25921 was reduced at a 50% rate while income 
                        between 21825 and 25921 was reduced at a 30% rate.  
                        So if a family's income was 26000, then the benefit would be 
                        reduced by (26000-25921)*0.3 + (25921-21825)*0.5
                        */
                     cben -= rr[i-1]*(nfinet - td[i-1]);
                     /* loop through any smaller turndowns */
                     for (j=i-1; j>0; j--) {
                        cben -= rr[j-1]*(td[j] - td[j-1]);
                     }
                     break;
                  }
               }
               /* the benefit cannot be smaller than the minimum benefit
                  if the income is under QFATCHITD */
               cben = maxn(cben, minben);
            }
           /* if the benefit is smaller than the minimum, set to zero */
           if (cben < MP.QFATCMIN) {
              cben = ZERO;
           } 

           /* Benefits assignment to the women  */
           /* A take up rate can be used if required */
           if (spfflag == 0) {
              if (ineld->id.idsex == 1 && (ineld->id.idrand[3] > (NUMBER) lkup2(MP.QFATCTK, MP.QFATCTKrows, (double) nneg(ineld->im.imqitot)))) {
                 ineld->im.imqfatc = cben   ;
              }
              else if (inspo->id.idsex == 1 && (inspo->id.idrand[3] > (NUMBER) lkup2(MP.QFATCTK, MP.QFATCTKrows, (double) nneg(inspo->im.imqitot)))) {
                 inspo->im.imqfatc = cben;
              }
           }
           else {
              /* Benefits assignment to the parent */
              /* A take up rate can be used if required */
              if (ineld->id.idrand[3] > (NUMBER) lkup2(MP.QFATCTK, MP.QFATCTKrows, (double) nneg(ineld->im.imqitot))) {
                 ineld->im.imqfatc = cben ;
              }
            }
         }
      }   
   }


	/* Quebec Work Premium  */
	/* Announced in 2004 Budget - to be implemented in 2005 */
	/* Refundable tax credit for families or individuals with employment income */
	/* Employment income is used to determine the maximum premium */
	/* Total income is used to clawback the premium */

   if (MP.QWPFLAG) {

      /* First, figure out if there are any eligible children, and if so, 
         what their lowest income is */
      /* Though we are going to assign the benefit at the nuclear family level, kids 18+
         who go to school full-time are elible as dependants for this program */

	  /* Do needed declarations/initialisations for each census family */
	  P_in   p_min  = NULL;   /* kid with smallest net income      */

	  DEBUG1("%s process census family children Pass #1\n");
	  /* process each census family in household */
	  for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          NUMBER inet  = ZERO;

	      /* process children in census family and save income of eligible kid with lowest income */
	      for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {

			    /**
			    * Filter condition:
			    *  either be < 18 or tranfser amount to parents for contribution to education
                *     or had the post-secondary amount (pre 2007)
			    **/

    		  	if (in->id.idage >= 18 && in->im.imqctpc<=0.0 && in->im.imqcchc<=0) {
                    /* not eligible */
    				continue;
    			}

    			/* Update pointer for lowest net income child */
    			inet = in->im.imqinet;

			    if (p_min == NULL || p_min->im.imqinet > inet) {
				    p_min = in;
	            }
          } /* end of children */

          /* Assign lowest income to parent */
          if (p_min != NULL) {
              ineld = cf->cfineld;
              ineld->im.imqidc = p_min->im.imqinet;
          }
	  }

      int spogets;            /* 0=elder gets, 1=spouse gets */
      NUMBER wprem = ZERO;
	  NUMBER awprem = ZERO;
      NUMBER empinc = ZERO;
      NUMBER totinc = ZERO;
	  int famtyp;
      int haskid=0;
	  int adaptedwp = 0;
   
      /* process each nuclear family in household */
      for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
		   
           /* initialise elder and spouse pointers */
           ineld = nf->nfineld;
           inspo = nf->nfinspo;   /* NB: is NULL if spouse not present */

		   /* Eligibility for the work premium 
		       - at least 18 or older, unless with a spouse or own kids 
			   - filer or spouse have employment/business income 
			   - no child assistance received with regard to filer
			   - not claimed as a dependent in post-secondary studies (or transfered credit)
			   - not designated as a dependent for work premium */

           if (ineld->im.imqcchc > 0 || ineld->im.imqctpc > 0|| (ineld->id.idage < 18 && 
           		(nf->nfspoflg == NULL || nf->nfnkids == 0))) {
		       DEBUG1("%s skipping nuclear families headed by 'dependent''\n");
		       continue;
		   }

		   /* calculate employment income and total income */
		   empinc = nneg(ineld->id.idiemp + ineld->id.idise
		   			- ineld->im.imqalexp - ineld->id.idclergy);	
		   totinc = ineld->im.imqinet ;
		   
		   haskid = 0;
		   adaptedwp = 0;
		   wprem = 0;
		   awprem = 0;

           if ( ineld->im.imqndc > 0 || ineld->im.imqtpcn>0 ) haskid = 1;
	     
		   /* Determine if eligible for basic or adapted work premium */
		   /* Eligible for adapted if received the disability tax credit or spouse receives disability tax credit */
		   
		   if ( ineld->im.imqdistc > 0 && MP.QAWPFLAG == 1) {
		   		adaptedwp = 1;
		   }
		   		
	       if (nf->nfspoflg) {
	       		empinc += nneg(inspo->id.idiemp + inspo->id.idise
	       					- inspo->im.imqalexp - inspo->id.idclergy);
	       		totinc += inspo->im.imqinet;
                if ( inspo->im.imqndc > 0 || inspo->im.imqtpcn>0) haskid = 1;
	       		if ( inspo->im.imqdistc > 0 && MP.QAWPFLAG == 1 ) adaptedwp = 1;
	       }

           /* If there is a dependant child, then total income includes the amount of income for the 
                designated "child" over the base amount.  
                We'll use the smallest income, which is always given to the elder
           */
           if ( haskid==1 && ineld->im.imqidc > MP.QBXM) {
                totinc += ineld->im.imqidc - MP.QBXM;
           }

		   
		   if ( haskid==0 && !nf->nfspoflg)	{   /*  alone, no kids */
					 famtyp = 0;
		   }
		   else if ( haskid==0 && nf->nfspoflg) {	 /* couple without kids */
					 famtyp = 1;
		   }
		   else if (!nf->nfspoflg) {	 /* single parent with kids */
				  famtyp = 2;
		   }
		   else {		/* couple with kids */
				  famtyp=3;
		   }

		   /* Calculate base benefit */
		   /* Must have employment income greater than a prescribed limit depending on family type */
		   /* Depending on the disability status of head/spouse give basic or adapted work premium */
		   	
		   if ( adaptedwp == 0 ) {  /* eligible for the basic work premium only */
		   		if (empinc > MP.QWPEIPI[famtyp]) {
		   			wprem = minn(MP.QWPMAX[famtyp], (MP.QWPCR[famtyp] * 
		   			 	(minn(empinc, MP.QWPTD[famtyp]) - MP.QWPEIPI[famtyp])));
		   		}
		   		else {
		   			wprem = 0;
		   		}

		   		/* Clawback work premium if total income is greater than turndown */
		   			   
		   		if (totinc > MP.QWPTD[famtyp] && wprem > 0) {
		   			wprem = nneg(wprem - (nneg(totinc - MP.QWPTD[famtyp]) * MP.QWPRR));
		   		}
		   }
		   else { /* calculate the adapted work premium */
		   		if (empinc > MP.QAWPEIPI[famtyp]) {
		   			awprem = minn(MP.QAWPMAX[famtyp], (MP.QAWPCR[famtyp] * 
		   			 	(minn(empinc, MP.QAWPTD[famtyp]) - MP.QAWPEIPI[famtyp])));
		   		}
		   		else {
		   			awprem = 0;
		   		}

		   		/* Clawback work premium if total income is greater than turndown */
		   			   
		   		if (totinc > MP.QAWPTD[famtyp] && awprem > 0) {
		   			awprem = nneg(awprem - (nneg(totinc - MP.QAWPTD[famtyp]) * MP.QAWPRR));
		   		}
		   }
		   
		   /* Ensure that both work premium programs are not being given, priority to adapted WP */
		   
		  /* if (awprem > 0) {
		   		wprem = 0;
		   	} */			   				

		   spogets = 0;

		   if (nf->nfspoflg) {

                /**
                * Optimize by assigning work premium to spouse with higher net income,
                * since in this way the family as a whole optimizes their
                * tax situation.
                **/

                DEBUG1("%s partner with higher net income receives premium\n");
                if (inspo->im.imqinet >= ineld->im.imqinet) {
                    DEBUG2("%s spouse claims premium=%.2f\n", wprem);
                    spogets = 1;
                }
                else {
                    DEBUG2("%s elder claims premium=%.2f\n", wprem);
                }
           }
			 
           if (spogets) {
               inspo->im.imqwrkpr = wprem;
			   inspo->im.imqawrkpr = awprem;
               inspo->im.imptc += (wprem + awprem);
           }
           else {
               ineld->im.imqwrkpr = wprem; 
			   ineld->im.imqawrkpr = awprem;
               ineld->im.imptc += (wprem + awprem);
           }
	  }
   }

    DEBUG_OFF("Atxqcalc");
}


