/***
*Atxque.cpp - Compute provincial taxes for Quebec
*
*$Id: TXQUE.cpp 6.98 2008/08/22 17:07:44 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   Atxque   [global]  Compute provincial taxes for Quebec
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Ttxque[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL

#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*Atxque - Compute provincial taxes for Quebec
*
*Synopsis:
*    void Atxque(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   txqinet  Compute net income (Quebec)
*   txqitax  Compute taxable income (Quebec)
*   txqhstr  Apply tax transfers between head and spouse (Quebec)
*   txqcalc  Calculate federal income tax
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void Atxque(
	P_hh hh
)
{
	DEBUG_ON("Atxque");

	txqinet(hh);
	txqitax(hh);
	txqhstr(hh);
	txqcalc(hh);

	DEBUG_OFF("Atxque");
}
