/***
*memo2.cpp - Compute consumable income, etc.
*
*$Id: memo2.cpp 6.100 2008/08/22 17:07:44 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | BBM | 02/06/12 | Add after tax LICO option: imatpinc variable            |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | BBM | 92/04/21 | Change def'n of efpovinc to be EF:immtot (incl ctc/stc) |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   memo2    [global]  Compute consumable income, etc.
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Smemo2[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,memo2)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*memo2 - Compute consumable income, etc.
*
*Synopsis:
*    void memo2(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Structure Members:
*   MP.CTFLAG       (i)  Commodity tax activation flag
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   im.imftran      (i)  Federal transfer income
*   im.immdisp      (i)  Disposable income
*   im.immtot       (i)  Total income
*   im.immtran      (i)  All transfer income
*   im.imptran      (i)  Provincial transfer income
*   im.imqtaa       (i)  Quebec tax abatement (applied)
*   im.imqtar       (i)  Quebec tax abatement (refundable)
*   im.imrepay      (i)  Social Benefits Repayments
*   im.imtxfc       (i)  Federal commodity taxes
*   im.imtxpc       (i)  Provincial commodity taxes
*   im.imuibr       (i)  UI benefit recovery
*   im.imftax       (io) Federal taxes
*   im.immicons     (io) Consumable income
*   im.immtax       (io) All taxes
*   im.imptax       (io) Provincial taxes
*   ef.efin         (o)  First person in economic family [pointer]
*   id.idef         (o)  Person's economic family [pointer]
*   im.imatpinc     (o)  After tax Income for low income measurement
*   im.imfedbal     (o)  Federal taxes less transfers
*   im.imnettr      (o)  Net transfers to person
*   im.imothrep     (o)  Other federal repayments
*   im.impovinc     (o)  Before Tax Income for low income measurement
*   im.imprvbal     (o)  Provincial taxes less transfers
*   im.imqta        (o)  Quebec tax abatement (total)
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void memo2(
	P_hh hh
)
{
	register P_in in;
	register int ini;

	DEBUG_ON("memo2");

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		if (MP.CTFLAG) {
			/* adjust memo items by commodity taxes */
			in->im.imftax   += in->im.imtxfc;
			in->im.imptax   += in->im.imtxpc;
			in->im.immtax   += in->im.imtxfc + in->im.imtxpc;

			in->im.immicons = in->im.immdisp
			                - in->im.imtxfc
			                - in->im.imtxpc;
			DEBUG5("%s ftax=%.2f, ptax=%.2f, mtax=%.2f, icons=%.2f\n",
			        in->im.imftax, in->im.imptax, in->im.immtax, in->im.immicons);
		}

		/* compute additional derived memo items */
		in->im.imnettr  = in->im.immtran - in->im.immtax;
		in->im.imothrep = in->im.imrepay - in->im.imuibr;
		in->im.imqta    = in->im.imqtaa + in->im.imqtar;
		in->im.imfedbal = in->im.imftax - in->im.imftran;
		in->im.imprvbal = in->im.imptax - in->im.imptran;

		/**
		* Cumulate the family income for poverty analysis purposes.
		* The before tax value is cumulated into the impovinc variable for the
		* first person in each economic family.  Note that this is
		* the current StatCan income definition used for LICOs. 
		* It includes the Child Tax Credit and Sales Tax Credit. 
		* After tax income (immicons) is then cumulated into the
		* imatpinc variable that can be compared to the After Tax 
		* LICO - PTFAT and efpvthat. 
		**/

		in->id.idef->efin->im.impovinc += in->im.immtot;
		in->id.idef->efin->im.imatpinc += in->im.immicons;

	}

	DEBUG_OFF("memo2");
}

