/***
*txalta.cpp - Compute provincial taxes for Alberta
*
*$Id: txalta.cpp 6.139 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 08/06/11 | Added ACGTCTD and a model option to ACGTCOPT and code   |
* | LP  | 08/06/03 | Removed caregivers (impcgtc) from transfer of credits   |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | CH  | 07/02/22 | Added AHCIPINCFLG - giving AHP to biggest earner        |
* | LP  | 07/02/15 | Added AHCIPFLG - Alta Health Premium activation flag    |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/19 | Removed AUCCBITF-UCCB removed from income as per forms  |
* | LP  | 06/12/14 | Switched iminet to imicnet (income tax form concept)    |
* | LP  | 06/06/12 | Added Phase in rates for Alta Family Employment Tx Cred |
* | LP  | 06/06/09 | Removed AAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added AUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/06/05 | Added PYINC to AFETC, revised application of PYINCP     |
* | LP  | 06/06/02 | Added Alberta Family Tax Credit code for calendar year  |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate ADTCRLC|
* | LP  | 05/11/04 | Added Alberta Resource Rebate (imresreb)                |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 05/05/16 | Added AHCIPSPI - Health premium seniors exempt phase-in |
* | LP  | 05/04/18 | Added AFETCAPC, removed AFETCBPC, updated code          |
* | LP  | 04/07/20 | Added Alta Health Care Insurance Plan (health premium)  |
* | LP  | 04/02/18 | Updated net income concepts used in tax credit tests    |
* | LP  | 03/12/22 | Added AMEDALL and AMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/09 | Added impothpe, AODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added ACGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/12 | Added ideducpm and AEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred.|
* | LP  | 01/05/09 | Added AAMTPCTM and added another option to AAMTOPT      |
* | LP  | 00/04/05 | Added Alberta Energy Rebate AHEATFLG and AHEATREB       |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/05/05 | Modified CGTC                                           |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/05/26 | Added flat tax on taxable income                        | 
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/02/25 | Added AFETCFLAG                                         | 
* | EGA | 97/10/21 | Political contribution calculation fixed                | 
* | BBM | 97/09/12 | Alberta Family Employment Tax Credit Implemented        |
* | BBM | 97/05/27 | drop idroyreb,idroytc,idstkccf,idrecssp (no data in 92) |
* | GJC | 93/02/16 | add idrecssp to income tax                              |
* | GJC | 93/02/16 | add idroytc, idstkccf to non-refundable tax credits     |
* | GJC | 93/02/15 | add the royalty tax rebate idroyreb                     |
* | BBM | 92/07/22 | Incorporated non-refundable provincial tax credit var   |
* | BBM | 92/07/09 | Modeled provincial tax credits, drop reference to idptc |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txalta   [global]  Compute provincial taxes for Alberta
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxalta[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txalta)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txalta - Compute provincial taxes for Alberta
*
*Synopsis:
*    void txalta(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   minn     Find the minimum of two numbers
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   lkup1    Table look-up with interpolation/extrapolation
*   taxbak1  One level tax-back function
*   lkup2    Table look-up without interpolation
*
*Structure Members:
*   MP.AAMTOPT      (i)  Alta alternative minimum tax option
*   MP.AAMTPCTF     (i)  Alta amt rate as pct of additional fed tax due to minimum tax
*   MP.AAMTPCTM     (i)  Alta amt rate as pct of federal minimum tax amount
*   MP.AAMTTX       (i)  Alta amt rate as tax on adjusted income
*   MP.AAXM         (i)  Alta Age Amount
*   MP.AAXRR        (i)  Alta Age Amount credit reduction rate
*   MP.AAXTD        (i)  Alta Age Amount net income turndown
*   MP.ABXM         (i)  Alta Basic Personal Exemption/amount
*   MP.ACGTC        (i)  Alta Caregiver Tax Credit
*   MP.ACGTCFLG     (i)  Alta Caregiver Tax Credit activation flag
*   MP.ACGTCOPT     (i)  Alta Caregiver Tax Credit Option (1=max, 2=greenbook, 3=model)
*   MP.ACGTCTD      (i)  Alta Caregiver Tax Credit income turndown
*   MP.ACHATL1      (i)  Alta Charitable Donations amount level 1
*   MP.ACHATR1      (i)  Alta Charitable Donations tax credit rate 1
*   MP.ACHATR2      (i)  Alta Charitable Donations tax credit rate 2
*   MP.ADTCR        (i)  Alta dividend tax credit rate
*   MP.ADTCRLC      (i)  Alta dividend tax credit rate from large (eligible) corporations
*   MP.AEDXPM       (i)  Alta Education Amount per month of full-time studies
*   MP.AEDXPMPT     (i)  Alta Education Amount per month of part-time studies
*   MP.AEMXM        (i)  Alta equivalent to married amount
*   MP.AEMXMT       (i)  Alta equivalent to married turndown level
*   MP.AFETCAPC     (i)  Alberta Family Employment Tax Credit Amount Per Child [number of children]
*   MP.AFETCAPCP    (i)  Alberta Family Employment Tax Credit Amount Per Child for 1st half of year
*   MP.AFETCBR      (i)  Alberta Family Employment Tax Credit Benefit Rate
*   MP.AFETCBRP     (i)  Alberta Family Employment Tax Credit Benefit Rate for 1st half of year
*   MP.AFETCCI      (i)  Alberta Family Employment Tax Credit Benefit Cut-in Level
*   MP.AFETCCIP     (i)  Alberta Family Employment Tax Credit Benefit Cut-in Level for 1st half of year
*   MP.AFETCFLAG    (i)  Alberta Family Employment Tax Credit Activation Flag
*   MP.AFETCNC      (i)  Alberta Family Employment Tax Credit Maximum Number of Children
*   MP.AFETCNCP     (i)  Alberta Family Employment Tax Credit Maximum Number of Children for 1st half of year
*   MP.AFETCPI      (i)  Alberta Family Employment Tax Credit phase in
*   MP.AFETCPIP     (i)  Alberta Family Employment Tax Credit phase in for 2nd half of year
*   MP.AFETCRR      (i)  Alberta Family Employment Tax Credit Reduction Rate
*   MP.AFETCRRP     (i)  Alberta Family Employment Tax Credit Reduction Rate for 1st half of year
*   MP.AFETCTD      (i)  Alberta Family Employment Tax Credit Turndown Level
*   MP.AFETCTDP     (i)  Alberta Family Employment Tax Credit Turndown Level for 1st half of year
*   MP.AFTAX        (i)  Alberta flat surtax rate on taxable income
*   MP.AHCIPF       (i)  Alberta Health Care Insurance Plan Family Premium
*   MP.AHCIPFLG     (i)  Alberta Health Care Insurance Plan activation flag
*   MP.AHCIPFSB     (i)  Alberta Health Care Insurance Plan Family Subsidy Table
*   MP.AHCIPFSBrows (i)  number of rows in AHCIPFSB
*   MP.AHCIPINCFLG  (i)  Alberta Health Care Insurance Plan flag to give to highest earner
*   MP.AHCIPIT      (i)  Alberta Health Care Insurance Plan Income Threshold [family type]
*   MP.AHCIPS       (i)  Alberta Health Care Insurance Plan Single Premium
*   MP.AHCIPSOPT    (i)  Alberta Health Care Insurance Plan Subsidy option (1=table, 2=flat rate)
*   MP.AHCIPSPI     (i)  Alberta Health Care Insurance Plan Senior Exemption Phase In
*   MP.AHCIPSR      (i)  Alberta Health Care Insurance Plan Subsidy Rate
*   MP.AHCIPSSB     (i)  Alberta Health Care Insurance Plan Single Subsidy Table
*   MP.AHCIPSSBrows (i)  number of rows in AHCIPSSB
*   MP.AHEATFLG     (i)  Alberta Energy Tax Refund activation flag
*   MP.AHEATREB     (i)  Alberta Energy Tax Refund amount
*   MP.ALTASOPT     (i)  Alberta seniors option [1=GIST,2=new seniors benefit 1994]
*   MP.AMAXDX       (i)  Alta Maximum Disability deduction/amount
*   MP.AMAXET       (i)  Alta maximum on transfer of education and tuition amount
*   MP.AMEDALL      (i)  Alta Medical allowance maximum lower limit
*   MP.AMEDANF      (i)  Alta Medical allowance lower limit net income fraction
*   MP.AMXM         (i)  Alta married amount
*   MP.AMXMT        (i)  Alta married amount turndown level
*   MP.AODISTC      (i)  Alta Maximum infirm dependent 18 or older tax credit
*   MP.APNTCR       (i)  Alta provincial non-refundable tax credit rate
*   MP.APTC         (i)  Alberta political contribution table [total donations,donation allowed]
*   MP.APTCBEN      (i)  Maximum Alberta political tax credit allowed
*   MP.APTCrows     (i)  number of rows in APTC
*   MP.APTF         (i)  Alberta provincial tax fraction
*   MP.APTX         (i)  Alta tax table [taxable income,basic provincial tax]
*   MP.APTXrows     (i)  number of rows in APTX
*   MP.ARESREB      (i)  Alberta Resource Rebate Amount
*   MP.ASBEMP       (i)  Alberta seniors benefit employment inclusion rate
*   MP.ASCI         (i)  Alberta surtax cut-in
*   MP.ASF          (i)  Alberta surtax fraction
*   MP.ATRBC        (i)  Alberta tax reduction basic claim
*   MP.ATRF         (i)  Alberta tax reduction fraction
*   MP.ATXFLG       (i)  Alta tax on taxable income activation flag
*   MP.AYPNDL       (i)  Alta Pension Income Deduction Amount
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   ef.efin         (i)  First person in economic family [pointer]
*   ef.efnpers      (i)  Number of persons in economic family
*   hh.cf           (i)  Census family data [array]
*   hh.ef           (i)  Economic family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnef        (i)  Number of economic families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.iddisab      (i)  Disability status (age 16+)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.idestat      (i)  Educational status
*   id.idialimo     (i)  Alimony income received
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idise        (i)  Self-employed income
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idsex        (i)  Sex
*   id.idspoflg     (i)  Person has spouse
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imamtfg      (i)  Minimum tax flag
*   im.imamtinc     (i)  Net adjusted income used for minimum tax
*   im.imamtprv     (i)  Federal minimum tax concept used for provincial minimum tax
*   im.imbft        (i)  Basic federal tax
*   im.imchara      (i)  Allowable charitable donations and gifts (calculated)
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedfn      (i)  All deductions from net income
*   im.imdepni      (i)  Dependant's net income
*   im.imiasb       (i)  Alberta seniors benefit
*   im.imicnet      (i)  Net Income (line 236)
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.iminet       (i)  Net income
*   im.imioas       (i)  OAS benefits
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imitax       (i)  Taxable income
*   im.imitot       (i)  Total income
*   im.imminamt     (i)  Minimum Amount due to Federal Minimum Tax
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   im.imbpt        (io) Basic provincial tax
*   im.impatxc      (io) Prov Age amount tax credit
*   im.impbtc       (io) Prov Basic amount
*   im.impcgtc      (io) Prov Caregiver Tax Credit Level
*   im.impcgtcp     (io) Provincial Dependent Caregiver Tax Credit Amount
*   im.impchrtc     (io) Prov Charitable Donations tax credit
*   im.impcpptc     (io) Prov CPP/QPP contributions tax credit
*   im.impdistc     (io) Prov Disability tax credit
*   im.impdtxc      (io) Provincial dividend tax credit
*   im.impedtxc     (io) Prov Education tax credit
*   im.impintsl     (io) Provincial interest on student loans tax credit
*   im.impmartc     (io) Prov Married and Equivalent to Married tax credit
*   im.impmeda      (io) Prov Medical expenses allowed (computed)
*   im.impmedtc     (io) Prov Medical Expenses tax credit
*   im.impminco     (io) Prov Minimum tax carryover
*   im.impnit       (io) Provincial net income tax
*   im.impothpe     (io) Provincial infirm dependents 18 or older tax credit
*   im.imppentc     (io) Prov Pension Income tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.imprvtax     (io) Prov gross provincial tax
*   im.impsur       (io) Provincial surtax
*   im.imptaxcr     (io) Prov total tax credits
*   im.imptr        (io) Provincial tax reduction
*   im.impttxct     (io) Prov total tax credits transferred
*   im.imptutxc     (io) Prov Tuition tax credit
*   im.impuictc     (io) Prov UI/EI contributions tax credit
*   im.imtxp        (io) Provincial income tax payable
*   im.imiafetc     (o)  Alberta Family Employment Tax Credit Benefits
*   im.imnptc       (o)  Non-refundable provincial tax credits
*   im.impatxcr     (o)  Prov total tax credits applied
*   im.impctxct     (o)  Prov tax credits transferred from children
*   im.impheatr     (o)  Provincial Home Heating Fuel/Energy Rebate
*   im.imphp        (o)  Provincial Health Premium
*   im.impstxct     (o)  Prov tax credits transferred from spouse
*   im.imresreb     (o)  Alberta Resource Rebate Amount
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void txalta(
	P_hh hh
)
{
    register P_in ineld;
    register P_in inspo;
    register P_nf nf;
	register P_cf cf;
    register int nfi;
    register int ini;
	register int cfi;
    register P_in in;
	P_ef ef;
			  
	register int efi;
    NUMBER poltcr = ZERO;    /* Political tax credit (raw) */
    NUMBER poltc = ZERO;     /* Political tax credit */
    NUMBER nfinet = ZERO;    /* AFETC Nuclear Family Net Income */
    NUMBER nfiemp = ZERO;    /* AFETC Nuclear Family Employment Income */
    NUMBER maxben = ZERO;    /* AFETC Nuclear Family Maximum Benefits */
    NUMBER nfben = ZERO;     /* AFETC Nuclear Family Allowed Benefits */  
    NUMBER maxben1 = ZERO;    /* AFETC Nuclear Family Maximum Benefits */
    NUMBER nfben1 = ZERO;     /* AFETC Nuclear Family Allowed Benefits */  
    NUMBER maxben2 = ZERO;    /* AFETC Nuclear Family Maximum Benefits */
    NUMBER nfben2 = ZERO;     /* AFETC Nuclear Family Allowed Benefits */  
    NUMBER afetckid = ZERO;  /* AFETC maximum number of allowable children */
    NUMBER provtax = ZERO;   /* Provincial tax */
    NUMBER taxcred = ZERO;   /* Provincial tax credits */
	NUMBER ahptinc = ZERO;   /* taxable income for Alberta Health Premium */
    int    ahpspo = 0;       /* Spouse has a higher income for Alberta Health Premium */
	NUMBER asbfinc = ZERO;   /* senior income for Alberta Health Premium */
	NUMBER ahpnkids = ZERO;  /* number of kids for Alberta Health Premium */
	NUMBER ahpsen = ZERO;    /* number of seniors for Alberta Health Premium */
	NUMBER ahpnadlt = ZERO;  /* number of adults for Alberta Health Premium */
	NUMBER ahpnpers = ZERO;  /* family size for Alberta Health Premium */
	NUMBER ahpbase = ZERO;   /* Alberta Health Premium base amount before subsidy */
	NUMBER ahp = ZERO;       /* Alberta Health Premium amount */
	NUMBER subrate = ZERO;   /* Alberta Health Premium subsidy rate */
	NUMBER efimcgtcp = ZERO;  /* combined cgtc potential amounts for ef */   

    DEBUG_ON("txalta");


    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.ATXFLG) { 
       /* Calculate the Caregiver tax credit */
       if (MP.ACGTCFLG  && MP.ACGTCOPT == 3) {
       
          int cgassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimcgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 65 and either a head or spouse or other family member who could be a parent (with takeup) */
                if (in->id.idage >=65 || (in->id.iddisab == 1 && in->id.idage>= 18 ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.impcgtcp = MP.APNTCR * minn(nneg(MP.ACGTCTD -in->im.iminet),MP.ACGTC);
                }
			   efimcgtcp += in->im.impcgtcp;
             }

			 cgassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (cgassign == 0 && in->id.idcgtc > 0 ) {
				   	if (in->id.idspoflg && inspo->im.impcgtcp > 0) {
				   		in->im.impcgtc = nneg(efimcgtcp - inspo->im.impcgtcp - in->im.impcgtcp );
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.impcgtc = nneg(efimcgtcp - in->im.impcgtcp);
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (cgassign == 0 && efimcgtcp > 0 && in->id.idcgtc > 0 && in->im.impcgtc == 0 ) {
		 			in->im.impcgtc = efimcgtcp;
		 			cgassign = 1;
		 	 	}
		    }
		  } 	 /* end of ef hh loop */
	   }  /* end of cgtc loop */

       /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;

          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */

          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni;
			 
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.APNTCR * nneg(MP.AEMXM - nneg(cinet - MP.AEMXMT));
             }
          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption 
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.APNTCR * nneg(MP.AMXM - nneg(inspo->im.imdepni - MP.AMXMT));
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.APNTCR * nneg(MP.AMXM - nneg(ineld->im.imdepni - MP.AMXMT));
             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
          }
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/
			 
			 /* calculate caregivers tax credit */
			 if (MP.ACGTCFLG && isnzero(in->id.idcgtc) && MP.ACGTCFLG < 3) {
				/* if AGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.ACGTCOPT == 1) {
					in->im.impcgtc = MP.ACGTC * MP.APNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.APNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }

             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.APNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.APNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.APNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.APNTCR * MP.ABXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc = in->im.imicnet ;

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.AAXM, MP.AAXRR * nneg(netinc - MP.AAXTD));
                adjamt = MP.AAXM - agered;
                in->im.impatxc = MP.APNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.APNTCR * minn(MP.AYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.AMEDALL, MP.AMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.APNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

			 /* calculate amount for infirm dependents age 18 or older */
			 /*  if the filer has an amount for idothpe, then give max credit */

			 if (isnzero(in->id.idothpe)) {
   				in->im.impothpe = MP.APNTCR * MP.AODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
			 }
		     
             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.ACHATL1, in->im.imchara)  * MP.ACHATR1
                            + nneg(in->im.imchara - MP.ACHATL1) * MP.ACHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.APNTCR * MP.AMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.APNTCR * MP.AMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             /* 10.0 redo */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.APNTCR * (in->id.ideducfm * MP.AEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.APNTCR * (in->id.ideducpm * MP.AEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.APNTCR * (in->id.ideducpm * MP.AEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }

			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.APNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }

             /* calculate federal dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.ADTCR)
								+ (in->im.imildivt * MP.ADTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }

			 /* calculate the Alberta minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.AAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use AAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.AAMTPCTF;
				 }
				 else if (MP.AAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using AAMTTX. */
						in->im.impminco = in->id.idmincar * MP.AAMTTX;
					  }
				 else if (MP.AAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using AAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.AAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
            
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */
			 if (in->im.imamtfg && MP.AAMTOPT == 3) {
                in->im.imprvtax = MP.AAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.APTX, MP.APTXrows, in->im.imitax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
                        + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
                       	+ in->im.impminco   /* provincial minimum tax carryover credit */
						+ in->im.impintsl   /* provincial interest on student loans credit */
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.APNTCR * MP.AMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.APNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.APNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                       + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco  /* provincial minimum tax carryover credit */
					   + in->im.impintsl  /* provincial interest on student loans credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
                    + in->im.imppentc   /* pension income tax credit             */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.APNTCR * MP.AMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.APNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.ABXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic));
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

	/* if we are calculating tax on taxable income, start by deriving
	   equivalent to married and married tax credits */

    /* process each person in household */
    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
           ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);
		/* taxes based on fraction of federal tax */
        /* original tax as percentage of federal tax */
        if (!MP.ATXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.APTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if ATXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.AAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.AAMTPCTF;
			}
			else {
				if ( in->im.imamtfg && MP.AAMTOPT == 4 ) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.AAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}
        /* compute surtax */
        in->im.impsur = MP.ASF * nneg(in->im.imtxp - MP.ASCI);

        /* apply surtax */
        in->im.imtxp += in->im.impsur;

        /* compute flat tax */
        in->im.impnit += MP.AFTAX * in->im.imitax;

        /* apply flat tax */
        in->im.imtxp += in->im.impnit;

        /* compute tax reduction not more than provincial tax */
        in->im.imptr =
            minn(in->im.imtxp, nneg(MP.ATRBC - MP.ATRF * in->im.imtxp));

        /* apply tax reduction */
        in->im.imtxp -= in->im.imptr;


        /* COMPUTE TAX CREDITS */
        /* calculate and apply provincial foreign tax credit */
        in->im.imnptc += minn(in->im.imtxp, in->id.idprvftc);
        in->im.imtxp -= minn(in->im.imtxp, in->id.idprvftc);


        /**
        * add up other provincial refundable tax credits:
        * these are the credits found in lines 444 through 478.
        * since they are not used to reduce net Alberta tax (line 427), we
        * have chosen to treat them as refundable credits. they are added to
        * the provincial transfers in memo1.c
        **/

        /* Non-refundable Alberta Political Contribution Tax Credit */
        if (isnzero(in->id.idprvpol)) {
            poltcr = (NUMBER) lkup1(MP.APTC, MP.APTCrows, in->id.idprvpol);
            poltc = minn(MP.APTCBEN, poltcr);
            in->im.impptc = minn(poltc, in->im.imtxp);
            DEBUG4("%s alt pol: idpolcon=%.2f poltcr=%.2f poltc=%.2f\n",
               in->id.idprvpol, poltcr, poltc);

            /* apply political contribution tax credit */
            in->im.imnptc += in->im.impptc;
            in->im.imtxp  -= in->im.impptc;

            DEBUG3("%s idpolcon =%.2f importc =%.2f\n",
               in->id.idprvpol, in->im.importc);
        }


        DEBUG4("%s ALTA bpt=%.2f, txp=%.2f, ptc=%.2f\n",
            in->im.imbpt, in->im.imtxp, in->im.imptc);

		/*  Alberta Energy Tax Refund  */

		/* Every Albertan 16 years of age and over will receive a one-time tax refund
		   of $300 in 2001.   */

		if ( MP.AHEATFLG && in->id.idage >= 16 ) {
			in->im.impheatr = MP.AHEATREB;
		}
			
    }

    
    /* Calculate Alberta Family Employment Tax Credit */
   	if (MP.AFETCFLAG) {
       /* process each nuclear family in household */
       for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {

          if (nf->nfnkids == 0) {
              DEBUG1("%s AFETC: skipping nuclear family with no `children'\n");
              continue;
          }

          /* Initialize local variables and head and spouse pointers */
          ineld = nf->nfineld;
          inspo = nf->nfinspo;

          /* Compute Family net income and family employment income */
          nfinet = nneg(ineld->im.imicnet - ineld->im.imuccb) * MP.PYINC;
          nfiemp = (ineld->id.idiemp + ineld->id.idise ) * MP.PYINC;

          if (nf->nfspoflg) {
             nfinet += nneg(inspo->im.imicnet - inspo->im.imuccb) * MP.PYINC;
             nfiemp += (inspo->id.idiemp + inspo->id.idise) * MP.PYINC;
          }

		  afetckid = minn(nf->nfnkids, MP.AFETCNC);

		  if (afetckid == 1) {
		   	    maxben2 = MP.AFETCAPC[0];
		  }
		  else if (afetckid == 2) {
		    	maxben2 = MP.AFETCAPC[0] + MP.AFETCAPC[1];
		  }
		  else if (afetckid == 3) {
		    	maxben2 = MP.AFETCAPC[0] + MP.AFETCAPC[1] + MP.AFETCAPC[2];
		  }
		  else {
		    	maxben2 = MP.AFETCAPC[0] + MP.AFETCAPC[1] + MP.AFETCAPC[2] + MP.AFETCAPC[3];
		  }	
			
          nfben2 = minn( maxben2,  MP.AFETCBR * nneg(nfiemp - MP.AFETCCI) );
          DEBUG2("%s nfben: gross earned inc benefit nfben=%.2f\n",
                   nfben2);

          if (isnzero(nfben2)) {
             taxbak1(nfinet, MP.AFETCTD, &nfben2, MP.AFETCRR);
          }
          DEBUG2("%s nfben: reduced earned inc benefit nfben=%.2f\n",
                   nfben2);

		  if (MP.FCBENCAL) {
			/* calculate the benefit for the calendar year */

		    /* Adjust net income to represent the value for 2 years previous */
			  nfinet = (nfinet / MP.PYINC) * MP.PYINCP; 
			  nfiemp = (nfiemp / MP.PYINC) * MP.PYINCP;
			
			  afetckid = minn(nf->nfnkids, MP.AFETCNCP);

			  if (afetckid == 1) {
			   	    maxben1 = MP.AFETCAPCP[0];
			  }
			  else if (afetckid == 2) {
			    	maxben1 = MP.AFETCAPCP[0] + MP.AFETCAPCP[1];
			  }
			  else if (afetckid == 3) {
			    	maxben1 = MP.AFETCAPCP[0] + MP.AFETCAPCP[1] + MP.AFETCAPCP[2];
			  }
			  else {
			    	maxben1 = MP.AFETCAPCP[0] + MP.AFETCAPCP[1] + MP.AFETCAPCP[2] + MP.AFETCAPC[3];
			  }	
				
	          nfben1 = minn( maxben1,  MP.AFETCBRP * nneg(nfiemp - MP.AFETCCIP) );
	          DEBUG2("%s nfben: gross earned inc benefit nfben=%.2f\n",
	                   nfben1);

	          if (isnzero(nfben1)) {
	             taxbak1(nfinet, MP.AFETCTDP, &nfben1, MP.AFETCRRP);
	          }
	          DEBUG2("%s nfben: reduced earned inc benefit nfben=%.2f\n",
	                   nfben1);	   

			  nfben = (nfben1/2) + ((nfben2/2) * MP.AFETCPIP);
		  }
		  else {  /* use the July to June amounts for the year */
			  nfben = nfben2 * MP.AFETCPI;
		  }
			
          /* assign family benefits to mother if present */

          if (nf->nfspoflg && (inspo->id.idsex == FEMALE)) {
              DEBUG1("%s cben: spouse is the mother, gets AFETC\n");
              inspo->im.imiafetc = nfben;

          }
          else {
              DEBUG1("%s cben: elder receives AFETC \n");
              ineld->im.imiafetc = nfben;

          }
       }
    }

	/* Alberta Health Care Insurance Plan  */

	if (MP.AHCIPFLG) {
    	for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {
        
        	ineld = cf->cfineld;
			inspo = cf->cfinspo;
			int ahpfamt;
			ahp = 0;
			ahpbase = 0;
			subrate = 0;
            ahpspo = 0;

			ahptinc = ineld->im.imitax * MP.PYINC;

			/* Count the number of eligible children in the census family 
	   	   		Eligibility - 20 or younger
	            	        - under 25 and attending school full time.
	                        - 21 or older - dependent due to disability 	*/

			ahpnadlt = 1;	 /* Count eldest as adult */
			ahpnkids = 0;	
        	for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
				if (in->id.idage < 21) {
			    	ahpnkids++;
				}
				else {   /* aged 21 to 24  */
					 if (in->id.idestat == 1 || in->id.idestat == 3 || in->id.iddisslf > 0) {
			 			ahpnkids++;
				 	 }
				 	 else {
						ahpnadlt++;
				 	 }
				}
			}
		
			if (ineld->id.idage > 64) {
				ahpsen++;
			}		 	 

			if (cf->cfspoflg) {
				ahpnadlt++;
				ahptinc += inspo->im.imitax * MP.PYINC;
				if (inspo->id.idage > 64) {
					ahpsen++;
				}
                /* if spouse has a bigger income, then note */
                if (inspo->im.imitax  > ineld->im.imitax ) ahpspo = 1;

			}
			ahpnpers = ahpnadlt + ahpnkids;

			if (ahpnpers == 1) {
				ahpbase = MP.AHCIPS;
			}
			else {
				ahpbase = MP.AHCIPF;
			}

			/* Determine family type 
			Non-senior - Single, Couple, Family with kids
			Senior - Single, Couple (both 65+), Couple (one 65+)
			*/

			if (ahpsen > 0) {
				if (ahpnpers == 1) {
					ahpfamt = 3;	 /* Single Senior */
				}
				else if (ahpsen == 2) {
					ahpfamt = 4;	 /* Senior Couple */
				}
				else {
					ahpfamt = 5;     /* Senior married to non-senior */
				}
			}
			else if (ahpnkids > 0) {
				ahpfamt = 2;		 /* Non-Senior with kids */
			}
			else if (ahpnadlt == 1) {
				ahpfamt = 0;		 /* Non-Senior Single */
			}						 
			else {
				ahpfamt = 1;		 /* Non-Senior Couple */
			}
		
			/* if a senior single or couple, income used in test is not taxable income
		   	but is the non-deductable income used for Alberta Seniors Benefit */

			if (ahpfamt > 2 && MP.ALTASOPT == 2) { 	 /* Alberta Seniors Benefit in effect */
	   			/* calculate income used in program.  It is total income (line 150 of tax guide)
	   		       minus OAS/GIS/SPA, CPP Death and Disabiliy, alimony received, worker's compensation,
	   		   	   social assistance, RRSP contributions, and 5% of employment.  since imitot already
	   		   	   exludes GIS/SPA, social assistance, and worker's compensation, it is not necessary
	   		       to deduct them.  Currently, CPP Death and disability are not included
	   		       as separate items in the model */
			
	   			ahptinc = MP.PYINC * (nneg(ineld->im.imitot 
			               - (MP.ASBEMP * ineld->id.idiemp)
						   - ineld->id.idialimo
			               - ineld->im.imioas 
			               - ineld->id.idrpp
			               - ineld->id.idrrsp));
			
				if (cf->cfspoflg) {
                    ahptinc += MP.PYINC * (nneg(inspo->im.imitot
			        	       - (MP.ASBEMP * inspo->id.idiemp)
							   - inspo->id.idialimo
			               	   - inspo->im.imioas 
			               	   - inspo->id.idrpp
			                   - inspo->id.idrrsp )); 
                    /* if spouse has a bigger income, then note */
                    if ( (nneg(inspo->im.imitot
			        	       - (MP.ASBEMP * inspo->id.idiemp)
							   - inspo->id.idialimo
			               	   - inspo->im.imioas 
			               	   - inspo->id.idrpp
			                   - inspo->id.idrrsp )) 
			                   > 
			                (nneg(ineld->im.imitot 
                              - (MP.ASBEMP * ineld->id.idiemp)
       						   - ineld->id.idialimo
    			               - ineld->im.imioas 
    			               - ineld->id.idrpp
    			               - ineld->id.idrrsp))   ) {
			               ahpspo = 1;
                    }
				}
			}	 
		
			if (MP.AHCIPSOPT == 2 || (ahpfamt > 2 && MP.ALTASOPT == 2)) {	  /* Non-senior & senior  */
				ahp = minn(ahpbase, nneg(ahptinc - MP.AHCIPIT[ahpfamt]) * MP.AHCIPSR);
			}
			else  {   /* different subsidy option, prior to 2002 for  */
				if (ahpfamt == 0) {  /* Single */
					subrate = (NUMBER) lkup2(MP.AHCIPSSB, MP.AHCIPSSBrows, (double) ahptinc);
				}
				if (ahpfamt == 1 || ahpfamt == 2) { /* Family */
					subrate = (NUMBER) lkup2(MP.AHCIPFSB, MP.AHCIPFSBrows, (double) ahptinc);
				}
				ahp = ahpbase * nneg(1.0 - subrate);
			}

			/* Seniors who are eligible for Alberta Seniors Benefit have their premiums waived	 */
			if (MP.ALTASOPT == 2 && (ineld->im.imiasb > 0 || (cf->cfspoflg && inspo->im.imiasb > 0))) {
				ahp = ZERO;
			}
			if (MP.ALTASOPT == 1 && ahpfamt > 2) { /* prior to Seniors Benefit, all seniors are exempt */
				ahp = ZERO;
			}

			if (ahpfamt > 2 ) {  /* seniors - exemption of premiums begins in October 2004 */
		    	ahp = ahp * ( 1 - MP.AHCIPSPI) ;
			}
            
            /* if flagged to give the person with the highest income and spouse has 
               higher income, then give to the spouse */
            if ( MP.AHCIPINCFLG && ahpspo==1) {
    			inspo->im.imphp = ahp;
            }
            else {
   			    /* Assign to the elder in the family */
			    ineld->im.imphp = ahp;
            }
		}
	}

    /* Alberta Resource Rebate - Alberta residents are entitled to a rebate due to
       higher than anticipated resource revenues. For the 2005 Resource Rebate, cheques
       will be issued in January 2006, but based on residency status in Sept 2005.  For
       SPSM, rebates will be granted in 2006 and residency requirement based on 2006.
       All residents receive the full rebate.  Rebates for children under 18 will 
       be assigned to the mother if she exists, otherwise the father.  */
       
	/* process each nuclear family in household */
	for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
        
		ineld = nf->nfineld;
		inspo = nf->nfinspo;

		/* add the kid amount to the mother if she exists */
 	   if (nf->nfspoflg ) {
 		   inspo->im.imresreb = MP.ARESREB + (nf->nfnkids * MP.ARESREB);
		   /* spouse only gets rebate for self */
		   ineld->im.imresreb = MP.ARESREB;
 	   }
	   else {   /* elder gets amounts for kids if they exist. */ 
 		   ineld->im.imresreb = MP.ARESREB + (nf->nfnkids * MP.ARESREB);
 		   /* indicate that person has income */
	   }
	}
				
    DEBUG_OFF("txalta");
}
