/***
*txinet.cpp - Compute net income
*
*$Id: txinet.cpp 6.139 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the txinet.c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 08/05/05 | Updated pension income splitting to match forms         |
* | LP  | 07/07/20 | Removed the nneg constraint on imitot (no longer applic)|
* | LP  | 07/07/16 | Bug fix, UCCB was double counted in imitot and imictot  |
* | CH  | 07/02/22 | Make pension income split variables fractional          |
* | CH  | 07/02/06 | Added pension income splitting                          |
* | LP  | 07/02/06 | Final test for uccb assignment to lower income spouse   |
* | LP  | 07/01/25 | Deleted CLERGYRD - not required, clergy always deducted |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 06/12/18 | Added tax form total/net income concepts imicnet imictot|
* | LP  | 06/06/16 | Add imbcss to net income for income tests               |
* | LP  | 06/06/13 | Subtracted UCCB from the OAS repayment income test      |
* | LP  | 06/06/09 | Removed OASTBPI, phase in not required after 1991       |
* | LP  | 06/05/30 | Added Universal Child Care Benefit imuccb to imitot     |
* | LP  | 06/05/15 | Added the split for large/small corp. dividends, new    |
* |     |          | variables imildivt and imisdivt and parameter - DIVPLC  |
* | CH  | 06/01/04 | Changed QPIP for self-employed                          |
* | LP  | 05/12/16 | Added the Que Parental Insurance premium, revise EI prem|
* | LP  | 05/07/13 | Added clergy residence deduction (CLERGYRD & idclergy)  |
* | LP  | 04/02/18 | Updated net income concepts for social ben repay calc.  |
* | LP  | 04/02/17 | Added idialimo (alimony received) to imitot             |
* | CH  | 03/10/17 | Deleted iditogv, replaced idiint + idioinv by idiinvrd  |
* | LP  | 01/07/10 | Added take up rate to EI contribution refund UIEIREFTK  |
* | CH  | 01/06/11 | Changed UI repayment to work with new database          |
* | LP  | 01/05/30 | Ensure that imicapgt does not become negative           |
* | LP  | 01/03/12 | Removed pre-1988 programs (exemptions, etc.)            |
* | CH  | 00/10/24 | Added CPP deduction for self-employed                   |
* | CH  | 00/10/24 | Added UI benefit recovery                               |
* | LP  | 00/06/01 | Added CPP Contribution Take ups by age range and income |                                          
* | CH  | 00/05/24 | Added iditrrsp                                          |
* | EGA | 99/05/25 | UITARGET by rpovince                                    |
* | EGA | 99/03/25 | Add UITARGET for case selection in UI                   |
* | BBM | 99/02/11 | Drop idiroom from imitot (now incl in idisenf as of 7.0)|
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | SFG | 98/05/14 | play with overlays to keep MS-DOS version working       |
* | CH  | 97/09/04 | modified UI and EI contributions                        |
* | CH  | 97/06/03 | changed EI refund calculation                           |
* | CH  | 97/04/01 | modified employment insurance reforms                   |
* | BBM | 95/12/22 | add employment insurance reforms                        |
* | BBM | 92/07/27 | incorporate new greenbook variables for 1988            |
* | SFG | 91/12/09 | move social programs repayment from txitax to txinet    |
* | SFG | 90/09/27 | re-assign imqtfa to higher inet just like imtfa         |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txinet   [global]  Compute net income
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxinet[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
//#pragma alloc_text(MODEL_TEXT,txinet)
#pragma alloc_text(ROOT_TEXT,txinet)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txinet - Compute net income
*
*Synopsis:
*    void txinet(         
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   round    Round to nearest integer
*   isnzero  Is argument non-zero ?
*   nneg     Change negative numbers to zero
*   minn     Find the minimum of two numbers
*   lkup2    Table look-up without interpolation
*   txccea   Compute child care expense allowance
*
*Structure Members:
*   MP.ALEXPP       (i)  Proportion of other allowable employment expenses to use as deduction
*   MP.CAPGIR       (i)  Capital gains inclusion rate
*   MP.CPP60T64     (i)  CPP/QPP Contributions Take up (60 - 64) by earnings with CPP income
*   MP.CPP60T64rows (i)  number of rows in CPP60T64
*   MP.CPPGT64      (i)  CPP/QPP Contributions Take up (over 64) by earnings with CPP income
*   MP.CPPGT64rows  (i)  number of rows in CPPGT64
*   MP.CPPLT60      (i)  CPP/QPP Contributions Take up (under 60) by earnings with CPP income
*   MP.CPPLT60rows  (i)  number of rows in CPPLT60
*   MP.CPPSEDEDFLG  (i)  CPP/QPP contribution deduction for self-employed earnings flag
*   MP.CPPSEDFRC    (i)  CPP/QPP contribution deduction fraction for self-employed earnings
*   MP.CPPXM        (i)  CPP/QPP exemptible earnings
*   MP.DIVPLC       (i)  Percentage of dividends from large (eligible) corporations
*   MP.FACTISENF    (i)  Scale-up factor for non-farm self-employment income
*   MP.FARR         (i)  Family allowance repayment rate
*   MP.FATBPI       (i)  Family allowance take-back phase in
*   MP.FATD         (i)  Family allowance turndown income
*   MP.FDGUR        (i)  Federal dividend gross-up rate
*   MP.FDGURLC      (i)  Federal dividend gross-up rate from large (eligible) corporations
*   MP.FNTCR        (i)  Federal non-refundable tax credit rate
*   MP.NETOASFLG    (i)  Net the clawback from Old Age Security
*   MP.OASRR        (i)  OAS reduction rate
*   MP.OASTD        (i)  OAS turndown income
*   MP.QPIPEMAX     (i)  Quebec parental insurance plan maximum insurable earnings
*   MP.QPIPEMIN     (i)  Quebec parental insurance plan minimum insurable earnings
*   MP.QPIPF        (i)  Quebec parental insurance plan contribution rate for paid workers
*   MP.QPIPFLAG     (i)  Quebec parental insurance plan activation flag
*   MP.QPIPFSE      (i)  Quebec parental insurance plan contribution rate for self-employed
*   MP.QUIPF        (i)  Quebec EI contribution rate
*   MP.REPNETFLAG   (i)  Social program repayments reduce net income flag
*   MP.SECF         (i)  CPP/QPP contribution rate on self-employment earnings
*   MP.SPLTPNSFLAG  (i)  Flag to split pension income
*   MP.TARGETYEAR   (i)  Year of analysis
*   MP.UIBRA        (i)  UI repayment base amount (UI and EI)
*   MP.UIBRANR      (i)  EI repayment base amount for non-repeaters (EI only)
*   MP.UIBRP        (i)  UI benefit recovery portion (UI only)
*   MP.UIEIBRP      (i)  Maximum clawback under EI reform [Past Wks ben][Rate] (EI only)
*   MP.UIEIBRProws  (i)  number of rows in UIEIBRP
*   MP.UIEINRT      (i)  UI EI Non repeater level for higher clawback turndown (EI only)
*   MP.UIEIOPT      (i)  UI Employment Insurance reform option [1=UI, 2=EI Dec'95]
*   MP.UIEIREF      (i)  EI contribution refund cut-in (EI only)
*   MP.UIEIREFTK    (i)  EI contribution refund take up
*   MP.UIEIREPOPT   (i)  EI Benefit repayment option (1=repeaters have higher rate,2=one rate)(EI only)
*   MP.UIEIRPR      (i)  UI EI benefit clawback rate (EI only)
*   MP.UIERNMAX     (i)  Maximum insurable earnings (UI and EI)
*   MP.UIMINPCT     (i)  Exemption limit (percent of maximum insurable earnings) (UI only)
*   MP.UIPF         (i)  UI contribution rate on earnings (UI and EI)
*   MP.UITARGET     (i)  UI/EI target for the number of cases (UI and EI)
*   MP.WSCF         (i)  CPP/QPP contribution rate on employment earnings
*   MP.WSCM         (i)  Ratio of self-employed to employed contribution fraction
*   MP.YMPE         (i)  CPP/QPP maximum pensionable earnings
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   hd.hdprov       (i)  Province
*   hh.cf           (i)  Census family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idalexp      (i)  Other allowable employment expenses (229)
*   id.idcarry      (i)  Carrying charges (221)
*   id.idcfrh       (i)  Relationship to census family head
*   id.idclergy     (i)  Clergy residential deduction
*   id.iddalimo     (i)  Alimony paid (220)
*   id.iddues       (i)  Union and professional dues (212)
*   id.idexplor     (i)  Exploration and development expenses (224)
*   id.idialimo     (i)  Alimony income received
*   id.idicapg      (i)  Capital gains (actual)
*   id.idicqp       (i)  CPP/QPP income (114)
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idiinvnd     (i)  Interest and other investment income
*   id.idiloss      (i)  Business investment losses (217)
*   id.idipens      (i)  Pension income (115)
*   id.idise        (i)  Self-employed income
*   id.iditoth      (i)  Other taxable income (130)
*   id.iditrrsp     (i)  Taxable RRSP withdrawals
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idlyww       (i)  Weeks worked
*   id.idmovexp     (i)  Imputed moving expenses (219)
*   id.idothded     (i)  Other deductions from total income (232)
*   id.idrand       (i)  Random numbers [array]
*   id.idrpp        (i)  Registered pension plan contributions (207)
*   id.idrrsp       (i)  RRSP calculated amount (208)
*   id.idtuitn      (i)  Tuition fees (320)
*   id.iduirank     (i)  Predicted ranked likelihood of getting UI
*   im.imbcss       (i)  British Columbia Seniors Supplement
*   im.imigis       (i)  GIS benefits
*   im.imiotg       (i)  Other taxable demogrants
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imispa       (i)  Spouse's allowance
*   im.imiuib       (i)  Unemployment Insurance/Employment Insurance benefits
*   im.impnsph      (i)  Pension income split for head
*   im.impnsps      (i)  Pension income split for spouse
*   im.imqpipcbn    (i)  Quebec parental insurance plan calendar year benefits
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   ub.ubcalpd      (i)  Benefits paid in calendar year
*   ub.ubeiwbp      (i)  Weeks of past EI benefits
*   uc.ucbtyp       (i)  Claim type
*   uc.ucstat       (i)  Claim status flag
*   im.imalexp      (io) Allowable employment expenses
*   im.imcppse      (io) CPP contributions deduction for self-emp
*   im.imcqppc      (io) CPP/QPP contributions
*   im.imdedft      (io) Deductions from total income
*   im.imfar        (io) Family allowance recovery
*   im.imicapgt     (io) Capital gains (taxable)
*   im.imicnet      (io) Net Income (line 236)
*   im.imictot      (io) Total Income (line 150)
*   im.imidivt      (io) Dividend income (taxable)
*   im.imildivt     (io) Dividend income from large (eligible) corporations (taxable)
*   im.imiloss      (io) Modelled Business investment losses (217)
*   im.imioas       (io) OAS benefits
*   im.imisdivt     (io) Dividend income from small corporations (taxable)
*   im.imitot       (io) Total income
*   im.imnfach      (io) Number of family allowance children claimed
*   im.imoasr       (io) OAS recovery
*   im.impentr      (io) Pension income transferred from spouse
*   im.impentrd     (io) Pension income deduction from transfer to spouse
*   im.imqpipp      (io) Quebec parental insurance plan premiums for paid workers
*   im.imqpippse    (io) Quebec parental insurance plan premiums for self-employed
*   im.imqpsedd     (io) Quebec parental insurance plan premium deduction for self-employed (federal)
*   im.imqtfa       (io) Quebec taxable family allowances
*   im.imrepay      (io) Social Benefits Repayments
*   im.imtfa        (io) Taxable family allowances
*   im.imuccb       (io) Universal Child Care Benefit amount
*   im.imuibr       (io) UI benefit recovery
*   im.imuic        (io) UIC contributions
*   im.imuicrf      (io) UI EI contribution refund
*   im.imcppctc     (o)  CPP contributions tax credit
*   im.iminet       (o)  Net income
*   im.imipnst      (o)  Taxable pension income (after splitting)
*   im.imqpiptc     (o)  Quebec parental insurance plan premium tax credit for paid workers (federal)
*   im.imqpsetc     (o)  Quebec parental insurance plan premium tax credit for self-employed (federal)
*   im.imtutxc      (o)  Tuition tax credit
*   im.imuictc      (o)  UIC contributions tax credit
*
*Called by:
*   drv      Tax/transfer calculator
*
*****************************************************************************/
void txinet(
	P_hh hh
)
{
	register P_in in;
	P_in ineld;
	P_in inspo;
	int spgetfa;
	int spgetuc;

	P_cf cf;
	register int cfi;
	P_nf nf;

	register int ini;
	register int wkwk;
	int nfi;

	NUMBER diff = ZERO;    	/* difference between MAX and MIN earnings subject */
			                /* to CPP/QPP contribution                         */

	NUMBER wscon  = ZERO;   /* CPP/QPP contribution from wages and salaries      */
	NUMBER secon  = ZERO;   /* CPP/QPP contribution from self-employed earnings  */
	NUMBER empinc = ZERO;   /* employment income (wage/salaries/self-emp) for CPP contrib */
	NUMBER avgye  = ZERO;
	NUMBER itot   = ZERO;
	NUMBER uictd    = ZERO;   /* UI clawback Turndown Level */
	NUMBER uictd1   = ZERO;   /* UI clawback Turndown Level, claim 1 */
	NUMBER uictd2   = ZERO;   /* UI clawback Turndown Level, claim 2 */
	NUMBER uicmpb   = ZERO;   /* UI clawback maximum proportion of benefits */
	NUMBER uicmpb1  = ZERO;   /* UI clawback maximum proportion of benefits, claim 1 */
	NUMBER uicmpb2  = ZERO;   /* UI clawback maximum proportion of benefits, claim 2 */
	NUMBER lowrep  = ZERO;   /* EI repayment for benefits with low lawback rate */
	NUMBER lowrate  = ZERO;  /* EI low clawback rate */
	NUMBER lowtd  = ZERO;    /* EI turndown for benefits with low clawback rate */
	NUMBER lowbens  = ZERO;  /* EI benefits qualifying for low clawback rate */
	NUMBER highrep  = ZERO;   /* EI repayment for benefits with high lawback rate */
	NUMBER highrate  = ZERO;  /* EI high clawback rate */
	NUMBER hightd  = ZERO;    /* EI turndown for benefits with high clawback rate */
	NUMBER highbens  = ZERO;  /* EI benefits qualifying for high clawback rate */
    NUMBER qualbens = ZERO;   /* EI benefits qualifyint for clawback */
	NUMBER netinc = ZERO;
	int prov ;
	NUMBER uipf = ZERO;       /* UI/EI premium rate, differs for Quebec */
    NUMBER headsplit = ZERO;   /* Amount of pension income head to give to spouse */
    NUMBER sposplit = ZERO;    /* Amount of pension income spouse to give to head */


	prov = hh->hd.hdprov;   /* for assignment of UI/EI premium for Quebec/provinces */

	DEBUG_ON("txinet");

	/* do basic tax calculation for all persons in household */

    /* Start by splitting pension income */
	/* process each census family in household */
	for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
        if ( MP.SPLTPNSFLAG ) {
            /* split pension income, if appropriate */
            /* amount to be split is first set to zero */
            headsplit = 0.0;
            sposplit = 0.0;

            /* calculate amount to be split */
            if (cf->cfspoflg) {
    			for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
                    if (in->id.idcfrh==0 && in->im.impnsph>0.0 && in->id.idipens>0) {
                        headsplit= round(in->id.idipens * in->im.impnsph);
                    }
                    else if (in->id.idcfrh==1 && in->im.impnsps>0.0 && in->id.idipens>0) {
                        sposplit= round(in->id.idipens * in->im.impnsps);
                    }
                }
            }
            /* now split income */
            /* note, only one of headsplit or sposplit might be greater than 0, so use this fact to our advantage */
    		for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
                if (in->id.idcfrh==0) {
                    in->im.imipnst = in->id.idipens - headsplit + sposplit;
					in->im.impentr = sposplit;
					in->im.impentrd = headsplit;
                }
                else if (in->id.idcfrh==1) {
                    in->im.imipnst = in->id.idipens + headsplit - sposplit;
					in->im.impentr = headsplit; 
					in->im.impentrd = sposplit;
                }
                else {
                    in->im.imipnst = in->id.idipens ;
					in->im.impentr = ZERO;
					in->im.impentrd = ZERO;
                }
            }
        }
        else {
            /* taxable pension income is equal to original pension income */
			for (ini=0, in=&cf->cfin[0]; ini<cf->cfnpers; ini++, in++) {
                 in->im.imipnst = in->id.idipens;
				 in->im.impentr = ZERO;
				 in->im.impentrd = ZERO;
			}
        }
	}

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		/*  first reset imicnet and iminet to zero */
	  	in->im.imicnet = ZERO;
		in->im.iminet = ZERO;  


		DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
		         ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);


		/* calculate other allowable employment expenses as */
		/* proportion of imputed amount                     */
		if (isnzero(in->id.idalexp)) {
			in->im.imalexp = in->id.idalexp * MP.ALEXPP;
			DEBUG2("%s alexp  =%.2f\n", in->im.imalexp);
		}

		/* calculate taxable capital gains */

		in->im.imicapgt = nneg(in->id.idicapg * MP.CAPGIR);
		DEBUG2("%s icapgt =%.2f\n", in->im.imicapgt);

		/* calculate taxable dividends */
		/* Beginning in 2006, dividends from large corporations will be 
		   subject to a larger gross-up rate (45%) with a larger dividend 
		   tax credit rate, while dividends from smaller corporations will
		   be treated the same as in the past (25% gross up, 13.5% tax credit.
		   Split dividends into large and small corporations as per the 
		   comparative results from Department of Finance.  */

		/* taxable amount of dividends from large corportations */
		in->im.imildivt = (in->id.ididiv * MP.DIVPLC) * MP.FDGURLC;
		DEBUG2("%s ildivt =%.2f\n", in->im.imildivt);

		/* taxable amount of dividends from small corporations */
		in->im.imisdivt = (in->id.ididiv * (1 - MP.DIVPLC)) *  MP.FDGUR;
		DEBUG2("%s idivt =%.2f\n", in->im.imisdivt);

		/* total amount of taxable dividends */
		in->im.imidivt = in->im.imildivt + in->im.imisdivt;

		/* calculate total income excluding family allowances */
		itot = in->id.idiemp      /* earnings from employment                */
		     + MP.FACTISENF * in->id.idise
				                /* self-employed earnings     */
									 /* optionally scaled                    */
		     + in->id.idiinvnd    /* investment income                       */
		     + in->id.idicqp      /* CPP/QPP received                        */
		     + in->id.idipens     /* pension income                          */
			 + in->im.impentr     /* pension income transferred from spouse  */
		     + in->id.iditrrsp    /* taxable RRSP withdrawals                */
		     + in->id.iditoth     /* other taxable income                    */
			 + in->im.imiuib      /* modelled UI benefits received           */
			 + in->im.imqpipcbn   /* modelled QPIP benefits received       	 */
		     + in->im.imiotg      /* modelled new taxable demogrants         */
		     + in->im.imioas      /* modelled OAS benefits                   */
		     + in->im.imidivt     /* modelled taxable amount of dividends    */
		     + in->im.imicapgt    /* modelled taxable capital gains          */
			 + in->id.idialimo	  /* Alimony and support payments received   */
			 ;

		/* ensure total income is not negative (old) */
		/* this was removed in version 15, as total income can be negative for the
		   years simulated in version 15.  Plus imitot is not complete yet, uccb added below */
		in->im.imitot = itot;
		DEBUG2("%s itot   =%.2f\n", in->im.imitot);

		/* create the tax form concept of total income - imictot */
		/* for 1992 onwards, total income contains SA + wcb + net federal supplements*/
		if (MP.TARGETYEAR > 1991) {
			in->im.imictot = in->im.imitot + in->im.imisa + in->im.imigis +
							  in->im.imispa + in->id.idiworkc + in->im.imbcss;
		}
		else {
			in->im.imictot = in->im.imitot;
		}

		/* calculate CPP/QPP contributions */
		diff = MP.YMPE - MP.CPPXM;
		wscon = MP.WSCF * minn(diff, nneg(in->id.idiemp - MP.CPPXM));

		/* calculate amount to contribute on all earnings (iemp+ise) */
		/* (subject to maximum) and then subtract amount already contributed */
		/* for wages and salaries                                            */

		secon = nneg(MP.SECF * minn(diff,
	                           nneg(in->id.idiemp + nneg(in->id.idise)
	                                               - MP.CPPXM)))
	    	    - MP.WSCM * wscon;

		DEBUG4("%s wscon  =%.2f, secon=%.2f, cqpp=%.2f\n",
		           wscon, secon, wscon+secon);
		in->im.imcqppc = wscon + secon;
		DEBUG2("%s cqppc  =%.2f\n", in->im.imcqppc);

		/*  Zero out CPP contributions if contributor is 17 or less or 
		/*  greater than age 70  */

		if (in->id.idage < 18 || in->id.idage > 70 ) {
		    in->im.imcqppc = ZERO;
		}

		/* Apply a takeup rate for contributions to CPP when in receipt of 
		/*  some sort of pension/income from CPP - (idicqp)
		/*  and with earnings >= to CPPXM .
		/*  The take up rate depends on employment earnings and age range (18-59,60-64,65-70)  */
		/*   Those under 18 and over 70 already have their contributions set to 0. */
	
		empinc = in->id.idiemp + in->id.idise;
		if (in->id.idicqp > 0 && in->im.imcqppc > 0 && empinc >= MP.CPPXM) {
		   /* for those in the 18 to 59 age range */	
           if (in->id.idage < 60 &&
               in->id.idrand[4] > (NUMBER) lkup2(MP.CPPLT60,MP.CPPLT60rows,(double)empinc)) {
		   	   in->im.imcqppc = ZERO;
		   }
		   /* for those in the 60 to 64 age range */
		   if (in->id.idage > 59 && in->id.idage < 65 && 
               in->id.idrand[4] > (NUMBER) lkup2(MP.CPP60T64,MP.CPP60T64rows,(double)empinc)) {
		   	   in->im.imcqppc = ZERO;
		   }
		   /*  for those in the 65 to 70 age range */
		   if (in->id.idage > 64 &&
               in->id.idrand[4] > (NUMBER) lkup2(MP.CPPGT64,MP.CPPGT64rows,(double)empinc)) {
		   	   in->im.imcqppc = ZERO;
		   }
		}


	 	if (isnzero(in->im.imcqppc)) {
	 		/* calculate CPP contributions tax credit */
            if (MP.CPPSEDEDFLG) {
                /* give the employee's portion of self-employed contributions
                    as a tax credit, the rest as a tax deduction */
                in->im.imcppse = MP.CPPSEDFRC * secon;
	 		    in->im.imcppctc = (in->im.imcqppc - in->im.imcppse) * MP.FNTCR;
            }
            else {
    			in->im.imcppctc = in->im.imcqppc * MP.FNTCR;
            }
			DEBUG2("%s cppctc =%.2f\n", in->im.imcppctc);
		}

		/* calculate UI contributions */
		DEBUG3("%s lyww=%d, iemp=%.2f\n", in->id.idlyww, in->id.idiemp);

		/* assume earnings are evenly distributed among weeks worked */
		/* if weekly earnings are greater than the minimum insurable */
		/*  earnings, calculate annual contribution                  */

		wkwk = in->id.idlyww;
		/* if weeks worked less than one, weeks worked are assumed to be 
		   equal to 12 weeks since there a number of people with income who
		   have no weeks worked */
		if (wkwk < 1) wkwk = 12;
		if (wkwk > 52) wkwk = 52;
		DEBUG2("%s weeks worked=%d\n", wkwk);

		avgye = in->id.idiemp/(NUMBER) wkwk;

		if ( MP.QPIPFLAG ) {
			/* when activated, Quebec pays a lower UI premium and a 
				separate premium for the Quebec Parental Insurance Plan */
			switch (hh->hd.hdprov) {
				case PRQUE:
					uipf = MP.QUIPF;   
					/* check for minimum insurable earnings */
					if (in->id.idiemp > MP.QPIPEMIN) {
						in->im.imqpipp = MP.QPIPF * minn(in->id.idiemp, MP.QPIPEMAX);
						in->im.imqpippse = MP.QPIPFSE * minn(nneg(MP.QPIPEMAX - in->id.idiemp),
													   nneg(in->id.idise));
					}
                    else if ((in->id.idise) > MP.QPIPEMIN) {
						in->im.imqpippse = MP.QPIPFSE * minn(MP.QPIPEMAX,
													   nneg(in->id.idise));
                    }
					break;
				default:
					uipf = MP.UIPF;
					break;
			}
		}
		else {
			/* one UI/EI premium rate for everyone */
			uipf = MP.UIPF;
		}
		
		
		/* if EI options */
    		 /* no weekly maximum.  All earnings up to the annual 
	    	     Maximum Insurable Earnings count */

		if (MP.UIEIOPT==2) {
			    in->im.imuic = uipf * minn(in->id.idiemp, MP.UIERNMAX * 52);
		}

        /* not EI system */
		else {
		       if (avgye < MP.UIMINPCT * MP.UIERNMAX) {
			   DEBUG1("%s uc=0.0\n");
		       }
		       else {
  		           in->im.imuic = wkwk * uipf * minn(avgye, MP.UIERNMAX);
			       DEBUG2("%s uc    =%.2f\n", in->im.imuic);
			   }
     	}

		/* calculate EI contribution refund for persons with low income */
  		if (MP.UIEIOPT==2) {
			if (in->id.idiemp <= MP.UIEIREF) {
			    in->im.imuicrf = in->im.imuic;

				/* In order to get the refund, one must apply for it.  */
				/*  Apply a take up rate for the EI Contribution refund. */
				if (in->im.imuicrf > 0 && in->id.idrand[0] > MP.UIEIREFTK ) {
				    in->im.imuicrf = ZERO;
				}
				   
				in->im.imuic = nneg(in->im.imuic - in->im.imuicrf);
			}
			else if ((in->id.idiemp - in->im.imuic) < MP.UIEIREF) {
				in->im.imuicrf = nneg(MP.UIEIREF - (in->id.idiemp-in->im.imuic));
				if (in->im.imuicrf < 1) in->im.imuicrf= ZERO;

				/* Apply a take up rate for the EI Contribution refund.  */
				if (in->im.imuicrf > 0 && in->id.idrand[0] > MP.UIEIREFTK ) {
				    in->im.imuicrf = ZERO;
				}
				
				in->im.imuic = nneg(in->im.imuic - in->im.imuicrf);
			}
        }  

   		if (isnzero(in->im.imuic)) {
   			/* calculate UI contributions tax credit */
   			in->im.imuictc = in->im.imuic * MP.FNTCR;
   			DEBUG2("%s uctc  =%.2f\n", in->im.imuictc);
   		}

		/* Calculate the tax credit for the Quebec Parental Insurance Plan premiums */

		if ( MP.QPIPFLAG ) {
			if (isnzero(in->im.imqpipp)) {
				in->im.imqpiptc = in->im.imqpipp * MP.FNTCR;
			}
			if (isnzero(in->im.imqpippse)) {
    			in->im.imqpsetc = in->im.imqpippse * (MP.QPIPF / MP.QPIPFSE) * MP.FNTCR;
    			in->im.imqpsedd = in->im.imqpippse * (1.0 - (MP.QPIPF / MP.QPIPFSE) );
			}
		}

		/* calculate other tax reform tax credits */

		if (isnzero(in->id.idtuitn)) {
			/* calculate tuition tax credit */
			in->im.imtutxc = in->id.idtuitn * MP.FNTCR;
			DEBUG2("%s tutxc  =%.2f\n", in->im.imtutxc);
		}
        /* apply capital gains rate to business investment losses */
        in->im.imiloss = in->id.idiloss * MP.CAPGIR;

		/* calculate deductions from total income */
		/* (not including Child Care Exemption)   */
		in->im.imdedft =
		    in->id.idrpp       /* registered pension plan contribution    */
	      + in->id.idrrsp      /* RRSP amount                             */
		  + in->im.impentrd    /* Pension income amount claimed by spouse */
	      + in->id.iddues      /* union and professional dues             */
	      + in->im.imiloss     /* capital loss on disp of CPCC equities   */
	      + in->id.idmovexp    /* moving expenses                         */
	      + in->id.iddalimo    /* Alimony allowance paid out              */
	      + in->id.idcarry     /* carrying charges                        */
	      + in->id.idexplor    /* exploration expenses                    */
	      + in->im.imalexp     /* other employment expenses               */
	      + in->id.idothded    /* other deductions from total income (232)*/
		  + in->id.idclergy    /* clergy residence deduction (231-after 99) (232-pre 99)*/
          + in->im.imqpsedd    /* QPIP deduction for self-employed workers*/
	      ;

        /* a portion of cpp contributions reprensenting the employer's portion
           of self-employment contributions can be deducted */
        if (MP.CPPSEDEDFLG) {
            in->im.imdedft += in->im.imcppse ;
        }

		DEBUG2("%s dedft  =%.2f\n", in->im.imdedft);

		/* calculate net income                   */
		/* (not including FA and CCE)             */
		in->im.iminet = nneg(in->im.imitot - in->im.imdedft);
		DEBUG2("%s inet   =%.2f\n", in->im.iminet);

		/* calculate the tax form concept of net income */
		in->im.imicnet = nneg(in->im.imictot - in->im.imdedft);
	}

	/* Both the UCCB and family allowances may need to get reassigned.  */
	/*  They were originally given to the eldest.  If they have to be */
	/*  reassigned, then total income and net income will need to be fixed. */

	DEBUG1("%s adjusting net income by family allowance for each NF\n");

	/* process each nuclear family in household */
	for (nfi=0, nf=&hh->nf[0]; nfi<hh->hhnnf; nfi++, nf++) {

		if (nf->nfnkids == 0) {
			DEBUG1("%s skipping nuclear family, (no kids)\n");
			continue;
		}

		/* initialise stuff to elder of nuclear family */

		ineld = nf->nfineld;
		spgetfa = 0;
		spgetuc = 0;

		if (nf->nfspoflg) {
			inspo = nf->nfinspo;
			if (ineld->im.imicnet < inspo->im.imicnet) {
				DEBUG1("%s spouse reports FA: higher net income\n");
				/* Spouse has the higher income.  So spouse gets the family allowance.  */
				spgetfa = 1;
			}
			else {
				DEBUG1("%s head   reports FA: higher net income\n");
				/* Spouse has the lower income.  So spouse gets the UCCB. */
				spgetuc = 1;
			}
		}
		else {
			DEBUG1("%s head   reports: no spouse present\n");
		}

		/* assign taxable portion of FA */
		/* also # FA children (Sask. Prov. Tax uses it) */
		/* adjust total and net income (for tax purposes) to include FA */
		if (spgetfa) {
			/* move amounts from eldest to spouse */
			inspo->im.imtfa   = ineld->im.imtfa;
			inspo->im.imqtfa  = ineld->im.imqtfa;
			ineld->im.imtfa   = ZERO;
			ineld->im.imqtfa  = ZERO;
			inspo->im.imnfach = ineld->im.imnfach;
			ineld->im.imnfach = ZERO;
			inspo->im.imitot += inspo->im.imtfa;
			inspo->im.imictot += inspo->im.imtfa;
			inspo->im.iminet = nneg(inspo->im.imitot - inspo->im.imdedft);
			inspo->im.imicnet = nneg(inspo->im.imictot - inspo->im.imdedft);
		}
		else {
			ineld->im.imitot += ineld->im.imtfa;
			ineld->im.imictot += ineld->im.imtfa;
			ineld->im.iminet = nneg(ineld->im.imitot - ineld->im.imdedft);
			ineld->im.imicnet = nneg(ineld->im.imictot - ineld->im.imdedft);
		}

		if (spgetuc) {
			/* move amounts from eldest to spouse */
			inspo->im.imuccb = ineld->im.imuccb;
			ineld->im.imuccb = ZERO;
			inspo->im.imitot += inspo->im.imuccb;
			inspo->im.imictot += inspo->im.imuccb;
			inspo->im.iminet = nneg(inspo->im.imitot - inspo->im.imdedft);
			inspo->im.imicnet = nneg(inspo->im.imictot - inspo->im.imdedft);
		}
		else {
			ineld->im.imitot += ineld->im.imuccb;
			ineld->im.imictot += ineld->im.imuccb;
			ineld->im.iminet = nneg(ineld->im.imitot - ineld->im.imdedft);
			ineld->im.imicnet = nneg(ineld->im.imictot - ineld->im.imdedft);
		}
			
	}

	/* now complete computation of net income by doing the */
	/* child care expense allowance */
	txccea(hh);

	/* compute social program repayments, based upon net income, */
	/* (or net income before adjustments in 1989 & subsequent years) */

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

		/* calculate all repayments and cumulate into imrepay */
		/* calculate UI benefits repayable  */
		if (isnzero(in->im.imiuib)) {
			
			if (MP.UIEIOPT==2) {

                /* option 1: There are two rates.
                             The low rate is used for sickness, maternity, and parental
                              benefits.  It is also used if a person has not used the ei system much
                              in the past.  (The weeks prior to the first claim are used to decide this fact).
                */

                if (MP.UIEIREPOPT==1) {

                    /* sickness, maternity, and parental benefits always qualify for the lower rate.
                       The benefit recovery amount for other kinds of benefits depends on previous
                       receipt of benefits */

	                lowrep  = ZERO;   /* EI repayment for benefits with low lawback rate */
	                lowrate  = ZERO;  /* EI low clawback rate */
	                lowtd  = ZERO;    /* EI turndown for benefits with low clawback rate */
	                lowbens  = ZERO;  /* EI benefits qualifying for low clawback rate */
	                highrep  = ZERO;   /* EI repayment for benefits with high lawback rate */
	                highrate  = ZERO;  /* EI high clawback rate */
	                hightd  = ZERO;    /* EI turndown for benefits with high clawback rate */
	                highbens  = ZERO;  /* EI benefits qualifying for high clawback rate */
                
                    /* first, calculate the amount of benefits which get the low rate */


                    /* Claim 1 */
                    /* are there benefits? */
                    if (in->id.uc1.ucstat && (in->id.iduirank/1000 < MP.UITARGET[hh->hd.hdprov])) {
                        /* if low weeks or special benefits, use the low rate */
                        if ((in->im.ub1.ubeiwbp<=MP.UIEINRT)
                            || (in->id.uc1.ucbtyp==UIMAT)||(in->id.uc1.ucbtyp==UISIC)||(in->id.uc1.ucbtyp==UIPAR)) {
                            lowbens = in->im.ub1.ubcalpd;
                        }
                        else {
                            highbens = in->im.ub1.ubcalpd;
                        }
                    }
                    /* Claim 2 */
                    /* are there benefits? */
                    if (in->id.uc2.ucstat && (in->id.iduirank/1000 < MP.UITARGET[hh->hd.hdprov])) {
                        /* if low weeks or special benefits, use the low rate */
                        if ((in->im.ub2.ubeiwbp<=MP.UIEINRT)
                            || (in->id.uc2.ucbtyp==UIMAT)||(in->id.uc2.ucbtyp==UISIC)||(in->id.uc2.ucbtyp==UIPAR)) {
                            lowbens += in->im.ub2.ubcalpd;
                        }
                        else {
                            highbens += in->im.ub2.ubcalpd;
                        }
                    }

			        lowtd  = (NUMBER) MP.UIBRANR;
				    lowrate = (NUMBER) (lkup2(MP.UIEIBRP, MP.UIEIBRProws, 0));
			        hightd  = (NUMBER) MP.UIBRA;
                    /* use the first claim weeks to see if over max */
				    highrate  = (NUMBER) (lkup2(MP.UIEIBRP, MP.UIEIBRProws, in->im.ub1.ubeiwbp));
					/* use the tax form concept of net income */
					/* Must add sa, gis, spa, bc sen supp and wcb to net income to match tax forms */
					netinc = nneg(in->im.imicnet - in->im.imuccb);

                    lowrep = minn( lowrate * lowbens, MP.UIEIRPR * nneg(netinc - lowtd) );
                    highrep = minn( highrate * highbens, MP.UIEIRPR * nneg(netinc - lowrep - hightd) );
				    in->im.imuibr = lowrep + highrep;
			    }
                /* in this option, the two rates and thresholds are eliminated.  The clawback
                   no longer depends on how often a person collects benefits.  Maternity, parental and 
                   sickness benefits are exempted from the repayment.  And first-time claimants are 
                   exempted from repaying
                */
                else if (MP.UIEIREPOPT==2) {
                    qualbens = ZERO;
                    if (in->id.uc1.ucstat && (in->id.iduirank/1000 < MP.UITARGET[hh->hd.hdprov])) {
                        /* First-time claimants and maternity, sickness, and parental benefits are exempt
                           from repayment */
                        /* we don't currently have a flag for first-time claimants */
                        if ((in->id.uc1.ucbtyp!=UISIC)&&(in->id.uc1.ucbtyp!=UIMAT)&&(in->id.uc1.ucbtyp!=UIPAR)) {
                            qualbens = in->im.ub1.ubcalpd;
                        }
                    }
                    /* Claim 2 */
                    /* are there benefits? */
                    if (in->id.uc2.ucstat && (in->id.iduirank/1000 < MP.UITARGET[hh->hd.hdprov])) {
                        /* First-time claimants and maternity, sickness, and parental benefits are exempt
                           from repayment */
                        /* we don't currently have a flag for first-time claimants */
                        if ((in->id.uc2.ucbtyp!=UISIC)&&(in->id.uc2.ucbtyp!=UIMAT)&&(in->id.uc2.ucbtyp!=UIPAR)) {
                            qualbens += in->im.ub1.ubcalpd;
                        }
                    }

					/* use the tax form concept of net income */
					/* Must add sa, gis, spa, bc sen supp and wcb to net income to match tax forms */
					netinc = nneg(in->im.imicnet - in->im.imuccb);

					in->im.imuibr = MP.UIEIRPR * minn( qualbens, nneg(netinc - MP.UIBRANR) );
                    
                }
            }
			else {

				/* Derive the net income concept */
			   	netinc = nneg(in->im.imicnet - in->im.imuccb);

				uictd  = MP.UIBRA;
				uicmpb = MP.UIBRP;
				in->im.imuibr = uicmpb * minn(in->im.imiuib, nneg(netinc - uictd));
			}
				
			DEBUG1("%s calculate UI benefits repayable under UI\n");
  			DEBUG2("%s ubr   =%.2f\n", in->im.imuibr);
			in->im.imrepay += in->im.imuibr;
  		}

		/* calculate FA and OAS repayments (optional algorithms) */
		if (isnzero(MP.FARR) && isnzero(in->im.imtfa)) {
			in->im.imfar = MP.FATBPI * minn(in->im.imtfa,
				MP.FARR * nneg((in->im.imicnet-in->im.imuibr) - MP.FATD));
			in->im.imrepay += in->im.imfar;
		}

		/* Derive the net income concept for OAS reduction test 
		   Based on net income (line 234) before adjustments, less
		   the uccb payments  */

		netinc = nneg(in->im.imicnet - in->im.imuccb); 

		if (isnzero(MP.OASRR) && isnzero(in->im.imioas)) {
			in->im.imoasr = minn(in->im.imioas,
				MP.OASRR * nneg((netinc-in->im.imuibr) - MP.OASTD));
			if (MP.NETOASFLG) {
				in->im.imioas -= in->im.imoasr;
				in->im.imitot = in->im.imitot - in->im.imoasr;
				in->im.imictot = in->im.imictot - in->im.imoasr;
				in->im.iminet = nneg(in->im.imitot - in->im.imdedft);
				in->im.imicnet = nneg(in->im.imictot - in->im.imdedft);
			}
			else {
				in->im.imrepay += in->im.imoasr;
			}
		}
		DEBUG2("%s repay=%.2f\n", in->im.imrepay);

		// Starting in 1989, repayments reduce net income. Previous to 1989,
		// repayments reduced taxable income.

		if (MP.REPNETFLAG) {
			/* include repayments in deductions from total income. */
			/* i.e. they means they should be removed from net income. */
			in->im.imdedft += in->im.imrepay;
			in->im.iminet  = nneg(in->im.imitot - in->im.imdedft);
			in->im.imicnet = nneg(in->im.imictot - in->im.imdedft);
		}
	}

	DEBUG_OFF("txinet");
}
