/***
*txns.cpp - Compute provincial taxes for Nova Scotia
*
*$Id: txns.cpp 6.143 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | LP  | 08/06/11 | Added VCGTCTD and a model option to VCGTCOPT and code   |
* | LP  | 08/06/03 | Removed caregivers (impcgtc) from transfer of credits   |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/19 | Removed VUCCBITF-UCCB removed from income as per forms  |
* | LP  | 07/01/15 | Add impyctxc, VYCTCAPM VYCTCPI, removed VUCCBTC,impucctc|
* | LP  | 06/12/13 | Switched iminet to imicnet, imitot to imictot (tax form)|
* | LP  | 06/06/13 | Add nneg to income tests that are reduced by imuccb     |
* | LP  | 06/06/09 | Removed VAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added VUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/06/05 | Revised the application of PYINCP                       |
* | LP  | 06/06/01 | Added impucctc - univ. child care non-ref tax credit    |
* |     |          | and made it non-transferable between parents - verify   |
* | LP  | 06/05/25 | Added N.S. child benefit by calendar year code/parms    |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate VDTCRLC|
* | LP  | 05/11/23 | Added the rebate for electricity VHEATERB               |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 05/05/05 | Added home-heating rebate VHEAT series of parameters    |
* | LP  | 04/02/17 | Added idiworkc to income tests for pharmacare, NSCTB and|
* |     |          | Direct Assistance, updated net income concept for cred  |
* | LP  | 03/12/22 | Added VMEDALL and VMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/22 | Updated idestat to reflect version 10 codes             |
* | LP  | 03/12/09 | Added impothpe, VODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added VCGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/12 | Added ideducpm and VEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | CH  | 03/05/16 | Taxpayer refund program implemented                     |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred.|
* | LP  | 01/06/08 | Modified low-income tax reduction income test           |
* | LP  | 01/05/09 | Added VAMTPCTM and added another option to VAMTOPT      |
* | BBM | 01/04/19 | Removed reference to hosspo/hosslf (not on 9.0 dbase)   |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/05/05 | Modified CGTC                                           |
* | LP  | 00/04/17 | Added NS direct assistance for families with kids flag  |
* | LP  | 00/04/14 | Added test to tax reduction before reducing imtxp       |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/04/20 | Added NS child benefit                                  |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | CH  | 98/05/22 | Modified Pharmacare Program                             |
* | CH  | 98/02/03 | added NS Direct Assistance iminsda                      |
* | SFG | 97/10/23 | Change overlay definitions to avoid breaking DOS limits |
* | BBM | 97/05/27 | Dropped idstksav -- Insufficient data in 1992           |
* | BBM | 95/06/21 | Add Nova Scotia Tax Reduction (Begins in 1994)          |
* | BBM | 95/05/19 | Add Nova Scotia Pharmacare program (April 1995)         |
* | GJC | 94/03/16 | add two-tiered surtax for 1994                          |
* | BBM | 92/07/22 | Incorporated non-refundable provincial tax credit var   |
* | BBM | 92/07/08 | Modeled provincial tax credits, drop reference to idptc |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txns     [global]  Compute provincial taxes for Nova Scotia
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxns[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(ROOT_TEXT,txns)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txns - Compute provincial taxes for Nova Scotia
*
*Synopsis:
*    void txns(           
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   minn     Find the minimum of two numbers
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   lkup1    Table look-up with interpolation/extrapolation
*
*Structure Members:
*   MP.CTCINC       (i)  Refundable CTC family income scaling factor
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.NSTXREFAMT   (i)  NS Taxpayer refund amount
*   MP.NSTXREFFLG   (i)  NS Taxpayer refund flag
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   MP.VAMTOPT      (i)  N.S. alternative minimum tax option
*   MP.VAMTPCTF     (i)  N.S. amt rate as pct of additional fed tax due to minimum tax
*   MP.VAMTPCTM     (i)  N.S. amt rate as pct of federal minimum tax amount
*   MP.VAMTTX       (i)  N.S. amt rate as tax on adjusted income
*   MP.VAXM         (i)  N.S. Age Amount
*   MP.VAXRR        (i)  N.S. Age Amount credit reduction rate
*   MP.VAXTD        (i)  N.S. Age Amount net income turndown
*   MP.VBXM         (i)  N.S. Basic Personal Exemption/amount
*   MP.VCBBAS1      (i)  NS child benefit for first child
*   MP.VCBBAS1P     (i)  NS child benefit for first child for 1st half of year
*   MP.VCBBAS2      (i)  NS child benefit for second child
*   MP.VCBBAS2P     (i)  NS child benefit for second child for 1st half of year
*   MP.VCBBAS3      (i)  NS child benefit for third or additional child
*   MP.VCBBAS3P     (i)  NS child benefit for third or additional child for 1st half of year
*   MP.VCBFLG       (i)  NS child benefit activation flag
*   MP.VCBPO        (i)  NS child benefit family income phase out
*   MP.VCBPOP       (i)  NS child benefit family income phase out for 1st half of year
*   MP.VCBTD        (i)  NS child benefit turndown
*   MP.VCBTDP       (i)  NS child benefit turndown for 1st half of year
*   MP.VCGTC        (i)  N.S. Caregiver Tax Credit
*   MP.VCGTCFLG     (i)  N.S. Caregiver Tax Credit activation flag
*   MP.VCGTCOPT     (i)  N.S. Caregiver Tax Credit Option (1=max, 2=greenbook, 3=model)
*   MP.VCGTCTD      (i)  N.S. Caregiver Tax Credit income turndown
*   MP.VCHATL1      (i)  N.S. Charitable Donations amount level 1
*   MP.VCHATR1      (i)  N.S. Charitable Donations tax credit rate 1
*   MP.VCHATR2      (i)  N.S. Charitable Donations tax credit rate 2
*   MP.VDACB        (i)  NS Direct assistance benefit for couples
*   MP.VDACI        (i)  NS Direct assistance maximum income for couples
*   MP.VDAFLAG      (i)  NS Direct assistance benefit flag
*   MP.VDAFWC       (i)  NS Direct assistance benefit for families with children flag
*   MP.VDASB        (i)  NS Direct assistance benefit for singles
*   MP.VDASI        (i)  NS Direct assistance maximum income for singles
*   MP.VDTCR        (i)  N.S. dividend tax credit rate
*   MP.VDTCRLC      (i)  N.S. dividend tax credit rate from large (eligible) corporations
*   MP.VEDXPM       (i)  N.S. Education Amount per month of full-time studies
*   MP.VEDXPMPT     (i)  N.S. Education Amount per month of part-time studies
*   MP.VEMXM        (i)  N.S. equivalent to married amount
*   MP.VEMXMT       (i)  N.S. equivalent to married turndown level
*   MP.VHEATERB     (i)  Nova Scotia home heating fuel rebate amount for electricity
*   MP.VHEATFLG     (i)  Nova Scotia Home Heating Assistance Rebate activation flag
*   MP.VHEATFTD     (i)  Nova Scotia home heating fuel rebate income turndown for families
*   MP.VHEATREB     (i)  Nova Scotia home heating fuel rebate amount for oil
*   MP.VHEATSTD     (i)  Nova Scotia home heating fuel rebate income turndown for singles
*   MP.VHEATTU      (i)  Nova Scotia home heating fuel rebate take up
*   MP.VLVCMAX      (i)  Maximum N.S. labour-sponsored funds tax credit allowed
*   MP.VLVCRT       (i)  Percent of N.S. labour-sponsored funds cost allowed as credit
*   MP.VMAXDX       (i)  N.S. Maximum Disability deduction/amount
*   MP.VMAXET       (i)  N.S. maximum on transfer of education and tuition amount
*   MP.VMEDALL      (i)  N.S. Medical allowance maximum lower limit
*   MP.VMEDANF      (i)  N.S. Medical allowance lower limit net income fraction
*   MP.VMXM         (i)  N.S. married amount
*   MP.VMXMT        (i)  N.S. married amount turndown level
*   MP.VODISTC      (i)  N.S. Maximum infirm dependent 18 or older tax credit
*   MP.VPHOPT       (i)  Nova Scotia pharmacare premium option (1=GIS, 2=income)
*   MP.VPHPREM      (i)  Nova Scotia pharmacare premium
*   MP.VPHRR        (i)  Nova Scotia pharmacare tax credit reduction rate
*   MP.VPHTC        (i)  Nova Scotia pharmacare refundable tax credit
*   MP.VPHTD        (i)  Nova Scotia pharmacare tax credit turndown
*   MP.VPHTDC       (i)  Nova Scotia pharmacare tax credit turndown for couples
*   MP.VPNTCR       (i)  N.S. provincial non-refundable tax credit rate
*   MP.VPTC         (i)  Nova Scotia political contribution table [total donations,donation allowed]
*   MP.VPTCBEN      (i)  Maximum Nova Scotia political tax credit allowed
*   MP.VPTCrows     (i)  number of rows in VPTC
*   MP.VPTF         (i)  Nova Scotia provincial tax fraction
*   MP.VPTX         (i)  N.S. tax table [taxable income,basic provincial tax]
*   MP.VPTXrows     (i)  number of rows in VPTX
*   MP.VSCI         (i)  Nova Scotia provincial tax above which surtax applies
*   MP.VSCI2        (i)  Nova Scotia provincial tax above which surtax applies (2nd level)
*   MP.VSF          (i)  Nova Scotia provincial surtax rate
*   MP.VSF2         (i)  Nova Scotia provincial surtax rate (2nd level)
*   MP.VTRBAS       (i)  Nova Scotia tax reduction basic amount
*   MP.VTREM        (i)  Nova Scotia tax reduction equivalent to spouse amount
*   MP.VTRKID       (i)  Nova Scotia tax reduction child amount
*   MP.VTRRR        (i)  Nova Scotia tax reduction family income reduction rate
*   MP.VTRSP        (i)  Nova Scotia tax reduction spouse amount
*   MP.VTRTD        (i)  Nova Scotia tax reduction family income turndown
*   MP.VTXFLG       (i)  N.S. tax on taxable income activation flag
*   MP.VYCTCAPM     (i)  N.S. Young Child Tax Credit Amount per Month
*   MP.VYCTCPI      (i)  N.S. Young Child Tax Credit Phase in
*   MP.VYPNDL       (i)  N.S. Pension Income Deduction Amount
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   ef.efin         (i)  First person in economic family [pointer]
*   ef.efnpers      (i)  Number of persons in economic family
*   fx.fxio         (i)  I/O expenditure categories [array]
*   hh.cf           (i)  Census family data [array]
*   hh.ef           (i)  Economic family data [array]
*   hh.fx           (i)  SHS data [struct]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnef        (i)  Number of economic families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.hhnnf        (i)  Number of nuclear families in household
*   hh.in           (i)  Individual data [array]
*   hh.nf           (i)  Nuclear family data [array]
*   id.idage        (i)  Age
*   id.idcf         (i)  Person's census family [pointer]
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.iddisab      (i)  Disability status (age 16+)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.idestat      (i)  Educational status
*   id.ididiv       (i)  Dividend income (actual)
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idnf         (i)  Person's nuclear family [pointer]
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idrand       (i)  Random numbers [array]
*   id.idsex        (i)  Sex
*   id.idspoflg     (i)  Person has spouse
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imamtfg      (i)  Minimum tax flag
*   im.imamtinc     (i)  Net adjusted income used for minimum tax
*   im.imamtprv     (i)  Federal minimum tax concept used for provincial minimum tax
*   im.imbft        (i)  Basic federal tax
*   im.imcemc       (i)  Child's equivalent to married credit
*   im.imchara      (i)  Allowable charitable donations and gifts (calculated)
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedfn      (i)  All deductions from net income
*   im.imdepni      (i)  Dependant's net income
*   im.imgisinc     (i)  Individual's income for GIS/SPA reduction
*   im.imicnet      (i)  Net Income (line 236)
*   im.imictot      (i)  Total Income (line 150)
*   im.imigis       (i)  GIS benefits
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.iminet       (i)  Net income
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imispa       (i)  Spouse's allowance
*   im.imitax       (i)  Taxable income
*   im.immartxc     (i)  Married tax credit claimed
*   im.imminamt     (i)  Minimum Amount due to Federal Minimum Tax
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   nf.nfinch       (i)  First child in nuclear family [pointer]
*   nf.nfineld      (i)  Eldest person in nuclear family [pointer]
*   nf.nfinspo      (i)  Spouse of eldest [pointer]
*   nf.nfnkids      (i)  Number of children in nuclear family
*   nf.nfnpers      (i)  Number of persons in nuclear family
*   nf.nfspoflg     (i)  Nuclear family contains married couple
*   id.idinspo      (io) Person's spouse [pointer]
*   im.imbpt        (io) Basic provincial tax
*   im.imnptc       (io) Non-refundable provincial tax credits
*   im.impatxc      (io) Prov Age amount tax credit
*   im.impbtc       (io) Prov Basic amount
*   im.impcgtc      (io) Prov Caregiver Tax Credit Level
*   im.impcgtcp     (io) Provincial Dependent Caregiver Tax Credit Amount
*   im.impchrtc     (io) Prov Charitable Donations tax credit
*   im.impcpptc     (io) Prov CPP/QPP contributions tax credit
*   im.impdistc     (io) Prov Disability tax credit
*   im.impdtxc      (io) Provincial dividend tax credit
*   im.impedtxc     (io) Prov Education tax credit
*   im.impeht       (io) Provincial elderly health tax
*   im.impehtc      (io) Provincial elderly health tax credit
*   im.impintsl     (io) Provincial interest on student loans tax credit
*   im.implvctc     (io) Provincial labour-sponsored funds tax credit
*   im.impmartc     (io) Prov Married and Equivalent to Married tax credit
*   im.impmeda      (io) Prov Medical expenses allowed (computed)
*   im.impmedtc     (io) Prov Medical Expenses tax credit
*   im.impminco     (io) Prov Minimum tax carryover
*   im.impothpe     (io) Provincial infirm dependents 18 or older tax credit
*   im.imppentc     (io) Prov Pension Income tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.imprvtax     (io) Prov gross provincial tax
*   im.impsur       (io) Provincial surtax
*   im.imptaxcr     (io) Prov total tax credits
*   im.imptr        (io) Provincial tax reduction
*   im.impttxct     (io) Prov total tax credits transferred
*   im.imptutxc     (io) Prov Tuition tax credit
*   im.impuictc     (io) Prov UI/EI contributions tax credit
*   im.impyctxc     (io) Prov Young Child tax credit
*   im.imtxp        (io) Provincial income tax payable
*   im.iminsda      (o)  NS Direct Assistance Program
*   im.imnscb       (o)  Nova Scotia Child Benefit
*   im.imnstxrf     (o)  Nova Scotia Taxpayer Refund
*   im.impatxcr     (o)  Prov total tax credits applied
*   im.impctxct     (o)  Prov tax credits transferred from children
*   im.impheatr     (o)  Provincial Home Heating Fuel/Energy Rebate
*   im.impstxct     (o)  Prov tax credits transferred from spouse
*   im.imptc        (o)  Refundable provincial tax credits
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void txns(
    P_hh hh
)
{
    register int ini;
    P_in in;
    register int nfi;
    register P_nf nf;
    register P_in ineld;  /* pointer to data of elder   */
    register P_in inspo;  /* pointer to data of younger */
    register P_cf cf;
    register int cfi;
	register P_fx fx;
	P_ef ef;
			  
	register int efi;

	fx = &hh->fx;

    NUMBER poltcr = ZERO;  /* Political tax credit (raw) */
    NUMBER poltc = ZERO;   /* Political tax credit */
    NUMBER ptc=ZERO;       /* Provincial Tax Credits */
    NUMBER faminc=ZERO;    /* Census family net income */
    NUMBER hc=ZERO;        /* HOSP Family Credit */
    NUMBER tcr=ZERO;       /* HOSP Tax Credit Rate from Table */
	NUMBER redinc=ZERO;    /* Income used for tax reduction */
    NUMBER totred=ZERO;    /* Total tax reduction amount */
    NUMBER ptr=ZERO;       /* Total tax reductions allowed */
    NUMBER lvctc = ZERO;   /* Labour-sponsored venture capital tax cred */
	NUMBER nscrr = ZERO;   /* NS child benefit reduction rate */
	NUMBER nscrr1 = ZERO;   /* NS child benefit reduction rate 1st half of year */
	NUMBER nscrr2 = ZERO;   /* NS child benefit reduction rate 2nd half of year */
	int gotheat;           /* temporary flag - 1 family/hhld get heat rebate */
	int yngkids;
	int oldkids;
	NUMBER efimcgtcp = ZERO;  /* combined cgtc potential amounts for ef */   

    DEBUG_ON("txns");

    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.VTXFLG) { 
       /* Calculate the Caregiver tax credit */
       if (MP.VCGTCFLG  && MP.VCGTCOPT == 3) {
       
          int cgassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimcgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 65 and either a head or spouse or other family member who could be a parent (with takeup) */
                if (in->id.idage >=65 || (in->id.iddisab == 1 && in->id.idage>= 18 ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.impcgtcp = MP.VPNTCR * minn(nneg(MP.VCGTCTD -in->im.iminet),MP.VCGTC);
                }
			   efimcgtcp += in->im.impcgtcp;
             }

			 cgassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (cgassign == 0 && in->id.idcgtc > 0 ) {
				   	if (in->id.idspoflg && inspo->im.impcgtcp > 0) {
				   		in->im.impcgtc = nneg(efimcgtcp - inspo->im.impcgtcp - in->im.impcgtcp );
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.impcgtc = nneg(efimcgtcp - in->im.impcgtcp);
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (cgassign == 0 && efimcgtcp > 0 && in->id.idcgtc > 0 && in->im.impcgtc == 0 ) {
		 			in->im.impcgtc = efimcgtcp;
		 			cgassign = 1;
		 	 	}
		    }
		  } 	 /* end of ef hh loop */
	   }  /* end of cgtc loop */

       /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;


          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */

		  yngkids = 0;
		  oldkids = 0;
		  	
          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni;
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.VPNTCR * nneg(MP.VEMXM - nneg(cinet - MP.VEMXMT));
             }
			 /* count the kids for the young child tax credit check */
			 if (in->id.idage <= 5) {
				yngkids += 1;
			 }
			 else {  /* must be 6 to 18 */
				oldkids += 1;
			 }
          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

		  /* New for 2006, young child tax credit.  The parent with the lower
		     net income can claim an amount per month per child under 6 as a tax credit.
		     Begins in July 2006.  The young child tax credit cannot be claimed for a 
		     child used as the equivalent to married amount.  Parameters are set to match tax form, which grants
		     an amount per month.  Hard coding the number of months per year as 12 and
		     adjusting for 2006 with a phase in rate of 0.5. */	

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption 
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.VPNTCR * nneg(MP.VMXM - nneg(inspo->im.imdepni - MP.VMXMT));
				/* elder has the higher net income, spouse gets young child credit */
				inspo->im.impyctxc = MP.VYCTCAPM * 12.0 * MP.VYCTCPI * yngkids * MP.VPNTCR;
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.VPNTCR * nneg(MP.VMXM - nneg(ineld->im.imdepni - MP.VMXMT));
				/* spouse has the higher net income, elder get young child credit */
				ineld->im.impyctxc = MP.VYCTCAPM * 12.0 * MP.VYCTCPI * yngkids * MP.VPNTCR;

             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
			 /* now test to see if there are young kids left for the young child credit */
			 if (cemc > 0 && yngkids > 0 ) {
				/* may still be eligible, check for older kids */
				if ( oldkids > 0 ) {
					ineld->im.impyctxc = MP.VYCTCAPM * 12.0 * MP.VYCTCPI * yngkids * MP.VPNTCR;
				}
				else {
					ineld->im.impyctxc = MP.VYCTCAPM * 12.0 * MP.VYCTCPI * nneg(yngkids-1) * MP.VPNTCR;
				}
			}
				
		  }
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/

			 /* calculate caregivers tax credit */
			 if (MP.VCGTCFLG && isnzero(in->id.idcgtc) && MP.VCGTCOPT < 3) {
				/* if VGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.VCGTCOPT == 1) {
					in->im.impcgtc = MP.VCGTC * MP.VPNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.VPNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }

             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.VPNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.VPNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.VPNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.VPNTCR * MP.VBXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc =  in->im.imicnet;

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.VAXM, MP.VAXRR * nneg(netinc - MP.VAXTD));
                adjamt = MP.VAXM - agered;
                in->im.impatxc = MP.VPNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.VPNTCR * minn(MP.VYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.VMEDALL, MP.VMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.VPNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

		     /* calculate amount for infirm dependents age 18 or older */
			 /*  if the filer has an amount for idothpe, then give max credit */

			 if (isnzero(in->id.idothpe)) {
   				in->im.impothpe = MP.VPNTCR * MP.VODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
			 }
		     
             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.VCHATL1, in->im.imchara)  * MP.VCHATR1
                            + nneg(in->im.imchara - MP.VCHATL1) * MP.VCHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.VPNTCR * MP.VMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.VPNTCR * MP.VMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.VPNTCR * (in->id.ideducfm * MP.VEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.VPNTCR * (in->id.ideducpm * MP.VEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.VPNTCR * (in->id.ideducpm * MP.VEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }

			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.VPNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }

             /* calculate federal dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.VDTCR)
								+ (in->im.imildivt * MP.VDTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }


			 /* calculate the Nova Scotia minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.VAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use VAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.VAMTPCTF;
				 }
				 else if (MP.VAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using VAMTTX. */
						in->im.impminco = in->id.idmincar * MP.VAMTTX;
					  }
				 else if (MP.VAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using VAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.VAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
            
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */
			 if (in->im.imamtfg && MP.VAMTOPT == 3) {
                in->im.imprvtax = MP.VAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.VPTX, MP.VPTXrows, in->im.imitax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
                        + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
						+ in->im.impminco    /* provincial minimum tax carryover credit */
						+ in->im.impintsl   /* provincial interest on student loans credit */
 	                    + in->im.impyctxc   /* provincial young child tax credit */                           
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.VPNTCR * MP.VMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.VPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.VPNTCR * in->im.imitax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                       + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco   /* provincial minimum tax carryover credit */
					   + in->im.impintsl   /* provincial interest on student loans credit */
					   + in->im.impyctxc   /* provincial young child tax credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
                    + in->im.imppentc   /* pension income tax credit 			  */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.VPNTCR * MP.VMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.VPNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.VBXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic));
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
        DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
            ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

        ptr=ZERO;
        totred=ZERO;

        /* COMPUTE BASIC PROVINCIAL TAX AND SURTAX */

        /* original tax as percentage of federal tax */
        if (!MP.VTXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.VPTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if VTXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.VAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.VAMTPCTF;
			}
			else {
				if ( in->im.imamtfg && MP.VAMTOPT == 4 ) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.VAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}


        /* compute and add in surtax */
        /* two-teir surtax only in effect for 1994 */

        in->im.impsur = MP.VSF * nneg(in->im.imtxp - MP.VSCI)
                          + MP.VSF2 * nneg(in->im.imtxp - MP.VSCI2);

        /* apply surtax */
        in->im.imtxp += in->im.impsur;

        DEBUG4("%s ns bpt=%.2f, txp=%.2f neti=%.2f\n",
                 in->im.imbpt, in->im.imtxp, in->im.imicnet);

        /* COMPUTE NOVA SCOTIA TAX REDUCTION (Reduction starts in 1994) 

        /**
        * Eligible persons are over age 18, or have a spouse, or have kids
        * We will give the tax reduction to the higher net income in a head/spouse family
        **/
        if (in->id.idage>18 || in->id.idspoflg || (in->id.idcfrh==0 && in->id.idnf->nfnkids))  {
            
            /* --- Assign the reduction amounts --- */
            /* Eligible individuals without a spouse get the basic reduction amount */
            if (!in->id.idspoflg) totred = MP.VTRBAS;
            
            /* Basic and spousal reduction amounts are assigned forthe head */
            if (in->id.idspoflg && in->id.idcfrh==0) {
                totred = MP.VTRBAS + MP.VTRSP;
            }

            /**
            * Equivalent to Married given to eldest in Single parent CF
            * (i.e. recipient of equivalent to married exemption) 
            **/
            if (in->id.idcfrh==0 && in->id.idcf->cfspoflg==0 && in->im.immartxc>0)  {
                totred += MP.VTREM; 
            }

            /**
            * Child dependant reductions are temporarily assigned to the head
            * Don't give the child deduction for children who triggered the equivalent to 
            * married exemption
            **/
            if (in->id.idcfrh==0 && in->id.idnf->nfnkids) {
                if (in->id.idcf->cfspoflg==0) {
                    totred +=  (in->id.idnf->nfnkids - in->im.immartxc>0) * MP.VTRKID;
                }
                else {
                    totred += in->id.idnf->nfnkids * MP.VTRKID;
                }
            }

            /* --- Calculate the allowed tax reduction --- */
            if (in->id.idspoflg)  {
				/* Calculate the net income concept used for the income test */
				redinc = nneg(in->im.imicnet + in->id.idinspo->im.imicnet  
							- in->im.imuccb - in->id.idinspo->im.imuccb);
                ptr = nneg(totred - (nneg(redinc - MP.VTRTD) * MP.VTRRR));
                if (in->im.imicnet >= in->id.idinspo->im.imicnet)  {
                    in->im.imptr = ptr;
                }
                else if (in->im.imicnet < in->id.idinspo->im.imicnet)  {
                    in->id.idinspo->im.imptr = ptr;
                }
            }
            else {
				redinc = nneg(in->im.imicnet - in->im.imuccb );
			    ptr = nneg(totred - (nneg(redinc - MP.VTRTD) * MP.VTRRR));
                in->im.imptr = ptr;
            }
        
            /* apply reduction individually */
            in->im.imptr = minn(in->im.imtxp,in->im.imptr);
            in->im.imtxp -= in->im.imptr;

        }


      
        /* COMPUTE PHARMACARE PROGRAM TAX AND REFUNDABLE TAX CREDIT (1995 Budget) */
        if (in->id.idage>=65)  {
    
            /* Calculate and apply pharmacare premium */
            in->im.impeht = MP.VPHPREM;
            in->im.imtxp += in->im.impeht;

            /* Calculate and apply pharmacare premium refundable tax credit */
			/* The refundable tax credit was eliminated in 1999 */
            /* Pharmacare depends on gis receipt */
            if (MP.VPHOPT==1 && in->im.imigis>0)  {
                in->im.impehtc = nneg(MP.VPHTC - nneg((in->im.imgisinc - MP.VPHTD) * MP.VPHRR));
                in->im.imptc += in->im.impehtc;
            }
            /* Pharmacare depends on family income */
            else if (MP.VPHOPT == 2) {
                faminc = in->im.imictot ;
                if (in->id.idspoflg) {
                    faminc += in->id.idinspo->im.imictot ;
                    in->im.impehtc = nneg(MP.VPHTC - nneg((faminc - MP.VPHTDC) * (MP.VPHRR/2)));
                    in->im.imptc += in->im.impehtc;
                }
                else {
                    in->im.impehtc = nneg(MP.VPHTC - nneg((faminc - MP.VPHTD) * (MP.VPHRR/2)));
                    in->im.imptc += in->im.impehtc;
                }
            }
        }
   
   
        /* COMPUTE TAX CREDITS */

        /* calculate and apply provincial foreign tax credit */
        in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
        in->im.imtxp -= in->im.imnptc;
        DEBUG5("%s ns: imbpt=%.2f impsur=%.2f imnptc=%.2f imtxp=%.2f\n",
              in->im.imbpt, in->im.impsur, in->im.imnptc, in->im.imtxp);


        /* Non-refundable Nova Scotia Political Contribution Tax Credit */
        if (isnzero(in->id.idprvpol)) {
            poltcr = (NUMBER) lkup1(MP.VPTC, MP.VPTCrows, in->id.idprvpol);
            poltc = minn(MP.VPTCBEN, poltcr);
            in->im.impptc = minn(poltc, in->im.imtxp);
            DEBUG4("%s ns pol: idpolcon=%.2f poltcr=%.2f poltc=%.2f\n",
               in->id.idprvpol, poltcr, poltc);

            /* apply political contribution tax credit */
            in->im.imnptc += in->im.impptc;
            in->im.imtxp  -= in->im.impptc;

            DEBUG3("%s ns pol: idpolcon =%.2f impptc =%.2f\n",
                    in->id.idprvpol, in->im.impptc);
        }

        /* NS labour-sponsored venture capital credit */
        if ( isnzero(in->id.idlabtxg) ) {
           lvctc = minn(MP.VLVCMAX, MP.VLVCRT * in->id.idlabtxg);
           in->im.implvctc = minn(lvctc, in->im.imtxp);
           in->im.imnptc += in->im.implvctc;
           in->im.imtxp  -= in->im.implvctc;
        }

        /**
        * Refundable Nova Scotia Home ownership saving plan tax credit
        * Based on amounts deposited into a Nova Scotia home ownership
        * savings plan (hosslf/hosspo).  In married/common-law couples, 
        * only one spouse can make a Federal claim and so we arbitrarily 
        * calculate it for the head.
        *
        * As of 1994 the rates differ for Individuals and couples.
        **/
		/*********************************************************************/
		/*  This following code commented out for version 9.0 as no greenbook
		/*  imputation of hosslf and hosspo was possible due to small sample size 
		**********************************************************************/
        /* if ( isnzero(in->id.idhosslf) || isnzero(in->id.idhosspo) ) { 
        if (in->id.idcfrh==0) { 
            DEBUG1("%s ns hos: always given to head of census family\n");
            faminc = in->im.iminet;
            hc = minn(in->id.idhosslf, MP.VHOSPMAX);

            /* Married or Common-law 
            if (in->id.idspoflg) {
                    DEBUG1("%s ns hos: spouse present\n");
                faminc += in->id.idinspo->im.iminet;
                faminc *= MP.VHOSPRM;
                hc += minn(in->id.idhosspo, MP.VHOSPMAX);
                    tcr = (NUMBER) lkup2(MP.VHOTM, MP.VHOTMrows, faminc);
            }
                /* Single parent 
            else if (isnzero(in->im.immartxc)) {
                    DEBUG1("ns hos: equivalent to married credit claimed\n");
                faminc *= MP.VHOSPR;
                    tcr = (NUMBER) lkup2(MP.VHOT, MP.VHOTrows, faminc);
            }
                /* Other individuals 
            else {
                    tcr = (NUMBER) lkup2(MP.VHOT, MP.VHOTrows, faminc);
            }
                DEBUG5("%s ns hos: hc=%.2f tcr=%.4f imphotc=%.2f faminc=%.2f\n",
                hc, tcr, in->im.imphotc, faminc);
         
            in->im.imphotc = tcr * hc;
            in->im.imptc += in->im.imphotc;
        }
        /* } 
		**********************************************************************/

        DEBUG4("%s NS summary: bpt=%.2f, txp=%.2f, ptc=%.2f\n",
            in->im.imbpt, in->im.imtxp, in->im.imptc);
    
        /* NS Taxpayer refund */

        /* The taxpayer refund is payable in 2003 if the person paid at
           least 1 dollar worth of NS taxes in 2001 or 2002.  It is payable 
           in 2004 if the person wasn't elible via the the 2001 and 2002 method, but 
           paid taxes in 2003. 

           For the SPSM implementation, the benefit is assigned in 2003 if there was 
           a non-zero taxes in 2003.

        */

        if (MP.NSTXREFFLG) {
            if (in->im.imtxp>0) {
                in->im.imnstxrf=MP.NSTXREFAMT;
            }
        }

        


    }

    /* COMPUTE NS Direct assistance if flag is on 
        OR if NS child benefit flag is on 
        OR if NS home heating fuel rebate flag is on */
    if (MP.VDAFLAG || MP.VCBFLG || MP.VHEATFLG) {

		gotheat = 0;

        /* process each nuclear family in household */
        for (nfi=0, nf=&hh->nf[0];  nfi<hh->hhnnf;  nfi++, nf++) {
            NUMBER dainc=ZERO;     /* Income for direct assistance */
            NUMBER nsda=ZERO;      /* Direct assistance benefit */
            NUMBER nfinet=ZERO;    /* Net income for Child Benefit */
            NUMBER nsctb=ZERO;     /* Amount of Child Benefit */
            NUMBER nsctb1=ZERO;    /* Amount of Child Benefit 1st half of year */
            NUMBER nsctb2=ZERO;    /* Amount of Child Benefit 2nd half of year */
			NUMBER hhreb=ZERO;     /* Amount of home heating fuel rebate */
			NUMBER nsreb=ZERO;     /* temporary value of N.S. home heating rebate */
			int inctest;           /* temporary flag for income testing for fuel rebate */

            /* get elder's pointer */
            ineld = nf->nfineld;
            if (nf->nfspoflg) {
                /* get spouse's pointer */
                inspo = nf->nfinspo;
            }

            if (MP.VDAFLAG) {
            
                /* derive income */

                dainc = MP.PYINC * ineld->im.imicnet ;

                if (nf->nfspoflg) {
                    /* derive income */
                    dainc += MP.PYINC * inspo->im.imicnet ;
                   /* couple qualifies if income is under VDACI, if neither spouse receives
                      social assistance, both were in school less than 4 months, and spouse is 19 also */
				   /* Version 10, assuming part time to be less than 4 months (idestat=2) for this purpose */
                   if (dainc<MP.VDACI && ineld->im.imisa==0 && inspo->im.imisa==0
                   && (ineld->id.idestat==0 || ineld->id.idestat == 2) 
                   && (inspo->id.idestat==0 || inspo->id.idestat == 2)
                   && ineld->id.idage>=19 && inspo->id.idage>=19) {
                      nsda = MP.VDACB;
                   }
                }
                /* if single and no equivalent to married tax credit */
                else if (ineld->im.immartxc==0 && ineld->id.idage>=19) {

                   /* eligible if net income less than VDACI,
                        not in school for at least 4 months, 
                        and who received sa for 3 months or less */
                   if (nneg(ineld->im.imicnet)<MP.VDASI &&
                       (ineld->id.idestat==0 || ineld->id.idestat ==2) 
                       && ineld->im.imisa==0) {
                      if (nf->nfnkids>0) {
                        nsda = MP.VDACB;
                      }
                      else {
                        nsda = MP.VDASB;
                      }
                   }
                }
                /* if single, but with equivalent to married tax credit, need to
                   add child's income.  But first check that other qualifications hold */
                else if (ineld->id.idage>=19 && (ineld->id.idestat==0 || ineld->id.idestat ==2)
                         && ineld->im.imisa==0) {

                    /* process kids in nuclear family */
                    for (ini=0, in=nf->nfinch;  ini<nf->nfnkids; ini++, in++) {
                        if (in->im.imcemc>0) {
                           dainc += MP.PYINC * in->im.imicnet;
                           /* note that equivalent to married is only applicable to children under
                              the age of 19, so no gis or spa */
                        }
                    }

                    if (dainc < MP.VDASI) {
                        nsda = MP.VDACB;
                    }
                }

				/*  The Nova Scotia 2000 Budget announced changes to the Direct Assistance
				    Progam to be exclusively for families with children.  
					If this flag (VDAFWC) is on, then only those families with children
					will be given the benefit. */

				if (MP.VDAFWC == 1) {
				   if (nf->nfnkids < 1) {
				      nsda = 0;
				   }
				}

                /**
                * In the absence of any other information,
                * assign to spouse with lower net income.
                **/
            
                if (nf->nfspoflg && (inspo->im.imicnet < ineld->im.imicnet)) {
                    inspo->im.iminsda = nsda;
                }
                else {
                    ineld->im.iminsda = nsda;
                }
                
            }
			/* Nova Scotia Home Heating Fuel Rebate programs 
			   For the years 2001 and 2003 it was called the Low Income Fuel Assitance Program.
			   For 2005, enhanced and renamed to Keep the Heat Program.
			   Eligible applicants - heat home with oil or propane.
			   Must apply for program, income tests are based on previous year's net income.
			   Seniors on GIS or SPA are eligible with out income testing.
			   SA recipients eligible without income testing .
			   For 2006, widened to include electricity users with a reduced rebate.
			*/
			
			if (MP.VHEATFLG) {
				
			    inctest = 1;
			        
                /* Compute net income */
                nfinet = MP.PYINC * ineld->im.imicnet;

			    if (ineld->im.imisa > 0 || ineld->im.imigis > 0 || ineld->im.imispa > 0) {
						inctest = 0;
					}

             	if (nf->nfspoflg) {
						 
                	 nfinet += MP.PYINC * inspo->im.imicnet ;
               		 if (inspo->im.imisa > 0 || inspo->im.imigis > 0 || inspo->im.imigis > 0) {
               		 	inctest = 0;
               		 }     
               	}

				/* Determine if nuclear family qualifies for a rebate and which one */	
					
				nsreb = ZERO;

				if (fx->fxio[14] > MP.VHEATREB) {
					/* give the larger rebate if eligible - heating oil */
					nsreb = MP.VHEATREB;
				}
				else { 
					if (fx->fxio[12] > MP.VHEATERB && MP.VHEATERB > 0) {
						nsreb = MP.VHEATERB;
					}
				}

				if (nf->nfnpers > 1) { /* treat as a family for income testing */
					if ( nfinet < MP.VHEATFTD || inctest == 0 ) {
						hhreb = nsreb;
					}	
				}

				else {  /* treat as single, no phase out of benefits */
					if ( nfinet < MP.VHEATSTD || inctest == 0 ) {
						hhreb = nsreb;
					}
				}
				/* apply a take up rate */
				if (hhreb > 0 && ineld->id.idrand[6] >= MP.VHEATTU) {
				    hhreb = ZERO;
				}
                    
                if (hhreb > 0 && gotheat == 0 ) {
					ineld->im.impheatr = hhreb;
					gotheat = 1;
				}
		  	}
  
            /* If flag is on, calculate NS Child Benefit */
            if (MP.VCBFLG) {

                 /* if there are no kids, then skip */
                 if (nf->nfnkids == 0) {
                       DEBUG1("%s cben: skipping nuclear family with no `children'\n");
                       continue;
                 }

                 /* Compute net income */
                 nfinet = MP.PYINC * nneg( ineld->im.imicnet - ineld->im.imuccb);

                 if (nf->nfspoflg) {
        		     nfinet += MP.PYINC * nneg(inspo->im.imicnet - inspo->im.imuccb);
                 }

    	         /* Scale income with user specified rates */
                 nfinet = nfinet * MP.CTCINC;

                 /* if net income is lower than turndown, parents get 100% of benefit.
                    otherwise, they get nothing in 1998 */
				 /* program was changed in 1999 to give partial benefits based on income */

                 if (nf->nfnkids > 0) {
                     nsctb2 =  ((MP.VCBBAS1) + (MP.VCBBAS2 * minn(nneg(nf->nfnkids-1),1)) +
                               (MP.VCBBAS3 * nneg(nf->nfnkids-2))) ;
                 	nscrr2 = nsctb2 / (MP.VCBPO - MP.VCBTD);
					nsctb2 = nneg(nsctb2 - nneg((nfinet-MP.VCBTD)*nscrr2));
                 }

				 /* Calculate for the calendar year if FCBENCAL is activated */

				 if (MP.FCBENCAL) {

					  /* Adjust net income to represent the value for 2 years previous */
					  nfinet = ( nfinet / MP.PYINC )  * MP.PYINCP;  

	                 if (nf->nfnkids > 0) {
	                     nsctb1 =  ((MP.VCBBAS1P) + (MP.VCBBAS2P * minn(nneg(nf->nfnkids-1),1)) +
	                               (MP.VCBBAS3P * nneg(nf->nfnkids-2))) ;
	                 	nscrr1 = nsctb1 / (MP.VCBPOP - MP.VCBTDP);
						nsctb1 = nneg(nsctb1 - nneg((nfinet-MP.VCBTDP)*nscrr1));
	                 }

					 nsctb = (nsctb1/2) + (nsctb2/2);

				 }
				 else {  /* use the july - june amounts for the year */
					nsctb = nsctb2;
				 }
				 
                 /* assign child benefits to mother if present */

                 if (nf->nfspoflg && (inspo->id.idsex == FEMALE)) {
                    DEBUG1("%s cben: spouse is the mother, gets CTC\n");
                    inspo->im.imnscb  = nsctb ;

                 }
                 else {
                     DEBUG1("%s cben: elder receives CTC\n");
                     ineld->im.imnscb  = nsctb ;
                 }
            }

        }	 /* end of nuclear family loop */
    }	  /* end of child benefit, direct assistance and heating rebate loop */

    DEBUG_OFF("txns");
}
