/***
*txont.cpp - Compute provincial taxes for Ontario
*
*$Id: txont.cpp 6.152 2008/08/22 17:07:45 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/07/21 | Replaced idisefm and idisenf by idise                   |
* | LP  | 08/06/11 | Added OCGTCTD and a model option to OCGTCOPT and code   |
* | LP  | 08/06/09 | Added imotxtgr, OTXTGRANT Ontario Textbook Grant        |
* | LP  | 08/06/09 | Added imondc - kids 18 and younger, added to programs   |
* | LP  | 08/06/03 | Added Senior Homeowner Property Tax Grant - OSHPTGFLG+  |
* | LP  | 08/06/03 | Removed caregivers (impcgtc) from transfer of credits   |
* | LP  | 08/05/27 | Added OCWISMIN Child Care Supplement minimum payment    |
* | LP  | 07/06/04 | Added the Ontario Child Benefit and OCCS phase out      |
* | LP  | 07/04/19 | Changed spouse cred transfer test from imicnet to imbpt |
* | LP  | 07/03/01 | Fixed income test for 91 for prop/sales tax credit      |
* | LP  | 07/02/15 | Added OHPFLAG - Ont. Health Premium activation flag     |
* | LP  | 07/02/11 | Prov. pension credit now based on imipnst, not idipens  |
* | LP  | 07/01/23 | Removed imninc - no income flag - no longer used        |
* | LP  | 07/01/19 | Removed OUCCBITF-UCCB removed from income as per forms  |
* | LP  | 06/12/13 | Switched iminet to imicnet, imitot to imictot (tax form)|
* | LP  | 06/07/04 | Added Ont. Home Electricity Relief Program - OHEATFLG+  |
* | LP  | 06/06/09 | Removed OAXPI - no need for a phase-in for prov. age tc |
* | LP  | 06/06/07 | Added OUCCBITF - Flag to remove UCCB from income tests  |
* | LP  | 06/06/05 | Revised the application of PYINCP                       |
* | LP  | 06/05/30 | Added Ont Child Care expense tax credit for calendar yr |
* | LP  | 06/05/15 | Added large corporation dividend tax credit rate ODTCRLC|
* | LP  | 05/12/28 | Removed ORDOPT & OMTY - pre 1991 parms for Ont tax red  |
* | LP  | 05/11/03 | Revised p.t. educ. credit to allow disab students ft amt|
* | LP  | 05/05/26 | Added OSTCNIBC - income turndown for senior couples     |
* | LP  | 04/06/04 | Added Ontario Health Premium - OHPBASE and OHPPI        |
* | LP  | 04/06/02 | Added OFTCOPT - foreign tax credit application option   |
* | LP  | 04/06/02 | Added OSPSMAX - Ont. prop/sales tax credit senior max   |
* |     |          | and created imopstc for Ont. prop/sales tax credit amt  |
* | LP  | 04/02/17 | Added idiworkc to income tests for Ontario property and |
* |     |          | sales tax credit and child benefit programs             |
* | LP  | 03/12/22 | Added OMEDALL and OMEDANF prov. non-ref medical exp cred|
* | LP  | 03/12/09 | Added part time studies to Ont Child Care supplement    |
* | LP  | 03/12/09 | Added impothpe, OODISTC infirm dep. (18+) tax credit    |
* | LP  | 03/11/17 | Added OCGTCOPT and revised caregiver credit calculation |
* | LP  | 03/11/17 | Added impintsl (provincial student loan interest credit)|
* |     |          | using the new database variable idintstu                |
* | LP  | 03/11/12 | Added ideducpm and OEDXPMPT - part-time months of educ. |
* | CH  | 03/10/17 | Deleted iddisab, replaced ideducm by ideducfm           |
* | LP  | 02/06/21 | Added OCCSEXT one time $100 payment to OCCS families    |
* | LP  | 01/07/18 | Renamed impmincar to impminco                           |
* | LP  | 01/06/12 | Added OPTCFLAG- political cont tx cred. refundable flag |
* | LP  | 01/06/12 | Added impmincar - provincial minimum tax carryover cred |
* | LP  | 01/05/09 | Added OAMTPCTM and added another option to OAMTOPT      |
* | BBM | 01/04/19 | Removed calculation of home ownership tax credit(nodata)|
* | LP  | 01/03/12 | Removed pre-1988 programs (exemptions, etc.)            |
* | CH  | 00/10/24 | Added CPP deduction for the self-employed               |
* | CH  | 00/05/12 | Added taxpayer dividend                                 |
* | LP  | 00/05/09 | Added OCGIRFLG, OCAPGIR Ont Capital Gains Inclusion Rate|
* | CH  | 00/05/05 | Modified CGTC                                           |
* | LP  | 00/05/03 | Added OCWISSP max annual benefit for single parents     |
* | CH  | 00/03/01 | Added provincial tax on taxable income                  |
* | CH  | 99/07/30 | Modifed WIS so that only students can claim CCEA        |
* | CH  | 99/03/11 | Added labour-sponsored venture capital program          |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | BBM | 98/05/13 | Added Ontario Child Care Supplement                     |
* | CCH | 98/03/23 | recoded child care exp. credit                          |
* | CCH | 98/02/25 | replaced idhomstu with OPCRA in code                    |
* | CCH | 98/02/19 | added ONCCEAPCT to child care exp. credit               |
* | BBM | 97/10/15 | added Refundable child care tax credit                  |
* | CCH | 97/08/12 | added Employer Health Tax                               |
* | GJC | 93/06/24 | fixed bug corresponding with netting tax of pol credit  |
* | GJC | 93/01/19 | fixed surtax bug, implelemented senior tax credits      |
* | SFG | 92/09/11 | moved inspo initialisation to beginning                 |
* | GJC | 92/08/19 | changed the calc. of the political contrib tax credit   |
* | BBM | 92/07/22 | Modeled provincial tax credits, drop reference to idptc |
* | SFG | 92/07/06 | Created                                                 |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   txont    [global]  Compute provincial taxes for Ontario
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "vsu.h"    /* Household variable sub-structure (user's)            */
#include "vs.h"     /* Household variable structure and sub-structures      */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== GLOBAL VARIABLE DEFINITIONS ======================= */

/* Give global string describing version of this module */

/*global*/ char FAR Stxont[] = "Untitled"
#ifdef _MSC_VER
" [" __TIMESTAMP__ "]"
#endif
;


/* ==================== COMPILER PRAGMAS ================================== */

#ifdef MSCOVL
#pragma alloc_text(MODEL_TEXT,txont)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*txont - Compute provincial taxes for Ontario
*
*Synopsis:
*    void txont(          
*    P_hh hh              
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   minn     Find the minimum of two numbers
*   nneg     Change negative numbers to zero
*   isnzero  Is argument non-zero ?
*   lkup1    Table look-up with interpolation/extrapolation
*   txhhexp  Compute and pro-rate household taxes, rent, etc.
*   maxn     Find the maximum of two numbers
*   taxbak1  One level tax-back function
*   lkup2    Table look-up without interpolation
*
*Structure Members:
*   MP.CAPGIR       (i)  Capital gains inclusion rate
*   MP.EDXPMPT      (i)  Education amount per month of part-time studies
*   MP.FCBENCAL     (i)  Calculate child benefits for calendar year flag
*   MP.OAMTOPT      (i)  Ont. alternative minimum tax option
*   MP.OAMTPCTF     (i)  Ont. amt rate as pct of additional fed tax due to minimum tax
*   MP.OAMTPCTM     (i)  Ont. amt rate as pct of federal minimum tax amount
*   MP.OAMTTX       (i)  Ont. amt rate as tax on adjusted income
*   MP.OAXM         (i)  Ont. Age Amount
*   MP.OAXRR        (i)  Ont. Age Amount credit reduction rate
*   MP.OAXTD        (i)  Ont. Age Amount net income turndown
*   MP.OBXM         (i)  Ont. Basic Personal Exemption/amount
*   MP.OCAPGIR      (i)  Ont. Capital Gains Inclusion Rate
*   MP.OCBAMT       (i)  Ontario Child Benefit amount per child
*   MP.OCBAMTP      (i)  Ontario Child Benefit amount per child for 1st half of year
*   MP.OCBFLAG      (i)  Ontario Child Benefit activation flag
*   MP.OCBMIN       (i)  Ontario Child Benefit minimum benefit
*   MP.OCBPIP       (i)  Ontario Child Benefit flag for 1st half of year
*   MP.OCBRR        (i)  Ontario Child Benefit reduction rate
*   MP.OCBRRP       (i)  Ontario Child Benefit reduction rate for 1st half of year
*   MP.OCBTD        (i)  Ontario Child Benefit family income turndown
*   MP.OCBTDP       (i)  Ontario Child Benefit family income turndown for 1st half of year
*   MP.OCCEAFLG     (i)  Ont. Child Care Expense Credit activation flag
*   MP.OCCEAPCT     (i)  Ont. Child Care Expense Credit qualifying percentage
*   MP.OCCEAPCTP    (i)  Ont. Child Care Expense Credit qualifying percentage for 1st half of year
*   MP.OCCEARR      (i)  Ont. Child Care Expense Credit benefit reduction rate
*   MP.OCCEARRP     (i)  Ont. Child Care Expense Credit benefit reduction rate for 1st half of year
*   MP.OCCEATD      (i)  Ont. Child Care Expense Credit family income turndown
*   MP.OCCEATDP     (i)  Ont. Child Care Expense Credit family income turndown for 1st half of year
*   MP.OCCEAYNG     (i)  Ont. Child Care Expense Credit allowance per child
*   MP.OCCEAYNGP    (i)  Ont. Child Care Expense Credit allowance per child for 1st half of year
*   MP.OCCSEXT      (i)  Ont. Child Care Supplement One-time payment of extra benefit
*   MP.OCCSOCB      (i)  Ont. Child Care Supplement reduced by Ontario Child Benefit flag
*   MP.OCCSPOPYR    (i)  Ont. Child Care Supplement phase out year proportion born pre July
*   MP.OCCSPOYR     (i)  Ont. Child Care Working Income Supplement phase out year
*   MP.OCGIRFLG     (i)  Ont. Capital Gains Inclusion Rate Activation Flag
*   MP.OCGTC        (i)  Ont. Caregiver Tax Credit
*   MP.OCGTCFLG     (i)  Ont. Caregiver Tax Credit activation flag
*   MP.OCGTCOPT     (i)  Ont. Caregiver Tax Credit Option (1=max, 2=greenbook, 3=model)
*   MP.OCGTCTD      (i)  Ont. Caregiver Tax Credit income turndown
*   MP.OCHATL1      (i)  Ont. Charitable Donations amount level 1
*   MP.OCHATR1      (i)  Ont. Charitable Donations tax credit rate 1
*   MP.OCHATR2      (i)  Ont. Charitable Donations tax credit rate 2
*   MP.OCWISFLG     (i)  Ont. Child Care Working Income Supplement activation flag
*   MP.OCWISFLGP    (i)  Ont. Child Care Working Income Supplement activation flag for 1st half of year
*   MP.OCWISMIN     (i)  Ont. Child Care Working Income Supplement minimum benefit
*   MP.OCWISRR      (i)  Ont. Child Care Working Income Supplement Rate [children][rr]
*   MP.OCWISRRP     (i)  Ont. Child Care Working Income Supplement Rate [children][rr] for 1st half of year
*   MP.OCWISRRProws (i)  number of rows in OCWISRRP
*   MP.OCWISRRrows  (i)  number of rows in OCWISRR
*   MP.OCWISSP      (i)  Ont. Child Care Working Income Supplement maximum benefit for single parents
*   MP.OCWISSPP     (i)  Ont. Child Care Working Income Supplement maximum benefit for single parents for 1st half of year
*   MP.OCWISTD      (i)  Ont. Child Care Working Income Supplement minimum earnings
*   MP.OCWISTDP     (i)  Ont. Child Care Working Income Supplement minimum earnings for 1st half of year
*   MP.ODTCR        (i)  Ont. dividend tax credit rate
*   MP.ODTCRLC      (i)  Ont. dividend tax credit rate from large (eligible) corporations
*   MP.OEDXPM       (i)  Ont. Education Amount per month of full-time studies
*   MP.OEDXPMPT     (i)  Ont. Education Amount per month of part-time studies
*   MP.OEHTDRR      (i)  OEHT - Income Tax Deduction Rate
*   MP.OEHTEX       (i)  OEHT - Exemption
*   MP.OEHTFLAG     (i)  OEHT - Ontario Employer Health Tax Flag
*   MP.OEHTRR1      (i)  OEHT - Rate 1
*   MP.OEHTRR2      (i)  OEHT - Rate 2
*   MP.OEHTRR3      (i)  OEHT - Rate 3
*   MP.OEHTTD1      (i)  OEHT - Turndown 1
*   MP.OEHTTD2      (i)  OEHT - Turndown 2
*   MP.OEMXM        (i)  Ont. equivalent to married amount
*   MP.OEMXMT       (i)  Ont. equivalent to married turndown level
*   MP.OFTCOPT      (i)  Ontario foreign tax credit application option
*   MP.OHEATFAM     (i)  Ontario Home Electricity Relief amount for families
*   MP.OHEATFLG     (i)  Ontario Home Electricity Relief activation flag
*   MP.OHEATFTD     (i)  Ontario Home Electricity Relief income turndown for families
*   MP.OHEATMIN     (i)  Ontario Home Electricity Relief minimum
*   MP.OHEATRR      (i)  Ontario Home Electricity Relief reduction rate
*   MP.OHEATSNG     (i)  Ontario Home Electricity Relief amount for singles
*   MP.OHEATSTD     (i)  Ontario Home Electricity Relief income turndown for singles
*   MP.OHPBASE      (i)  Ontario Health Premium table [taxable income,premium]
*   MP.OHPBASErows  (i)  number of rows in OHPBASE
*   MP.OHPFLAG      (i)  Ontario Health Premium activation flag
*   MP.OHPPI        (i)  Ontario Health Premium program phase in
*   MP.OLVCMAX      (i)  Maximum Ont. labour-sponsored funds tax credit allowed
*   MP.OLVCRT       (i)  Percent of Ont. labour-sponsored funds cost allowed as credit
*   MP.OMAXDX       (i)  Ont. Maximum Disability deduction/amount
*   MP.OMAXET       (i)  Ont. maximum on transfer of education and tuition amount
*   MP.OMEDALL      (i)  Ont. Medical allowance maximum lower limit
*   MP.OMEDANF      (i)  Ont. Medical allowance lower limit net income fraction
*   MP.OMXM         (i)  Ont. married amount
*   MP.OMXMT        (i)  Ont. married amount turndown level
*   MP.OODISTC      (i)  Ont. Maximum infirm dependent 18 or older tax credit
*   MP.OPCRA        (i)  Ontario property tax college residence amount
*   MP.OPNTCR       (i)  Ont. provincial non-refundable tax credit rate
*   MP.OPOCAR       (i)  Ontario property tax percent of occupancy cost
*   MP.OPOCM        (i)  Ontario property tax maximum occupancy cost
*   MP.OPRIR        (i)  Ontario property tax credit rental inclusion rate
*   MP.OPSMAX       (i)  Ontario property/sales tax credit maximum
*   MP.OPTC         (i)  Ontario political contribution table [total donations,donation allowed]
*   MP.OPTCBEN      (i)  Maximum Ontario political tax credit allowed
*   MP.OPTCFLAG     (i)  Ontario political contribution refundable tax credit flag
*   MP.OPTCrows     (i)  number of rows in OPTC
*   MP.OPTF         (i)  Ontario provincial tax fraction
*   MP.OPTX         (i)  Ont. tax table [taxable income,basic provincial tax]
*   MP.OPTXrows     (i)  number of rows in OPTX
*   MP.OPYTAX       (i)  Estimate of previous year's Ontario taxes
*   MP.OSHPTGAMT    (i)  Ont. Senior Homeowners' Property Tax Grant amount
*   MP.OSHPTGFLG    (i)  Ont. Senior Homeowners' Property Tax Grant activation flag
*   MP.OSHPTGMPT    (i)  Ont. Senior Homeowners' Property Tax Grant minimum property tax allowed
*   MP.OSHPTGPOC    (i)  Ont. Senior Homeowners' Property Tax Grant income phase out for senior couples
*   MP.OSHPTGPOS    (i)  Ont. Senior Homeowners' Property Tax Grant income phase out for single seniors
*   MP.OSHPTGTDC    (i)  Ont. Senior Homeowners' Property Tax Grant income turndown for senior couples
*   MP.OSHPTGTDS    (i)  Ont. Senior Homeowners' Property Tax Grant income turndown for single seniors
*   MP.OSPOCM       (i)  Ontario seniors property tax maximum occupancy cost
*   MP.OSPSMAX      (i)  Ontario seniors property/sales tax credit maximum
*   MP.OSSML        (i)  Ontario surtax first cut-in level
*   MP.OSSML2       (i)  Ontario surtax second cut-in level
*   MP.OSSMR        (i)  Ontario surtax first level rate
*   MP.OSSMR2       (i)  Ontario surtax second level rate
*   MP.OSSTCP       (i)  Ontario seniors sales tax credit percent of income
*   MP.OSTCB        (i)  Ontario sales tax credit basic credit
*   MP.OSTCD        (i)  Ontario sales tax credit dependant credit
*   MP.OSTCFLAG     (i)  Ontario seniors tax credit claimed flag
*   MP.OSTCNIBA     (i)  Ontario seniors tax credit net income base amount
*   MP.OSTCNIBC     (i)  Ontario seniors tax credit net income base amount for couples
*   MP.OSTCP        (i)  Ontario sales tax credit percent of income
*   MP.OSTCS        (i)  Ontario sales tax credit spouse credit
*   MP.OTCNIBA      (i)  Ontario tax credit net income base amount
*   MP.OTRBPA       (i)  Ontario tax reduction basic personal amount
*   MP.OTRDCA       (i)  Ontario tax reduction dependant child amount
*   MP.OTRDDA       (i)  Ontario tax reduction disabled dependant amount
*   MP.OTRF         (i)  Ontario tax reduction Ontario tax multiplier
*   MP.OTRFP        (i)  Ontario tax reduction personal amount multiplier
*   MP.OTXDVMAX     (i)  Maximum taxpayer dividend
*   MP.OTXDVMIN     (i)  Minimum taxpayer dividend for people with non-zero taxes
*   MP.OTXFLG       (i)  Ont. tax on taxable income activation flag
*   MP.OTXTGRANT    (i)  Ontario Textbook and Technology Grant amount
*   MP.OYPNDL       (i)  Ont. Pension Income Deduction Amount
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   MP.TARGETYEAR   (i)  Year of analysis
*   cf.cfin         (i)  First person in census family [pointer]
*   cf.cfinch       (i)  First child in census family [pointer]
*   cf.cfineld      (i)  Eldest person in census family [pointer]
*   cf.cfinspo      (i)  Spouse of eldest [pointer]
*   cf.cfnchild     (i)  Number of children (including 18+)
*   cf.cfnkids      (i)  Number of children in census family
*   cf.cfnpers      (i)  Number of persons in census family
*   cf.cfspoflg     (i)  Census family contains married couple
*   ef.efin         (i)  First person in economic family [pointer]
*   ef.efnpers      (i)  Number of persons in economic family
*   hd.hdageeld     (i)  Age of eldest in household
*   hh.cf           (i)  Census family data [array]
*   hh.ef           (i)  Economic family data [array]
*   hh.hhncf        (i)  Number of census families in household
*   hh.hhnef        (i)  Number of economic families in household
*   hh.hhnin        (i)  Number of individuals in household
*   hh.in           (i)  Individual data [array]
*   id.idage        (i)  Age
*   id.idcapgex     (i)  Capital gains exemptions (254)
*   id.idcfrh       (i)  Relationship to census family head
*   id.idcgtc       (i)  Caregivers tax credit (database) (315)
*   id.iddisab      (i)  Disability status (age 16+)
*   id.iddisoth     (i)  Disability amount for dependants (318)
*   id.iddisslf     (i)  Disability amount for self (316)
*   id.ideducfm     (i)  Eligible full-time months of education
*   id.ideducpm     (i)  Eligible part-time months of education
*   id.idestat      (i)  Educational status
*   id.idhomstu     (i)  College res/resdnt homeowner assist (558)
*   id.idicapg      (i)  Capital gains (actual)
*   id.ididiv       (i)  Dividend income (actual)
*   id.idiemp       (i)  Wages & salaries
*   id.idiloss      (i)  Business investment losses (217)
*   id.idinspo      (i)  Person's spouse [pointer]
*   id.idintstu     (i)  Interest paid on student loans (319)
*   id.idise        (i)  Self-employed income
*   id.idiworkc     (i)  Workers compensation benefits
*   id.idlabtxg     (i)  Gross Labour funds bought (413)
*   id.idmedgro     (i)  Medical expenses, gross (330)
*   id.idmincar     (i)  Minimum tax carryover (504)
*   id.idnf         (i)  Person's nuclear family [pointer]
*   id.idothpe      (i)  Amounts for Infirm Dependants (306)
*   id.idprvftc     (i)  Provincial foreign tax credit (Form T2036)
*   id.idprvpol     (i)  Provincial political contributions (565)
*   id.idrand       (i)  Random numbers [array]
*   id.idschtp      (i)  School type
*   id.idsex        (i)  Sex
*   id.idspoflg     (i)  Person has spouse
*   id.idstkded     (i)  Stock option deduction (249)
*   id.idtuitn      (i)  Tuition fees (320)
*   im.imamtfg      (i)  Minimum tax flag
*   im.imamtinc     (i)  Net adjusted income used for minimum tax
*   im.imamtprv     (i)  Federal minimum tax concept used for provincial minimum tax
*   im.imbft        (i)  Basic federal tax
*   im.imccea       (i)  Child care expenses allowed
*   im.imchara      (i)  Allowable charitable donations and gifts (calculated)
*   im.imcppse      (i)  CPP contributions deduction for self-emp
*   im.imcqppc      (i)  CPP/QPP contributions
*   im.imdedfn      (i)  All deductions from net income
*   im.imdedft      (i)  Deductions from total income
*   im.imdepni      (i)  Dependant's net income
*   im.imicapgt     (i)  Capital gains (taxable)
*   im.imicnet      (i)  Net Income (line 236)
*   im.imictot      (i)  Total Income (line 150)
*   im.imigis       (i)  GIS benefits
*   im.imildivt     (i)  Dividend income from large (eligible) corporations (taxable)
*   im.iminet       (i)  Net income
*   im.imipnst      (i)  Taxable pension income (after splitting)
*   im.imisa        (i)  Social assistance (or replacement program)
*   im.imisdivt     (i)  Dividend income from small corporations (taxable)
*   im.imispa       (i)  Spouse's allowance
*   im.imitax       (i)  Taxable income
*   im.imminamt     (i)  Minimum Amount due to Federal Minimum Tax
*   im.improptx     (i)  Imputed property tax paid
*   im.imrentpd     (i)  Imputed rent paid
*   im.imuccb       (i)  Universal Child Care Benefit amount
*   im.imuic        (i)  UIC contributions
*   nf.nfnkids      (i)  Number of children in nuclear family
*   im.imbpt        (io) Basic provincial tax
*   im.imiotax      (io) Ontario taxable income
*   im.imnptc       (io) Non-refundable provincial tax credits
*   im.imocbyng     (io) Ontario Child Benefit amount for Child Care Supplement dependents
*   im.imondc       (io) Ontario number of dependent children 18 or younger
*   im.imonteht     (io) Ontario Employers Heath Tax (Self-employed)
*   im.imopstc      (io) Ont. Property and Sales Tax Credit
*   im.imotxdv      (io) Ontario Taxpayer Dividend
*   im.impatxc      (io) Prov Age amount tax credit
*   im.impbtc       (io) Prov Basic amount
*   im.impcgtc      (io) Prov Caregiver Tax Credit Level
*   im.impcgtcp     (io) Provincial Dependent Caregiver Tax Credit Amount
*   im.impchrtc     (io) Prov Charitable Donations tax credit
*   im.impcpptc     (io) Prov CPP/QPP contributions tax credit
*   im.impdistc     (io) Prov Disability tax credit
*   im.impdtxc      (io) Provincial dividend tax credit
*   im.impedtxc     (io) Prov Education tax credit
*   im.impintsl     (io) Provincial interest on student loans tax credit
*   im.implvctc     (io) Provincial labour-sponsored funds tax credit
*   im.impmartc     (io) Prov Married and Equivalent to Married tax credit
*   im.impmeda      (io) Prov Medical expenses allowed (computed)
*   im.impmedtc     (io) Prov Medical Expenses tax credit
*   im.impminco     (io) Prov Minimum tax carryover
*   im.impothpe     (io) Provincial infirm dependents 18 or older tax credit
*   im.imppentc     (io) Prov Pension Income tax credit
*   im.impptc       (io) Provincial Political Contrib Tax Credit
*   im.imprvtax     (io) Prov gross provincial tax
*   im.impsur       (io) Provincial surtax
*   im.imptaxcr     (io) Prov total tax credits
*   im.imptr        (io) Provincial tax reduction
*   im.impttxct     (io) Prov total tax credits transferred
*   im.imptutxc     (io) Prov Tuition tax credit
*   im.impuictc     (io) Prov UI/EI contributions tax credit
*   im.imtxp        (io) Provincial income tax payable
*   im.imocb        (o)  Ontario Child Benefit
*   im.imoccea      (o)  Ont. Child Care Exp. credit allowed (Family)
*   im.imoccext     (o)  Ont. Child Care Supplement one-time extra benefit
*   im.imoshptg     (o)  Ontario Senior Homeowners' Property Tax Grant
*   im.imotxtgr     (o)  Ontario Textbook and Technology Grant
*   im.impatxcr     (o)  Prov total tax credits applied
*   im.impctxct     (o)  Prov tax credits transferred from children
*   im.impheatr     (o)  Provincial Home Heating Fuel/Energy Rebate
*   im.imphp        (o)  Provincial Health Premium
*   im.impstxct     (o)  Prov tax credits transferred from spouse
*   im.imptc        (o)  Refundable provincial tax credits
*
*Called by:
*   txprov   Compute provincial taxes
*
*****************************************************************************/
void txont(
	P_hh hh
)
{
	register int ini;
	register P_in in;
	register P_in inspo;
	register P_nf nf;
	P_in recip;		 /* Pointer to recipent */
	register P_cf cf;
    register P_in ineld;
	int cfi;
	P_ef ef;
			  
	register int efi;
	NUMBER opocm;          /* temp value for MP.OPOCM or MP.OSPOCM */
	NUMBER ostcp;			  /* temp value for MP.OSTCP or MP.OSSTCP */
	NUMBER otcniba;		  /* temp value for MP.OTCNIBA or MP.OSTCNIBA */
	NUMBER persamt=ZERO;   /* personal amount calculation for tax redction */
	NUMBER oc=ZERO;        /* Occupancy cost */
	NUMBER prtc=ZERO;      /* Property Tax Credit */
	NUMBER stc=ZERO;       /* Sales Tax Credit */
	NUMBER faminet=ZERO;   /* Head/spouse Net Income for net PSTC Calc */
	NUMBER famearn=ZERO;   /* Head/spouse Earnings for Child Care supplement */
	NUMBER pstc=ZERO;      /* Property plus sales Tax Credit */
	NUMBER pfaminet=ZERO;  /* %age of net income for net PSTC Calc */
	NUMBER netpstc=ZERO;   /* net pstc */
	NUMBER hosinet=ZERO;   /* Head/spouse Net Income for net HOSTC Calc */
	NUMBER poltcr = ZERO;  /* Political tax credit (raw) */
	NUMBER poltc=ZERO;     /* political tax credit */
	NUMBER ehtded=ZERO;    /* Employers Health Tax Deduction */
	NUMBER seinc=ZERO;     /* Self employment income */
	NUMBER eht;            /* Employer Health Tax; */
	NUMBER ccea = ZERO;    /* child care expenses claimed for entire family */
	NUMBER occea = ZERO;   /* Ontario's portion of child care expenses claimed for entire family */
	NUMBER owfis = ZERO;   /* Ontario's working family income supplement */
	NUMBER ccea1 = ZERO;    /* child care expenses claimed for entire family */
	NUMBER occea1 = ZERO;   /* Ontario's portion of child care expenses claimed for entire family */
	NUMBER owfis1 = ZERO;   /* Ontario's working family income supplement */
	NUMBER ben = ZERO;     /* Maximum Ontario working family income supplement- CCE Credit */
	NUMBER ynglim = ZERO;  /* limit of occea for young children */
	NUMBER ben1 = ZERO;     /* Maximum Ontario working family income supplement- CCE Credit */
	NUMBER ben2 = ZERO;     /* Maximum Ontario working family income supplement- CCE Credit */
	NUMBER ynglim1 = ZERO;  /* limit of occea for young children */
	NUMBER yngkids = ZERO; /* Number of young children */
	NUMBER occsext = ZERO; /* One time payment to Ontario Child Care Supplement families */
	int spoflg;
    NUMBER lvctc = ZERO;   /* Labour-sponsored venture capital tax cred */
	NUMBER odedfn = ZERO;  /* deductions from net using Ontario Capital Gains Inclusion Rate */
	NUMBER odedft = ZERO;  /* deductions from total using Ontario Capital Gains Inclusion Rate */
	NUMBER onet = ZERO;    /* net income using Ontario Capital Gains Inclusion Rate  */
    NUMBER pytax = ZERO;   /* Estimate of previous year's taxes */
    NUMBER otxdv = ZERO;   /* Ontario taxpayer dividend */
	NUMBER heatinc = ZERO;  /* Ontario home electricity relief net income */
	NUMBER heatthr = ZERO;  /* Ontario home electricity relief income threshold */
	NUMBER heatrel = ZERO;  /* Ontario home electricity relief amount */

	NUMBER ocbkids = ZERO;  /* Number of kids eligible for Ontario Child Benefit */
	NUMBER occskids = ZERO;  /* Number of kids eligible for Ontario Child Care Supplement */
	NUMBER amtocb1 = ZERO;  /* Amount OCB for 1st half of year */
	NUMBER amtocb2 = ZERO;  /* Amount OCB for 2nd half of year */
	NUMBER amtocb = ZERO;   /* Amount of OCB */
	NUMBER ocbyng1 = ZERO;  /* Amount of OCB for OCCS eligible - 1st half of year */
	NUMBER ocbyng2 = ZERO;  /* Amount of OCB for OCCS eligible - 2nd half of year */
	NUMBER ocbyng = ZERO;   /* Amount of OCB for OCCS eligible */

	NUMBER inctest = ZERO;  /* income concept for Senior Homeowners' Property Tax Grant */
	int    gotgrant;        /* flag to determine if household got property tax grant already */
	NUMBER ptgrr = ZERO;    /* reduction rate for Senior Homeowner's Property Tax Grant */
	NUMBER ptgrant = ZERO;  /* temporary value of property tax grant */
	NUMBER proptax = ZERO;  /* temporary value of property tax */
	int nkids = 0;        /* Number of kids 18 and younger */
	NUMBER efimcgtcp = ZERO;  /* combined cgtc potential amounts for ef */   



	DEBUG_ON("txont");

    /* process each census family in household */
    for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {
       /* process children in census family */
	   nkids = 0;
       for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
			if (in->id.idage < 19) nkids ++;
	   }
	   /* now assign to elder or spouse with higher net income */
       ineld = cf->cfineld;
       inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

       if (cf->cfspoflg) {
			 if (inspo->im.imicnet > ineld->im.imicnet) {
				inspo->im.imondc = nkids;
			 }
			 else {
				ineld->im.imondc = nkids;
			 }
	   }	 
	   else {
	   		ineld->im.imondc = nkids;
	   }	
	}		 

    /*******************************************************************
    *****  Calculations for tax on taxable income, 
    ********************************************************************/

    if (MP.OTXFLG) { 
       /* Calculate the Caregiver tax credit */
       if (MP.OCGTCFLG  && MP.OCGTCOPT == 3) {
       
          int cgassign;                  
          /*process each economic family in household */
          for (efi=0, ef=&hh->ef[0];  efi<hh->hhnef;  efi++, ef++) {
             int    spoflg = 0;      /* Spouse flag */ 
             efimcgtcp = 0;

			 /* First step: Do a family loop in order to calculate the potential credit
			      from potential donors as well as the potential recipient of the credit */
             
             /* process each individual in the economic family */
             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
				/* if over 65 and either a head or spouse or other family member who could be a parent (with takeup) */
                if (in->id.idage >=65 || (in->id.iddisab == 1 && in->id.idage>= 18 ) ) {
				   /* potential benefit to be assigned to some caregiver */
                   in->im.impcgtcp = MP.OPNTCR * minn(nneg(MP.OCGTCTD -in->im.iminet),MP.OCGTC);
                }
			   efimcgtcp += in->im.impcgtcp;
             }

			 cgassign = 0;

             for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
 				 /* initialize pointers to spouse, and set spouse flag */
                 if (in->id.idspoflg == 1 ) {
                   	inspo   = in->id.idinspo;
                   	spoflg = 1;
				 }
				 if (cgassign == 0 && in->id.idcgtc > 0 ) {
				   	if (in->id.idspoflg && inspo->im.impcgtcp > 0) {
				   		in->im.impcgtc = nneg(efimcgtcp - inspo->im.impcgtcp - in->im.impcgtcp );
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				   	else {
				   		in->im.impcgtc = nneg(efimcgtcp - in->im.impcgtcp);
				   		efimcgtcp = efimcgtcp - in->im.impcgtc;
				   		if (efimcgtcp <= 0) {
				   			cgassign = 1;
				   		}
				   	}
				 }	 
			 }

		 	/* one final ef loop to assign the credit if not already assigned to the right person */
         	for (ini=0, in=ef->efin;  ini<ef->efnpers;  in++, ini++) {   
		 		if (cgassign == 0 && efimcgtcp > 0 && in->id.idcgtc > 0 && in->im.impcgtc == 0 ) {
		 			in->im.impcgtc = efimcgtcp;
		 			cgassign = 1;
		 	 	}
		    }
		  } 	 /* end of ef hh loop */
	   }  /* end of cgtc loop */

       /****************************************************************
       *** Implement the Ontario Capital Gains Inclusion Rate
       ***  NOTE:  As of the time of this coding, the legislation for 
       ***         this implementation had not been written and requires
       ***         approval from CCRA.  Ontario's objective is to 
       ***         have the lower inclusion rate to be used to determine
       ***         taxable income for Ontario tax.  If the CCRA are not
       ***         willing to implement it as such, then difference in
       ***         the inclusion rates would be given as a tax credit.
       ***  VERIFY:  Legislation is not expected before the fall of 2000.
       ***           Verify the implementation of the lower capital
       ***           gains inclusion rate as announced in Ontario 2000
       ***           Budget (proposed to be effective January 1, 2001) 
       *****************************************************************/
	   
		if (MP.OCGIRFLG) {
		   /* if the Ontario Capital Gains Inclusion Rate Flag is activated,
		      reduce taxable income by the difference in the federal and
			  the Ontario capital gains inclusion rates                  */
		    /* process each person in household */
		    for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
			 
					if (isnzero(in->im.imicapgt)) {
				      /* reduce the capital gains deduction amount */
				  	  odedfn = nneg(in->im.imdedfn - (in->id.idcapgex * (MP.CAPGIR - MP.OCAPGIR)));
					 }
					 else {
					    odedfn = in->im.imdedfn;
					 }
				  /* reduce the stock option deduction */
				  odedfn = nneg(odedfn + (in->id.idstkded * (MP.CAPGIR - MP.OCAPGIR)));

				  /* reduce the business investment losses deduction */
				   
			      odedft = nneg(in->im.imdedft - (in->id.idiloss * (MP.CAPGIR - MP.OCAPGIR)));
     
				  onet = nneg(in->im.imictot - (in->id.idicapg * (MP.CAPGIR - MP.OCAPGIR)) - odedft);
				  in->im.imiotax = nneg(onet - odedfn );
			}
		}

        /* process each census family in household */
       for (cfi=0, cf=&hh->cf[0]; cfi<hh->hhncf; cfi++, cf++) {

          /* Do needed declarations/initialisations for each census family */
          NUMBER ctxcrt = ZERO;   /* total children's transferable credits  */
          NUMBER tfr1 = ZERO;     /* transfer type 1 */
          NUMBER tfr2 = ZERO;     /* tranfer type 2 */
          NUMBER txcrt = ZERO;    /* tax credits */
          NUMBER agered = ZERO;
          NUMBER adjamt = ZERO;
		  NUMBER netinc = ZERO;

          NUMBER cemc   = ZERO;   /* value of equivalent to married tax credit */
          NUMBER cinet   = ZERO;   /* value of current child's income */
          NUMBER minet   = ZERO;   /* value of child's income for child with least income */
          NUMBER inetfg   = ZERO;   /* went through process once */

          /*******************************************************************
          *****  Calculation of married and equivalent to married tax credits
          ********************************************************************/
         
          /* calculate equivalent to married credit */
 
          /* process children in census family */
          for (ini=0, in=cf->cfinch; ini<cf->cfnchild; ini++, in++) {
             /* Filter condition (ineligible for EM credit)  */
             if (in->id.idage > 18) {
                continue;
             }
             cinet = in->im.imdepni ;
             /* calculate EM credit if income is lower or if not done before */
             if (!cf->cfspoflg && (inetfg==ZERO || minet > cinet)) {
				inetfg=1;
				minet = cinet;
                cemc = MP.OPNTCR * nneg(MP.OEMXM - nneg(cinet - MP.OEMXMT));
             }
          }
          /* initialise pointers for head and spouse modelled variables */
          ineld = cf->cfineld;
          inspo = cf->cfinspo;     /* NB: is NULL if no spouse present */

          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
             /**
             * If the net income of the head is greater than the net
             * income of the spouse, the head gets the married
             * exemption 
             **/
             if (ineld->im.imicnet >= inspo->im.imicnet) {
                DEBUG1("%s head receives married tax credit\n");
                ineld->im.impmartc = MP.OPNTCR * nneg(MP.OMXM -	nneg(inspo->im.imdepni - MP.OMXMT));
             }
             else {
                DEBUG1("%s spouse receives married tax credit\n");
                inspo->im.impmartc = MP.OPNTCR * nneg(MP.OMXM - nneg(ineld->im.imdepni - MP.OMXMT));
             }
          }
          else { /* no spouse present */
             ineld->im.impmartc = cemc;
          }
     
          /* note, we are already in a census family loop */
 
          /* process persons in census family */
          for (ini=0, in=cf->cfin;  ini<cf->cfnpers;  ini++, in++) {
             /*********************************************************
             *****  Calculation of non-family tax credits
             **********************************************************/

			 /* calculate caregivers tax credit */
			 if (MP.OCGTCFLG && isnzero(in->id.idcgtc) && MP.OCGTCOPT < 3) {
				/* if OGCTGOPT == 1 then */
				/* apply the maximum amount if they have any caregivers amount */
				 if (MP.OCGTCOPT == 1) {
					in->im.impcgtc = MP.OCGTC * MP.OPNTCR;
				 }
				 else {   /* use the greenbook imputed amount */
				 	in->im.impcgtc = in->id.idcgtc * MP.OPNTCR;
				 }
				 DEBUG2("%s cgtc =%.2f\n", in->im.impcgtc);
			 }

             /* calculate cpp/qpp contributions tax credit  */
             in->im.impcpptc = (in->im.imcqppc - in->im.imcppse) * MP.OPNTCR;
             DEBUG2("%s cppctc =%.2f\n", in->im.impcpptc);

             /* calculate ui contributions tax credit  */
             in->im.impuictc = in->im.imuic * MP.OPNTCR;
             DEBUG2("%s uctc  =%.2f\n", in->im.impuictc);

             /* calculate tuition tax credit */
             if (isnzero(in->id.idtuitn)) {
                in->im.imptutxc = in->id.idtuitn * MP.OPNTCR;
                DEBUG2("%s tutxc  =%.2f\n", in->im.imptutxc);
             }

             /* calculate personal tax credit */
             in->im.impbtc = MP.OPNTCR * MP.OBXM;
                DEBUG2("%s btc    =%.2f\n", in->im.impbtc);

			 /*  Calculate the net income concept to match tax forms */
			 /*  This income is used in the age and medical tax credit income tests */

			 netinc = in->im.imicnet ;

             /*  calculate age amount tax credit  */
             if (in->id.idage >= 65) {
                DEBUG1("%s age tax credit\n");
                agered = minn(MP.OAXM, MP.OAXRR * nneg(netinc - MP.OAXTD));
                adjamt = MP.OAXM - agered;
                in->im.impatxc = MP.OPNTCR * adjamt;
             }
             DEBUG2("%s atxc   =%.2f\n", in->im.impatxc);

             /* calculate prov pension income tax credit    */
             in->im.imppentc = MP.OPNTCR * minn(MP.OYPNDL, in->im.imipnst);
             DEBUG2("%s pentxc =%.2f\n", in->im.imppentc);
 
			 /* compute allowable medical expenses using the value on the database */
			 if (isnzero(in->id.idmedgro)) {
			 	in->im.impmeda = nneg(in->id.idmedgro -
			 		minn(MP.OMEDALL, MP.OMEDANF * netinc));
			  	DEBUG3("%s idmedgro= %.2f impmeda =%.2f\n",
			  		in->id.idmedgro, in->im.impmeda);
			 }

			 /* calculate medical expenses tax credit */
			 if (isnzero(in->im.impmeda)) {
			  	in->im.impmedtc = in->im.impmeda * MP.OPNTCR;
			   	DEBUG2("%s pmedtc =%.2f\n", in->im.impmedtc);
			 }

			 /* calculate amount for infirm dependents age 18 or older */
			 /*  if the filer has an amount for idothpe, then give max credit */

			 if (isnzero(in->id.idothpe)) {
   				in->im.impothpe = MP.OPNTCR * MP.OODISTC;
                DEBUG2("%s othpe =%.2f\n", in->im.impothpe);
			 }
		     
             /* calculate prov charitable donations tax credit */
             if (isnzero(in->im.imchara)) {
                in->im.impchrtc = (minn(MP.OCHATL1, in->im.imchara)  * MP.OCHATR1
                            + nneg(in->im.imchara - MP.OCHATL1) * MP.OCHATR2);
                DEBUG2("%s chartc =%.2f\n", in->im.impchrtc);
             }
 
             /* set prov disability tax credit for self to maximum    */
             if (isnzero(in->id.iddisslf)) {
                in->im.impdistc = MP.OPNTCR * MP.OMAXDX;
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* set disability tax credit for other to maximum    */
             if (isnzero(in->id.iddisoth)) {
               in->im.impdistc += (MP.OPNTCR * MP.OMAXDX);
             }
             DEBUG2("%s disatc =%.2f\n", in->im.impdistc);

             /* calculate prov education allowance tax credit */
             if (isnzero(in->id.ideducfm)) {
                in->im.impedtxc = MP.OPNTCR * (in->id.ideducfm * MP.OEDXPM);
                DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
             }
			 /*  add in the tax credit amount associated with part-time studies */
			 if (isnzero(in->id.ideducpm)) {
				if (isnzero(in->id.iddisslf) && in->id.ideducfm == 0 ) {
				    /* give only part time students the full time credit
					   if they also have an amount for disability */
					in->im.impedtxc = MP.OPNTCR * (in->id.ideducpm * MP.OEDXPM);
				}
				else {  /* has some part and/or full time and not disabled */
					in->im.impedtxc += MP.OPNTCR * (in->id.ideducpm * MP.OEDXPMPT);
                	DEBUG2("%s edtxc  =%.2f\n", in->im.impedtxc);
				}
			 }

			 /* calculate the interest paid on student loans tax credit */
			 /*  new for version 10.0, apply the Greenbook value idintstu */
			 if (isnzero(in->id.idintstu)) {
				in->im.impintsl = MP.OPNTCR * in->id.idintstu;
				DEBUG2("%s intsl =%.2f\n", in->im.impintsl);
			 }

             /* calculate provincial dividend tax credit */
             if (isnzero(in->id.ididiv)) {
				in->im.impdtxc = (in->im.imisdivt * MP.ODTCR)
								+ (in->im.imildivt * MP.ODTCRLC);
	    	   	DEBUG2("%s fdtxc  =%.2f\n", in->im.impdtxc);
             }

			 /* calculate the Ontario minimum tax carry-over tax credit */
			 if (isnzero(in->id.idmincar)) {
				if (MP.OAMTOPT == 2) {
				   /* Alternative minimum taxes are calculated, use OAMTPCTF, the
				   /*  amt rate as pct of additional fed tax due to minimum tax
				   /*  to calculte the minimum tax carry-over credit. */
				   in->im.impminco = in->id.idmincar * MP.OAMTPCTF;
				 }
				 else if (MP.OAMTOPT == 3) { 
						/* Alternative minimum taxes are calculated using OAMTTX. */
						in->im.impminco = in->id.idmincar * MP.OAMTTX;
					  }
				 else if (MP.OAMTOPT == 4) {
						/* Alternative minimum taxes are calculated using OAMTPCTM. */
						 in->im.impminco = in->id.idmincar * MP.OAMTPCTM;
				 }
				 DEBUG2("%s pminco =%.2f\n", in->im.impminco);
			 }
            
             /*********************************************************
             *****  Calculation of basic personal tax
             **********************************************************/
			            
			 /* initialize the Ontario taxable income to federal taxable income */
			 /*  when the OCAPGIR is not used    */

			 if (!MP.OCGIRFLG) {
			    in->im.imiotax = in->im.imitax;
			 }
             /* calculate provincial tax before tax credits    */
             /* if alternative minimum tax , option 3 = tax structure on federal net adjusted income */

			 if (in->im.imamtfg && MP.OAMTOPT == 3) {
                in->im.imprvtax = MP.OAMTTX * in->im.imamtinc;
             }
             else {
                in->im.imprvtax = (NUMBER) lkup1(MP.OPTX, MP.OPTXrows, in->im.imiotax);
             }
             in->im.imbpt = in->im.imprvtax;

             /* census family children */
             if (in->id.idcfrh >= 2) {
                /* total non-transferable tax credits applicable to dependent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                        + in->im.impatxc     /* age tax credit                  */
                        + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                        + in->im.impchrtc   /* charitable donations tax credit */
                        + in->im.impcpptc   /* CPP contributions tax credit    */
                        + in->im.impmedtc   /* medical expenses tax credit     */
                        + in->im.imppentc   /* pension income tax credit       */
                        + in->im.impuictc    /* UI contributions tax credit     */
                        + in->im.impdtxc    /* provincial dividend tax credit     */
				        + in->im.impcgtc     /* caregiver tax credit                  */
						+ in->im.impminco    /* minimum tax carryover credit    */
						+ in->im.impintsl   /* provincial interest on student loans credit */
                        ;
                DEBUG2("%s child's  non-transferable tax credits=%.2f\n",
                        in->im.imtaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s child's bft    =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impdistc;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s child's  transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if child's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.imttxcrt = 0.0  and no need to increase ctxcrt  */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the child has some tax credit left for a parent    */
                      /* reduce basic federal tax to zero and calculate     */
                      /* actual amount of transferable tax credits          */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                      tfr2 = minn(MP.OPNTCR * MP.OMAXET,tfr2);
                      /* first calculate disability credit, then education amount */
  
                      tfr1 = nneg(tfr1  /* disability credit */
                              - nneg( (MP.OPNTCR * in->im.imiotax)     /* taxable income * credit tax rate 
                                                                            to preserve idendity */
                                      - in->im.impbtc      /* basic personal tax credit       */
                                      - in->im.impatxc     /* age tax credit                  */
                                     /* next two credits equal zero for CF children */
                                     /*- in->im.immartxc      married tax credit */
                                     - in->im.impcpptc   /* CPP contributions tax credit    */
                                     - in->im.impuictc    /* UI contributions tax credit     */
                                     - in->im.imppentc));  /* pension income tax credit       */

                      tfr2 = nneg(tfr2  /* education and tuition */
                              - nneg( (MP.OPNTCR * in->im.imiotax)     /* taxable income * credit tax rate 
                                                                         to preserve idendity */
                                    - in->im.impbtc      /* basic personal tax credit       */
                                    - in->im.impatxc     /* age tax credit                  */
                                    /* next two credits equal zero for CF children */
                                    /*- in->im.immartxc      married tax credit */
                                    - in->im.impcpptc   /* CPP contributions tax credit    */
                                    - in->im.impuictc    /* UI contributions tax credit     */
                                    - in->im.imppentc));  /* pension income tax credit       */

             
                      in->im.impttxct = tfr1+tfr2;
                      /* add extra to family ctxcrt */
                      ctxcrt += in->im.impttxct;
                      DEBUG2("%s ctxcrt = %.2f\n", ctxcrt);
                   }
                }
                DEBUG4("%s child's bft    =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                      in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             }

             /* head or spouse of census family */
             if (in->id.idcfrh < 2) {
                /* total non-transferable tax credits applicable to parent */
                in->im.imptaxcr = in->im.impbtc      /* basic personal tax credit       */
                       + in->im.impothpe    /* infirm dependents 18 and older tax credit */
                       + in->im.impchrtc   /* charitable donations tax credit */
                       + in->im.impcpptc   /* CPP contributions tax credit    */
                       + in->im.impmartc   /* married tax credit              */
                       + in->im.impmedtc   /* medical expenses tax credit     */
                       + in->im.impuictc    /* UI contributions tax credit     */
				       + in->im.impdtxc    /* provincial dividend tax credit     */
					   + in->im.impminco   /* minimum tax carryover credit  */
					   + in->im.impintsl   /* provincial interest on student loans credit */
                       ;
                DEBUG2("%s parent's non-transferable tax credits=%.2f\n",
                     in->im.imptaxcr);
                in->im.impatxcr = ZERO;
                if (isnzero(in->im.imptaxcr)) {
                   /* if parent's basic federal tax is more than transferable */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > in->im.imptaxcr) {
                      in->im.impatxcr += in->im.imptaxcr;
                      in->im.imbpt -= in->im.imptaxcr;
                   }
                   else { /* in->im.imbpt <= in->im.imptaxcr */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
                   }
                   DEBUG3("%s parent's bft   =%.2f, atxcrt=%.2f\n",
                         in->im.imbpt, in->im.impatxcr);
                }
                /* total transferable tax credits must be processed in 2 parts*/
                /* tfr2 contains the education allowance and tuition tax      */
                /*      credits which are subject to a maximum transferable   */
                /*      calculation dependent on the parameter MAXET          */
                /* tfr1 contains all other transferable tax credits           */
                tfr1 = in->im.impatxc   /* age tax credit                        */
                    + in->im.impdistc   /* disability tax credit                 */
                    + in->im.imppentc   /* pension income tax credit             */
                    ;
                tfr2 = in->im.impedtxc + in->im.imptutxc;
                txcrt = tfr1 + tfr2;
                DEBUG2("%s parent's transferable tax credits    =%.2f\n", txcrt);
                in->im.imptaxcr += txcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transferable         */
                   /* tax credits, apply total amount to reduce tax          */
                   if (in->im.imbpt > txcrt) {
                      in->im.impatxcr += txcrt;
                      /* in->im.impttxct = 0.0 */
                      in->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= in->im.imbpt */
                      /* the parent has some tax credits left for a         */
                      /* their spouse (if present) so reduce basic federal  */
                      /* tax to zero and calculate actual amount of         */
                      /* transferable tax credits                           */
                      in->im.impatxcr += in->im.imbpt;
                      in->im.imbpt = ZERO;
  
                      tfr2 = minn(MP.OPNTCR * MP.OMAXET, tfr2);
                       /* NB: use expanded net income (as on tax form) */
                      /* follows Schedule 2 */
                      in->im.impttxct = nneg(tfr1 + tfr2
                                         - MP.OPNTCR * nneg(in->im.imdepni -in->im.imdedfn - MP.OBXM -
                                                         (in->im.imcqppc - in->im.imcppse) - in->im.imuic ));
   
                   }
                }
                DEBUG4("%s parent's bft   =%.2f, atxcrt=%.2f, ttxcrt=%.2f\n",
                    in->im.imbpt, in->im.impatxcr, in->im.impttxct);
             } /* end of head/spouse */
          } /* end of person in census family loop.  
              we are still in a census family loop */                           

          /* if there is a spouse in the family */
          if (cf->cfspoflg) {
             DEBUG1("%s spouse present\n");
 
             /* if provincial taxes owing of the elder is greater than the taxes   */
             /* owing of the spouse, the elder receives any tax credits  */
             /* not needed by the spouse and children                    */
             /* The impcgtc will be claimed by the person with the most taxes owing */
             if (ineld->im.imbpt >= inspo->im.imbpt) {
                txcrt = inspo->im.impttxct + ctxcrt;
                ineld->im.imptaxcr += txcrt;
                DEBUG2("%s elder receives transfered tax credits=%.2f\n", txcrt);
                ineld->im.impstxct = inspo->im.impttxct;
                ineld->im.impttxct = ZERO;
                ineld->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (ineld->im.imbpt > txcrt) {
                      ineld->im.impatxcr += txcrt;
                      ineld->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= ineld->im.imbpt */
                      ineld->im.impatxcr += ineld->im.imbpt;
                      ineld->im.imbpt = ZERO;
                   }	 
                }
                DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                    ineld->im.imbpt, ineld->im.imstxcrt, ineld->im.impctxct);
             } 		 
             else { /* spouse has greater income */
                txcrt = ineld->im.impttxct + ctxcrt;
                inspo->im.imptaxcr += txcrt;
                DEBUG2("%s spouse receives transfered tax credits=%.2f\n", txcrt);
                inspo->im.impstxct = ineld->im.impttxct;
                inspo->im.impttxct = ZERO;
                inspo->im.impctxct = ctxcrt;
                if (isnzero(txcrt)) {
                   /* if basic federal tax is more than transfered           */
                   /* tax credits, apply total amount to reduce tax          */
                   if (inspo->im.imbpt > txcrt) {
                      inspo->im.impatxcr += txcrt;
                      inspo->im.imbpt -= txcrt;
                   }
                   else { /* txcrt >= inspo->im.imbpt */
                      inspo->im.impatxcr += inspo->im.imbpt;
                      inspo->im.imbpt = ZERO;
                   }
                }
                DEBUG4("%s spouse's bft=%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                  inspo->im.imbpt, inspo->im.impstxct, inspo->im.impctxct);
             }
          } 
          else { /* no spouse present */
             txcrt = ctxcrt;
             ineld->im.imptaxcr += txcrt;
             DEBUG2("%s elder (alone) receives transfered tax credits=%.2f\n", txcrt);
             ineld->im.impstxct = ZERO;
             ineld->im.impttxct = ZERO;
             ineld->im.impctxct = ctxcrt;
             if (isnzero(txcrt)) {
                /* if basic federal tax is more than transfered           */
                /* tax credits, apply total amount to reduce tax          */
                if (ineld->im.imbpt > txcrt) {
                   ineld->im.impatxcr += txcrt;
                   ineld->im.imbpt -= txcrt;
                }
                else { /* txcrt >= ineld->im.imbpt */
                   ineld->im.impatxcr += ineld->im.imbpt;
                   ineld->im.imbpt = ZERO;
                }
             }
             DEBUG4("%s elder's bft  =%.2f, stxcrt=%.2f, ctxcrt=%.2f\n",
                ineld->im.imbpt, ineld->im.impstxct, ineld->im.impctxct);
          } 	 
       } 
    } /* end of tax on taxable income calculation */


    /*******************************************************************
    *****  Calculate provincial taxes 
    ********************************************************************/

	/* call the function which performs the inputation of paid rent
	*  and property taxes to individuals in the hhld
	*/
	txhhexp(hh);

	/* process each person in household */
	for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {
		DEBUG6("%s person %d: age=%d, sex=%d, marst=%d, cfrh=%d\n",
		   ini, in->id.idage, in->id.idsex, in->id.idmarst, in->id.idcfrh);

		/* initialise spouse and census family pointer */
	    nf = in->id.idnf;
      	inspo = in->id.idinspo;
		
		/****   initialize the Ontario Taxable Income concept to be used
		 ****    throughout Ontario tax module.  The Ontario Taxable Income
		 ****    concept takes into account the reduced capital gains 
		 ****    inclusion rate as announced in the Ontario 2000 Budget ***/

		if (!MP.OTXFLG && !MP.OCGIRFLG) {
		    in->im.imiotax = in->im.imitax;
		}
		if (!MP.OTXFLG && MP.OCGIRFLG) {
			if (isnzero(in->im.imicapgt)) {
		      /* reduce the capital gains deduction amount */
		  	  odedfn = nneg(in->im.imdedfn - (in->id.idcapgex * (MP.CAPGIR - MP.OCAPGIR)));
			 }
			 else {
			    odedfn = in->im.imdedfn;
			 }
		  /* reduce the stock option deduction */
		  odedfn = nneg(odedfn + (in->id.idstkded * (MP.CAPGIR - MP.OCAPGIR)));

		  /* reduce the business investment losses deduction */
		   
	      odedft = nneg(in->im.imdedft - (in->id.idiloss * (MP.CAPGIR - MP.OCAPGIR)));

		  onet = nneg(in->im.imictot - (in->id.idicapg * (MP.CAPGIR - MP.OCAPGIR)) - odedft);
		  in->im.imiotax = nneg(onet - odedfn);
		}    

		/***  
		 *** COMPUTE INCOME TAXES
		 ***/

		/* basic provincial tax */
		
        /* original tax as percentage of federal tax */
        if (!MP.OTXFLG) { /* basic provincial tax */
            in->im.imbpt = in->im.imbft * MP.OPTF;
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
        }
        /* if OTXFLG is turned on, then it's the tax on taxable income
           the bpt has already been calculated in the census family loop */
		else {
			if (in->im.imamtfg && MP.OAMTOPT == 2) {
			   /* minimum tax is equal to a percentage of the additional federal tax
			   	   attributable to the alternative minimum tax */
               in->im.imbpt += in->im.imamtprv * MP.OAMTPCTF;
			}
			else {
				if ( in->im.imamtfg && MP.OAMTOPT == 4 ) {
				  /* minimum tax is equal to a percentage of the federal minimum
				     amount attributable to the alternative minimum tax */
					in->im.imbpt = in->im.imminamt * MP.OAMTPCTM;
				}
			}
            in->im.imtxp = in->im.imbpt;   /* initialise tax paid to BPT */
		}

		/* compute surtax */

		in->im.impsur	= MP.OSSMR * nneg(in->im.imtxp - MP.OSSML)
							+ MP.OSSMR2 * nneg(in->im.imtxp - MP.OSSML2);

		/* apply surtax */
		
		in->im.imtxp += in->im.impsur;

		/* calculate and apply provincial foreign tax credit */
		/* only when OFTCOPT is set to 1 - calculate and apply prior to tax reduction */

		if (MP.OFTCOPT == 1) {
			in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
	        in->im.imtxp -= in->im.imnptc;
		}

		/* compute and apply the tax reduction */
		
        /* reduction based on dependants and ontario tax */
        /* note: only one person can claim this reduct'n */
        /* in the family - the spouse with the lowest    */
        /* net income or highest family allownce claim   */
        /* we use net income only                        */

  		DEBUG1("%s ont red: based on dependants and ontario tax\n");
  		persamt = MP.OTRBPA + (in->im.imondc * MP.OTRDCA);

  		if ( isnzero(in->id.iddisoth) ) {
  			persamt += MP.OTRDDA;
  		}

  		DEBUG4("%s ont red: persamt=%.2f imondc=%.2f nfnkids=%d\n",
  			    persamt, in->im.imondc, nf->nfnkids);

  		if (persamt >= in->im.imtxp) {
  			in->im.imptr =	in->im.imtxp;
  		}
  		else {
       		in->im.imptr =	nneg( (MP.OTRFP * persamt)
                   		 - (MP.OTRF * in->im.imtxp) );
  		}
  		DEBUG4("%s ont tax summary: ininet=%.2f imtxp=%.2f imptr=%.2f\n",
  			    in->im.imicnet, in->im.imtxp, in->im.imptr);

  		in->im.imtxp -= in->im.imptr;

		

		/* calculate and apply provincial foreign tax credit */
		/* only when OFTCOPT is set to 2 - calculate and apply after tax reduction */
		/* This adjustment was announced in the Ontario 2004 budget - effective for 2004 */

		if (MP.OFTCOPT == 2) {
			in->im.imnptc = minn(in->im.imtxp,in->id.idprvftc);
	        in->im.imtxp -= in->im.imnptc;
		}


        /***
        * Taxpayer dividend 
        ****/
        /* The taxpayer dividend provides taxpayers refunds of their previous year's income taxes
           including surtaxes and tax reduction but excluding non-refundable tax credits.  
           Since we do not have the previous year's taxes,
           we estimate them with a parameter */
        pytax = MP.OPYTAX * in->im.imtxp;
        /* taxpayers get up to OTXDVMAX back */
        otxdv = minn(pytax, MP.OTXDVMAX);
        /* taxpayers who paid over 0$ in taxes get at least OTXDVMIN back */
        if (pytax>0) {
            in->im.imotxdv = maxn(otxdv, MP.OTXDVMIN);
        }
        else {
            in->im.imotxdv = ZERO;
        }
        /* add to refundable tax credits */
        in->im.imptc += in->im.imotxdv;


		/**
		* Ontario Investment and worker ownership program tax credit,
		* which began in taxation year 1991, is
		* not implemented.
		**/

        /* Ont labour-sponsored venture capital credit */
        if ( isnzero(in->id.idlabtxg) ) {
           lvctc = minn(MP.OLVCMAX, MP.OLVCRT * in->id.idlabtxg);
           in->im.implvctc = minn(lvctc, in->im.imtxp);
           in->im.imnptc += in->im.implvctc;
           in->im.imtxp  -= in->im.implvctc;
        }
      

		/***
		 *** COMPUTE TAX CREDITS
		 ***/


        /* Property tax and sales tax credits */
		/* initialise temporary credit variables */
		oc = ZERO;
		prtc = ZERO;
		stc = ZERO;

        /* Some tax credit parameters differ for seniors for years
			in which they are eligible for tax credits
		*/
		if (in->id.idage>64 && MP.OSTCFLAG==1) {
		   DEBUG1("%s ont prptc: senior mprs\n");
			opocm   = MP.OSPOCM;
			ostcp   = MP.OSSTCP;
			if (in->id.idspoflg) {  /* different income turndown for couples */
				otcniba = MP.OSTCNIBC;
			}
			else {  /* single senior */
				otcniba = MP.OSTCNIBA;
			}
		}
		else {
		   DEBUG1("%s ont prptc: not-senior mprs\n");
			opocm   = MP.OPOCM;
			ostcp   = MP.OSTCP;
			otcniba = MP.OTCNIBA;
		}

		/* gross property tax credit */
		if ( (in->id.idage<65 || (in->id.idage>64 && MP.OSTCFLAG==1) )
			&& (isnzero(in->im.imrentpd)
			|| isnzero(in->im.improptx)
			|| isnzero(in->id.idhomstu))) {
		   DEBUG1("%s ont prptc: compute oc\n");
            oc = in->im.imrentpd * MP.OPRIR
				  	+ in->im.improptx;
	        if (isnzero(in->id.idhomstu)) {
		       oc += MP.OPCRA;
		    }
            prtc = minn(opocm, oc) + (MP.OPOCAR * oc);
		}
		else {
		   DEBUG1("%s ont prptc: skip oc calculation\n");
			prtc = ZERO;
		}
	     DEBUG3("%s ont prptc: prtc=%.2f  oc=%.2f\n", prtc, oc);


        /* gross sales tax credit */

		if ( in->id.idage<65 || (in->id.idage>64 && MP.OSTCFLAG==1)) {
			DEBUG1("%s ont saltc: based on family credits\n");
            if ( in->id.idcfrh==0 ) { /* associate to the cf head */
				stc= MP.OSTCB
				   + MP.OSTCS * in->id.idspoflg
				  	+ MP.OSTCD * nf->nfnkids;
			}
			else if (in->id.idcfrh==2 && in->id.idage>17) {
				/* associate to the non-dependent older kids */
				stc = MP.OSTCB;
			}
			else {
				stc = ZERO;
			}
        }
		else {
			DEBUG1("%s ont saltc: no credits for kids or seniors prev to 1992\n");
				stc = ZERO;
		}
	     DEBUG3("%s ont saltc: stc=%.2f nfnkids=%.2f\n", stc, nf->nfnkids);

		/* Ontario home ownership savings plan tax credit */
		/* don't worry about double counting benefits here */
		/*-------------------------------------------------------------------*/
		/* Following code commented out strating with Version 9.0 as there	*/
		/* were not enough records on the greenbook to impute hosspo/hosslf */
		/*-------------------------------------------------------------------*/
        /* hoscslf = minn(in->id.idhosslf,MP.OHCMAX);   */
        /* hoscspo = minn(in->id.idhosspo,MP.OHCMAX);	*/
		/* hoscont = hoscslf + hoscspo; 				*/

	    /* DEBUG3("%s ont hos: hosslf=%.2f hosspo=%.2f\n",				   */
	   	/*      in->id.idhosslf, in->id.idhosspo);					   */
	    /* DEBUG4("%s ont hos: hoscslf=%.2f hoscspo=%.2f hoscont=%.2f\n", */
	   	/*      hoscslf, hoscspo, hoscont);							   */


		/* Calculate NET Property and Sales Tax Credits */
		/* to do this we must first calculate the family net income */
		/* and income concept for HOSP credit */


		DEBUG1("%s ont net pstc: use family net income income concept\n");
	    if ( in->id.idcfrh==0 || (in->id.idcfrh==2 && in->id.idage>18)) {
					DEBUG1("%s ont net pstc: cf head and old kids\n");
	        if (in->id.idspoflg) { 
					DEBUG1("%s ont net pstc: spouse present\n");
			   	hosinet = nneg(in->im.imicnet + inspo->im.imicnet 
			   	             - in->im.imuccb - inspo->im.imuccb);
				/* although net fed supp, work comp and sa were not included
				   in net income in 1991, however, the tax form for 1991 indicates
				   that these amounts are added to net income for the tax credits */
				if (MP.TARGETYEAR == 1991) {
					hosinet += (in->im.imisa + in->im.imispa + in->im.imigis 
								+ in->id.idiworkc + inspo->im.imisa 
								+ inspo->im.imispa + inspo->im.imigis 
								+ inspo->id.idiworkc);
				}
				DEBUG2("%s ont net pstc: hosinet=%.2f\n", hosinet);
	        }
			else {    
					DEBUG1("%s ont net pstc: no spouse present\n");
	           	hosinet = nneg(in->im.imicnet - in->im.imuccb );
				/* although net fed supp, work comp and sa were not included
				   in net income in 1991, however, the tax form for 1991 indicates
				   that these amounts are added to net income for the tax credits */
				if (MP.TARGETYEAR == 1991) {
					hosinet += (in->im.imisa + in->im.imispa + in->im.imigis 
								+ in->id.idiworkc);
				}
				DEBUG2("%s ont net pstc: hosinet=%.2f\n", hosinet);
	        }
			faminet = nneg( hosinet - otcniba);
				DEBUG3("%s ont net pstc: hosinet=%.2f faminet=%.2f\n", hosinet, faminet);

		   	/* Calculate NET property and sales tax credits */
		   	pstc		= prtc + stc;
		   	pfaminet	= ostcp * faminet;

			/* Apply the maximum for seniors and non-seniors.  Announced in the 
			   Ontario 2004 Budget, seniors have a higher maximum than non-seniors. */

			if (in->id.idage>64 && MP.OSTCFLAG==1) {
		   	   DEBUG1("%s ont opstc: senior mprs\n");
		   	   netpstc = minn( MP.OSPSMAX, nneg( pstc - pfaminet ));
			}
			else {
			   DEBUG1("%s ont opstc: not-senior mprs\n");
	           netpstc = minn( MP.OPSMAX, nneg( pstc - pfaminet ));
			}

		   	/* calculate home ownership savings plan tax credit */
			/*-------------------------------------------------------------------*/
			/* Following code commented out starting with Version 9.0 as there	*/
			/* were not enough records on the greenbook to impute hosspo/hosslf */
			/*-------------------------------------------------------------------*/
		   	/*if (isnzero(hoscont)) {													*/
			/*		DEBUG1("%s ont hosptc: cf head\n");									*/
	      	/*	if ( in->id.idspoflg || isnzero(in->im.immartxc) ) {					*/
	        /*		hosinet *= ((NUMBER)1.0 - MP.OHIRF);								*/
	      	/*	}																		*/
		    /*	hostcr = (NUMBER) lkup1(MP.OHT, MP.OHTrows, hosinet);					*/
		    /*	hostc = hoscont * hostcr;												*/
			/*	DEBUG3("%s ont hosptc: hostcr=%.4f hosinet=%.2f\n",						*/
			/*			hostcr, hosinet);												*/
			/*	DEBUG3("%s ont hosptc: hostc=%.2f hoscont=%.2f\n",						*/
			/*			hostc, hoscont);												*/
	 		/*	}																		*/
			/*	else {																	*/
			/*		hostc = ZERO;														*/
			/*	}																		*/
		}																			
		else { 																	
			DEBUG1("%s ont net pstc: zero for dep spouses and kids\n");				
			netpstc = ZERO;															
		}																			
	 			
	 	in->im.imopstc = netpstc;																				
		in->im.imptc += netpstc;
		DEBUG5("%s ont net pstc: pstc=%.2f netpstc=%.2f faminet=%.2f otcniba=%.2f\n",
				   pstc, netpstc, faminet, otcniba);								 

	      /* Ontario Political Contribution Tax Credit */
	      /* note: this is a function of the net prop/sales tax credit when*/
		  /* OPTCFLAG is set to 0 for non-refundable tax credit */
		if (isnzero(in->id.idprvpol)) {
			DEBUG1("%s ont poltc\n");
			poltcr = (NUMBER) lkup1(MP.OPTC, MP.OPTCrows, in->id.idprvpol);
			poltc	= minn(MP.OPTCBEN, poltcr);
			if (!MP.OPTCFLAG) {   /* treat as a non-refundable credit */
				in->im.impptc  = minn(poltc, nneg(in->im.imtxp-netpstc));
				DEBUG4("%s ont pol: idpolcon=%.2f poltcr=%.2f poltc=%.2f\n",
	    	       	in->id.idprvpol, poltcr, poltc);
					/* apply non-refundable political contribution tax credit*/
				in->im.imnptc += in->im.impptc;
				in->im.imtxp -= in->im.impptc;
			}
			else { /* political contribution tax credit is refundable */
				in->im.impptc = poltc;
				/* add to refundable tax credits */
				in->im.imptc += in->im.impptc;
			}

		   		DEBUG3("%s ont pol: idpolcon =%.2f impptc =%.2f\n",
	           		in->id.idprvpol, in->im.impptc);
	    }
		else {
			in->im.impptc = ZERO;
		}


		/* Ontario Home Electricity Relief Program */

		/* Give the one time rebate to tax filers in fall of 2006, based on 2005 tax return */
		/* Give to the person who claimed the property tax credit */
		if (MP.OHEATFLG && isnzero(in->im.imopstc)) {
			heatinc = ZERO;
			heatthr = ZERO;
			if (in->id.idspoflg) {
				heatinc = MP.PYINC * (in->im.imicnet + inspo->im.imicnet);
				heatthr = MP.OHEATFTD;
				heatrel = MP.OHEATFAM;
			}
			else {
				heatinc = MP.PYINC * in->im.imicnet;
				heatthr = MP.OHEATSTD;
				heatrel = MP.OHEATSNG;
			}

            taxbak1(heatinc, heatthr, &heatrel, MP.OHEATRR);

			/* Minimum payment is $10. */
			if (heatrel > 0 && heatrel < MP.OHEATMIN) {
				heatrel = MP.OHEATMIN;
			}

			if (isnzero(heatrel)) {
				in->im.impheatr = heatrel;
			} 
			
		}

		/***
		 *** Ontario Employers Health Tax
		 ***/

		seinc= nneg(in->id.idise);

		if (MP.OEHTFLAG && (seinc > MP.OEHTEX))  {
		 
			/* This code should be equivalent to bit below (except 
			   doesn't account for lower limit) - keep for reference
				
				eht1 = minn((MP.OEHTTD1-MP.OEHTEX)*MP.OEHTRR1,maxn(0,seinc-MP.OEHTEX)*MP.OEHTRR1);
	  			eht2 = minn((MP.OEHTTD2-MP.OEHTTD1)*MP.OEHTRR2,(MP.OEHTRR2*maxn(0,(seinc-MP.OEHTTD1))));
	 			eht3 = maxn(0,MP.OEHTRR3*(seinc-MP.OEHTEX)-MP.OEHTRR3*(MP.OEHTTD2-MP.OEHTEX));
				eht   = eht1 + eht2 + eht3;
				*/
				

			if (seinc <= MP.OEHTTD1)
				eht = (seinc-MP.OEHTEX) * MP.OEHTRR1;
			else if ((seinc > MP.OEHTTD1) && (seinc <= MP.OEHTTD2))
				eht = ((maxn(0,(MP.OEHTTD1 - MP.OEHTEX))) * MP.OEHTRR1) +
				(MP.OEHTRR2 * (seinc - maxn(MP.OEHTTD1,MP.OEHTEX)));
			else
				eht = (seinc-MP.OEHTEX) * MP.OEHTRR3;

		    ehtded = eht * MP.OEHTDRR;

		    in->im.imonteht = eht-ehtded;
	          /* add to provincial taxes */
	        in->im.imtxp += in->im.imonteht;
		}

		/* Ontario Health Premium */
		
		if (in->im.imitax > 0 && MP.OHPFLAG) {
        	in->im.imphp = nneg((NUMBER) lkup1(MP.OHPBASE, MP.OHPBASErows, in->im.imitax)) * MP.OHPPI;
		}

		/* Ontario Textbook and Technology Grant */
		/* Announced in 2008 Budget, provides grants to full time college and university students */
		if (in->id.idestat == 1 && (in->id.idschtp == 2 || in->id.idschtp == 3) && in->id.ideducfm > 3) {
			in->im.imotxtgr = MP.OTXTGRANT;
		}
   }
   
   /*****  Ontario Child Benefit ******/
   /*  Program design for July 2007 to June 2008.
   *    one-time Benefit of $250 per dependent under age 18 */
   /* Program design for July 2008 onwards */
   /*  amount per dependent under age 18  */
   /*  amounts for dependents eligible for the OCCS are retained */
   /*  and are subtracted from the OCCS later on */

   if (MP.OCBFLAG) {
		/* process each census family in household */
		for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			
		   if (cf->cfnkids == 0) {
			   DEBUG1("%s skipping census family with no young `children'\n");
			   continue;
		   }

		   /* First we count up the families earnings, net income and child care expenses */
		   faminet = ZERO; 
		   occskids = ZERO;
		   ocbkids = ZERO;
		   amtocb2 = ZERO;
		   amtocb1 = ZERO;
		   amtocb = ZERO;
		   ocbyng1 = ZERO;
		   ocbyng2 = ZERO;
		   ocbyng = ZERO;

		   /* get elder's pointer */
		   ineld = cf->cfineld;
		   /* calculate net income */
	       faminet = nneg(ineld->im.imicnet - ineld->im.imuccb ) * MP.PYINC;

		   /* determine if spouse is present in census family and calculate family net income */
		   spoflg = cf->cfspoflg;
		   if (spoflg) {
		   		DEBUG1("%s spouse present\n");
				/* get spouse's pointer */
				inspo = cf->cfinspo;
				/* calculate net income */
	         	faminet += (nneg(inspo->im.imicnet - inspo->im.imuccb ) * MP.PYINC);
		   }

		   /* Second we count the number of eligible children in the census family */
		   /* process kids in census family in order to calculate number of kids under 7 */
		   for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++) {
				ini++;
			   
				// count the number of children eligible for the ontario
				//  child care supplement for working families

	            if (in->id.idage <= 6 ) {
    	            if (/* born in 2011 but before July  */
        	            (((MP.TARGETYEAR-MP.OCCSPOYR)==in->id.idage) && (in->id.idrand[7]<MP.OCCSPOPYR)) || 
            	        /* born before 2011 */
                	    ((MP.TARGETYEAR-MP.OCCSPOYR)<in->id.idage)) {
                    	       occskids += (NUMBER) 1;
                	}
            	}

				ocbkids += (NUMBER) 1.0;	
			}

			amtocb2 = ocbkids * MP.OCBAMT;

			/* Next we tax back the benefits based on family income */
			taxbak1(faminet, MP.OCBTD , &amtocb2, MP.OCBRR);

		   	/* apply the minimum amount */
		   	/* this is only applicable for the 2007 one-time payment */
		   	if (amtocb2>0 && MP.OCBMIN > 0) {
		   		amtocb2 = maxn(amtocb2, MP.OCBMIN);
		   	} 

		   	/* separate the amount for Ont Child Care Supplement for Working Families */
		   	ocbyng2 = amtocb2 * (occskids/ocbkids);

			/* now calculate for the calendar year if flagged on */

			if (MP.FCBENCAL) {
				
				/* update the income test for 2 years prior */
				faminet = (faminet / MP.PYINC) * MP.PYINCP;
				amtocb1 = ocbkids * MP.OCBAMTP;

				taxbak1(faminet, MP.OCBTDP, &amtocb1, MP.OCBRRP);

				ocbyng1 = amtocb1 * (occskids/ocbkids);

				/* take half of each calculation */
				/* for the year 2007, no OCCS clawback is implemented and 
				   the payment for 2007 is a one-time payment */
				if (MP.OCBPIP) { /* years after 2007, take half of each */
					amtocb = (amtocb1/2) + (amtocb2/2);
					
					ocbyng = (ocbyng1/2) + (ocbyng2/2);
				}
				else {  /* works for 2007, use the second half of year only */   
					amtocb = amtocb2;
					ocbyng = ocbyng2;
				}

			}
			else {

				amtocb = amtocb2;
				ocbyng = ocbyng2;
			}

			/* Give the benefit to the same person as the OCCS */

		    if (spoflg && (inspo->id.idsex == FEMALE)) {
				inspo->im.imocb = amtocb;
				inspo->im.imocbyng = ocbyng;
			}
			else {
				ineld->im.imocb = amtocb;
				ineld->im.imocbyng = ocbyng;
			}

		}
	}
			
			   	
	/**
	* Now calculate the Ontario refundable child care tax credit or
	* the Ontario Child Care Supplement for Working Families
	**/
	if (MP.OCCEAFLG || MP.OCWISFLG)  {
   	
		/* process each census family in household */
		for (cfi=0, cf=&hh->cf[0];  cfi<hh->hhncf;  cfi++, cf++) {
			
		   if (cf->cfnkids == 0) {
			   DEBUG1("%s skipping census family with no young `children'\n");
			   continue;
		   }


			/* First we count up the families earnings, net income and child care expenses */
			famearn = ZERO; 
			faminet = ZERO; 
			ynglim = ZERO;
			yngkids = ZERO;
			occsext = ZERO;	 
			ynglim1 = ZERO;
			occea = ZERO;
			occea1 = ZERO;
			owfis = ZERO;
			owfis1 = ZERO;
			ben = ZERO;
			ben1 = ZERO;
			ben2 = ZERO;

			/* get elder's pointer */
			ineld = cf->cfineld;
			/* calculate net income */
	        famearn = ineld->id.idiemp + ineld->id.idise;
	        faminet = nneg(ineld->im.imicnet - ineld->im.imuccb );

			
			/* family ccea */
			ccea = ineld->im.imccea;

			/* determine if spouse is present in census family and calculate family net income */
		  	spoflg = cf->cfspoflg;
			if (spoflg) {
				DEBUG1("%s spouse present\n");
				/* get spouse's pointer */
				inspo = cf->cfinspo;
				/* calculate net income */
	  	        famearn += inspo->id.idiemp + inspo->id.idise;
	         	faminet += nneg(inspo->im.imicnet - inspo->im.imuccb );
				/* family ccea */
				ccea += inspo->im.imccea;
			}

			/* Second we count the number of eligible children in the census family */
			/* process kids in census family in order to calculate number of kids under 7 */
			for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++) {
				ini++;

				/* Announced in the Ontario 2000 Budget, the Child Care Supplement for
				   Working Families was enhanced to include a new single parent's benefit 
				   which would increase the maximum annual benefit for single parents */

				/* subject to limits as a function of age */

				/* With the introduction of the Ontario Child Benefit, the 
				   Ontario Child Care Supplement for working families will 
				   gradually be phased out.  New regulations state the eligible 
				   dependents for the OCCS must be born before July 1, 2011. */

	            if (in->id.idage <= 6 ) {
    	            if (/* born in 2011 but before July  */
        	            (((MP.TARGETYEAR-MP.OCCSPOYR)==in->id.idage) && (in->id.idrand[7]<MP.OCCSPOPYR)) || 
            	        /* born before 2011 */
                	    ((MP.TARGETYEAR-MP.OCCSPOYR)<in->id.idage)) {
				    	/* check if this is a single parent family and apply the higher amount */
						if (!spoflg) {
					   		ynglim += MP.OCWISSP;
						}
						else {
					   		ynglim += MP.OCCEAYNG;
						}
						yngkids += (NUMBER) 1.0;

					}
				}

			}
			
			/**
			* Third we determine the child care expense related benefits for families 
			* In 1997 through June 1998 this refundable child care expense tax credit 
			* was the only Benefit available.  Starting July 1998 under the Ontario 
			* Child Care Supplement for Working Families, The child care expense provisions
			* apply to everyone.  For parents with income below the earnings threshold 
			* and who have child care expenses and who have attended school or obtained 
			* training are eligible for child care expense benefits. Families with both 
			* earnings over the threshold and child care expenses are eligible for the
			* greater of child care expenses credit and the working income supplement.
			**/

			/* first limit is the claimed child care expenses times OCCEAPCT, line 23 */
			occea = ccea * MP.OCCEAPCT;

			/* if occea is not zero then take the lower of it and the sum of limit per child */
			if (occea>0) {
				occea = minn(occea,ynglim);
			}
	    	
			/**
			* Fourth we calculate the working income supplement if activated and
			* then determine which is bigger; the CCE portion or the WIS portion
			* If the working income supplement is not active (and the CCE portion is)
			* then the benefit to be taxed back is defined as the CCE portion.
			**/

			if (MP.OCWISFLG)  {
			    
			    /* adjust family income to represent previous year income */
				famearn = famearn * MP.PYINC;
				faminet = faminet * MP.PYINC;

				owfis = minn(ynglim, 
						       nneg(famearn-MP.OCWISTD) * lkup2(MP.OCWISRR,MP.OCWISRRrows, (double) yngkids));
				/* Note that only child care expenses can only be claimed by full-time students.  
				   Otherwise, they only claim the earned income supplement.  We assume that students
				   used all their child care expenditures while studying */
				/* Also take into account months of part time study for years where 
				   it is used in the Federal Schedule 11 (1998 on) */
				if ((ineld->id.ideducfm>0) || (ineld->id.ideducpm>0 && MP.EDXPMPT > 0)) {
					ben2 = maxn(occea, owfis);
				}
				else if (spoflg) {
					if ((inspo->id.ideducfm>0) || (inspo->id.ideducpm>0 && MP.EDXPMPT > 0)) {
						ben2 = maxn(occea, owfis);
					}
					else ben2=owfis;
				}
				else {
					ben2 = owfis;
				}
			}
			else {
				ben2 = occea;
			}


			/* Fifth we tax back the benefits based on family income */
			taxbak1(faminet, MP.OCCEATD , &ben2, MP.OCCEARR);
			
			if (MP.FCBENCAL) {  /* calculate for the year */

				yngkids = ZERO;

				/* Second we count the number of eligible children in the census family */
				/* process kids in census family in order to calculate number of kids under 7 */
				for (ini=0, in=cf->cfinch;  ini<cf->cfnkids;  in++) {
					ini++;

					/* Announced in the Ontario 2000 Budget, the Child Care Supplement for
					   Working Families was enhanced to include a new single parent's benefit 
					   which would increase the maximum annual benefit for single parents */

					/* subject to limits as a function of age */

	            	if (in->id.idage <= 6 ) {
    	            	if (/* born in 2011 but before July  */
        	            	(((MP.TARGETYEAR-MP.OCCSPOYR)==in->id.idage) && (in->id.idrand[7]<MP.OCCSPOPYR)) || 
            	        	/* born before 2011 */
                	    	((MP.TARGETYEAR-MP.OCCSPOYR)<in->id.idage)) {
				    		/* check if this is a single parent family and apply the higher amount */
							if (!spoflg) {
						   		ynglim1 += MP.OCWISSPP;
							}
							else {
						   		ynglim1 += MP.OCCEAYNGP;
							}
							yngkids += (NUMBER) 1.0;
						}
					}

				}
				
				/**
				* Third we determine the child care expense related benefits for families 
				* In 1997 through June 1998 this refundable child care expense tax credit 
				* was the only Benefit available.  Starting July 1998 under the Ontario 
				* Child Care Supplement for Working Families, The child care expense provisions
				* apply to everyone.  For parents with income below the earnings threshold 
				* and who have child care expenses and who have attended school or obtained 
				* training are eligible for child care expense benefits. Families with both 
				* earnings over the threshold and child care expenses are eligible for the
				* greater of child care expenses credit and the working income supplement.
				**/

				/* first limit is the claimed child care expenses times OCCEAPCT, line 23 */
				occea1 = ccea * MP.OCCEAPCTP;

				/* if occea is not zero then take the lower of it and the sum of limit per child */
				if (occea1>0) {
					occea1 = minn(occea1,ynglim1);
				}
		    	
				/**
				* Fourth we calculate the working income supplement if activated and
				* then determine which is bigger; the CCE portion or the WIS portion
				* If the working income supplement is not active (and the CCE portion is)
				* then the benefit to be taxed back is defined as the CCE portion.
				**/

				if (MP.OCWISFLGP)  {
				    
				    /* adjust family income to represent income from 2 years prior */
					famearn = (famearn / MP.PYINC ) * MP.PYINCP;
					faminet = (faminet / MP.PYINC ) * MP.PYINCP;
					
					owfis1 = minn(ynglim1, 
							       nneg(famearn-MP.OCWISTDP) * lkup2(MP.OCWISRRP,MP.OCWISRRProws, (double) yngkids));
					/* Note that only child care expenses can only be claimed by full-time students.  
					   Otherwise, they only claim the earned income supplement.  We assume that students
					   used all their child care expenditures while studying */
					/* Also take into account months of part time study for years where 
					   it is used in the Federal Schedule 11 (1998 on) */
					if ((ineld->id.ideducfm>0) || (ineld->id.ideducpm>0 && MP.EDXPMPT > 0)) {
						ben1 = maxn(occea1, owfis1);
					}
					else if (spoflg) {
						if ((inspo->id.ideducfm>0) || (inspo->id.ideducpm>0 && MP.EDXPMPT > 0)) {
							ben1 = maxn(occea1, owfis1);
						}
						else ben1=owfis1;
					}
					else {
						ben1 = owfis1;
					}
				}
				else {
					ben1 = occea1;
				}


				/* Fifth we tax back the benefits based on family income */
				taxbak1(faminet, MP.OCCEATDP , &ben1, MP.OCCEARRP);

				/* take half of each calculation to report for the calendar year */
				ben = (ben1/2) + (ben2/2);
		 	}
		 	else {   /* use the July to June amounts for the year */
		 		ben = ben2;
		 	}		 		

    		/* Sixth and lastly we assign the benefits									  */
    		/* determine who receives occea												  */
			/* Benefits first go to the parent who received the federal child tax benefit */
			/* Ontario child care supplement for working families rules                   */
			/* (i.e. the mother if present, or the head)								  */
			/* We do not test based on fcben as the user may disable fcben and still      */
			/* wish to retain the Ontario Program                                         */
			/* Users note that the rules are applied retroactively to the 1997 CCE credit */
			/* That 1997 credit could be applied to either the head or the spouse and was */
			/* originally implemented in 6.0 as an arbitrary assignment based on          */
			/* the receipt of federal ccea and higher net income.  The female in the      */
			/* house rule was explicit in 1998 and to aviod spurious individual losers    */
			/* we apply the same rules to both.											  */
		    if (spoflg && (inspo->id.idsex == FEMALE)) {
				recip = inspo;
			}
			else {
				recip = ineld;
			}

 			/***
			Announced in the Ontario 2001 Ontario Economic Outlook, the Ontario government
			provided a one time payment of $100 per child under 7 to families
			eligible for the Ontario Child Care Supplement.  Families eligible
			for at least $1 of the Supplement received $100 per child under 7
			in November 2001.  
			***/
			
			if (ben > 0) {
				occsext = MP.OCCSEXT * yngkids;
			}
			recip->im.imoccext = occsext; 

			/* New with the introduction of the Ontario Child Benefit */
			/* reduce the OCCS by the amount of Ontario Child Benefit
			   allocated to children eligible for the OCCS */

			if (MP.OCCSOCB) {
				ben = maxn(ben - recip->im.imocbyng, 0); 
			}

			/* Test if below minimum amount and give minimum */
			if (ben > 0 && ben < MP.OCWISMIN) {
				ben = MP.OCWISMIN;
			}

			recip->im.imoccea = ben;

		}
	}

	/* Ontario Senior Homeowners' Property Tax Grant */
	/* Announced in the 2008 Budget, grants to begin in 2009 based on 2008 income and property taxes */
	
	if (MP.OSHPTGFLG) {
		if (hh->hd.hdageeld < 65) {
			DEBUG1("%s eldest in household is under 65\n");
			return;
		}
		 
		gotgrant = 0;

		/* process each person in household */
		for (ini=0, in=&hh->in[0]; ini<hh->hhnin; ini++, in++) {

			/* initialize spouse pointer */
	   		inspo = in->id.idinspo;

			/* compute family GIS tax-back income */
			inctest = nneg(in->im.imicnet - in->im.imuccb) * MP.PYINC;
			proptax = in->im.improptx * MP.PYINC;
			if (in->id.idspoflg) {
				inctest += nneg(inspo->im.imicnet - inspo->im.imuccb) * MP.PYINC;
				proptax += inspo->im.improptx * MP.PYINC;
			}  


			if (in->id.idspoflg) {
				/* treat as a couple */
				if ((in->id.idage > 64 || inspo->id.idage > 64) && proptax >= MP.OSHPTGMPT
					&& inctest < MP.OSHPTGPOC) {
					/* eligible for a grant - check if maximum can be applied if income less than turndown */
					if (inctest < MP.OSHPTGTDC) {
						ptgrant = MP.OSHPTGAMT;
					}
					else {		/* apply a reduced grant */
						ptgrr = MP.OSHPTGAMT / (MP.OSHPTGPOC - MP.OSHPTGTDC);
						ptgrant = nneg(MP.OSHPTGAMT - nneg((inctest - MP.OSHPTGTDC) * ptgrr));
					}
				}
			}
			else { /* no spouse present */
				if (in->id.idage > 64 && proptax >= MP.OSHPTGMPT && inctest < MP.OSHPTGPOS) {
					/* eligible for a grant - check if maximum can be applied if income less than turndown */
					if (inctest < MP.OSHPTGTDS) {
						ptgrant = MP.OSHPTGAMT;
					}
					else {		/* apply a reduced grant */
						ptgrr = MP.OSHPTGAMT / (MP.OSHPTGPOS - MP.OSHPTGTDS);
						ptgrant = nneg(MP.OSHPTGAMT - nneg((inctest - MP.OSHPTGTDS) * ptgrr));
					}
				}
			}
					
			/* assign the grant to the person with the higher net income and only if not claimed by this household */

			if (gotgrant == 0 && ptgrant > 0) {
				if ( in->id.idspoflg && (inspo->im.imicnet > in->im.imicnet)) {
					inspo->im.imoshptg = ptgrant;
					gotgrant = 1;
				}
				else {
					in->im.imoshptg = ptgrant;
					gotgrant = 1;
				}
			}
		}
	}						
			



	DEBUG_OFF("txont");
}
