/***
*mpd1.cpp - Tax/transfer model parameter definitions
*
*$Id: mpd1.cpp 6.155 2008/08/22 17:07:44 hickcha Exp $
*
*   SPSD/M Copyright (c) 1987-1995, Minister of Industry.
*   All rights reserved.  Use, reproduction, or disclosure of this work,
*   as well as publication of results derived from this work, are subject
*   to the conditions specified in the SPSD/M License Agreement.
*
*Purpose:
*   This file contains the definition of the .c function.
*
*Revisions:
* +-----+----------+---------------------------------------------------------+
* | Who |   Date   | Description                                             |
* +-----+----------+---------------------------------------------------------+
* | CH  | 08/06/12 | Increased width of  UIEITKP                             |
* | LP  | 08/06/04 | Added GIS employment exemption GISEMPEXP & GISEMPEXM    |
* | CH  | 08/04/22 | Added MPRDESCE and MPRDESCF, deleted MPRDESC            |
* | LP  | 08/04/14 | Changed the format for OASTD to 9.2                     |
* | CH  | 07/03/12 | Added labels to UIEWK and QNBFA                         |
* | LP  | 06/06/14 | Removed OASTBPI - OAS tax back completely phased in 91  |
* | LP  | 06/05/23 | Added previous years' parms for Alta/Quebec child ben.  |
* | CH  | 06/01/19 | Added QPIPEADJ, deleted QPIPEARN                        |
* | CH  | 06/01/04 | Deleted QPIPSEES                                        |
* | LP  | 05/12/20 | Added Quebec Parental Insurance Plan parameters         |
* | LP  | 05/11/17 | Removed pre 1991 UI parameters                          |
* | BBM | 05/01/11 | Added MPRSFILE                                          |
* | BBM | 04/11/09 | Added MPRVER                                            |
* | CH  | 03/10/17 | Deleted FORAVGFLAG                                      |
* | LP  | 01/07/10 | Added UIEIREFTK take up for EI contribution refund      |
* | CH  | 01/06/12 | Changed EI code to incorporate 9.0 database variables   | 
* | LP  | 01/03/12 | Removed pre-1988 program parameters                     |
* | LP  | 01/03/05 | Removed Seniors Benefits parameters                     |                                          
* | CH  | 00/12/19 | Family supplement additions                             |                                          
* | CH  | 00/10/24 | Added UIEIREPOPT                                        |                                          
* | CH  | 00/05/08 | Removed FDFATCFLAG                                      |                                          
* | CH  | 00/05/01 | Added flag for type and modified training               |                                          
* | CH  | 00/04/06 | Add    QNBPOFLAG QNBPOPYR QNBPOYR                       |
* | CH  | 00/03/30 | Add QCBCFLG                                             |
* | EGA | 99/05/25 | UITARGET by province                                    |
* | EGA | 99/05/21 | Modification of UIEITKP                                 |
* | CH  | 99/03/26 | Add GSTREBFLAG                                          |
* | EGA | 99/03/25 | Add UITARGET                                            |
* | CH  | 99/03/22 | Drop LABTXCFLAG: program is now modelled                |
* | BBM | 99/03/09 | Add UIMAXPARWKS for parental benefits                   |
* | BBM | 99/02/99 | Drop FSITCFLAG, XII2FLAG: id variables no longer exist  |
* | TCB | 98/05/28 | Renamed *.cpp, ANSI function headers                    |
* | EGA | 98/05/21 | Added UIEITKP                                           |
* | BBM | 98/05/11 | Add GISPTDFLAG parameter                                |
* | CH  | 98/04/01 | Added FORAVGFLAG FDFATCFLAG     |
* | EGA | 98/03/17 | QNORTHFLAG QPARTLOFLAG QEMPLOFLAG Moved to mpd3 and     |
* |     |          | import QFAIFALG and QNBRFLAG from mpd3.                 |
* | CH  | 97/10/02 | Add UIEIFSRR UIEIFSRL                                   |
* | EGA | 97/09/10 | Add QNORTHFLAG QPARTLOFLAG QEMPLOFLAG                   |
* | CH  | 97/08/21 | Removed SBTURFLAG SBST SBCT SBOT SBSPAT SBESPAT         |
* | CH  | 97/08/13 | Added UIFSHERMIN                                        |
* | CH  | 97/06/18 | Removed UIWBP5 and UIWBP5R                              |
* | CH  | 97/05/29 | Added UIEIRPR                                           |
* | BBM | 97/05/27 | Drop parameters STKCCFFLAG and RECSSPFLAG(no data in 92)|
* | CH  | 97/04/07 | Added new EI parameters                                 |
* | BBM | 96/08/28 | Add parameter to net clawback from OAS (NETOASFLAG)     |
* | BBM | 96/05/09 | Add Suite of parameters for Seniors Benefit             |
* | GJC | 94/04/05 | add 1994 UI reform parms                                |
* | RSE | 94/02/22 | add Alberta and Quebec child benefit parameters         |
* | GJC | 93/03/19 | fix flags so new ui parm are saclars 0 to 1             |
* | GJC | 93/03/08 | add duration factors for ui modeling                    |
* | GJC | 93/02/17 | Added database variable activation flags                |
* | BBM | 92/07/07 | Added PFACFLAG to disable prov. FA configurations       |
* | BBM | 91/07/05 | Changed order/labels of Quebec FA parms                 |
* | SFG | 90/03/02 | Redistributed parms among mpd[123].c cuz mpd1.c too big |
* | --- | 89/10/13 | Release 4.0                                             |
* |     |          |                                                         |
* +-----+----------+---------------------------------------------------------+
*
*Definitions:
*   mpd1     [global]  Tax/transfer model parameter definitions
*
*****************************************************************************/


/* ==================== C-RUNTIME LIBRARY INCLUDE FILES =================== */

#include <stdio.h>  /* definitions/declarations for standard I/O routines   */


/* ==================== SPSD/M INCLUDE FILES ============================== */

#include "spsm.h"   /* SPSM common manifest constants and macros            */
#include "sub.h"    /* Subroutine library declarations                      */
#include "pm.h"     /* Parameter manipulation package declarations          */
#include "mpu.h"    /* Tax/transfer parameter declarations (user's)         */
#include "mp.h"     /* Tax/transfer parameter declarations                  */


/* ==================== STATIC VARIABLE DEFINITIONS ======================= */

static char NEAR F_WEEKS[]  = "3";
static char NEAR F_FRACT[]  = "8.5";
static char NEAR F_LKTUR[]  = "8.0 8.3 10.4";
static char NEAR F_LKTURI[] = "8.0 8.0 10.4";

/* ==================== COMPILER PRAGMAS ================================== */

// eliminate spurious 'near/far mismatch' from MS-DOS compiler
#if defined(_MSDOS)
#pragma warning(disable : 4762)
#endif

#ifdef MSCOVL
#pragma alloc_text(INIT_TEXT,mpd1)
#endif


/* ==================== FUNCTION DEFINITIONS ============================== */


/*****************************************************************************
*mpd1 - Tax/transfer model parameter definitions
*
*Synopsis:
*    void mpd1(           
*    P_pc pcp             
*    )                    
*
*Description:
*
*Return Value:
*   None.
*
*Calls:
*   pmaddent Define and add a parameter
*
*Structure Members:
*   MP.ACBC1        (i)  Alberta Child benefit per child aged 0 - 6
*   MP.ACBC1P       (i)  Alberta Child benefit per child aged 0 - 6 for 1st half of year
*   MP.ACBC2        (i)  Alberta Child benefit per child aged 7 - 11
*   MP.ACBC2P       (i)  Alberta Child benefit per child aged 7 - 11 for 1st half of year
*   MP.ACBC3        (i)  Alberta Child benefit per child aged 12- 15
*   MP.ACBC3P       (i)  Alberta Child benefit per child aged 12- 15 for 1st half of year
*   MP.ACBC4        (i)  Alberta Child benefit per child aged 16- 17
*   MP.ACBC4P       (i)  Alberta Child benefit per child aged 16- 17 for 1st half of year
*   MP.AFAC1        (i)  Alberta FA benefit per child aged 0 - 6
*   MP.AFAC2        (i)  Alberta FA benefit per child aged 7 - 11
*   MP.AFAC3        (i)  Alberta FA benefit per child aged 12- 15
*   MP.AFAC4        (i)  Alberta FA benefit per child aged 16- 17
*   MP.BESPA        (i)  Basic GIS portion of extended SPA
*   MP.BGISM        (i)  Basic GIS - married
*   MP.BGISS        (i)  Basic GIS - single
*   MP.BOAS         (i)  Basic OAS
*   MP.ESPAT        (i)  Extended SPA take-up rate by benefit level [benefit,rate]
*   MP.ESPATrows    (i)  number of rows in ESPAT table
*   MP.FAFLAG       (i)  Family allowance flag
*   MP.FARR         (i)  Family allowance repayment rate
*   MP.FATBPI       (i)  Family allowance take-back phase in
*   MP.FATD         (i)  Family allowance turndown income
*   MP.GISCT        (i)  GIS take-up rate: pensioner couple by benefit level [benefit,rate]
*   MP.GISCTrows    (i)  number of rows in GISCT table
*   MP.GISEMPEXM    (i)  GIS employment income exemption - maximum
*   MP.GISEMPEXP    (i)  GIS employment income exemption - percentage of earnings
*   MP.GISFLAG      (i)  Federal GIS/SPA/ESPA flag
*   MP.GISOASFLAG   (i)  1984 GIS top-up to OAS residence shortfall flag
*   MP.GISOT        (i)  GIS take-up rate: one pensioner couple by benefit level [benefit,rate]
*   MP.GISOTrows    (i)  number of rows in GISOT table
*   MP.GISRLM       (i)  Basic GIS reduction level: married pensioners
*   MP.GISRLS       (i)  Basic GIS reduction level: single pensioners
*   MP.GISRRM       (i)  Basic GIS reduction rate: married pensioners
*   MP.GISRRS       (i)  Basic GIS reduction rate: single pensioners
*   MP.GISST        (i)  GIS take-up rate: single pensioner by benefit level [benefit,rate]
*   MP.GISSTrows    (i)  number of rows in GISST table
*   MP.GISTURFLAG   (i)  GIS take-up flag: apply 5 take-up tables
*   MP.GSTREBFLAG   (i)  Database variable(gstreb) activation flag
*   MP.MPRDESCE     (i)  English description of tax/transfer parameter file
*   MP.MPRDESCF     (i)  French description of tax/transfer parameter file
*   MP.MPRSFILE     (i)  Starting tax/transfer parameter file
*   MP.MPRVER       (i)  SPSD/M Release Version
*   MP.NETOASFLG    (i)  Net the clawback from Old Age Security
*   MP.OASFLAG      (i)  Old age security flag
*   MP.OASRR        (i)  OAS reduction rate
*   MP.OASTD        (i)  OAS turndown income
*   MP.PFACFLAG     (i)  Use provincial configuration flag
*   MP.PYINC        (i)  Deflator to calculate previous year income
*   MP.PYINCP       (i)  Deflator to calculate income from 2 years prior
*   MP.QAAFA        (i)  Quebec availability allowance (supplement) [parity,$]
*   MP.QAAFAFLAG    (i)  Availability Allowance: Supplement to Quebec FA
*   MP.QAAFArows    (i)  number of rows in QAAFA
*   MP.QCBC1        (i)  Quebec Child benefit for 1st child
*   MP.QCBC1P       (i)  Quebec Child benefit for 1st child for 1st half of year
*   MP.QCBC2        (i)  Quebec Child benefit for 2nd child
*   MP.QCBC2P       (i)  Quebec Child benefit for 2nd child for 1st half of year
*   MP.QCBC3        (i)  Quebec Child benefit for 3rd child
*   MP.QCBC3P       (i)  Quebec Child benefit for 3rd child for 1st half of year
*   MP.QCBCFLG      (i)  Quebec Child benefit flag for Quebec calculation methodology
*   MP.QCBCFLGP     (i)  Quebec Child benefit flag for Quebec calculation methodology for 1st half of year
*   MP.QCBCS        (i)  Quebec Child benefit supplement child aged 12- 17
*   MP.QCBCSP       (i)  Quebec Child benefit supplement child aged 12- 17 for 1st half of year
*   MP.QFAIFLAG     (i)  Quebec inclusion of federal family allowance in income flag
*   MP.QFFSL        (i)  Federal family allowance: Quebec Configuration[parity,$]
*   MP.QFFSLrows    (i)  number of rows in QFFSL
*   MP.QFPSL        (i)  Quebec provincial family allowance [parity,$]
*   MP.QFPSLrows    (i)  number of rows in QFPSL
*   MP.QFS          (i)  Federal supplement per child 12-17 in Quebec configuration
*   MP.QNBFA        (i)  Quebec newborn family allowance [parity,age]
*   MP.QNBFAFLAG    (i)  Quebec newborn family allowance flag
*   MP.QNBFArows    (i)  number of rows in QNBFA
*   MP.QNBPOFLAG    (i)  Quebec newborn family allowance phase out flag
*   MP.QNBPOPYR     (i)  Quebec newborn family allowance phase-out year proportion
*   MP.QNBPOYR      (i)  Quebec newborn family allowance phase out year
*   MP.QNBRFLAG     (i)  Quebec newborn family random increase of child rank
*   MP.QPIPBMTR     (i)  Quebec parental insurance plan maternity weeks benefit rate - basic plan
*   MP.QPIPBMTW     (i)  Quebec parental insurance plan maximum maternity weeks - basic plan
*   MP.QPIPBPRR1    (i)  Quebec parental insurance plan parental initial weeks benefit rate - basic plan
*   MP.QPIPBPRR2    (i)  Quebec parental insurance plan parental additional weeks benefit rate - basic plan
*   MP.QPIPBPRW1    (i)  Quebec parental insurance plan maximum parental initial weeks - basic plan
*   MP.QPIPBPRW2    (i)  Quebec parental insurance plan maximum parental additional weeks - basic plan
*   MP.QPIPBPTR     (i)  Quebec parental insurance plan paternity benefit rate - basic plan
*   MP.QPIPBPTW     (i)  Quebec parental insurance plan maximum paternity weeks - basic plan
*   MP.QPIPEADJ     (i)  Quebec parental insurance plan earnings adjustment
*   MP.QPIPEADJrows (i)  number of rows in QPIPEADJ
*   MP.QPIPEMAX     (i)  Quebec parental insurance plan maximum insurable earnings
*   MP.QPIPEMIN     (i)  Quebec parental insurance plan minimum insurable earnings
*   MP.QPIPF        (i)  Quebec parental insurance plan contribution rate for paid workers
*   MP.QPIPFLAG     (i)  Quebec parental insurance plan activation flag
*   MP.QPIPFSE      (i)  Quebec parental insurance plan contribution rate for self-employed
*   MP.QPIPSMTR     (i)  Quebec parental insurance plan maternity benefit rate - special plan
*   MP.QPIPSMTW     (i)  Quebec parental insurance plan maximum maternity weeks - special plan
*   MP.QPIPSPRR     (i)  Quebec parental insurance plan parental benefit rate - special plan
*   MP.QPIPSPRW     (i)  Quebec parental insurance plan maximum parental weeks - special plan
*   MP.QPIPSPTR     (i)  Quebec parental insurance plan paternity benefit rate - special plan
*   MP.QPIPSPTW     (i)  Quebec parental insurance plan maximum paternity weeks - special plan
*   MP.QPIPTU       (i)  Quebec parental insurance plan option take up
*   MP.QUIPF        (i)  Quebec EI contribution rate
*   MP.SPAOASRR     (i)  OAS portion of SPA taxback rate
*   MP.SPARL        (i)  SPA reduction point: one married/ widowed
*   MP.SPAT         (i)  SPA take-up rate by benefit level [benefit,rate]
*   MP.SPATrows     (i)  number of rows in SPAT table
*   MP.STDFA        (i)  Standard federal family allowance per child
*   MP.TARGETYEAR   (i)  Year of analysis
*   MP.UIBAF        (i)  UI benefit recovery base amount factor (UI and EI)
*   MP.UIBAFNR      (i)  UI EI benefit recovery base amount factor for non-repeaters (EI only)
*   MP.UIBASFLAG    (i)  Basic phase calculation flag (UI and EI)
*   MP.UIBASRATE    (i)  Benefit rate for basic phase (UI only)
*   MP.UIBRP        (i)  UI benefit recovery portion (UI only)
*   MP.UIDEPOPT     (i)  UI dependency option [1=normal, 2=Feb'94] (UI only)
*   MP.UIEFFFLAG    (i)  Observed effective weekly benefit rate flag (UI and EI)
*   MP.UIEIBRP      (i)  Maximum clawback under EI reform [Past Wks ben][Rate] (EI only)
*   MP.UIEIBRProws  (i)  number of rows in UIEIBRP
*   MP.UIEIDIV      (i)  EI minimum divisor [uer][divisor] (EI only)
*   MP.UIEIDIVFLG   (i)  EI minimum divisor flag (EI only)
*   MP.UIEIDIVrows  (i)  number of rows in UIEIDIV
*   MP.UIEIFIFLG    (i)  EI intensity rule exemption for family supplement receivers (EI only)
*   MP.UIEIFS1      (i)  Max fam supplement 1 child (EI only)
*   MP.UIEIFS1rows  (i)  number of rows in UIEIFS1
*   MP.UIEIFS2      (i)  Max fam supplement 2 children (EI only)
*   MP.UIEIFS2rows  (i)  number of rows in UIEIFS2
*   MP.UIEIFS3      (i)  Max fam supplement 3+ children (EI only)
*   MP.UIEIFS3rows  (i)  number of rows in UIEIFS3
*   MP.UIEIFSFLG    (i)  UI EI calculate family supplement flag (EI only)
*   MP.UIEIFSOPT    (i)  UI EI calculate family supplement option (EI only)
*   MP.UIEIFSRL     (i)  UI EI family supplement reduction level(EI only)
*   MP.UIEIFSRR     (i)  UI EI family supplement reduction rate (EI only)
*   MP.UIEIFSTOPUP  (i)  Max fam supplement top up for more than 3 kids (EI only)
*   MP.UIEIFSTOPUProws (i)  number of rows in UIEIFSTOPUP
*   MP.UIEIFSYNG    (i)  Max fam supplement top up for young children (EI only)
*   MP.UIEIFSYNGrows (i)  number of rows in UIEIFSYNG
*   MP.UIEIMFSP     (i)  EI Maximum family supplement percent of earnings (EI only)
*   MP.UIEIMTYPFLG  (i)  Flag to model main benefit type (UI and EI)
*   MP.UIEINEREFLG  (i)  Flag to check for hours for new entrants and re-entrants (EI only)
*   MP.UIEINRT      (i)  UI EI Non repeater level for higher clawback turndown (EI only)
*   MP.UIEIOPT      (i)  UI Employment Insurance reform option [1=UI, 2=EI Dec'95]
*   MP.UIEIRATE     (i)  Benefit rate under EI reform [Past Wks ben][Rate] (EI only)
*   MP.UIEIREF      (i)  EI contribution refund cut-in (EI only)
*   MP.UIEIREFTK    (i)  EI contribution refund take up
*   MP.UIEIREPOPT   (i)  EI Benefit repayment option (1=repeaters have higher rate,2=one rate)(EI only)
*   MP.UIEIRPR      (i)  UI EI benefit clawback rate (EI only)
*   MP.UIEIRrows    (i)  number of rows in UIEIRATE
*   MP.UIEITKP      (i)  UI EI takeup rate [index] : index based on prov, sex, age, weeks
*   MP.UIEITKProws  (i)  number of rows in UIEITKP
*   MP.UIEIYRS      (i)  Number of years of previous EI benefits calculated (EI only)
*   MP.UIENSRATE    (i)  Enhanced rate for basic phase (UI only)
*   MP.UIENSRTCO    (i)  Enhanced rate cutoff (UI only)
*   MP.UIENTFLAG    (i)  Basic entrance requirements flag (UI and EI)
*   MP.UIERNMAX     (i)  Maximum insurable earnings (UI and EI)
*   MP.UIEWK        (i)  Entitlement (weeks) [wkwrk, uer] (UI and EI)
*   MP.UIEWKrows    (i)  number of rows in UIEWK
*   MP.UIEXTMATWKS  (i)  Additional weeks for maternity - behavioural response (UI and EI)
*   MP.UIFLAG       (i)  UI/EI Activation flag (UI and EI)
*   MP.UIFSHERMIN   (i)  Min. earnings to qualify for fish. benefits [unempl. rate] (EI only)
*   MP.UIFSHErows   (i)  number of rows in UIFSHERMIN
*   MP.UIFSHMINWK   (i)  Minimum weeks to qualify for fishing benefits (UI only)
*   MP.UILFERATE    (i)  Benefit rate for labour force extended phase (UI only)
*   MP.UIMATHRMIN   (i)  Minimum hours to qualify for maternity benefits (EI only)
*   MP.UIMATMINWK   (i)  Minimum weeks to qualify for maternity benefits (UI only)
*   MP.UIMAXDUR     (i)  Maximum duration of a UI claim (UI and EI)
*   MP.UIMAXFSHWKS  (i)  Maximum number of weeks - fishing (UI and EI)
*   MP.UIMAXMATWKS  (i)  Maximum number of weeks - maternity (UI and EI)
*   MP.UIMAXPARWKS  (i)  Maximum number of weeks - parental (UI and EI)
*   MP.UIMAXSICWKS  (i)  Maximum number of weeks - sickness (UI and EI)
*   MP.UIMINPCT     (i)  Exemption limit (percent of maximum insurable earnings) (UI only)
*   MP.UINEREHRMIN  (i)  Minimum hours to qualify for new entrant or re-entrant (EI only)
*   MP.UIOTHBNFCT   (i)  UI other benefits per week adjustment factor (EI and EI)
*   MP.UIOTHWKFCT   (i)  UI other benefits weeks adjustment factor (UI and EI)
*   MP.UIPF         (i)  UI contribution rate on earnings (UI and EI)
*   MP.UIREGHRMIN   (i)  Min. hours to qualify for reg. benefits [unempl. rate] (EI only)
*   MP.UIREGMINWK   (i)  Minimum weeks to qualify for regular benefits (UI only)
*   MP.UIREGWKFCT   (i)  UI regular benefit weeks adjustment factor (UI and EI)
*   MP.UIRGERATE    (i)  Benefit rate for regional extended phase (UI only)
*   MP.UIRMHRrows   (i)  number of rows in UIREGHRMIN
*   MP.UISICHRMIN   (i)  Minimum hours to qualify for sickness benefits (EI only)
*   MP.UISICMINWK   (i)  Minimum weeks to qualify for sickness benefits (UI only)
*   MP.UITARGET     (i)  UI/EI target for the number of cases (UI and EI)
*   MP.UITARGETrows (i)  number of rows in UITARGET
*   MP.UITRNBNFCT   (i)  UI training benefit per week adjustment factor (EI and EI)
*   MP.UITRNWKFCT   (i)  UI training benefit weeks adjustment factor (UI and EI)
*   MP.UIWAITWKS    (i)  Minimum waiting period all claims (UI and EI)
*
*Called by:
*   mpd      Tax/transfer model parameter definitions
*
*****************************************************************************/
void mpd1(
	P_pc pcp
)
{
	DEBUG_ON("mpd1");

	/* Government Transfers and Personal Income Taxes */
	pmaddent(pcp, "2.3",       NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);

	/* Variant Description */
	pmaddent(pcp, "2.3.0",     NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "MPRDESCE", (char *)&MP.MPRDESCE[0], NULL,    P_SCL, C_STR, 0,      0, NULL, LABSIZE);
	pmaddent(pcp, "MPRDESCF", (char *)&MP.MPRDESCF[0], NULL,    P_SCL, C_STR, 0,      0, NULL, LABSIZE);
	pmaddent(pcp, "MPRVER",  (char *)&MP.MPRVER,     NULL,    P_SCL, C_NUM, E_NOCH,    0, NULL, 0);
	pmaddent(pcp, "MPRSFILE",(char *)&MP.MPRSFILE[0], NULL,   P_SCL, C_STR, E_NOCH,    0, NULL, LABSIZE);
	pmaddent(pcp, "TARGETYEAR",(char *)&MP.TARGETYEAR,NULL,   P_SCL, C_INT,    0,      0, NULL, 0);

	/* Database variable activation flags */
	pmaddent(pcp, "2.3.1",     NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "GSTREBFLAG",   (char *)&MP.GSTREBFLAG,   NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	/* Government Transfers and Personal Income Taxes */
	/*     Government Transfers */
	pmaddent(pcp, "2.3.2",     NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);

	/*          Unemployment Insurance */
	pmaddent(pcp, "2.3.2.1",   NULL,                    NULL, P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIFLAG",   (char *)&MP.UIFLAG,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "UIERNMAX", (char *)&MP.UIERNMAX,     NULL, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIMINPCT", (char *)&MP.UIMINPCT,  F_FRACT, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIPF",     (char *)&MP.UIPF,      F_FRACT, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIEIREF",  (char *)&MP.UIEIREF,      NULL, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIEIREFTK", (char *)&MP.UIEIREFTK,  F_FRACT, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UITARGET", (char *)&MP.UITARGET[0], F_FRACT, P_VCT,   C_NUM, 0, UITARGETMAX, &MP.UITARGETrows, 0, "prov");

	/*               Minimum Weeks for Qualification */
	pmaddent(pcp, "2.3.2.1.1",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIREGMINWK",  (char *)&MP.UIREGMINWK,  NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIMATMINWK",  (char *)&MP.UIMATMINWK,  NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UISICMINWK",  (char *)&MP.UISICMINWK,  NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIFSHMINWK",  (char *)&MP.UIFSHMINWK,  NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIREGHRMIN",  (char *)&MP.UIREGHRMIN[0][0],F_LKTURI, P_LKPXY, C_NUM,    0, UIRMHRMAX, &MP.UIRMHRrows, 0);
	pmaddent(pcp, "UIFSHERMIN",  (char *)&MP.UIFSHERMIN[0][0],F_LKTURI, P_LKPXY, C_NUM,    0, UIFSHEMAX, &MP.UIFSHErows, 0);
	pmaddent(pcp, "UIMATHRMIN",  (char *)&MP.UIMATHRMIN,  NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UISICHRMIN",  (char *)&MP.UISICHRMIN,  NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UINEREHRMIN", (char *)&MP.UINEREHRMIN, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIEINEREFLG", (char *)&MP.UIEINEREFLG, NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);

	/*               Basic Parameters */
	pmaddent(pcp, "2.3.2.1.2",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIWAITWKS",   (char *)&MP.UIWAITWKS,   NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIDEPOPT",    (char *)&MP.UIDEPOPT,    NULL,   P_SCL, C_INT,    E_OPT,  2, NULL, 0);
	pmaddent(pcp, "UIEIOPT",     (char *)&MP.UIEIOPT,     NULL,   P_SCL, C_INT,    E_OPT,  2, NULL, 0);
	pmaddent(pcp, "UIREGWKFCT",  (char *)&MP.UIREGWKFCT,  F_FRACT,   P_SCL, C_NUM, E_FRCT,    0, NULL, 0);
	pmaddent(pcp, "UITRNWKFCT",  (char *)&MP.UITRNWKFCT,  F_FRACT,   P_SCL, C_NUM, E_FRCT,    0, NULL, 0);
	pmaddent(pcp, "UITRNBNFCT",  (char *)&MP.UITRNBNFCT,  F_FRACT,   P_SCL, C_NUM, E_FRCT,    0, NULL, 0);
	pmaddent(pcp, "UIOTHWKFCT",  (char *)&MP.UIOTHWKFCT,  F_FRACT,   P_SCL, C_NUM, E_FRCT,    0, NULL, 0);
	pmaddent(pcp, "UIOTHBNFCT",  (char *)&MP.UIOTHBNFCT,  F_FRACT,   P_SCL, C_NUM, E_FRCT,    0, NULL, 0);
	pmaddent(pcp, "UIEIYRS",     (char *)&MP.UIEIYRS,     NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIEIDIVFLG",  (char *)&MP.UIEIDIVFLG,  NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	pmaddent(pcp, "UIEIDIV",     (char *)&MP.UIEIDIV[0][0], NULL, P_LKPXY, C_NUM, 0, UIEIDIVMAX, &MP.UIEIDIVrows, 0);
	pmaddent(pcp, "UIEITKP",     (char *)&MP.UIEITKP[0],  "7.5",  P_VCT, C_NUM, 0, UIEITKPMAX, &MP.UIEITKProws, 0, "prov");
	pmaddent(pcp, "UIEIMTYPFLG", (char *)&MP.UIEIMTYPFLG, NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	pmaddent(pcp, "UIEIREPOPT",  (char *)&MP.UIEIREPOPT,  NULL,   P_SCL, C_INT,    E_OPT,  2, NULL, 0);


	/*               Benefit Duration */
	pmaddent(pcp, "2.3.2.1.3",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIMAXMATWKS", (char *)&MP.UIMAXMATWKS, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIEXTMATWKS", (char *)&MP.UIEXTMATWKS, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIMAXSICWKS", (char *)&MP.UIMAXSICWKS, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIMAXFSHWKS", (char *)&MP.UIMAXFSHWKS, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIMAXPARWKS", (char *)&MP.UIMAXPARWKS, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIMAXDUR",    (char *)&MP.UIMAXDUR,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "UIEWK",       (char *) MP.UIEWK,       F_WEEKS,   P_TBL, C_CHR, E_FIXL, UIEWKRMAX, &MP.UIEWKrows, UIEWKCMAX, "whwork", "uerelig");

	/*               Benefit Rates */
	pmaddent(pcp, "2.3.2.1.4",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIBASRATE",   (char *)&MP.UIBASRATE,   F_FRACT,P_SCL, C_NUM, 0,      0, NULL, 0);
	pmaddent(pcp, "UIENSRATE",   (char *)&MP.UIENSRATE,   F_FRACT,P_SCL, C_NUM, 0,      0, NULL, 0);
	pmaddent(pcp, "UIENSRTCO",   (char *)&MP.UIENSRTCO,   F_FRACT,P_SCL, C_NUM, 0,      0, NULL, 0);
	pmaddent(pcp, "UILFERATE",   (char *)&MP.UILFERATE,   F_FRACT,P_SCL, C_NUM, 0,      0, NULL, 0);
	pmaddent(pcp, "UIRGERATE",   (char *)&MP.UIRGERATE,   F_FRACT,P_SCL, C_NUM, 0,      0, NULL, 0);
	pmaddent(pcp, "UIEIRATE",    (char *)&MP.UIEIRATE[0][0],F_LKTUR, P_LKPXY, C_NUM, E_FRCT, UIEIRMAX, &MP.UIEIRrows, 0);

	/*               Option Activation */
	pmaddent(pcp, "2.3.2.1.5",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIENTFLAG",   (char *)&MP.UIENTFLAG,   NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	pmaddent(pcp, "UIBASFLAG",   (char *)&MP.UIBASFLAG,   NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	pmaddent(pcp, "UIEFFFLAG",   (char *)&MP.UIEFFFLAG,   NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);

	/*               Family supplement */
	pmaddent(pcp, "2.3.2.1.10",   NULL,                   NULL,  P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIEIFSFLG",   (char *)&MP.UIEIFSFLG,   NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	pmaddent(pcp, "UIEIFSOPT",   (char *)&MP.UIEIFSOPT,   NULL,    P_SCL, C_INT,    E_OPT,  2, NULL, 0);
	pmaddent(pcp, "UIEIFSRL",    (char *)&MP.UIEIFSRL,    NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "UIEIFSRR",    (char *)&MP.UIEIFSRR,    F_FRACT,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "UIEIFIFLG",   (char *)&MP.UIEIFIFLG,   NULL,   P_SCL, C_INT,    E_FLAG,   0, NULL, 0);
	pmaddent(pcp, "UIEIMFSP",    (char *)&MP.UIEIMFSP,    F_FRACT,   P_SCL, C_NUM, E_FRCT,    0, NULL, 0);
	pmaddent(pcp, "UIEIFS1",     (char *)&MP.UIEIFS1[0][0],F_LKTUR, P_LKPXY, C_NUM, E_NONE, UIEIFS1MAX, &MP.UIEIFS1rows, 0);
	pmaddent(pcp, "UIEIFS2",     (char *)&MP.UIEIFS2[0][0],F_LKTUR, P_LKPXY, C_NUM, E_NONE, UIEIFS2MAX, &MP.UIEIFS2rows, 0);
	pmaddent(pcp, "UIEIFS3",     (char *)&MP.UIEIFS3[0][0],F_LKTUR, P_LKPXY, C_NUM, E_NONE, UIEIFS3MAX, &MP.UIEIFS3rows, 0);
	pmaddent(pcp, "UIEIFSTOPUP", (char *)&MP.UIEIFSTOPUP[0][0],F_LKTUR, P_LKPXY, C_NUM, E_NONE, UIEIFSTOPUPMAX, &MP.UIEIFSTOPUProws, 0);
	pmaddent(pcp, "UIEIFSYNG",   (char *)&MP.UIEIFSYNG[0][0],F_LKTUR, P_LKPXY, C_NUM, E_NONE, UIEIFSYNGMAX, &MP.UIEIFSYNGrows, 0);

	/*               Repayment (through tax system) */
	pmaddent(pcp, "2.3.2.1.11",   NULL,                   NULL,  P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "UIBAF",    (char *)&MP.UIBAF,      F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIBAFNR",  (char *)&MP.UIBAFNR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIBRP",    (char *)&MP.UIBRP,      F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIEIRPR",  (char *)&MP.UIEIRPR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "UIEIBRP",  (char *)&MP.UIEIBRP[0][0],F_LKTUR, P_LKPXY, C_NUM, E_FRCT, UIEIBRPMAX, &MP.UIEIBRProws, 0);
	pmaddent(pcp, "UIEINRT",  (char *)&MP.UIEINRT, NULL,   P_SCL, C_INT,    0,      0, NULL, 0);


	/*          Quebec Parental Insurance Program */
	pmaddent(pcp, "2.3.2.1.12",   NULL,                   NULL, P_DUMMY, C_STR, 0, 0, NULL, 0);
	pmaddent(pcp, "QPIPFLAG",    (char *)&MP.QPIPFLAG,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QPIPTU",      (char *)&MP.QPIPTU,    F_FRACT,    P_SCL, C_NUM, 0,       0, NULL, 0);
	pmaddent(pcp, "QPIPSMTW",    (char *)&MP.QPIPSMTW,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPSPRW",    (char *)&MP.QPIPSPRW,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPSPTW",    (char *)&MP.QPIPSPTW,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPBMTW",    (char *)&MP.QPIPBMTW,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPBPRW1",   (char *)&MP.QPIPBPRW1,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPBPRW2",   (char *)&MP.QPIPBPRW2,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPBPTW",    (char *)&MP.QPIPBPTW,    NULL,   P_SCL, C_INT,    0,      0, NULL, 0);
	pmaddent(pcp, "QPIPSMTR",    (char *)&MP.QPIPSMTR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPSPRR",    (char *)&MP.QPIPSPRR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPSPTR",    (char *)&MP.QPIPSPTR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPBMTR",    (char *)&MP.QPIPBMTR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPBPRR1",   (char *)&MP.QPIPBPRR1,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPBPRR2",   (char *)&MP.QPIPBPRR2,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPBPTR",    (char *)&MP.QPIPBPTR,    F_FRACT,  P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPEMAX",    (char *)&MP.QPIPEMAX,     NULL, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPEMIN",    (char *)&MP.QPIPEMIN,     NULL, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QUIPF",       (char *)&MP.QUIPF,      F_FRACT, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPF",       (char *)&MP.QPIPF,      F_FRACT, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPFSE",     (char *)&MP.QPIPFSE,    F_FRACT, P_SCL,   C_NUM, 0,    0, NULL, 0);
	pmaddent(pcp, "QPIPEADJ",    (char *)&MP.QPIPEADJ,   F_FRACT, P_LKPXY,   C_NUM, 0,    QPIPEADJMAX, &MP.QPIPEADJrows, 0);

	/*          Family Allowance */
	pmaddent(pcp, "2.3.2.2",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "FAFLAG",   (char *)&MP.FAFLAG,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "PFACFLAG", (char *)&MP.PFACFLAG,  NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);

	/*               All Provinces Except Alberta and Quebec */
	pmaddent(pcp, "2.3.2.2.1",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "STDFA",    (char *)&MP.STDFA,     NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);

	/*               Alberta */										  
	pmaddent(pcp, "2.3.2.2.2",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "AFAC1",   (char *)&MP.AFAC1,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "AFAC2",   (char *)&MP.AFAC2,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "AFAC3",   (char *)&MP.AFAC3,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "AFAC4",   (char *)&MP.AFAC4,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC1",   (char *)&MP.ACBC1,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC2",   (char *)&MP.ACBC2,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC3",   (char *)&MP.ACBC3,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC4",   (char *)&MP.ACBC4,      NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	/*               Alberta for the calendar year */										  
	pmaddent(pcp, "2.3.2.2.2.1",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "ACBC1P",  (char *)&MP.ACBC1P,     NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC2P",  (char *)&MP.ACBC2P,     NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC3P",  (char *)&MP.ACBC3P,     NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "ACBC4P",  (char *)&MP.ACBC4P,     NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);

	/*                Quebec */
	pmaddent(pcp, "2.3.2.2.3",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "QAAFA",     (char *)&MP.QAAFA[0][0],  "8.0 8.2 8.2", P_LKPSL, C_NUM, 0,      QAAFAMAX, &MP.QAAFArows, 0);
	pmaddent(pcp, "QAAFAFLAG", (char *)&MP.QAAFAFLAG,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QCBCFLG",   (char *)&MP.QCBCFLG,     NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QCBC1",     (char *)&MP.QCBC1,        NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QCBC2",     (char *)&MP.QCBC2,        NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QCBC3",     (char *)&MP.QCBC3,        NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QCBCS",     (char *)&MP.QCBCS,        NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QFAIFLAG",  (char *)&MP.QFAIFLAG,     NULL,    P_SCL, C_INT, E_FLAG,  0, NULL, 0);
	pmaddent(pcp, "QFFSL",     (char *)&MP.QFFSL[0][0],  "8.0 8.2 8.2", P_LKPSL, C_NUM, 0,      QFFSLMAX, &MP.QFFSLrows, 0);
	pmaddent(pcp, "QFPSL",     (char *)&MP.QFPSL[0][0],  "8.0 8.2 8.2", P_LKPSL, C_NUM, 0,      QFPSLMAX, &MP.QFPSLrows, 0);
	pmaddent(pcp, "QFS",       (char *)&MP.QFS,           NULL,   P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QNBFA",     (char *)MP.QNBFA,         "8.2",   P_TBL,    C_NUM, E_FIXL, QNBFARMAX, &MP.QNBFArows, QNBFACMAX, "qnbfarank", "qnbfaage");
	pmaddent(pcp, "QNBFAFLAG", (char *)&MP.QNBFAFLAG,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QNBRFLAG",  (char *)&MP.QNBRFLAG,     NULL,    P_SCL, C_INT, E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QNBPOFLAG", (char *)&MP.QNBPOFLAG,    NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QNBPOPYR",  (char *)&MP.QNBPOPYR,   F_FRACT,  P_SCL,   C_NUM, 0,       0, NULL, 0);				  
	pmaddent(pcp, "QNBPOYR",   (char *)&MP.QNBPOYR,   NULL,    P_SCL, C_INT,    0,      0, NULL, 0);

	/*                Quebec */
	pmaddent(pcp, "2.3.2.2.3.1",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "QCBCFLGP",  (char *)&MP.QCBCFLGP,     NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "QCBC1P",    (char *)&MP.QCBC1P,       NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QCBC2P",    (char *)&MP.QCBC2P,       NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QCBC3P",    (char *)&MP.QCBC3P,       NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "QCBCSP",    (char *)&MP.QCBCSP,       NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);

	/*                 Repayment (through tax system) */
	pmaddent(pcp, "2.3.2.2.4",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "FATBPI",  (char *)&MP.FATBPI,     F_FRACT, P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "FATD",    (char *)&MP.FATD,       NULL,    P_SCL, C_NUM,  0,     0, NULL, 0);
	pmaddent(pcp, "FARR",    (char *)&MP.FARR,       F_FRACT, P_SCL, C_NUM,  0,     0, NULL, 0);

	/*          Old Age Security (OAS) */
	pmaddent(pcp, "2.3.2.3",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "OASFLAG",  (char *)&MP.OASFLAG,   NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "BOAS",    (char *)&MP.BOAS,       NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);

	/*                 Repayment (through tax system) */
	pmaddent(pcp, "2.3.2.3.1", NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "NETOASFLG",(char *)&MP.NETOASFLG,   NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "OASTD",   (char *)&MP.OASTD,      "9.2",    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "OASRR",   (char *)&MP.OASRR,      F_FRACT, P_SCL, C_NUM,  0,      0, NULL, 0);

	/*          Guaranteed Income Supplement */
	pmaddent(pcp, "2.3.2.4",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "GISFLAG",  (char *)&MP.GISFLAG,   NULL,    P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "GISOASFLAG",(char *)&MP.GISOASFLAG,NULL,   P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "2.3.2.4.1",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "BGISS",   (char *)&MP.BGISS,      NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "BGISM",   (char *)&MP.BGISM,      NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "BESPA",   (char *)&MP.BESPA,      NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "PYINC",   (char *)&MP.PYINC,      F_FRACT, P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "PYINCP",  (char *)&MP.PYINCP,     F_FRACT, P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "GISRLS",  (char *)&MP.GISRLS,     NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "GISRLM",  (char *)&MP.GISRLM,     NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "SPARL",    (char *)&MP.SPARL,     NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "GISRRS",  (char *)&MP.GISRRS,     F_FRACT,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "GISRRM",  (char *)&MP.GISRRM,     F_FRACT,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "SPAOASRR", (char *)&MP.SPAOASRR,  F_FRACT,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "GISEMPEXP", (char *)&MP.GISEMPEXP,  F_FRACT,    P_SCL, C_NUM,  0,      0, NULL, 0);
	pmaddent(pcp, "GISEMPEXM",   (char *)&MP.GISEMPEXM, NULL,    P_SCL, C_NUM,  0,      0, NULL, 0);

	/*               GIS Take-up Rates */
	pmaddent(pcp, "2.3.2.4.2",   NULL,                     NULL,    P_DUMMY, C_STR, 0,    0, NULL, 0);
	pmaddent(pcp, "GISTURFLAG", (char *)&MP.GISTURFLAG,NULL,  P_SCL, C_INT,  E_FLAG, 0, NULL, 0);
	pmaddent(pcp, "GISST",   (char *)&MP.GISST[0][0],F_LKTUR, P_LKPXY,C_NUM, E_FRCT, GISSTMAX, &MP.GISSTrows, 0);
	pmaddent(pcp, "GISCT",   (char *)&MP.GISCT[0][0],F_LKTUR, P_LKPXY,C_NUM, E_FRCT, GISCTMAX, &MP.GISCTrows, 0);
	pmaddent(pcp, "GISOT",   (char *)&MP.GISOT[0][0],F_LKTUR, P_LKPXY,C_NUM, E_FRCT, GISOTMAX, &MP.GISOTrows, 0);
	pmaddent(pcp, "SPAT",    (char *)&MP.SPAT[0][0], F_LKTUR, P_LKPXY,C_NUM, E_FRCT, SPATMAX, &MP.SPATrows, 0);
	pmaddent(pcp, "ESPAT",   (char *)&MP.ESPAT[0][0],F_LKTUR, P_LKPXY,C_NUM, E_FRCT, ESPATMAX, &MP.ESPATrows, 0);


	DEBUG_OFF("mpd1");
}
